#!/bin/sh
#
# GSI configuration script
# By W.H.Scholten, 1998 - 2000
#
# This file is public domain.
#
# Wowzers! Look at all the options Penny!


cd Config
# default/other script
Ask="sh ./Choose.sh"


echo "GSI configuration:"

# Figure out what kind of system we are on:
system=`uname`
if [ "X$system" = "X" ]; then
	echo "uname failed!"
	exit 1
else
	echo "It seems we are running on $system."
fi

# Feedback request:
if [ "X$system" != "XOpenBSD" ] && [ "X$system" != "XFreeBSD" ]; then
	echo " => I don't have access to a computer with your OS. Please let me"
	echo "    know about any problems."
fi
echo


#-----------------------------------------------------------------------------#
# Here we go.
#-----------------------------------------------------------------------------#
extraobjs=""
extralibs=""
extralibs_mixer=""
extralibs_pcm=""
extralibs_synth=""
extralibs_cd=""

config_file="../lib/include/gsi/config.h"

rm -f $config_file

# All the configuration options:
echo "/* GSI configuration created by Configure.sh */" >$config_file

D="#define "
U="#undef"

#echo "$D SYSTEM $system" >>$config_file


# Byte order
echo "#define SYSCONFIG_LITTLE_ENDIAN 1234" >>$config_file
echo "#define SYSCONFIG_BIG_ENDIAN 4321" >>$config_file
if [ "X$system" = "XOpenBSD" ] || [ "X$system" = "XNetBSD" ]; then
#	Net/OpenBSD: defined in <stdio.h> -> <sys/types.h> -> <machine/endian.h> (but only if not strict ANSI -> include <sys/types.h> is better? (need to add extra -I/usr/include/sys then)
	echo "#include <stdio.h>" >>$config_file
elif [ "X$system" = "XLinux" ]; then
#	Linux: defined in <endian.h>
	echo "#include <endian.h>" >>$config_file
elif [ "X$system" = "XFreeBSD" ]; then
#       Defined in <machine/endian.h>
	echo "#include <machine/endian.h>" >>$config_file
elif [ "`$Ask 'Is your machine/C-compiler little endian (usually on intel)?' 'y'`" != "n" ]; then
	echo "$D SYSCONFIG_LITTLE_ENDIAN_MACHINE" >>$config_file
	echo "$D SYSCONFIG_MACHINE_BYTE_ORDER  SYSCONFIG_LITTLE_ENDIAN" >>$config_file
else
	echo "$D SYSCONFIG_BIG_ENDIAN_MACHINE" >>$config_file
	echo "$D SYSCONFIG_MACHINE_BYTE_ORDER  SYSCONFIG_BIG_ENDIAN" >>$config_file
fi
if [ "X$system" = "XOpenBSD" ] || [ "X$system" = "XNetBSD" ] || [ "X$system" = "XFreeBSD" ] || [ "X$system" = "XLinux" ]; then
	echo "#if BYTE_ORDER==LITTLE_ENDIAN" >>$config_file
	echo "$D SYSCONFIG_LITTLE_ENDIAN_MACHINE" >>$config_file
	echo "$D SYSCONFIG_MACHINE_BYTE_ORDER  SYSCONFIG_LITTLE_ENDIAN" >>$config_file
	echo "#else" >>$config_file
	echo "$D SYSCONFIG_BIG_ENDIAN_MACHINE" >>$config_file
	echo "$D SYSCONFIG_MACHINE_BYTE_ORDER  SYSCONFIG_BIG_ENDIAN" >>$config_file
	echo "#endif" >>$config_file
fi
#if [ "X$system" = "XSunOS" ]; then
#fi


# Audio API:
# ----------
echo "Choose your audio system API:"
echo "  1. OSS (open sound system, version 3.8 or higher)"
echo "     -- For the  commercial, free (linux/FreeBSD) and emulated (OpenBSD/"
echo "        NetBSD & OSS emulation of FreeBSD's pcm driver) versions"
echo "  2. Ultrasound drivers (these only work with linux 2.0.x, 2.1.x)"
echo "        The ultra support uses native MIDI (you must compile libgus with"
echo "        MIDI support), native mixer, and /dev/dsp OSS compatibility for"
echo "        pcm (I can't get native pcm to work properly, help!))."
echo "  3. FreeBSD audio driver"
echo "  4. OpenBSD/NetBSD audio driver"
echo "  5. generic /dev/audio (8 kHz mono mulaw)"
echo
api_choice="0"
api_default=1

if [ "X$system" = "XFreeBSD" ]; then
    api_default=3
fi
if [ "X$system" = "XOpenBSD" ] || [ "X$OS" = "XNetBSD" ]; then
    api_default=4
fi

while [ "$api_choice" = "0" ]; do
    api_choice="`$Ask "Which one? " $api_default`"
    case $api_choice in
	1 ) api_choice="OSS"
	    ;;

	2 ) api_choice="ULTRASOUND"
	    extralibs_mixer="$extralibs_mixer -lgus"
	    extralibs_pcm="$extralibs_pcm -lgus"
	    extralibs_synth="$extralibs_synth -lgus"
	    ;;

	3 ) api_choice="FBSD"
	    ;;

	4 ) api_choice="OPENBSD_AUDIO"
	    ;;

	5 ) api_choice="DEV_AUDIO"
	    ;;

	* ) echo "Bad choice"
	    api_choice="0"
	    ;;
    esac
done
echo
echo "Using "$api_choice

APIS="OSS FBSD DEV_AUDIO ULTRASOUND OPENBSD_AUDIO"
for i in $APIS; do
	if [ $api_choice != $i ]; then
		echo "$D USE_"$i" 0" >>$config_file
	else
		echo "$D USE_"$i" 1" >>$config_file
	fi
done

use_midi=0
use_opl3=0
use_gus=0
use_awe32=0
# --------------
if [ $api_choice = "ULTRASOUND" ]; then
  #  use_gus=1
	use_gus=0
fi

# --------------
if [ $api_choice = "OSS" ]; then
	oss_includes="`$Ask "For the commercial OSS version enter the include directory:" ''`"
	if [ "X$oss_includes" != "X" ]; then
	    echo "$D OSS_INCLUDES \"$oss_includes\"" >> $config_file
	fi

	# MIDI support?
	if [ "`$Ask '  Compile in MIDI output support?' 'y'`" != "n" ]; then
	use_midi=1
	fi

	# OPL3 support?
	if [ "`$Ask '\n  Compile in OPL3 synth support?' 'y'`" != "n" ]; then
	use_opl3=1
	oss_opl_default_instr_dir="/etc/midi"
	if [ "X$oss_includes" != "X" ]; then
	    oss_opl_default_instr_dir=$oss_includes
	fi

	oss_opl_instr_dir=`$Ask '    In which directory can the OPL instrument files (drums.sb/std.sb/\n    drums.o3/std.o3) be found?' $oss_opl_default_instr_dir`
	echo "$D OSS_OPL_INSTRUMENT_DIR \"$oss_opl_instr_dir\"" >>$config_file

	echo
	if [ "`$Ask '    Use gsi_drums.sb? (sound better than the default drums.sb/drums.o3)' 'y'`" != "n" ]; then
		echo "$D USE_GSI_DRUMS 1" >>$config_file
		echo "      Aahh, much better! Note that you must install these drums by copying"
		echo "      them to the above instrument directory."
	else
		echo "$D USE_GSI_DRUMS 0" >>$config_file
		echo "      Are you sure that's a good idea?"
	fi

	echo
	if [ "`$Ask '    Assume CTL_MAIN_VOLUME (midi) on OPL is buggy?\n    (true for OSS 3.8 and earlier)' 'y'`" = "n" ]; then
		echo "$D OSS_OPL_CTL_MAIN_VOLUME_OK" >>$config_file
	fi
fi

	# GUS support?
	if [ "`$Ask '\n  Compile in GUS synth support?' 'y'`" != "n" ]; then
		use_gus=1

		gus_instrument_dir=/dos/ultrasnd/midi
		gus_instrument_dir="`$Ask '    In which directory can the GUS instruments be found?' $gus_instrument_dir`"
		echo "$D GUS_INSTRUMENT_DIR \"$gus_instrument_dir\"" >>$config_file

		if [ "`$Ask '    Compile in all GUS instruments (i.e. include tremolo/vox)?' 'y'`" != "n" ]; then
			echo "$D GUS_HAVE_ALL_INSTRUMENTS" >>$config_file
		fi
	fi

	# AWE32 support?
	if [ "`$Ask '\n  Compile in AWE32 synth support (also for AWE64)?' 'n'`" != "n" ]; then
		use_awe32=1
	fi
	# all done.
	echo
fi #api_choice="OSS"

echo "$D USE_MIDI "$use_midi >>$config_file
echo "$D USE_OPL3_SYNTH "$use_opl3 >>$config_file
echo "$D USE_GUS_SYNTH "$use_gus >>$config_file
echo "$D USE_AWE_SYNTH "$use_awe32 >>$config_file


#-----------------------------------------------------------------------------#
# This should be in a sysconfig script (config/sysconfig/location=system)
#-----------------------------------------------------------------------------#
# Shared memory:
# --------------
echo
use_shm=0

have_shm=0
if [ "`$Ask 'SYSV shared memory/semaphore support?' 'y'`" != "n" ]; then
	use_shm=1
	have_shm=1
	echo "$D HAVE_SYSV_SHARED_MEMORY "$have_shm >>$config_file
fi
have_shm=0
if [ "`$Ask 'POSIX shared memory/semaphore support?' 'n'`" != "n" ]; then
	use_shm=1
	have_shm=1
	echo "$D HAVE_POSIX_SHARED_MEMORY "$have_shm >>$config_file
fi
echo "$D USE_SHARED_MEMORY "$use_shm >>$config_file
echo


# Libraries:
# ----------
# Dynamic/static choice for all libraries:
lib_flags_choice="not_done"

lib_flags_libgsi="dynamic"
lib_flags_libgsif="dynamic"
lib_flags_libgsimixer="dynamic"
lib_flags_libgsipcm="dynamic"
lib_flags_libgsisynth="dynamic"
lib_flags_libgsimidi_pcm="dynamic"
lib_flags_libgsicd="dynamic"

echo "Dynamic or static libraries? (to profile the library code, use static libraries)"
done="false"
while [ "X$done" = "Xfalse" ]; do
	echo "1. Toggle libgsi (currently $lib_flags_libgsi)"
	echo "2. Toggle libgsif (currently $lib_flags_libgsif)"
	echo "3. Toggle libgsimixer (currently $lib_flags_libgsimixer)"
	echo "4. Toggle libgsipcm (currently $lib_flags_libgsipcm)"
	echo "5. Toggle libgsisynth (currently $lib_flags_libgsisynth)"
	echo "6. Toggle libgsimidi_pcm (currently $lib_flags_libgsimidi_pcm)"
	echo "7. Toggle libgsicd (currently $lib_flags_libgsicd)"
	echo "a. All libraries dynamic"
	echo "b. All libraries static"
	lib_flags_choice="`$Ask 'lib action?' 'done'`"
	case $lib_flags_choice in
	1 ) if [ "X$lib_flags_libgsi" = "Xdynamic" ]; then lib_flags_libgsi="static"; else lib_flags_libgsi="dynamic"; fi
	;;
	2 ) if [ "X$lib_flags_libgsif" = "Xdynamic" ]; then lib_flags_libgsif="static"; else lib_flags_libgsif="dynamic"; fi
	;;
	3 ) if [ "X$lib_flags_libgsimixer" = "Xdynamic" ]; then lib_flags_libgsimixer="static"; else lib_flags_libgsimixer="dynamic"; fi
	;;
	4 ) if [ "X$lib_flags_libgsipcm" = "Xdynamic" ]; then lib_flags_libgsipcm="static"; else lib_flags_libgsipcm="dynamic"; fi
	;;
	5 )  if [ "X$lib_flags_libgsisynth" = "Xdynamic" ]; then lib_flags_libgsisynth="static"; else lib_flags_libgsisynth="dynamic"; fi
	;;
	6 ) if [ "X$lib_flags_libgsimidi_pcm" = "Xdynamic" ]; then lib_flags_libgsimidi_pcm="static"; else lib_flags_libgsimidi_pcm="dynamic"; fi
	;;
	7 ) if [ "X$lib_flags_libgsicd" = "Xdynamic" ]; then lib_flags_libgsicd="static"; else lib_flags_libgsicd="dynamic"; fi
	;;

	a ) lib_flags_libgsi="dynamic"
	    lib_flags_libgsif="dynamic"
	    lib_flags_libgsimixer="dynamic"
	    lib_flags_libgsipcm="dynamic"
	    lib_flags_libgsisynth="dynamic"
	    lib_flags_libgsimidi_pcm="dynamic"
	    lib_flags_libgsicd="dynamic"
	;;

	b ) lib_flags_libgsi="static"
	    lib_flags_libgsif="static"
	    lib_flags_libgsimixer="static"
	    lib_flags_libgsipcm="static"
	    lib_flags_libgsisynth="static"
	    lib_flags_libgsimidi_pcm="static"
	    lib_flags_libgsicd="static"
	;;

	*) done="true"
	esac
done

use_dynamic_libs=0
if [ "X$lib_flags_libgsi" = "Xdynamic" ]; then
    libgsi="\$(DYNAMIC_LIB)"; use_dynamic_libs=1;
else
    libgsi="\$(STATIC_LIB)"
fi
if [ "X$lib_flags_libgsif" = "Xdynamic" ]; then
    libgsif="\$(DYNAMIC_LIB)"; use_dynamic_libs=1;
else
    libgsif="\$(STATIC_LIB)"
fi
if [ "X$lib_flags_libgsimixer" = "Xdynamic" ]; then
    libgsimixer="\$(DYNAMIC_LIB)"; use_dynamic_libs=1;
else
    libgsimixer="\$(STATIC_LIB)"
fi
if [ "X$lib_flags_libgsipcm" = "Xdynamic" ]; then
    libgsipcm="\$(DYNAMIC_LIB)"; use_dynamic_libs=1;
else
    libgsipcm="\$(STATIC_LIB)"
fi
if [ "X$lib_flags_libgsisynth" = "Xdynamic" ]; then
    libgsisynth="\$(DYNAMIC_LIB)"; use_dynamic_libs=1;
else
    libgsisynth="\$(STATIC_LIB)"
fi

if [ "X$lib_flags_libgsimidi_pcm" = "Xdynamic" ]; then libgsimidi_pcm="\$(DYNAMIC_LIB)"; use_dynamic_libs=1; else libgsimidi_pcm="\$(STATIC_LIB)"; fi

if [ "X$lib_flags_libgsicd" = "Xdynamic" ]; then libgsicd="\$(DYNAMIC_LIB)"; use_dynamic_libs=1; else libgsicd="\$(STATIC_LIB)"; fi


if [ "X$use_dynamic_libs" = "X1" ]; then
    sysconfig_dynamic_lib_supported="YES"
    sysconfig_dynamic_lib_compile_flags="-fPIC"
#For CC:
#    sysconfig_dynamic_lib_link_flags="-Wl,-shared,-soname,\$(DYNAMIC_LIB_NAME)"
#For LD:
    sysconfig_dynamic_lib_link_flags="-shared -soname \$(DYNAMIC_LIB_NAME)"

#    if [ "X$system" = "XOpenBSD" ]; then
#	sysconfig_dynamic_lib_link_flags="ld  -Bshareable"
#    fi
    sysconfig_dynamic_lib_symlink="YES"
    if [ "X$system" = "XOpenBSD" ]; then
	sysconfig_dynamic_lib_symlink="NO"
    fi
else
    sysconfig_dynamic_lib_supported="NO"
fi
# END libs static/dynamic


# Threads:
# --------
# Are threads supported and what are the thread compile/link flags?
sysconfig_threads_supported="YES"
sysconfig_threads_makelib_cflags="-D_REENTRANT -D_THREAD_SAFE"
sysconfig_threads_lflags="-lpthread"
sysconfig_threads_cflags=""
if [ "X$system" = "XFreeBSD" ]; then
	#sysconfig_threads_use_cflags="-pthread"
	sysconfig_threads_lflags="-pthread"
fi
if [ "X$system" = "XOpenBSD" ]; then
	sysconfig_threads_cflags="-pthread"
	sysconfig_threads_lflags="-lc_r"
fi

# POSIX real time scheduling:
echo
if [ "`$Ask 'Does this OS have POSIX real time scheduling (POSIX 1003.1b)' 'y'`" != "n" ]; then
    sysconfig_posix_rt="1"
    echo "$D USE_POSIX_RT 1" >> $config_file
fi


# Compiler & optimize flags:
# --------------------------
# Compiler:
echo
cc="gcc"
cc=`$Ask 'What compiler do you want to use?' "$cc"`


# Optimize & debug:
echo
cflags_choice="not_done"
cflags_standard="-O2 -fomit-frame-pointer"
cflags_debug="-O2 -g"
cflags_debug_all="-O2 -g -DDEBUG"
cflags_profile="-O2 -pg"

while [ "X$cflags_choice" = "Xnot_done" ]; do
	echo "Compiler flags?"
	echo "  1. Standard: "$cflags_standard
	echo "  2. Debug: "$cflags_debug
	echo "  3. Debug + environment controlled output: "$cflags_debug_all
	echo "  4. Profiling: "$cflags_profile
	echo "  or enter custom flags."

	cflags_choice="`$Ask 'cflags?' '3'`"
	case $cflags_choice in
	1 ) cflags=$cflags_standard
	;;
	2 ) cflags=$cflags_debug
	;;
	3 ) cflags=$cflags_debug_all
	;;
	4 ) cflags=$cflags_profile
	;;
	* ) cflags=$cflags_choice
	esac
done

if [ "X$cflags_choice" = "X2" ] || [ "X$cflags_choice" = "X3" ]; then
	# Use the NANA debugging library?
	echo "You chose debugging compile flags, so:"
	if [ "`$Ask '  Use the NANA debugging library? (probably only usable for GCC)' 'n'`" != "n" ]; then
		#debug=$debug" -DUSE_NANA"
		cflags=$cflags" -DUSE_NANA"
		extralibs="$extralibs -lnana"
	fi
fi


#-----------------------------------------------------------------------------#
# Create a new makeflags.app file
rm -f makeflags.app
echo
echo "Creating new makeflags.app"

cf="makeflags.app"
echo "CC = "$cc > $cf
#echo "OPTIMIZE = "$optimize >> $cf
echo "OPTIMIZE = "$cflags >> $cf
echo "FEATURES = "$features >> $cf
echo "EXTRALIBS = "$extralibs >> $cf
echo "EXTRALIBS_MIXER = "$extralibs_mixer >> $cf
echo "EXTRALIBS_PCM = "$extralibs_pcm >> $cf
echo "EXTRALIBS_SYNTH = "$extralibs_synth >> $cf
echo "EXTRALIBS_CD = "$extralibs_cd >> $cf
#echo "DEBUG = "$debug >> $cf

echo >> $cf
echo "# Libaries:" >> $cf
echo "LIBGSI = $libgsi" >> $cf
echo "LIBGSIF = $libgsif" >> $cf
echo "LIBGSIMIXER = $libgsimixer" >> $cf
echo "LIBGSIPCM = $libgsipcm" >> $cf
echo "LIBGSISYNTH = $libgsisynth" >> $cf
echo "LIBGSIMIDI_PCM = $libgsimidi_pcm" >> $cf
echo "LIBGSICD = $libgsicd" >> $cf

echo >> $cf
echo "# Sysconfig:" >> $cf
echo "SYSCONFIG_DYNAMIC_LIB_SUPPORTED = $sysconfig_dynamic_lib_supported" >> $cf
echo "SYSCONFIG_DYNAMIC_LIB_COMPILE_FLAGS = $sysconfig_dynamic_lib_compile_flags" >> $cf
echo "SYSCONFIG_DYNAMIC_LIB_LINK_FLAGS = $sysconfig_dynamic_lib_link_flags" >> $cf
echo "SYSCONFIG_DYNAMIC_LIB_SYMLINK = $sysconfig_dynamic_lib_symlink" >> $cf

echo "SYSCONFIG_THREADS_SUPPORTED = $sysconfig_threads_supported" >> $cf
echo "SYSCONFIG_THREADS_MAKELIB_CFLAGS = $sysconfig_threads_makelib_cflags" >> $cf
echo "SYSCONFIG_THREADS_CFLAGS = $sysconfig_threads_cflags" >> $cf
echo "SYSCONFIG_THREADS_LFLAGS = $sysconfig_threads_lflags" >> $cf

## Now combine all makeflags files into one
sh ./Makeflags.sh

# EOF
