/* GSI - general sound interface
 *
 * gsifile (main.c):  gives information on sound/music files and converts them
 * to other fileformats and sample types.
 *
 * By W.H.Scholten 1998 - 2001
 *
 * This software is placed in the public domain and can be used freely
 * for any purpose. It comes without any kind of warranty, either
 * expressed or implied, including, but not limited to the implied
 * warranties of merchantability or fitness for a particular purpose.
 * Use it at your own risk. the author is not responsible for any damage
 * or consequences raised by use or inability to use this program.
 */


/* See the man page for all options and commands
 *
 *
 * export GSIAPP_DEBUG_LEVEL= n for debug output.
 */


/*__________________INCLUDES__________________*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <unistd.h>
#include <signal.h>

/* These two includes are really for internal GSI use .. */
#include <gsi/whs_types.h>
#define DEBUG_PREFIX "GSIAPP_"
#include <gsi/gsi_debug.h>

#include <gsi/gsif.h>
#include <gsi/gsif_midi.h>

#include <limits.h>
#include "util.h"



/*__________________PROGRAM___________________*/
void use() {
	fprintf(stderr,
#if 0
// "\nHELP ON GSIFILE:\n"
			"Use:\n"
			"  gsifile [file]\n"
			"\n"
			"  gsifile [-C file1 file2 [-f file_format] [-s samplerate] [-c channels] [-t type]]\n"
			"\nSee gsifile(1) for more information.\n"
#endif
#if 0
			"Use: gsifile [file]\n"
"\n"
			"     gsifile [-C file1 file2 [-f file_format] [-s samplerate] [-c channels] [-t type]]\n"
			"\nSee gsifile(1) for more information.\n"
#endif
#if 1
			"Use (see gsifile(1) for more information):\n"
			"  gsifile [file]\n"
			"\n"
			"  gsifile [-C file1 file2 [-f file_format] [-s samplerate] [-c channels] [-t type]]\n"
#endif
			"\n");
	exit(0);
}


void signal_handler(int signal) {
	if (signal == SIGCONT) {
		/* reinit curses or whatever */
	}
}


int main(int argc, char *argv[]) {
	int  i, j, k;
	int  new_samplerate, new_channels, new_type, new_file_type;
	char *old_filename, *new_filename;
	struct gsi_sound *snd;
	struct gsi_song  *sng;

	signal(SIGTTOU, SIG_IGN); /* otherwise the process stops at output when running in the background */
	signal(SIGCONT, signal_handler);

	if (argc == 1) {
		use();
	}

	for (j=1; j<argc; j++) {
		if (argv[j][0] == '-') {
			/* An option.
			 */

			if (argc <= j+1) use();
			switch(argv[j][1]) {
			case 'C':
				printf("File format conversion not yet implemented\n");
				exit(1);

				/* convert files
				 */

				old_filename = argv[j+1];
				new_filename = argv[j+2];
				/* init so that we don't change old values: */
				new_samplerate = -1;
				new_channels = -1;
				new_type = -1;
				new_file_type = -1;

				for (k=j+3; k<argc; k++) {
					if (argc >= k+1 && !strcmp(argv[k], "-f")) {
						new_file_type = sample_fileformat (argv[k+1]);
						status_check("file type", new_file_type, argv[k+1]);
					}
					else if (argc >= k+1 && !strcmp(argv[k], "-s")) {
						new_samplerate = (int) atol(argv[k+1]);
						range_check("sample rate", new_samplerate, 1, INT_MAX);
					}
					else if (argc >= k+1 && !strcmp(argv[k], "-c")) {
						new_channels = (int) atol(argv[k+1]);
						range_check("channels", new_channels, 1, INT_MAX);
					}
					else if (argc >= k+1 && !strcmp(argv[k], "-t")) {
						new_type = sample_num_from_name (argv[k+1]);
						status_check("sample type", new_type, argv[k+1]);
					}

					if (snd->filename) free(snd->filename);
					snd->filename = strdup(argv[j+2]);
					//gsif_get_structure(argv[j], 1, 0, 0, 0, 0, 0, &status);
					//gsif_convert_data (input_type, input_endian, input_data, samples, output_type, output_endian, converted_size);
					//gsif_save_sample(snd, new_samplerate, new_channels, new_type);
				}
				break;
			default: use();
			}
			j++;
		}
		else {
			/* give file information:
			 */
			float total_time = 0;
			float block_time;
			int   tempo_change = False;
			int   status = 0;

			sng = NULL;
			snd = gsif_get_structure(argv[j], 1, 0, 0, 0, 0, 0, &status);
			if (snd != NULL) {
				/* We found a valid sample:
				 */
				printf("\nFile \"%s\":\n format:  %s\n", argv[j], sample_fileformat_name(snd->file_format));
				if (snd->comment != NULL) {
					printf(" comment: %s\n", snd->comment);
				}
				else {
					printf(" no comment\n");
				}

				printf(" blocks: %d\n", snd->blocks);
				for (i=0; i<snd->blocks; i++) {
					struct gsi_block *blk;
					int  sample_i;

					blk = snd->block[i];
					sample_i = sample_num_from_type(blk->type);

					DEB1(DPRT("   -> sample type = %d\n", blk->type));

					printf(" %-2d: samplerate = %d, channels = %d, type = %s\n", i, blk->samplerate, blk->channels, sample_type_names[ sample_i ]);
					printf("     data offset: %ld\n     data_length: %ld bytes (%ld samples)\n", blk->data_offset, blk->data_length, (blk->data_length) / (blk->channels * bytes_per_sample[ sample_i ]));
					block_time = ((float) blk->data_length) / (blk->channels * bytes_per_sample[ sample_i ] * blk->samplerate);
					//if (block_time < 0) block_time /= -8; // for ADPCM 3,4,5 bits per sample
					printf("     duration %.2f s\n", block_time);
					total_time += block_time;
				}
				printf("\n total duration %.2f s\n\n", total_time);
				gsif_free_sound(snd);
			}
			else {
				sng = gsif_load_song(argv[j], 0 /*offset*/, &status);
			}

			if (sng != NULL) {
				/* We found a valid song:
				 */
				char   *format;
				struct gsi_song_status *song_status;
				struct gsi_song_event event;
				int    *track_ticks_of;
				double *track_time_of;
				int     instruments = 0;
				int     instrument[8] = { 0,0,0,0,0,0,0,0 };

				format = song_fileformat_name(sng->file_format);
				printf("\nFile \"%s\":\n format:  %s\n", argv[j], format);

				song_status = gsif_make_song_status(sng);
				track_ticks_of = malloc(sizeof(int) * sng->tracks);
				track_time_of = malloc(sizeof(double) * sng->tracks);

				for(;;) {
					int  ev_status;

					ev_status = gsif_midi_next_event(INT_MAX, song_status, -1, -1, &event);
					//fprintf(stderr, "ev_status=%d\n", ev_status);
					if (ev_status != 0)
						break;
					if (event.event == MIDIMETA_END_OF_TRACK) {
						track_ticks_of[ event.track ] = event.ticks;
						/* use song_status->time here for the real track
						 * duration as there may be tempo changes in the song..
						 */
						track_time_of[ event.track ] = song_status->time;
					}
					if (event.event == MIDIMETA_SET_TEMPO) {
						if (event.ticks > 0) {
							/* tempo change.. */
							tempo_change = True;
						}
					}

					if ( ((event.event &0xF0) == MIDI_PGM_CHANGE && ! (1 << (event.event & 0x0F) & song_status->percussion_chn)) || ((event.event & 0xF0) == MIDI_NOTE_ON && (song_status->percussion_chn & (1 << (event.event & 0x0F)) ) )) {
						/* add instrument to the list */
						int  pgm = event.data[0];

						if ((event.event & 0xF0) == MIDI_NOTE_ON) pgm |= 128;
						instruments ++;
						instrument[ pgm >> 5 ] |= (1 << (pgm & 31));
					}
					if (event.event == MIDIMETA_TEXT || event.event == MIDIMETA_COPYRIGHT_NOTICE || (event.event == MIDIMETA_MARKER && event.track == 0 && event.ticks == 0) || (event.event == MIDIMETA_SEQUENCE_NAME && event.track == 0 && event.ticks == 0) ) {
						//if (event.event == MIDIMETA_TEXT || event.event == MIDIMETA_COPYRIGHT_NOTICE) {
						/* add event data to comment string, or should this be
						 * done in gsif_load_song() ?
						 */
						char *str;
						int  comment_len;

						comment_len = 0;
						if (sng->comment) {
							comment_len = strlen(sng->comment);
							str = malloc(comment_len+2+event.length+1);
							memcpy(str, sng->comment, comment_len);
							str[ comment_len ] = ' ';
							memcpy(str+comment_len+1, event.data, event.length);
							str[ comment_len+2+event.length ] = 0;
							free(sng->comment);
						}
						else {
							str = malloc(comment_len+1+event.length+1);
							memcpy(str, event.data, event.length);
							str[ event.length ] = 0;
						}
						sng->comment = str;
						//fprintf(stderr, "event %d (trk=%d, ticks=%d), string = %s\n", event.event, event.track, event.ticks, str+comment_len);
					}

					/* All info in the file */
#if 0
					if (event.event == MIDIMETA_TEXT || event.event == MIDIMETA_COPYRIGHT_NOTICE || event.event == MIDIMETA_MARKER || event.event == MIDIMETA_SEQUENCE_NAME) {
						char *new_info_str;
						int   new_info_len;

						new_info_str = malloc(event_length + 32);
						sprintf(new_info_str, "%s at %d", event_name(event.event), event.ticks);
						new_info_len = strlen(new_info_str);
						strcpy(new_info_str + new_info_len, event.data, event_length);
						new_info_len += event_length;
						my_realloc(&info, info_len + new_info_len, MY_ALLOC__FATAL);
					}
#endif
/* test: */
/*
					if (event.event >= 3 && event.event <= 7) {
						char *str2;

						str2 = malloc(event.length+1);
						memcpy(str2, event.data, event.length);
						str2[ event.length ]=0;
fprintf(stderr, "event %d (trk=%d, ticks=%d), string = %s\n", event.event, event.track, event.ticks, str2);
						free(str2);
					}
*/


				} /* loop going through all song events */

				if (instruments == 0) {
					instruments ++;
					instrument[0] = 1; /* piano. */
				}

				if (sng->comment != NULL) {
					printf(" comment: %s\n", sng->comment);
				}
				else {
					printf(" no comment\n");
				}
				printf(" midi format: %d (%s)\n", sng->midi_format, midi_format_str(sng->midi_format));
				printf(" midi division: %d\n", sng->midi_division);
				printf(" midi tempo: %s\n", tempo_change ? "variable" : "fixed");

				printf(" tracks: %d\n", sng->tracks);
				for (i=0; i<sng->tracks; i++) {
					struct gsi_track *trk;
					int  track_ticks = track_ticks_of[i];
					double track_time = track_time_of[i];

					trk = sng->track[i];
					//track_time = (double) track_ticks *((double) song_status->midi_tempo *1.0e-6)/ (double) song_status->midi_division;

					if (i == 0)
						printf(" %-2d: %5ld bytes  %6d ticks (%6.2f s)\n", i, trk->data_length, track_ticks, (float) track_time);
					else
						printf(" %-2d: %5ld  ,,    %6d  ,,   (%6.2f s)\n", i, trk->data_length, track_ticks, (float) track_time);

					if (sng->midi_format < 2) {
						if (track_time >total_time) total_time = track_time;
					}
					else {
						total_time += track_time;
					}
				}
				printf("\n total duration %.2f s\n\n", total_time);

				printf(" Intruments used:\n");
				for (i=0; i<256; i++) {
					if (instrument[ i >> 5 ] & (1 << (i & 31))) {
						char *instrument = (i < 128) ? gsi_gmvoice_names[i] : gsi_gmdrum_names[ i & 127 ];
						if (instrument == NULL) instrument = "<non general midi drum>";
						printf("   %3d = %s\n", i, instrument);
					}
				}

				gsif_free_song(sng);
			}
			if (snd == NULL && sng == NULL) {
				printf("\nFile \"%s\": cannot load file (gsif: %s)\n", argv[j], gsif_strerror(status));
				continue;
			}
		}
	}

	return 0;
}


/*  EOF main.c */
