/* GSI - general sound interface
 *
 * gsifile (util.c):  utility functions & data tables.
 *
 * By W.H.Scholten 1998 - 2001
 *
 * This software is placed in the public domain and can be used freely
 * for any purpose. It comes without any kind of warranty, either
 * expressed or implied, including, but not limited to the implied
 * warranties of merchantability or fitness for a particular purpose.
 * Use it at your own risk. the author is not responsible for any damage
 * or consequences raised by use or inability to use this program.
 */



/*__________________  INCLUDES  __________________*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/* These two includes are really for internal GSI use .. */
#include <gsi/whs_types.h>
#define DEBUG_PREFIX "GSIAPP_"
//#include <gsi/gsi_debug.h>

#include <gsi/gsif.h>
#include "util.h"


void  status_check(char *wrt, int status, char *input) {
	if (status < 0) {
		fprintf(stderr, "Invalid value %s for %s\n", input, wrt);
		exit(1);
	}
}


void  range_check(char *wrt, int value, int _min, int _max) {
	if (value < _min || value > _max) {
		fprintf(stderr, "Invalid value %d for %s, range is %d - %d\n", value, wrt, _min, _max);
		exit(1);
	}
}


void  replace_ctrl_chars(char *str) {
	int i, str_len;

	str_len = strlen(str);
	for (i=0; i<str_len; i++) {
		if (str[i] < 32) {
			if (str[i] != '\n') str[i] = '_';
		}
	}
}


int my_realloc(char **ptr, int new_len, int error_handling) {
	char *new_ptr = realloc(*ptr, new_len);
	if (new_ptr == NULL) {
		if (error_handling == MY_ALLOC__FATAL) {
			fprintf(stderr, "Out of memory\n");
			exit(1);
		}
		else return -1;
	}

	(*ptr) = new_ptr;
	return 0;
}


/*
 * Samples:
 */
char *sample_fileformat_names[6] = {
	"RAW", "AU", "AIFF", "AIFC", "VOC", "WAV"
};

int  sample_fileformats[] = {
	GSI_FILE_RAW, GSI_FILE_AU, GSI_FILE_AIFF, GSI_FILE_AIFF, GSI_FILE_VOC, GSI_FILE_WAV
};

int sample_fileformat(char *name) {
	int  i;

	for (i=0; i<6; i++) {
		if (!strcmp(name, sample_fileformat_names[i]))
			return sample_fileformats[i];
	}
	return -1;
}

char *sample_fileformat_name(int format) {
	int  i;

	for (i=0; i<6; i++) {
		if (format == sample_fileformats[i])
			return sample_fileformat_names[i];
	}
	return NULL; /* return "unknown"; ?? */
}

char *sample_type_names[] = {
	"8BIT_ALAW", "8BIT_MULAW", "8BIT", "8BIT_SIGNED",
	"16BIT", "16BIT_LE", "24BIT", "24BIT_LE",
	"4BIT_ADPCM", "3BIT_ADPCM", "5BIT_ADPCM"
};
int  sample_types[] = {
	GSI_8BIT_ALAW, GSI_8BIT_MULAW, GSI_8BIT, GSI_8BIT_SIGNED,
	GSI_16BIT, GSI_16BIT_LE, GSI_24BIT, GSI_24BIT_LE,
	GSI_4BIT_ADPCM, GSI_3BIT_ADPCM, GSI_5BIT_ADPCM
};
int  bytes_per_sample[] = {
	1, 1, 1, 1, 2, 2, 3, 3, -4, -3, -5
};

int sample_num_from_type(int type) {
	int  i;

	for (i=0; i<8; i++)
		if (sample_types[i] == type) return i;

	return -1;
}

int sample_num_from_name(char *name) {
	int  i;

	for (i=0; i<8; i++) {
		if (!strcmp(name, sample_type_names[i]))
			return sample_types[i];
	}
	return -1;
}


/*
 * Songs:
 */
char *song_fileformat_name(int type) {
	switch (type) {
	case GSI_FILE_MIDI:
		return "MIDI";
	case GSI_FILE_HMP:
		return "HMP";
	case GSI_FILE_MUS:
		return "MUS";
	}
	return "unknown";
}

char *midi_format_str(int format) {
	switch(format) {
	case 0: return "one track";
	case 1: return "multiple simultaneous tracks";
	case 2: return "multiple consecutive tracks";
	}
	return "unknown";
}


/*  EOF util.c */
