/* GSI - general sound interface
 *
 * gsiplay (cd_action.c): cd playing commands & cdi.
 *
 * By W.H.Scholten 1998 - 2001
 *
 * This software is placed in the public domain and can be used freely
 * for any purpose. It comes without any kind of warranty, either
 * expressed or implied, including, but not limited to the implied
 * warranties of merchantability or fitness for a particular purpose.
 * Use it at your own risk. the author is not responsible for any damage
 * or consequences raised by use or inability to use this program.
 */


/*__________________INCLUDES___________________*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
//#include <time.h>
#include <unistd.h>

/* These two includes are really for internal GSI use .. */
#include <gsi/whs_types.h>
#define DEBUG_PREFIX "GSIAPP_"
//#include <gsi/gsi_debug.h>

#include <gsi/gsi.h>

#ifdef USE_CURSES
#  include <ncurses.h>
#endif

extern void use();
extern void close_curses();
extern void init_curses();

extern void cd_callback(int ch);


/*__________________VARIABLES___________________*/
extern gsi_server_t *c;
extern int j;

extern int my_cd;
extern int cd_status, cd_track, cd_m, cd_s, cd_f;
extern int got_tracks;
extern int got_status;

char *cd_status_string[6] = { "NONE", "PLAYING", "PAUSED", "NO_DISK", "NO_DRIVE", "UNAVAILABLE" };


/*__________________PROGRAM___________________*/
int _player_share_cd(gsi_server_t *c, int cd) {
	gsi_share_cd (c, my_cd);
	for(;;) {
		gsi_wait_for_event(c, -1,0); gsi_next_event(c);
		if (c->gsi_event_data[0] == GSI_EV_CD_BUSY && c->gsi_event_data[1] == cd) {
			fprintf(stderr, "CD device busy\n");
			return -1;
		}
		else if (c->gsi_event_data[0] == GSI_EV_CD_NO_DISK && c->gsi_event_data[1] == cd) {
			fprintf(stderr, "No CD in the drive\n");
		}
		else if (c->gsi_event_data[0] == GSI_EV_CD_SHARED && c->gsi_event_data[1] == cd) {
			break;
		}
	}
	return 0;
}
int _player_sync(gsi_server_t *c) {
#warning "should fix this to process events.. (and change gsi_sync in libgsi)"
	gsi_sync(c);
	return 0;
}

int cd_action(int argc, char **argv) {
	int  args_used = 1;

	if (!strcmp(argv[j], "cddevice")) {
		if (argc <= j+1) use();
		gsi_send_commands(c, 2, GSI_CMD_ADD_DEVICE, GSI_CD);
		//gsi_flush_all(c);
		gsi_send_string(c, argv[j+1]);
		/*gsi_send_commands(3, GSI_CMD_SELECT_DEVICE, GSI_CD, dev);*/
		gsi_flush(c);
		args_used++;
	}
	else if (!strcmp(argv[j], "cdplay")) {
		_player_share_cd(c, my_cd);
		gsi_play_cd (c, my_cd,  0,0,0,0, 255,255,255,255); /* entire cd */
		_player_sync(c);
	}
	else if (!strcmp(argv[j], "cdpause")) {
		_player_share_cd(c, my_cd);
		gsi_pause_cd (c, my_cd);
		_player_sync(c);
	}
	else if (!strcmp(argv[j], "cdresume")) {
		_player_share_cd(c, my_cd);
		gsi_resume_cd (c, my_cd);
		_player_sync(c);
	}
	else if (!strcmp(argv[j], "cdstop")) {
		_player_share_cd(c, my_cd);
		gsi_stop_cd (c, my_cd);
		_player_sync(c);
	}
	else if (!strcmp(argv[j], "cdnext")) {
		_player_share_cd(c, my_cd);
		got_status = False; gsi_get_cd_status (c, my_cd);
		while(!got_status) { gsi_wait_for_event(c, -1,0); gsi_next_event(c); }
		gsi_play_cd (c, my_cd,  c->gsi_cd_track_no+1, 0,0,0, 255,255,255,255);
		_player_sync(c);
	}
	else if (!strcmp(argv[j], "cdprev")) {
		_player_share_cd(c, my_cd);
		got_status = False; gsi_get_cd_status (c, my_cd);
		while(!got_status)  { gsi_wait_for_event(c, -1,0); gsi_next_event(c); }
		gsi_play_cd (c, my_cd,  c->gsi_cd_track_no-1, 0,0,0, 255,255,255,255);
		_player_sync(c);
	}
	else if (!strcmp(argv[j], "cdtrack")) {
		int  trk;

		if (j+1 >= argc) use();
		trk = atol(argv[j+1]);
		args_used++;
		_player_share_cd(c, my_cd);
		gsi_play_cd (c, my_cd,  trk, 0,0,0, 255,255,255,255);
		_player_sync(c);
	}
	else if (!strcmp(argv[j], "cdstatus")) {
		int  i;

		if (_player_share_cd(c, my_cd)) return -1;
		got_status = False;
		gsi_get_cd_status (c, my_cd);

		got_tracks = False;
		gsi_get_cd_tracks (c, my_cd);
		while(!got_tracks || !got_status) {
			gsi_wait_for_event(c, -1,0);
			gsi_next_event(c);
		}
		close_curses();

		printf("cd status: %s\n", (cd_status >= 0 && cd_status < 6) ? cd_status_string[ cd_status ] : "unknown");
		printf("current track: %d (msf %2d: %02d. %02d)\n", cd_track, cd_m, cd_s, cd_f);
		//for (i=0; i<=gsi_cd_track_no+1; i++) {
		printf("\ncontents:\n");
		printf("Track  start (msf)   duration (msf)\n");
		for (i=0; i<=c->gsi_cd_track_no; i++) {
			printf("%3d   ", i);
			if (c->gsi_cd_track_list[i].type == 0) {
				printf(" DATA\n");
			}
			else if (c->gsi_cd_track_list[i].type == 1) {
				int  lba, dm,ds,df;

				printf("%3d: %02d. %02d",
					   c->gsi_cd_track_list[i].min,
					   c->gsi_cd_track_list[i].sec,
					   c->gsi_cd_track_list[i].frame);

				lba = (c->gsi_cd_track_list[i+1].min - c->gsi_cd_track_list[i].min)*60*75 + (c->gsi_cd_track_list[i+1].sec - c->gsi_cd_track_list[i].sec)*75 + (c->gsi_cd_track_list[i+1].frame - c->gsi_cd_track_list[i].frame);

				dm =  lba / (60*75);
				ds = (lba / 75) % 60;
				df = (lba % 75);
				printf("   %3d: %02d. %02d\n", dm, ds, df);

				if ( (i == c->gsi_cd_track_no) ||
					 (i < c->gsi_cd_track_no && c->gsi_cd_track_list[i+1].type == 0) ) {
					printf("       end (msf)\n      %3d: %02d. %02d\n\n",
						   c->gsi_cd_track_list[ i+1 ].min,
						   c->gsi_cd_track_list[ i+1 ].sec,
						   c->gsi_cd_track_list[ i+1 ].frame);
				}
			}
			//else { // end of CD
			//	printf("          end   m.s.f %2d: %2d: %2d\n", gsi_cd_track_list[i].min, gsi_cd_track_list[i].sec, gsi_cd_track_list[i].frame);
			//}
		}
		//init_curses();
	}
	else if (!strcmp(argv[j], "cdeject")) {
		_player_share_cd(c, my_cd);
		gsi_eject_cd(c, my_cd);
		_player_sync(c);
	}
	else if (!strcmp(argv[j], "cdinsert")) {
		_player_share_cd(c, my_cd);
		gsi_insert_cd (c, my_cd);
		_player_sync(c);
	}
	else if (!strcmp(argv[j], "cdchange")) {
		int  slot;

		_player_share_cd(c, my_cd);
		if (j+1 >= argc) use();
		slot = atol(argv[j+1]);
		args_used++;
		gsi_select_cd_slot (c, my_cd, slot);
		_player_sync(c);
	}
	else if (!strcmp(argv[j], "cdi")) {
		/* interactive cd player! see, cdi isn't dead. */
#ifdef USE_CURSES
		int  play_cd = True;
		int  cd_paused = False;

		init_curses();
		/* should get cd status to find out if cd is already playing a track */

		_player_share_cd(c, my_cd);
		gsi_set_channel_callback(c, -2, cd_callback);

		cd_track = 0;
		while (play_cd) {
			fd_set  sel;
			int     max_sel;
			int     status;

			clear();
			move(0, 0); refresh();
			printf("Interactive cd player: (0..9 v bn pr s +- ei q)");
			move(1, 0); refresh();
			printf("Playing track: ");
			if (cd_track) {
				printf("%d", cd_track);
			} else {
				printf("none");
			}
			printf(" (volume %3.2f)", gsi_get_mixer_volume(c, GSI_CD, my_cd)/256.0);
			if (cd_paused) {
				printf(" PAUSED");
			}

			clrtoeol();
			refresh();
			FD_ZERO(&sel);
			FD_SET(STDIN_FILENO, &sel); /* watch stdin */
			FD_SET(gsi_event_fd(c), &sel);
			max_sel = gsi_event_fd(c) + STDIN_FILENO + 1;
//DEB(DPRT("before select (watching stdin=%d, gsi_event_fd=%d)\n", STDIN_FILENO, gsi_event_fd(c)));
			status = select (max_sel, &sel, NULL, NULL, NULL);
			DEB(DPRT("select status = %d\n", status));

			if (status > 0) {
//DEB(DPRT("status > 0\n"));
				if (FD_ISSET(STDIN_FILENO, &sel)) {
					byte b;

					b = getchar();
//DEB(DPRT("b=%0x\n", (int) b));
//DEB(sleep(1));
					if (b >= '0' && b <= '9') {
						gsi_play_cd (c, my_cd,  b-'0', 0,0,0, 255,255,255,255);
					} else {
						switch(b) {
						case 'v': gsi_play_cd (c, my_cd,  0,0,0,0,255,255,255,255);
							break;
						case KEY_RIGHT:
						case 'n':
							gsi_play_cd (c, my_cd,  cd_track+1, 0,0,0, 255,255,255,255);
							break;
						case KEY_LEFT:
						case 'b':
							gsi_play_cd (c, my_cd,  cd_track-1, 0,0,0, 255,255,255,255);
							break;
						case 'p': gsi_pause_cd (c, my_cd);
							cd_paused = True;
							break;
						case 'r': gsi_resume_cd (c, my_cd);
							cd_paused = False;
							break;
						case 's': gsi_stop_cd (c, my_cd);
							break;
						case 'e': gsi_eject_cd (c, my_cd); cd_track = 0;
							break;
						case 'i': gsi_insert_cd (c, my_cd);
							break;
						case 'q': gsi_stop_cd (c, my_cd); play_cd = False;
							break;
						case '-':
						case '_':
							gsi_set_cd_volume (c, my_cd, gsi_get_mixer_volume(c, GSI_CD, my_cd) - 5);
							break;
						case '=':
						case '+':
							gsi_set_cd_volume (c, my_cd, gsi_get_mixer_volume(c, GSI_CD, my_cd) + 5);
							break;
						}
					}
				}
				_player_sync(c);
			} else {
				/* read events from the server */
				int  available;
				DEB(DPRT("event available\n"));
				available = gsi_next_event(c);
			}
		}
#else
		/* Well, yes, actually it is. */
		close_curses();
		printf("gsiplay needs to be compiled with curses for cdi!\n");
		exit(1);
#endif
	}
	else args_used = 0;

	return args_used;
}

/*    EOF cd_action.c */
