/* GSI - general sound interface
 *
 * gsiplay (main.c): play a sample/song/cd and/or set mixer volumes.
 *
 * By W.H.Scholten 1998 - 2000
 *
 * This software is placed in the public domain and can be used freely
 * for any purpose. It comes without any kind of warranty, either
 * expressed or implied, including, but not limited to the implied
 * warranties of merchantability or fitness for a particular purpose.
 * Use it at your own risk. the author is not responsible for any damage
 * or consequences raised by use or inability to use this program.
 */


/* See the man page for all options and commands.
 *
 * export GSIAPP_DEBUG_LEVEL= n for debug output.
 *
 * ?? add options: -samplerate <n> -bit <8/16> -mono/-stereo for raw samples ?
 * ?? or pcmio like options ?:
 * ??    pcmio +play,stereo,44100,s16,loop 50s:20s song
 *
 */


/*__________________INCLUDES___________________*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <unistd.h>
#include <signal.h>

/* These two includes are really for internal GSI use .. */
#include <gsi/whs_types.h>
#define DEBUG_PREFIX "GSIPLAY_"
#include <gsi/gsi_debug.h>

#include <gsi/gsi.h>
#include <gsi/gsif.h>

#ifdef USE_CURSES
#  include <ncurses.h>
#endif


#define max(a,b) ((a)>(b) ?(a) :(b))

#define _PLAY_FILE 1


/*__________________VARIABLES___________________*/
gsi_server_t *c;
int  pcm_channels = 1;
int  pcm_fragsize = 8;
int  pcm_type = GSI_8BIT;
int  pcm_samplerate = 8000;


/*__________________PROTOTYPES___________________*/
extern int cd_action(int argc, char **argv);
extern int mixer_action(int argc, char **argv);

void close_curses();
void init_curses();
void cd_callback(int ch);


/*__________________PROGRAM___________________*/
void use() {
	close_curses();
	fprintf(stderr, "\nHELP ON GSIPLAY:\ngsiplay [-S] [-m master] [-p pcm vol] [-s synth vol] [-c cd vol]\n"
			"        [-V volume] [-v volume] [[r] file ] ...\n"
			"        [cdplay] [cdstop] [cdpause] [cdresume] [cdnext] [cdprev] [cdtrack <n>]\n"
			"        [cdstatus] [cdeject] [cdinsert] [cdchange <slot>]"
#ifdef USE_CURSES
			" [cdi]"
#endif
			" [cddevice <device>]"
			"\n\n"
			"Volumes are 0-256, but numbers prefixed by +/- change the current volume. The default softvolume is 128 (to avoid clipping)."
#ifdef USE_CURSES
			" While playing samples/songs or during cdi, +/- alter the volume, p pauses, r resumes, n next file/track, b previous file/track, q quits."
#endif
			"\n\nSee gsiplay(1) for more information.\n"
			"\n");
	exit(0);
}



enum { _PLAYSTATUS_NONE, _PLAYSTATUS_PLAYING, _PLAYSTATUS_PAUSED };

int  play_status = _PLAYSTATUS_NONE;

int  called_back = False;
int  play_started = False;
int  play_stopped = False;

void my_callback(int ch) {
	int  play_started = False;

	DEB1(DPRT("gsiplay: my_callback (ch=%d)\n", ch));
	if (ch == 0 && c->gsi_event_data[0] == GSI_EV_CHANNEL_END) {
//		if (play_status == _PLAYSTATUS_PLAYING) {
			play_stopped = True;
//		}
	}
	if (ch == 0 && c->gsi_event_data[0] == GSI_EV_CHANNEL_STARTED) {
		play_started = True;
	}
	called_back = True;
}


/* to go back to a previous sample/song: */
int  *file_index;
int  current_file;

int  last_error = -1;

void error_callback(int error) {
	DEB1(DPRT("gsiplay: error_callback (event no=%d)\n", error));
	last_error = error;
	if (error == GSI_EV_SERVER_KILLED) {
		close_curses();
		fprintf(stderr, "Server killed\n");
		exit(-1);
	}

	if (error == GSI_EV_NO_PCM) {
		close_curses();
		fprintf(stderr, "No pcm device available\n");
		exit(-1);
	}
	if (error == GSI_EV_NO_SYNTH) {
		close_curses();
		fprintf(stderr, "No synth device available\n");
		exit(-1);
	}
	if (error == GSI_EV_NO_SUCH_FILE) {
		last_error = -1;
		file_index[ current_file ] = 0;
		play_started = False;
		play_stopped = True;
	}

	called_back = True;
}

#warning my_cd fixed at 0
int  my_cd = 0;

int  cd_status, cd_track, cd_m, cd_s, cd_f;
int  got_tracks;
int  got_status;
void cd_callback(int ch) {
	DEB1(DPRT("gsiplay: cd_callback (cd=%d, cmd=%d data=%d)\n", (int) c->gsi_event_data[1], (int) c->gsi_event_data[0], (int) c->gsi_event_data[2]));
	if (c->gsi_event_data[0] == GSI_EV_CD_TRACK_STARTED) cd_track = c->gsi_event_data[2];
	if (c->gsi_event_data[0] == GSI_EV_CD_TRACK_END) cd_track = 0;
	if (c->gsi_event_data[0] == GSI_EV_CD_TRACK_LIST) got_tracks = True;
	if (c->gsi_event_data[0] == GSI_EV_CD_STATUS) {
		cd_status = c->gsi_event_data[2];
		cd_track = c->gsi_event_data[3]; cd_m = c->gsi_event_data[4]; cd_s = c->gsi_event_data[5]; cd_f = c->gsi_event_data[6];
		got_status = True;
	}
}

void info_callback (int cmd) {
	DEB1(DPRT("gsiplay: info_callback (cmd=%d)\n", (int) c->gsi_event_data[0]));
}


int  curses_initialized = False;
void init_curses() {
#ifdef USE_CURSES
	if (!curses_initialized) {
		/* see man (n)curses for more info on these functions: */
		initscr();
		cbreak();
		//nonl(); /* no newline */
		intrflush(stdscr, FALSE);
		keypad(stdscr, TRUE);

		curses_initialized = True;
	}
#endif
}
void close_curses() {
#ifdef USE_CURSES
	if (curses_initialized) {
		nocbreak(); /* buffer IO again */
		endwin();

		curses_initialized = False;
	}
#endif
}


void signal_handler(int signal) {
	if (signal == SIGCONT) {
		/* reinitialize curses */
#ifdef USE_CURSES
		if (curses_initialized) {
			init_curses();
			refresh(); /* won't repaint screen ?? */
		}
#endif
	}
}


int  j; /* index into current argument from argv */
int  softvolume = 128; /* the software mixing volume */
int  defaultvolume = 128;

int main(int argc, char *argv[]) {
	char pwd_buf[256];
	char *ptr;
	int  playing; /* flag for FILE */
	int  repeat = False;
	int  paused = False;


	signal(SIGTTOU, SIG_IGN); /* otherwise the process stops at output when running in the background */
	signal(SIGCONT, signal_handler);

	ptr = getcwd(pwd_buf, 256); /* longer and you're screwed */
	if (ptr == NULL) { perror("getcwd"); exit(1); }

	if (argc == 1) {
		use();
	}
#ifdef USE_CURSES
	//init_curses();
	atexit(close_curses);
#endif

DEB(DPRT("gsi_init now\n"));
	c = gsi_init(NULL);
DEB(DPRT("gsi_init done\n"));
	if (c == NULL) {
		close_curses();
		fprintf(stderr, "Failed to connect to gsi_server\n");
		return -1;
	}
	gsi_chdir(c, pwd_buf);
DEB(DPRT("gsi_chdir done\n"));

	gsi_set_error_callback(c, error_callback);
	gsi_set_info_callback(c, info_callback);
	gsi_set_channel_callback(c, -2, cd_callback); /* cd track's */

	file_index = calloc(1, sizeof(int) * argc);
	for (j = 1; j < argc; ) {
		int  args_done;

		args_done = mixer_action(argc, argv);
DEB(DPRT("mixer args %d\n", args_done));
		j += args_done;
		if (args_done) continue;
		args_done = cd_action(argc, argv);
DEB(DPRT("cd args %d\n", args_done));
		j += args_done;
		if (args_done) continue;

		if (!strcmp( argv[j], "r")) {
			repeat = True;
			j++;
			continue;
		}

		/* sample/song playing: */
		{
		int  kill_sent = False;

		current_file = j; /* needed in case of errors */

		file_index[j] = _PLAY_FILE + 1; /* this arg is a sample */
//		pcm_fragsize = 8+pcm_channels-1+gsif_sizeof_sample(pcm_type)-1+2; /* fast response is not needed */
//		gsi_init_pcm(c, pcm_samplerate, pcm_channels, pcm_type, pcm_fragsize);
//gsi_set_pcm_dt(dt);
		gsi_enable_events(c, GSI_EVMASK_CHANNEL);
		gsi_send_commands(c, 1, GSI_CMD_USE_BEST_PCM_SETTINGS);
		gsi_load_sound(c, 0, argv[j]);
		gsi_set_channel_callback(c, 0, my_callback);

		gsi_play_sound(c, 0, 0, repeat ? 0 : 1);
		gsi_set_channel_volume(c, 0, softvolume);
		playing = _PLAY_FILE;


		/* Now wait for the playing-done callback (or an error) */
		if (!curses_initialized) init_curses();

		called_back = False;
		kill_sent = False;
		paused = False;
		for (;;) {
			fd_set  sel;
			int     max_sel;

			gsi_flush(c); /* also handles events */
			while (gsi_next_event(c) == 0) ; /* read all events */

			if (play_started) {
				play_status = _PLAYSTATUS_PLAYING;
				play_started = False;
			}

			if (play_stopped) {
				play_status = _PLAYSTATUS_NONE;
				play_stopped = False;
				break;
			}

			FD_ZERO(&sel);
#ifdef USE_CURSES
			FD_SET(0, &sel); /* watch stdin for -/+ p r b n q */
			if (!kill_sent) {
				clear();
				refresh();
				move(1,0);
				//printf("Playing: %s (volume=%d)", argv[j], softvolume);
				printf("Playing: %s (volume=%3.2f)", argv[j], softvolume/256.0);
				if (paused) printf(" PAUSE ");
				refresh();
			}
#endif
			FD_SET(gsi_event_fd(c), &sel);
			max_sel = gsi_event_fd(c)+1;
			select (max_sel, &sel, NULL, NULL, NULL);
#ifdef USE_CURSES
			if (FD_ISSET(0, &sel)) {
				byte b;
				b = getchar();
				if (b == '-' /*|| b == '_'*/) {
					softvolume -= 5; if (softvolume<0) softvolume = 0;
						gsi_set_channel_volume(c, 0 /* channel */, softvolume);
				}
				if (/*b == '=' ||*/ b == '+') {
					softvolume += 5; if (softvolume > 256) softvolume = 256;
						gsi_set_channel_volume(c, 0, softvolume);
				}
				if (b == 'q') exit(0);
				if (b == 'n') { /* next sample/song */
					if (!kill_sent) {
						gsi_stop_channel(c, 0);
						kill_sent = True;
					}
				}
				if (b == 'b') { /* back to previous sample/song */
					do {
						j--;
					} while (j > 0 && file_index[j] == 0);
					if (j < 1) j = 1;
					j--;

					gsi_stop_channel(c, 0);
					kill_sent = True;
				}
				if (b == 'p') {
					if (!paused) {
						//gsi_send_commands(c, 2, GSI_CMD_PAUSE, 0);
						gsi_pause_channel(c, 0);
						paused = True;
					}
				}
				if (b == 'r') {
					if (paused) {
						//gsi_send_commands(c, 2, GSI_CMD_CONTINUE, 0);
						gsi_resume_channel(c, 0);
						paused = False;
					}
				}
			}
#endif /* USE_CURSES */
		}

		/* set defaults for next sample/song: */
		softvolume = defaultvolume;
		repeat = False;
		} /* sample/song playing */
		j++;
	} /* loop through all arguments */

	gsi_close(c);

	return 0;
}

/*    EOF main.c */
