/* GSI - general sound interface
 *
 * gsiplay (mixer_action.c): set mixer volumes.
 *
 * By W.H.Scholten 1998 - 2001
 *
 * This software is placed in the public domain and can be used freely
 * for any purpose. It comes without any kind of warranty, either
 * expressed or implied, including, but not limited to the implied
 * warranties of merchantability or fitness for a particular purpose.
 * Use it at your own risk. the author is not responsible for any damage
 * or consequences raised by use or inability to use this program.
 */


/*__________________INCLUDES___________________*/
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <unistd.h>
#include <signal.h>

/* These two includes are really for internal GSI use .. */
#include <gsi/whs_types.h>
#define DEBUG_PREFIX "GSIAPP_"
//#include <gsi/gsi_debug.h>

#include <gsi/gsi.h>
#include <gsi/gsif.h>

extern void use();
extern void close_curses();
extern void init_curses();

extern gsi_server_t *c;
extern int j, softvolume, defaultvolume;


/*__________________PROGRAM___________________*/
int mixer_action(int argc, char **argv) {
	int args_used = 0;

	if (argv[j][0] == '-' && strlen(argv[j]) == 2) {
		char option = argv[j][1];

		if (option == 'S') {
			/* Show mixer volumes */
			int  k, dev;

			gsi_sync(c);
			close_curses();
			printf("\nGSI volumes/multipliers (0-256):\n" \
				   "Volume:\n" \
				   "  master: %3d\n\n", gsi_get_master_volume(c));
			printf("Multipliers:\n");
			for (k=1; k<GSIDEV_NUM_DEVICE_TYPES; k++) {
				for (dev=0; dev < c->_gsi_num_devices[k]; dev++) {
					printf("  %-8s %2d: %3d\n", gsi_device_type_name[k-1], dev, gsi_get_mixer_volume(c, k, dev));
				}
			}
			args_used = 1;
		}
		else {
			/* Set sample/mixer volumes */
			int  old_volume = 0;
			int  dev = 0;

			if (argc <= j+1) use();
			if (option == 'p' || option == 's' || option == 'c') {
				if (argc <= j+2) use();
				dev = atol(argv[j+1]);
				args_used++;
			}

			/* relative change: */
			if (argv[j+1][0] == '-' || argv[j+1][0] == '+') {
				switch(option) {
				case 'v':
				case 'V': break;

				case 'm': old_volume = gsi_get_master_volume(c);
					break;

				case 'p': old_volume = gsi_get_mixer_volume(c, GSI_PCM, dev);
					break;
				case 's': old_volume = gsi_get_mixer_volume(c, GSI_SYNTH, dev);
					break;
				case 'c': old_volume = gsi_get_mixer_volume(c, GSI_CD, dev);
					break;
				default: use();
				}
			}
			old_volume += atol(argv[j+1+args_used]);
			gsi_grab_mixer(c);
			switch(option) {
			case 'v': softvolume = old_volume;
				break;
			case 'V': softvolume = defaultvolume = old_volume;
				break;

			case 'm': gsi_set_master_volume(c, old_volume);
				break;

			case 'p': gsi_set_mixer_volume(c, GSI_PCM, dev, old_volume);
				break;
			case 's': gsi_set_mixer_volume(c, GSI_SYNTH, dev, old_volume);
				break;
			case 'c': gsi_set_mixer_volume(c, GSI_CD, dev, old_volume);
				break;
			default: use();
			}
			gsi_release_mixer(c);

			args_used += 2; /* option & volume & device; args_used was incremented for device earlier */
		}
	}

	return args_used;
}

/*    EOF mixer_action.c */
