/* GSI - general sound interface
 *
 * midi_control_null.c: controls the synthesizer with midi type events
 *
 *
 * Copyright (C) W.H.Scholten 1998 - 2001
 *
 * Redistribution and use in source and binary forms, with or
 * without modification, are permitted provided that the following
 * conditions are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS
 * IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
 * AUTHOR OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


/*__________________INCLUDES___________________*/
//#include <gsi/gsisynth_config.h>
#include <gsi/config.h>
/* Compile this code for all API's without hardware synthesizer support. */
#if  (USE_DEV_AUDIO || USE_FBSD || USE_OPENBSD_AUDIO)

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <sys/ioctl.h>
#include <unistd.h>


#include <gsi/whs_types.h>
#define DEBUG_PREFIX "GSISYNTH_"
//#include <gsi/gsi_debug.h>
#include <gsi/gsif_midi.h>
#include <gsi/synth_data.h>

#include "../gsisynth.h"


/*__________________EXTERNALS___________________*/
extern void *ultra_synth_handle;
extern gsisynth_device_t *synth_devices;


void api_seq_prepare(gsi_song_playstatus_t *d) {

}

/*---------------------------------------------------------------------------*
 *      Set a instrument for channel chn to pgm.
 *      Returns 0 if ok, -1 if an error occurred (instrument not set).
 *---------------------------------------------------------------------------*/
int api_seq_set_instrument(gsi_song_playstatus_t *d, int chn, int pgm) {
//	gsi_midi_status_t *mdata = d->song;

//UP	if (mdata->midi_force_chn_pgm[chn])
//UP		pgm = mdata->midi_force_chn_pgm[chn] - 1;

//UP	mdata->midi_channel[chn].program = pgm;

	return 0;
}


void api_seq_stop_note(gsi_song_playstatus_t *d, int chn, int note, int vel) {
//	gsi_midi_status_t *mdata = d->song;

//UP	mdata->midi_note_vel[chn][note] = 0;
}


void api_seq_key_pressure(gsi_song_playstatus_t *d, int chn, int note, int vel) {
}


void api_seq_start_note(gsi_song_playstatus_t *d, int chn, int note, int vel) {
//	gsi_midi_status_t *mdata = d->song;

//UP	mdata->midi_note_vel[chn][note] = vel;
}


void api_seq_control(gsi_song_playstatus_t *d, int chn, int p1, int p2) {
}


void api_seq_chn_pressure(gsi_song_playstatus_t *d, int chn, int vel) {
//	gsi_midi_status_t *mdata = d->song;

//UP	mdata->midi_channel[chn].pressure = vel;
}


void api_seq_bender(gsi_song_playstatus_t *d, int chn, int p1, int p2) {
//	gsi_midi_status_t *mdata = d->song;
//	int  val;

//UP	val = (p2 << 7) + p1;
//UP	mdata->midi_channel[chn].bender = val;
}


/*---------------------------------------------------------------------------*
 *      Various MIDI stuff.
 *---------------------------------------------------------------------------*/
void api_seq_midi_out(gsi_song_playstatus_t *d, int ch, int data)
{
#if  USE_MIDI
#endif
}


/*---------------------------------------------------------------------------*
 *      Pause and continue use a workaround for OSS; MIDI timer doesn't seem
 *      to stop.
 *---------------------------------------------------------------------------*/
void api_seq_pause (gsi_song_playstatus_t *d) {
	//gus_midi_timer_stop();
	// mute the synth:
	//gus_midi_master_volume(GUS_MIDID_COMMON, 0); // vol=0-127;

	gsisynth_seq_flush(d);
}


// Make relative ticks from absolute ones with this:
int  last_ticks = 0x7fffffff;

void api_seq_continue (gsi_song_playstatus_t *d) {
	//gsi_midi_status_t *mdata = d->song;
	gsisynth_device_t *hw = d->hw;
	//gsisynth_devicedata_t *hw_status = d->hw_status;

	int  i;

		for (i = 0; i < hw->num_voices; i++) {
#  if 1
		//SEQ_START_NOTE(hw->listno, i, voices[i].note, 10);
#  else
		// experimental voice restarting:
		if (mdata->voices[i].channel == chn && voices[i].note == note) {
			mdata->voices[i].available = 0;
			mdata->voices[i].timestamp = ?;
			if (!mdata->midi_channel[chn].controller[MIDICTL_SUSTAIN] && !IS_PERC(chn)) {
				//gus_do_chn_note_on(gus_synth_handle, i, note, 64);
			}
		}
#  endif
	}

	// replace this section by gsisynth_seq_start_timer():
		//gus_midi_timer_continue();
	last_ticks = 0;
	// unmute the synth:
	//gus_midi_master_volume(GUS_MIDID_COMMON, 127); // vol=0...127;

	gsisynth_seq_flush(d);
}


/*---------------------------------------------------------------------------*
 *      MIDI timing.
 *---------------------------------------------------------------------------*/
void api_seq_start_timer(gsi_song_playstatus_t *d)
{
	//gus_midi_timer_start();
	last_ticks = 0;

	gsisynth_seq_flush(d);
}


void api_seq_wait_time(gsi_song_playstatus_t *d, int ticks)
{
	//gus_midi_wait((ticks-last_ticks)); last_ticks = ticks;
}



/*---------------------------------------------------------------------------*
 *      Send the sequencer data to the sound card.
 *---------------------------------------------------------------------------*/
int  api_seq_flush(gsi_song_playstatus_t *d)
{
	gsisynth_device_t *hw = d->hw;
	gsisynth_devicedata_t *hw_status = d->hw_status;

	if (hw_status->fd > 0) {
		if (!gsisynth_buffer_full(hw, hw_status)) {
			//status = gus_midi_write();
		}
		//gus_queue_flush(gus_synth_handle);

		//TEST:
		gsisynth_buffer_full(hw, hw_status);
	}

	return 0;
}
#endif /* (USE_DEV_AUDIO || USE_FBSD || USE_OPENBSD_AUDIO) */

/*      EOF midi_control_null.c */
