/* GSI - general sound interface
 *
 * midi_control_oss.c: controls the synthesizer with midi type events
 * requires: libgsif header files.
 *
 * Copyright (C) W.H.Scholten 1998 - 2001
 *
 * Redistribution and use in source and binary forms, with or
 * without modification, are permitted provided that the following
 * conditions are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS
 * IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
 * AUTHOR OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


/*__________________INCLUDES___________________*/
//#include <gsi/gsisynth_config.h>
#include <gsi/config.h>
#if  USE_OSS

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <sys/ioctl.h>
#include <unistd.h>


#include <gsi/whs_types.h>
#define DEBUG_PREFIX "GSISYNTH_"
#include <gsi/gsi_debug.h>
#include <gsi/gsif_midi.h>
#include <gsi/synth_data.h>

#include "../gsisynth.h"

#include "oss_instrument.h"
#include "../api.h"

#include "oss_includes.h"


/*__________________EXTERNALS___________________*/
SEQ_USE_EXTBUF();

extern gsisynth_device_t *synth_devices;

void api_seq_prepare(gsi_song_playstatus_t *d) {
	int chn;
	gsi_midi_status_t *mdata = d->song;

	for (chn = 0; chn < 16; chn++) {
//X		dev = mdata->midi_channel_output[chn];
//X		DEB1(DPRT("MIDI   init ch %d: dev=%d\n", chn, dev));
		gsisynth_seq_control(d, chn, MIDICTL_BANK_SELECT, 0);
		gsisynth_seq_control(d, chn, MIDICTL_EXT_EFF_DEPTH, mdata->reverb);
		gsisynth_seq_control(d, chn, MIDICTL_CHORUS_DEPTH, mdata->chorus);
		gsisynth_seq_control(d, chn, MIDICTL_MAIN_VOLUME, 127);
		gsisynth_seq_chn_pressure(d, chn, 127);
	}
}

/*---------------------------------------------------------------------------*
 *      Set a instrument for channel chn to pgm.
 *      Returns 0 if ok, -1 if an error occurred (instrument not set).
 *---------------------------------------------------------------------------*/
int api_seq_set_instrument(gsi_song_playstatus_t *d, int chn, int pgm) {
	gsi_midi_status_t *mdata = d->song;

	//gsisynth_device_t *hw = d->hw;
	gsisynth_devicedata_t *hw_status = d->hw_status;

	switch(synth_devices[mdata->midi_channel_output[chn]].type) {
	case MIDI_HW:
#if  USE_MIDI
		SEQ_MIDIOUT(mdata->midi_channel_output[chn], MIDI_PGM_CHANGE + chn);
		SEQ_MIDIOUT(mdata->midi_channel_output[chn], pgm);
#endif
		break;
	case SYNTH_HW_OPL:
#if  USE_OPL3_SYNTH
		if (hw_status->instrtype[pgm] == _INSTRUMENT_NONE) {

			//if (IS_PERC(chn))
			return -1;
		}
#endif
		break;
	case SYNTH_HW_GUS:
#if  USE_GUS_SYNTH
		DEB1(DPRT("   For GUS\n"));
		if (hw_status->instrtype[pgm] == _INSTRUMENT_NONE) {
			if (IS_PERC(chn))
				return -1;
			else {
				/* must replace any missing one, with knowledge of which one
				 * can be missed at the moment. For now, we simply replace the
				 * missing one with the first loaded gus program (make a
				 * similarity table ? or the closest in pgm number ?)
				 */
				for (pgm = 0; pgm<256; pgm++) {
					if (hw_status->instrtype[pgm] != _INSTRUMENT_NONE) break;
				}
			}
		}
#endif
		break;
	case SYNTH_HW_AWE32:
#if  USE_AWE_SYNTH
		SEQ_SET_PATCH(mdata->midi_channel_output[chn], chn, pgm);
#endif
		break;
	case SYNTH_PCM_EMULATION:
		break; /* not yet implemented */
	}
	DEB1(DPRT(" #api_seq_set_instrument#\n"));

	return 0;
}


void api_seq_stop_note(gsi_song_playstatus_t *d, int chn, int note, int vel) {
	gsi_midi_status_t *mdata = d->song;
	gsisynth_device_t *hw = d->hw;
	//gsisynth_devicedata_t *hw_status = d->hw_status;

	DEB2(DPRT("   ("__FILE__"): *gsisynth_seq_stop_note*\n"));
	DEB2(DPRT("     chn=%d, note=%d, vel=%d\n", chn, note, vel));
//UP	mdata->midi_note_vel [ chn ][ note ] = 0;

	switch(synth_devices[mdata->midi_channel_output[chn]].type) {
	case MIDI_HW:
#if  USE_MIDI
		SEQ_MIDIOUT(hw->listno, MIDI_NOTE_OFF + chn);
		SEQ_MIDIOUT(hw->listno, note);
		SEQ_MIDIOUT(hw->listno, vel);
#endif
		break;
	case SYNTH_HW_AWE32:
#if  USE_AWE_SYNTH
		SEQ_STOP_NOTE(hw->listno, chn, note, vel);
#endif
		break;
	default: // GUS, OPL
	{
		int  i;

		for (i = 0; i < hw->num_voices; i++) {
			if (mdata->voices[i].channel == chn && mdata->voices[i].note == note) {
				mdata->voices[i].available = 1;
				mdata->voices[i].timestamp /= 2;
				/* redefine CTL_SUSTAIN in gsi for all systems */
				if (!mdata->midi_channel[chn].controller[ MIDICTL_SUSTAIN ] && !IS_PERC(chn))
					SEQ_STOP_NOTE(hw->listno, i, note, vel);
			}
		}
	}
	break;
	}

	DEB2(DPRT("   #api_seq_stop_note#\n"));
}


void api_seq_key_pressure(gsi_song_playstatus_t *d, int chn, int note, int vel) {
	gsi_midi_status_t *mdata = d->song;
	gsisynth_device_t *hw = d->hw;
	//gsisynth_devicedata_t *hw_status = d->hw_status;

	DEB2(DPRT("   ("__FILE__"): *api_seq_key_pressure*\n"));
	DEB2(DPRT("     chn=%d, note=%d, vel=%d\n", chn, note, vel));

	switch(synth_devices[mdata->midi_channel_output[chn]].type) {
	case MIDI_HW:
#if  USE_MIDI
		SEQ_MIDIOUT(hw->listno, MIDI_KEY_PRESSURE + chn);
		SEQ_MIDIOUT(hw->listno, note);
		SEQ_MIDIOUT(hw->listno, vel);
#endif
		break;
	case SYNTH_HW_AWE32:
#if  USE_AWE_SYNTH
		AWE_KEY_PRESSURE(hw->listno, chn, note, vel);
#endif
		break;
	default: // GUS & OPL
	{
		int  i;

		for (i = 0; i < hw->num_voices; i++) {
			if (mdata->voices[i].channel == chn && mdata->voices[i].note == note) {
				SEQ_KEY_PRESSURE(hw->listno, i, note, vel);
			}
		}
	}
	break;
	}

	DEB2(DPRT("   #api_seq_key_pressure#\n"));
}


/* Returns oldest voice (first started) */
int new_voice(gsi_song_playstatus_t *d, int chn) {
	int  i, oldest, last;
	gsi_midi_status_t *mdata = d->song;
	gsisynth_device_t *hw = d->hw;
	gsisynth_devicedata_t *hw_status = d->hw_status;

	DEB2(DPRT("   ("__FILE__"): *new_voice*\n"));
	DEB2(DPRT("     chn=%d\n", chn));
	if (IS_FM(chn) && hw_status->instrtype[mdata->midi_channel[chn].program] == _INSTRUMENT_OPL3)

		last = 6; /* 4-op voice can only use the first 6 voices */
	else
		last = hw->num_voices;

	for (i = oldest = 0; i < last; i++) {
		if (mdata->voices[i].timestamp < mdata->voices[oldest].timestamp)
			oldest = i;
	}

	DEB2(DPRT("   #new_voice# (new=%d)\n", oldest));
	return oldest;
}


void api_seq_start_note(gsi_song_playstatus_t *d, int chn, int note, int vel) {
	gsi_midi_status_t *mdata = d->song;
	gsisynth_device_t *hw = d->hw;
	//gsisynth_devicedata_t *hw_status = d->hw_status;

	DEB2(DPRT("   ("__FILE__"): *api_seq_start_note*\n"));
	DEB2(DPRT("     chn=%d, note=%d, vel=%d\n", chn, note, vel));
//UP	mdata->midi_note_vel [ chn ][ note ] = vel;

	switch(synth_devices[mdata->midi_channel_output[chn]].type) {
	case MIDI_HW:
#if  USE_MIDI
		SEQ_MIDIOUT(hw->listno, MIDI_NOTE_ON + chn);
		SEQ_MIDIOUT(hw->listno, note);
		SEQ_MIDIOUT(hw->listno, vel);
#endif
		break;
	case SYNTH_HW_AWE32:
#if  USE_AWE_SYNTH
		if (vel == 0)
			gsisynth_seq_stop_note(d, chn, note, 64);
		else
			SEQ_START_NOTE(hw->listno, chn, note, vel);
#endif
		break;
	default: // GUS & OPL
		if (vel == 0)
			gsisynth_seq_stop_note(d, chn, note, 64);
		else {
			int  c; // chorus
			int  voice;

			voice = new_voice(d, chn);
			SEQ_SET_PATCH(hw->listno, voice, mdata->midi_channel[chn].program);
			DEB1(DPRT("set instrument voice=%d, ch=%d, progr=%d\n", voice, chn, mdata->midi_channel[chn].program));
//OLD (trying again, gives OPL sound?):
//			SEQ_BENDER_RANGE(hw->listno, voice, channel[chn].bender_range * 100);
//NEW:
			SEQ_CONTROL(hw->listno, voice, CTRL_PITCH_BENDER_RANGE, (mdata->midi_channel[chn].bender_range * 100));
			SEQ_BENDER(hw->listno, voice, mdata->midi_channel[chn].bender);
			SEQ_CONTROL(hw->listno, voice, CTL_PAN, mdata->midi_channel[chn].controller[MIDICTL_PAN]);
			SEQ_START_NOTE(hw->listno, voice, note, vel);
			mdata->voices[voice].note = note;
			mdata->voices[voice].channel = chn;
			mdata->voices[voice].timestamp = d->song_status->last_synth_ticks;
			mdata->voices[voice].available = 0;

			// Chorus handling
			if ((c = mdata->midi_channel[chn].controller[MIDICTL_CHORUS_DEPTH] * 8)) {
				if (mdata->midi_channel[chn].bender_range)
					c /= mdata->midi_channel[chn].bender_range;
				voice = new_voice(d, chn);
				SEQ_SET_PATCH(hw->listno, voice, mdata->midi_channel[chn].program);

//OLD:
//				SEQ_BENDER_RANGE(hw->listno, voice, mdata->midi_channel[chn].bender_range * 100);
//NEW:
				SEQ_CONTROL(hw->listno, voice, CTRL_PITCH_BENDER_RANGE, (mdata->midi_channel[chn].bender_range * 100));
				if (mdata->midi_channel[chn].bender + c < 0x4000) {
					SEQ_BENDER(hw->listno, voice, mdata->midi_channel[chn].bender + c);
				}
				else {
					SEQ_BENDER(hw->listno, voice, mdata->midi_channel[chn].bender - c);
				}
				// put chorus note on the left/right
				c = mdata->midi_channel[chn].controller[MIDICTL_PAN];
#if 1 /* accentuate chorus direction (ooh, roomy :) */
				if (c < 64)
					c = 0;
				else if (c > 64)
					c = 127;
#endif
				SEQ_CONTROL(hw->listno, voice, CTL_PAN, c);

#warning OPL: is the OSS octave bug still present? Assuming yes (adding 12 to 'note' in case of OPL).
				if (synth_devices[mdata->midi_channel_output[chn]].type == SYNTH_HW_OPL) note +=12;
				SEQ_START_NOTE(hw->listno, voice, note, vel);
				mdata->voices[voice].note = note;
				mdata->voices[voice].channel = chn;

				// chorus note lifespan
				mdata->voices[voice].timestamp = d->song_status->last_synth_ticks / 2;
				mdata->voices[voice].available = 0;
			}
		}
	}

	DEB2(DPRT("   #api_seq_start_note#\n"));
}


void api_seq_control(gsi_song_playstatus_t *d, int chn, int p1, int p2) {
	gsi_midi_status_t *mdata = d->song;
	gsisynth_device_t *hw = d->hw;
	//gsisynth_devicedata_t *hw_status = d->hw_status;

	DEB2(DPRT("   ("__FILE__"): *api_seq_control*\n"));
	DEB2(DPRT("     chn=%d, controller parm1=%d, conroller parm2=%d\n", chn, p1, p2));

//UP	mdata->midi_channel[ chn ].controller[ p1 ] = p2;

	// took a while before I found this to be the problem with OPL playing..
	// GUS & AWE are unaffected.
#  ifndef OSS_OPL_CTL_MAIN_VOLUME_OK
	// volume setting (=midi controller 7) really messes up OPL playing.
	if (synth_devices[mdata->midi_channel_output[chn]].type == SYNTH_HW_OPL) {
		if (p1 == 7 || p1 == 39)
			return;
	}
#  endif

	if (IS_MIDI(chn)) {
#if  USE_MIDI
		SEQ_MIDIOUT(hw->listno, MIDI_CTL_CHANGE + chn);
		SEQ_MIDIOUT(hw->listno, p1);
		SEQ_MIDIOUT(hw->listno, p2);
#endif
	}

	switch (p1) {
	case MIDICTL_SUSTAIN:
		switch(synth_devices[mdata->midi_channel_output[chn]].type) {
		case MIDI_HW:
			break;
		case SYNTH_HW_AWE32:
			SEQ_CONTROL(hw->listno, chn, p1, p2);
			break;
		default: // GUS & OPL
			if (p1 == MIDICTL_SUSTAIN && !p2) {
				int  i;

				for (i = 0; i < hw->num_voices; i++)
					if (mdata->voices[i].channel == chn && mdata->voices[i].available) {
						SEQ_STOP_NOTE(hw->listno, i, mdata->voices[i].note, 64);
						mdata->voices[i].available = 0;
					}
			}
			break;
		}
		break;

	case MIDICTL_REGIST_PARM_NUM_MSB:
		rpn_msb[chn] = p2;
		break;

	case MIDICTL_REGIST_PARM_NUM_LSB:
		rpn_lsb[chn] = p2;
		break;

	case MIDICTL_DATA_ENTRY:
		if (rpn_msb[chn] == 0 && rpn_lsb[chn] == 0) {
			mdata->midi_channel[chn].bender_range = p2;
			rpn_msb[chn] = rpn_lsb[chn] = 127;

			switch(synth_devices[mdata->midi_channel_output[chn]].type) {
			case MIDI_HW:
				break;
			case SYNTH_HW_AWE32:
				// must find voice number ..
				//SEQ_CONTROL(hw->listno, voice, CTRL_PITCH_BENDER_RANGE, (mdata->midi_channel[chn].bender_range * 100));
				break;
			default: // GUS & OPL
			{
				int  i;

				for (i = 0; i < hw->num_voices; i++) {
//NEW:
					SEQ_CONTROL(hw->listno, i, CTRL_PITCH_BENDER_RANGE, p2 * 100);
//OLD:
//					SEQ_BENDER_RANGE(hw->listno, i, p2 * 100);
				}
			}
			break;
			}
		}
		break;

	default:
		switch(synth_devices[mdata->midi_channel_output[chn]].type) {
		case MIDI_HW:
			break;
		case SYNTH_HW_AWE32:
			SEQ_CONTROL(hw->listno, chn, p1, p2);
			break;
		default: // GUS & OPL
			if ((p1 & 0xf0) == 0x00 || (p1 & 0xf0) == 0x50) {
				int  i;

				DEB2(DPRT("     hw->num_voices=%d\n", hw->num_voices));
				for (i = 0; i < hw->num_voices; i++)
					if (mdata->voices[i].channel == chn)
						SEQ_CONTROL(hw->listno, i, p1, p2);
			}
			break;
		}
		break;
	}

	DEB2(DPRT("   #api_seq_control#\n"));
}


void api_seq_chn_pressure(gsi_song_playstatus_t *d, int chn, int vel) {
	gsi_midi_status_t *mdata = d->song;
	gsisynth_device_t *hw = d->hw;
	//gsisynth_devicedata_t *hw_status = d->hw_status;

	DEB2(DPRT("   seq_chn_pressure:\n"));
	DEB2(DPRT("     chn=%d, vel=%d\n", chn, vel));
//UP	mdata->midi_channel[ chn ].pressure = vel;

	switch(synth_devices[mdata->midi_channel_output[chn]].type) {
	case MIDI_HW:
#if  USE_MIDI
		SEQ_MIDIOUT(hw->listno, MIDI_CHN_PRESSURE + chn);
		SEQ_MIDIOUT(hw->listno, vel);
#endif
		break;
	case SYNTH_HW_AWE32:
#if  USE_AWE_SYNTH
		AWE_CHN_PRESSURE(hw->listno, chn, vel);
#endif
		break;
	default: //GUS & OPL
	{
		int  i;

		for (i = 0; i < hw->num_voices; i++) {
			if (mdata->voices[i].channel == chn) {
				SEQ_KEY_PRESSURE(hw->listno, i, mdata->voices[i].note, vel);
			}
		}
	}
	break;
	}

	DEB2(DPRT("   #api_seq_chn_pressure#\n"));
}


void api_seq_bender(gsi_song_playstatus_t *d, int chn, int p1, int p2) {
	gsi_midi_status_t *mdata = d->song;
	gsisynth_device_t *hw = d->hw;
	//gsisynth_devicedata_t *hw_status = d->hw_status;

	int  val;

	DEB2(DPRT("   ("__FILE__"): *api_seq_bender*\n"));
	DEB2(DPRT("     chn=%d, p1=%d, p2=%d\n", chn, p1, p2));

	val = (p2 << 7) + p1;
	mdata->midi_channel[ chn ].bender = val;

	switch(synth_devices[mdata->midi_channel_output[chn]].type) {
	case MIDI_HW:
#if  USE_MIDI
		SEQ_MIDIOUT(hw->listno, MIDI_PITCH_BEND + chn);
		SEQ_MIDIOUT(hw->listno, p1);
		SEQ_MIDIOUT(hw->listno, p2);
#endif
		break;
	case SYNTH_HW_AWE32:
#if  USE_AWE_SYNTH
		SEQ_BENDER(hw->listno, chn, val);
#endif
		break;
	default: // GUS & OPL
	{
		int  i;

		for (i = 0; i < hw->num_voices; i++) {
			if (mdata->voices[i].channel == chn) {
				SEQ_BENDER(hw->listno, i, val);
			}
		}
	}
	break;
	}

	DEB2(DPRT("  #api_seq_bender#\n"));
}


/*---------------------------------------------------------------------------*
 *      Various MIDI stuff.
 *---------------------------------------------------------------------------*/
void api_seq_midi_out(gsi_song_playstatus_t *d, int ch, int data) {
#if  USE_MIDI
	SEQ_MIDIOUT(ch, data);
#endif
}


/*---------------------------------------------------------------------------*
 *      Pause and continue use a workaround for OSS; MIDI timer doesn't seem
 *      to stop.
 *---------------------------------------------------------------------------*/
void api_seq_pause (gsi_song_playstatus_t *d) {
	int  i;
	gsi_midi_status_t *mdata = d->song;

	gsisynth_device_t *hw = d->hw;
	//gsisynth_devicedata_t *hw_status = d->hw_status;

	//ioctl(hw_status->fd, SNDCTL_SEQ_RESET); // ??

	for (i = 0; i < hw->num_voices; i++) {

#    if 1
		SEQ_STOP_NOTE(hw->listno, i, mdata->voices[i].note, 64);
#    else
		if (mdata->voices[i].channel == chn && voices[i].note == note) {
			mdata->voices[i].available = 1;
			mdata->voices[i].timestamp /= 2;
			if (!mdata->midi_channel[chn].controller[MIDICTL_SUSTAIN] && !IS_PERC(chn))
				SEQ_STOP_NOTE(hw->listno, i, note, vel);
		}
#    endif
	}


	SEQ_STOP_TIMER();

	gsisynth_seq_flush(d);
}


void api_seq_continue (gsi_song_playstatus_t *d) {
	//gsi_midi_status_t *mdata = d->song;
	gsisynth_device_t *hw = d->hw;
	//gsisynth_devicedata_t *hw_status = d->hw_status;

	int  i;

		for (i = 0; i < hw->num_voices; i++) {
#  if 1
		//SEQ_START_NOTE(hw->listno, i, voices[i].note, 10);
#  else
		// experimental voice restarting:
		if (mdata->voices[i].channel == chn && voices[i].note == note) {
			mdata->voices[i].available = 0;
			mdata->voices[i].timestamp = ?;
			if (!mdata->midi_channel[chn].controller[MIDICTL_SUSTAIN] && !IS_PERC(chn)) {
				SEQ_START_NOTE(hw->listno, i, note, vel);
			}
		}
#  endif
	}

	// replace this section by gsisynth_seq_start_timer():
	//SEQ_CONTINUE_TIMER();
	SEQ_START_TIMER(); // using workaround for OSS midi timer bug.

	gsisynth_seq_flush(d);
}


/*---------------------------------------------------------------------------*
 *      MIDI timing.
 *---------------------------------------------------------------------------*/
void api_seq_start_timer(gsi_song_playstatus_t *d)
{
	SEQ_START_TIMER();

	gsisynth_seq_flush(d);
}


void api_seq_wait_time(gsi_song_playstatus_t *d, int ticks)
{
	SEQ_WAIT_TIME(ticks);
}



/*---------------------------------------------------------------------------*
 *      Send the sequencer data to the sound card.
 *---------------------------------------------------------------------------*/
int  api_seq_flush(gsi_song_playstatus_t *d)
{
	seqbuf_dump();

	return 0;
}
#endif /* USE_OSS */

/*      EOF midi_control_oss.c */
