/* GSI - general sound interface
 *
 * midi_control_ultra.c: controls the synthesizer with midi type events
 *
 *
 * Copyright (C) W.H.Scholten 1998 - 2001
 *
 * Redistribution and use in source and binary forms, with or
 * without modification, are permitted provided that the following
 * conditions are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS
 * IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
 * AUTHOR OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


/*__________________INCLUDES___________________*/
//#include <gsi/gsisynth_config.h>
#include <gsi/config.h>
#if  USE_ULTRASOUND

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <sys/ioctl.h>
#include <unistd.h>

#include <libgus.h>

#include <gsi/whs_types.h>
#define DEBUG_PREFIX "GSISYNTH_"
#include <gsi/gsi_debug.h>
#include <gsi/gsif_midi.h>
#include <gsi/synth_data.h>

#include "../gsisynth.h"
#include "../api.h"



/*__________________EXTERNALS___________________*/
extern void *ultra_synth_handle;

void api_seq_prepare(gsi_song_playstatus_t *d) {
#warning "ultra: api_seq_prepare empty, ok?"
}
/*---------------------------------------------------------------------------*
 *      Set a instrument for channel chn to pgm.
 *      Returns 0 if ok, -1 if an error occurred (instrument not set).
 *---------------------------------------------------------------------------*/
int api_seq_set_instrument(gsi_song_playstatus_t *d, int chn, int pgm) {
	gsi_midi_status_t *mdata = d->song;

	//gsisynth_device_t *hw = d->hw;
	gsisynth_devicedata_t *hw_status = d->hw_status;

	DEB1(DPRT("("__FILE__"): *api_seq_set_instrument*\n"));
	DEB1(DPRT("   ch=%d, pgm=%d\n", chn, pgm));

//UP	if (mdata->midi_force_chn_pgm[chn])
//UP		pgm = mdata->midi_force_chn_pgm[chn] - 1;

	switch(gsisynth_devices[mdata->midi_channel_output[chn]].type) {
	case MIDI_HW:
#if  USE_MIDI
		SEQ_MIDIOUT(mdata->midi_channel_output[chn], MIDI_PGM_CHANGE + chn);
		SEQ_MIDIOUT(mdata->midi_channel_output[chn], pgm);
#endif
		break;
	case SYNTH_HW_GUS:
		DEB1(DPRT("   For GUS\n"));
//XXX		if (gsisynth_devices[mdata->midi_channel_output[chn]].instrtype[pgm] == _INSTRUMENT_NONE) {
		if (hw_status->instrtype[pgm] == _INSTRUMENT_NONE) {

			if (IS_PERC(chn))
				return -1;
			else {
				/* must replace any missing one, with knowledge of which one
				 * can be missed at the moment. For now, we simply replace the
				 * missing one with the first loaded gus program (make a
				 * similarity table ? or the closest in pgm number ?)
				 */
				for (pgm = 0; pgm<256; pgm++) {
//XXX					if (gsisynth_devices[mdata->midi_channel_output[chn]].instrtype[pgm] != _INSTRUMENT_NONE) break;
					if (hw_status->instrtype[pgm] != _INSTRUMENT_NONE) break;
				}
			}
		}
		break;
	case SYNTH_PCM_EMULATION:
		break; /* not yet implemented */
	}
	DEB1(DPRT(" #api_seq_set_instrument#\n"));

//UP	mdata->midi_channel[chn].program = pgm;

	return 0;
}


void api_seq_stop_note(gsi_song_playstatus_t *d, int chn, int note, int vel) {
	gsi_midi_status_t *mdata = d->song;
	gsisynth_device_t *hw = d->hw;
	gsisynth_devicedata_t *hw_status = d->hw_status;


	DEB2(DPRT("   ("__FILE__"): *api_seq_stop_note*\n"));
	DEB2(DPRT("     chn=%d, note=%d, vel=%d\n", chn, note, vel));
//UP	mdata->midi_note_vel[chn][note] = 0;

	gus_midi_note_off(GUS_MIDID_COMMON, chn, note, vel);

	DEB2(DPRT("   #api_seq_stop_note#\n"));
}


void api_seq_key_pressure(gsi_song_playstatus_t *d, int chn, int note, int vel) {
	gsi_midi_status_t *mdata = d->song;
	gsisynth_device_t *hw = d->hw;
	gsisynth_devicedata_t *hw_status = d->hw_status;

	DEB2(DPRT("   ("__FILE__"): *api_seq_key_pressure*\n"));
	DEB2(DPRT("     chn=%d, note=%d, vel=%d\n", chn, note, vel));

	gus_midi_note_pressure(GUS_MIDID_COMMON, chn, note, vel);

	DEB2(DPRT("   #api_seq_key_pressure#\n"));
}


void api_seq_start_note(gsi_song_playstatus_t *d, int chn, int note, int vel) {
	gsi_midi_status_t *mdata = d->song;
	gsisynth_device_t *hw = d->hw;
	gsisynth_devicedata_t *hw_status = d->hw_status;

	DEB2(DPRT("   ("__FILE__"): *api_seq_start_note*\n"));
	DEB2(DPRT("     chn=%d, note=%d, vel=%d\n", chn, note, vel));
//UP	mdata->midi_note_vel[chn][note] = vel;

	gus_midi_note_on(GUS_MIDID_COMMON, chn, note, vel);

	DEB2(DPRT("   #api_seq_start_note#\n"));
}


void api_seq_control(gsi_song_playstatus_t *d, int chn, int p1, int p2) {
	gsi_midi_status_t *mdata = d->song;
	gsisynth_device_t *hw = d->hw;
	gsisynth_devicedata_t *hw_status = d->hw_status;

	DEB2(DPRT("   ("__FILE__"): *api_seq_control*\n"));
	DEB2(DPRT("     chn=%d, controller parm1=%d, conroller parm2=%d\n", chn, p1, p2));

//UP	mdata->midi_channel[chn].controller[p1] = p2;

	gus_midi_control(GUS_MIDID_COMMON, chn, p1, p2);

	DEB2(DPRT("   #api_seq_control#\n"));
}


void api_seq_chn_pressure(gsi_song_playstatus_t *d, int chn, int vel) {
	gsi_midi_status_t *mdata = d->song;
	gsisynth_device_t *hw = d->hw;
	gsisynth_devicedata_t *hw_status = d->hw_status;

	DEB2(DPRT("   seq_chn_pressure:\n"));
	DEB2(DPRT("     chn=%d, vel=%d\n", chn, vel));
//UP	mdata->midi_channel[chn].pressure = vel;

	//gus_midi_note_pressure(GUS_MIDID_COMMON, chn, /*p1*/ mdata->voices[i].note, /*p2*/ vel);
	gus_midi_note_pressure(GUS_MIDID_COMMON, chn, /*p1*/ chn, /*p2*/ vel);

	DEB2(DPRT("   #api_seq_chn_pressure#\n"));
}


void api_seq_bender(gsi_song_playstatus_t *d, int chn, int p1, int p2) {
	gsi_midi_status_t *mdata = d->song;
	gsisynth_device_t *hw = d->hw;
	gsisynth_devicedata_t *hw_status = d->hw_status;

	int  val;

	DEB2(DPRT("   ("__FILE__"): *api_seq_bender*\n"));
	DEB2(DPRT("     chn=%d, p1=%d, p2=%d\n", chn, p1, p2));
//UP	val = (p2 << 7) + p1;
//UP	mdata->midi_channel[chn].bender = val;

	gus_midi_bender(GUS_MIDID_COMMON, chn, /*pitchbend*/ val);

	DEB2(DPRT("  #api_seq_bender#\n"));
}


/*---------------------------------------------------------------------------*
 *      Various MIDI stuff.
 *---------------------------------------------------------------------------*/
void api_seq_midi_out(gsi_song_playstatus_t *d, int ch, int data) {
#if  USE_MIDI
#endif
}


/*---------------------------------------------------------------------------*
 *      Pause and continue use a workaround for OSS; MIDI timer doesn't seem
 *      to stop.
 *---------------------------------------------------------------------------*/
void api_seq_pause (gsi_song_playstatus_t *d) {
	int  i;
	gsi_midi_status_t *mdata = d->song;

	gsisynth_device_t *hw = d->hw;
	//gsisynth_devicedata_t *hw_status = d->hw_status;

	for (i = 0; i < hw->num_voices; i++) {
		//gus_do_chn_note_off(gus_synth_handle, i, note, /*vel*/ 64);
	}


	//for (i=0; i<16; i++)
	//		gus_midi_note_off(GUS_MIDID_COMMON, i, note, 64);
	gus_midi_timer_stop();
	// mute the synth:
	gus_midi_master_volume(GUS_MIDID_COMMON, 0); // vol=0-127;

	gsisynth_seq_flush(d);
}


// Make relative ticks from absolute ones with this:
int  last_ticks = 0x7fffffff;

void api_seq_continue (gsi_song_playstatus_t *d) {
	//gsi_midi_status_t *mdata = d->song;
	gsisynth_device_t *hw = d->hw;
	//gsisynth_devicedata_t *hw_status = d->hw_status;

	int  i;

		for (i = 0; i < hw->num_voices; i++) {
#  if 1
		//SEQ_START_NOTE(hw->listno, i, voices[i].note, 10);
#  else
		// experimental voice restarting:
		if (mdata->voices[i].channel == chn && voices[i].note == note) {
			mdata->voices[i].available = 0;
			mdata->voices[i].timestamp = ?;
			if (!mdata->midi_channel[chn].controller[MIDICTL_SUSTAIN] && !IS_PERC(chn)) {
				//gus_do_chn_note_on(gus_synth_handle, i, note, 64);
			}
		}
#  endif
	}

	// replace this section by gsisynth_seq_start_timer():
	gus_midi_timer_continue();
	last_ticks = 0;
	// unmute the synth:
	gus_midi_master_volume(GUS_MIDID_COMMON, 127); // vol=0...127;

	gsisynth_seq_flush(d);
}


/*---------------------------------------------------------------------------*
 *      MIDI timing.
 *---------------------------------------------------------------------------*/
void api_seq_start_timer(gsi_song_playstatus_t *d)
{
	gus_midi_timer_start();
	last_ticks = 0;

	gsisynth_seq_flush(d);
}


void api_seq_wait_time(gsi_song_playstatus_t *d, int ticks)
{
	//gus_midi_wait(ticks);
	gus_midi_wait((ticks-last_ticks)); last_ticks = ticks;
}



/*---------------------------------------------------------------------------*
 *      Send the sequencer data to the sound card.
 *---------------------------------------------------------------------------*/
int  api_seq_flush(gsi_song_playstatus_t *d)
{
	gsisynth_device_t *hw = d->hw;
	gsisynth_devicedata_t *hw_status = d->hw_status;

	int  status;

	if (hw_status->fd > 0) {
		if (!gsisynth_buffer_full(hw, hw_status)) {
			status = gus_midi_write();
			//status = gus_midi_flush();
		}
		//gus_queue_flush(gus_synth_handle);

		//TEST:
		gsisynth_buffer_full(hw, hw_status);
	}

	return 0;
}
#endif /* USE_ULTRASOUND */

/*      EOF midi_control_ultra.c */
