/* GSI - general sound interface
 *
 * synth_oss_gus.c: initializes synthesizer & loads instruments for GUS
 * soundcards.
 *
 *
 * Copyright (C) W.H.Scholten 1998 - 2001
 *
 * Redistribution and use in source and binary forms, with or
 * without modification, are permitted provided that the following
 * conditions are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS
 * IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
 * AUTHOR OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


/*__________________INCLUDES___________________*/
//#include <gsi/gsisynth_config.h>
#include <gsi/config.h>
#if  USE_OSS
#  if  USE_GUS_SYNTH

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <unistd.h>
#include <fcntl.h>

#include <sys/ioctl.h>


#include <gsi/whs_types.h>
#define DEBUG_PREFIX "GSISYNTH_"
#include <gsi/gsi_debug.h>
#include <gsi/gsi_instrumentnames_gus.h>

#include "../gsisynth.h"


#include "oss_instrument.h"
#include "oss_includes.h"
#include "../io.h"
#include "../log.h"


/*__________________DEFINITIES___________________*/
#ifndef GUS_INSTRUMENT_DIR
#  warning  assuming GUS_INSTRUMENT_DIR to be /dos/ultrasnd/midi
#  define GUS_INSTRUMENT_DIR "/dos/ultrasnd/midi/"
#endif


/*__________________EXTERN___________________*/
SEQ_USE_EXTBUF();


/*__________________FUNCTIE PROTOTYPES___________________*/


/*__________________VARIABELEN___________________*/
char *gus_instrument_dir = GUS_INSTRUMENT_DIR;


/* This is the header as used in instrument files */
struct pat_header {
	char            magic[ 12 ];
	char            version[ 10 ];
	char            description[ 60 ];
	unsigned char   instruments;
	char            voices;
	char            channels;
	unsigned short  nr_waveforms;
	unsigned short  master_volume;
	unsigned int    data_size;
};


/*---------------------------------------------------------------------------*
 *      Init GUS (reset, set voices=24, get available memory).
 *---------------------------------------------------------------------------*/
int oss_gus_init(gsisynth_device_t *hw, gsisynth_devicedata_t *hw_status)
{
	struct synth_info this_card;
	int  dev;

	DEB1(DPRT("*gus_init*\n"));

	/* Print out available memory */
	oss_gus_instrument_memory_available(hw, hw_status);

	oss_gus_clear_instruments(hw, hw_status);

#if 0
	dev = hw->listno;
	GUS_NUMVOICES(dev, 24); /* set number of voices */
	SEQ_DUMPBUF();
	/* now we must check to see if the nr of voices has changed */
	this_card.device = dev;
	status = ioctl(hw_status->fd, SNDCTL_SYNTH_INFO, &this_card);
#endif
	hw->num_voices = this_card.nr_voices;

//XXX	if (hw_status->voices != NULL) free(hw_status->voices);
//XXX	hw_status->voices = malloc(sizeof(struct voice_status) * this_card.nr_voices);

//YY	hw_status->initialized = True;

	DEB1(DPRT("#gus_init#\n"));

	return 0;
}


int oss_gus_done(gsisynth_device_t *hw, gsisynth_devicedata_t *hw_status)
{
	DEB1(DPRT("*gus_done*\n"));
//XXX	if (hw_status->voices != NULL) free(hw_status->voices);
//YY	hw_status->initialized = False;
	DEB1(DPRT("#gus_done#\n"));

	return 0;
}


int  oss_gus_write_instrument (gsisynth_device_t *hw, gsisynth_devicedata_t *hw_status, int pgm, int type, int size, char *data) {
	/* Not yet implemented */
	return -1;
}

/*---------------------------------------------------------------------------*
 *      Load a instrument, returns 0 if ok, -1 otherwise.
 *      Currently does not do a check to see if loading failed when not
 *      enough memory is available on the sound card, due to the OSS macro's.
 *      (workaround: use write() or check to see if available memory changed))
 *---------------------------------------------------------------------------*/
//int gus_load (gsisynth_device_t *hw, gsisynth_devicedata_t *hw_status, int pgm, int bits /* 8/16 */) {
/* 8 bits loading is to conserve memory (lower sample rate also?) */
int oss_gus_load_instrument (gsisynth_device_t *hw, gsisynth_devicedata_t *hw_status, int pgm)
{
	int  i;
	int  instrument_fd;
	char instrument_filename[ 256 ];
	byte buf[ 256 ];
	int  offset;
	struct pat_header       header;
	struct sample_header    sample;
	struct patch_info  *instrument = NULL;

	DEB1(DPRT("*gus_load*\n"));

	if (pgm < 0 || pgm > 255)
		return -1; /* ignore out of range values */

	if (hw_status->instrmap[ pgm ] == pgm)
		return 0; /* already loaded */

	hw_status->instrmap[ pgm ] = pgm;

	DEB1(DPRT("instrument name is '%s'\n", gus_instrument_names[ pgm ]));
	if (gus_instrument_names[ pgm ] == NULL || gus_instrument_names[ pgm ][0] == 0) /* Not defined */
		return -1;

	sprintf(instrument_filename, "%s/%s.pat", gus_instrument_dir, gus_instrument_names[ pgm ]);
	DEB1(DPRT("Loading instrument %d from instrument file \"%s\"\n", pgm, instrument_filename));

	if ((instrument_fd = open (instrument_filename, O_RDONLY, 0)) == -1) {
		log_message(GSI_MESSAGE_ERROR, "(oss) %s: open failed (%s)", instrument_filename, strerror(errno));
		return -1;
	}

	if (read (instrument_fd, buf, 0xef) != 0xef) {
		/* Instrument file is too short */
		return GSISYNTH_ERROR_BAD_INSTRUMENTFILE;
	}

	memcpy ((char *) &header, buf, sizeof (header));

	if (strncmp (header.magic, "GF1PATCH110", 12)) {
		/* Not a instrument file */
		return GSISYNTH_ERROR_BAD_INSTRUMENTFILE;
	}

	if (strncmp (header.version, "ID#000002", 10)) {
		/* Incompatible instrument file version */
		return GSISYNTH_ERROR_BAD_INSTRUMENTFILE;
	}

	header.nr_waveforms  = (uint16) lsb_int16_at(&buf[85]);
	header.master_volume = (uint16) lsb_int16_at(&buf[87]);

	offset = 0xef;

	if (header.nr_waveforms < 1) return -1;

	for (i = 0; i < header.nr_waveforms; i++) {
		if (lseek (instrument_fd, offset, 0) == -1) {
			log_message(GSI_MESSAGE_ERROR, "(oss) %s: open failed (%s)", instrument_filename, strerror(errno));
			return -1;
		}

		if (read (instrument_fd, &buf, sizeof (sample)) != sizeof (sample)) {
			log_message(GSI_MESSAGE_ERROR, "(oss) instrument file %s is too short", instrument_filename);
			return GSISYNTH_ERROR_BAD_INSTRUMENTFILE;
		}

		memcpy ((char *) &sample, buf, sizeof (sample));

		sample.fractions     = (char) buf[7];
		sample.len           = lsb_int32_at(&buf[8]);
		sample.loop_start    = lsb_int32_at(&buf[12]);
		sample.loop_end      = lsb_int32_at(&buf[16]);
		sample.base_freq     = (uint16) lsb_int16_at(&buf[20]);
		sample.low_note      = lsb_int32_at(&buf[22]);
		sample.high_note     = lsb_int32_at(&buf[26]);
		sample.base_note     = lsb_int32_at(&buf[30]);
		sample.detune        = lsb_int16_at(&buf[34]);
		sample.panning       = buf[36];

		memcpy (sample.envelope_rate, &buf[37], 6);
		memcpy (sample.envelope_offset, &buf[43], 6);

		sample.tremolo_sweep   = buf[49];
		sample.tremolo_rate    = buf[50];
		sample.tremolo_depth   = buf[51];

		sample.vibrato_sweep   = buf[52];
		sample.vibrato_rate    = buf[53];
		sample.vibrato_depth   = buf[54];
		sample.modes           = buf[55];
		sample.scale_frequency = lsb_int16_at(&buf[56]);
		sample.scale_factor    = (uint16) lsb_int16_at(&buf[58]);

		offset = offset + 96;

		/* Now allocate memory for the instrument structure and sample data.
		 * The information in the temporary sample struct is then copied
		 * to the instrument struct.
		 */


//		gus_upload_sample(&sample);
//	{}
//
//void gus_upload_sample(&sample) {
//}

		instrument = (struct patch_info *) malloc (sizeof (struct patch_info) + sample.len);
		if (instrument == NULL) {
			return GSISYNTH_ERROR_OUT_OF_MEMORY;
		}

		instrument->key = GUS_PATCH;
		instrument->device_no = hw->listno;
		instrument->instr_no = pgm;

		/* Copy the temporary sample info to the instrument struct */
		instrument->mode = sample.modes | WAVE_TREMOLO | WAVE_VIBRATO | WAVE_SCALE;

//		if (instrument->mode & WAVE_16_BITS && (bits == 8)) {
//			instrument->mode &= ~WAVE_16_BITS;
//			for (j = 0; j < instrument->len/2; j++) /* instrument->len or instrument->len/2 ? */
//				instrument->data[j] = instrument->data[1 + (j<<1)]; /* just use the MSB */
//		}

		instrument->len = sample.len;
		instrument->loop_start = sample.loop_start;
		instrument->loop_end = sample.loop_end;
		instrument->base_note = sample.base_note;
		instrument->high_note = sample.high_note;
		instrument->low_note = sample.low_note;
		instrument->base_freq = sample.base_freq;
		instrument->detuning = sample.detune;
		instrument->panning = (sample.panning - 7) * 16;

		memcpy (instrument->env_rate, sample.envelope_rate, 6);
		memcpy (instrument->env_offset, sample.envelope_offset, 6);

		instrument->tremolo_sweep = sample.tremolo_sweep;
		instrument->tremolo_rate = sample.tremolo_rate;
		instrument->tremolo_depth = sample.tremolo_depth;

		instrument->vibrato_sweep = sample.vibrato_sweep;
		instrument->vibrato_rate = sample.vibrato_rate;
		instrument->vibrato_depth = sample.vibrato_depth;

		instrument->scale_frequency = sample.scale_frequency;
		instrument->scale_factor = sample.scale_factor;


		instrument->volume = header.master_volume;

		/* some sanity checking */
		if (lseek (instrument_fd, offset, 0) == -1) {
			log_message(GSI_MESSAGE_ERROR, "(oss) %s: open failed (%s)", instrument_filename, strerror(errno));
			return -1;
		}

		/* some more sanity checking */
		if (read (instrument_fd, instrument->data, sample.len) != sample.len) {
			log_message(GSI_MESSAGE_ERROR, "(oss) instrument file %s is too short", instrument_filename);
			return GSISYNTH_ERROR_BAD_INSTRUMENTFILE;
		}

		/* Send the instrument to the sound card via OSS */
#define seqfd hw_status->fd
		SEQ_WRPATCH (instrument, sizeof (*instrument) + sample.len);
		free(instrument);

		offset = offset + sample.len;
	}

	if (instrument->mode & WAVE_16_BITS) {
		//OLD card_info[ dev ].instrument_loaded[ pgm ] = _INSTRUMENT_GUS_16BIT;
		hw_status->instrtype[ pgm ] = _INSTRUMENT_GUS_16BIT;
	}
	else {
		//OLD card_info[ dev ].instrument_loaded[ pgm ] = _INSTRUMENT_GUS_8BIT;
		hw_status->instrtype[ pgm ] = _INSTRUMENT_GUS_8BIT;
	}

	close(instrument_fd);

	DEB1(DPRT("#gus_load#\n"));

	return 0;
}


int  oss_gus_instrument_memory_available (struct gsisynth_device *hw, struct gsisynth_devicedata *hw_status) {
	int  dev;
	int  status;

	dev = hw->listno;
	status = ioctl(hw_status->fd, SNDCTL_SYNTH_MEMAVL, &dev);

	if (status < 0) {
		log_message(GSI_MESSAGE_ERROR, "%s (devno %d): SNDCTL_SYNTH_MEMAVL: %s\n", hw->device, dev, strerror(errno));
		return 0;
	}

	DEB1(DPRT("   GUS memory is %d\n", dev));

	return dev;
}


/* Returns the size of default patch file: */
int  oss_gus_sizeof_default_instrument (gsisynth_device_t *hw, gsisynth_devicedata_t *hw_status, int pgm) {
	/* Finish this! */
	return 4096*1024/192; /* for now, give the average */
}


int oss_gus_clear_instrument (gsisynth_device_t *hw, gsisynth_devicedata_t *hw_status, int pgm)
{
	int  status;
#ifdef SNDCTL_SYNTH_REMOVESAMPLE
	struct remove_sample sample;

	DEB1(DPRT("("__FILE__"): *gus_clear_instrument*\n"));
	sample.devno = hw->listno;
	sample.instrno = pgm & 127;
	sample.bankno = (pgm >> 7) & 1;
	status = ioctl(hw_status->fd, SNDCTL_SYNTH_REMOVESAMPLE, &sample);
	//OLD card_info[ dev ].instrument_loaded[ pgm ] = _INSTRUMENT_NONE;
	hw_status->instrtype[ pgm ] = _INSTRUMENT_NONE;

	DEB1(DPRT("#gus_unload# (pgm %d)\n", pgm));
#else
	status = -1;
#endif
	return status;
}


int oss_gus_clear_instruments(gsisynth_device_t *hw, gsisynth_devicedata_t *hw_status)
{
	int  pgm;
	int  dev = hw->listno;

	if (ioctl (hw_status->fd, SNDCTL_SEQ_RESETSAMPLES, &dev) == -1) {
		/* Can't reset GM programs on device */
		log_message(GSI_MESSAGE_ERROR, "(oss) %s (devno %d): Can't reset GM programs", hw->device, dev);
	return -1;
	}
//YY	hw_status->instrument_memory_available = dev;
	/* update available memory: */
//YY	ioctl(hw_status->fd, SNDCTL_SYNTH_MEMAVL, &hw_status->instrument_memory_available);

	for (pgm=0; pgm<256; pgm++) {
		hw_status->instrtype[ pgm ] = _INSTRUMENT_NONE;
	}

	for (pgm=0; pgm<256; pgm++) {
		hw_status->instrmap[ pgm ] = 0;
	}
	hw_status->instrmap[0] = -1;

	return 0;
}
#  endif /* USE_GUS_SYNTH */
#endif /* USE_OSS */


/*      EOF synth_oss_gus.c */
