/* GSI - general sound interface
 *
 * synth_oss_opl.c: initializes synthesizer & loads instruments for OPL2/OPL3
 * soundcards
 *
 *
 * Copyright (C) W.H.Scholten 1998 - 2001
 *
 * Redistribution and use in source and binary forms, with or
 * without modification, are permitted provided that the following
 * conditions are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS
 * IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
 * AUTHOR OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


/*__________________INCLUDES___________________*/
//#include <gsi/gsisynth_config.h>
#include <gsi/config.h>
#if  USE_OSS

#  if  USE_OPL3_SYNTH

#  ifndef OSS_OPL_INSTRUMENT_DIR
#    error "OSS_OPL_INSTRUMENT_DIR must be defined"
#  endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <unistd.h>
#include <fcntl.h>

#include <sys/ioctl.h>


#include <gsi/whs_types.h>
#define DEBUG_PREFIX "GSISYNTH_"
#include <gsi/gsi_debug.h>
#include <gsi/synth_data.h>

#include "../gsisynth.h"


#include "oss_instrument.h"
#include "oss_includes.h"
#include "../log.h"


/* The following is used to let the OSS macros use the parameter seq_fd
 * instead of the global variable seqfd:
 */
//#define seqfd gsisynth_fd


/*__________________DEFINITIES___________________*/
//#define USE_GSI_DRUMS 1
#define USE_GSI_VOICES 0

/* Defaults: */
#if USE_GSI_VOICES
#  define SB_INSTR_FILE    OSS_OPL_INSTRUMENT_DIR"/gsi_voices.sb"
#  define OPL3_INSTR_FILE  OSS_OPL_INSTRUMENT_DIR"/gsi_voices.sb"
#else
#  define SB_INSTR_FILE    OSS_OPL_INSTRUMENT_DIR"/std.sb"
#  define OPL3_INSTR_FILE  OSS_OPL_INSTRUMENT_DIR"/std.o3"
#endif

#if  USE_GSI_DRUMS
#  define SB_DRUMS_FILE    OSS_OPL_INSTRUMENT_DIR"/gsi_drums.sb"
#  define OPL3_DRUMS_FILE  OSS_OPL_INSTRUMENT_DIR"/gsi_drums.sb"
#else
#  define SB_DRUMS_FILE    OSS_OPL_INSTRUMENT_DIR"/drums.sb"
#  define OPL3_DRUMS_FILE  OSS_OPL_INSTRUMENT_DIR"/drums.o3"
#endif


/*__________________EXTERN___________________*/
SEQ_USE_EXTBUF();


/*__________________VARIABELEN___________________*/
char *instr_file;
char *drums_file;

int	 drum_fd = -1, instr_fd = -1;
/* global, we keep them opened unless special actions are done (loading a
 * instrument from memory) */



/*---------------------------------------------------------------------------*
 *      Init OPL (set OPL2 or OPL3, set voices=12 for OPL3 4-Operator).
 *---------------------------------------------------------------------------*/
int oss_opl_init(gsisynth_device_t *hw, gsisynth_devicedata_t *hw_status)
{
	struct synth_info this_card;
	int  status;
	int  dev = hw->listno;

	DEB1(DPRT("("__FILE__"): *opl_init*\n"));

	this_card.device = hw->listno;
	status = ioctl(hw_status->fd, SNDCTL_SYNTH_INFO, &this_card);

	if (this_card.capabilities & SYNTH_CAP_OPL3) {
		DEB1(DPRT("  We have a OPL 3!\n"));
//#warning disabled opl3 (gives more voices)
#if !USE_GSI_VOICES
		hw_status->flags |= USE_OPL3;
	    ioctl(hw_status->fd, SNDCTL_FM_4OP_ENABLE, &dev); /* already done in seq_reset, but here is better? */
#endif
	}
	else {
		DEB1(DPRT("  We don't have a OPL 3 (i.e. OPL2)\n"));
	}

//XXX	if (!hw_status->initialized) {
		if (hw_status->flags & USE_OPL3) {
			if (instr_file == NULL) instr_file = OPL3_INSTR_FILE;
			if (drums_file == NULL) drums_file = OPL3_DRUMS_FILE;
		}
		else {
			if (instr_file == NULL) instr_file = SB_INSTR_FILE;
			if (drums_file == NULL) drums_file = SB_DRUMS_FILE;
		}

		if (instr_fd < 0) {
			if ((instr_fd = open(instr_file, O_RDONLY, 0)) == -1) {
				log_message(GSI_MESSAGE_ERROR, "(oss) %s: open failed (%s)", instr_file, strerror(errno));
				return -1;
			}
		}
		if (drum_fd < 0) {
			if ((drum_fd = open(drums_file, O_RDONLY, 0)) == -1) {
				log_message(GSI_MESSAGE_ERROR, "(oss) %s: open failed (%s)", drums_file, strerror(errno));
				return -1;
			}
		}
//XXX	}
//XXX	hw_status->initialized = True;

	// there are 12 4-operator voices.
	if (hw_status->flags & USE_OPL3)
		this_card.nr_voices = 12;

//OLD	card_info[dev].nr_voices = this_card.nr_voices;
	hw->num_voices = this_card.nr_voices;

//OLD	if (voices[dev] != NULL) free(voices[dev]);
//OLD	voices[dev] = malloc(sizeof(struct voice_status) *card_info[dev].nr_voices);


//XXX	if (synth_devices[dev].voices != NULL) free(synth_devices[dev].voices);
//XXX	synth_devices[dev].voices = malloc(sizeof(struct voice_status) * synth_devices[dev].num_voices);
	DEB1(DPRT("#opl_init#\n"));

	return 0;
}


int oss_opl_done(gsisynth_device_t *hw, gsisynth_devicedata_t *hw_status)
{
	DEB1(DPRT("("__FILE__"): *opl_done*\n"));
//XXX	if (synth_devices[dev].voices != NULL) free(synth_devices[dev].voices);
	if (instr_fd > 0) { close(instr_fd); instr_fd = -1; }
	if (drum_fd > 0) { close(drum_fd); drum_fd = -1; }
//XXX	hw_status->initialized = False;
	DEB1(DPRT("#opl_done#\n"));

	return 0;
}


int oss_opl_load (gsisynth_device_t *hw, gsisynth_devicedata_t *hw_status, int type, int pgm)
{
	int  i;
	unsigned char buf[ 256 ];
	struct sbi_instrument instrument;
	int  instrument_size;
	int  file_instrument_size;
	char *name;

	DEB1(DPRT("("__FILE__"): *opl_load*\n"));

	if (instr_fd == -1 || drum_fd == -1)
		return -1;

	if (pgm < 0 || pgm > 255)
		return -1; /* ignore out of range values */

	if (hw_status->instrmap[ pgm ] == pgm)
		return 0; /* already loaded */

#if  ! USE_GSI_DRUMS
#  warning "some drums are not used as they sound too ugly. Try gsi_drums.sb and compare.."
	//if (pgm == 174) return -1;
	if (pgm == 168) return -1;
	if (pgm == 170) return -1;
	if (pgm == 181) return -1; /* too loud */
	if (pgm == 206) return -1;
#endif

	instrument.device  = hw->listno;
	instrument.channel = pgm;


	if (hw_status->flags & USE_OPL3) {
		file_instrument_size = 60;
	}
	else {
		file_instrument_size = 52;
	}
	if (pgm < 128) {
		/* melodic instruments */
#if  USE_GSI_VOICES
		file_instrument_size = 52; /* for opl3 also use sb (gsi_voices.sb). */
#endif
		name = instr_file;
		if (lseek(instr_fd, pgm* file_instrument_size, SEEK_SET) == -1) {
			log_message(GSI_MESSAGE_ERROR, "(oss) %s: open failed (%s)", instr_file, strerror(errno));
			return -1;
		}
		if (read(instr_fd, buf, file_instrument_size) != file_instrument_size) {
			log_message(GSI_MESSAGE_ERROR, "(oss) %s: open failed (%s)", instr_file, strerror(errno));
			return -1;
		}
	}
	else {
		/* drum instruments */
		name = drums_file;
#if  USE_GSI_DRUMS
		file_instrument_size = 52; /* for opl3 also use sb (gsi_drums.sb). */
#endif
		if (lseek(drum_fd, (pgm-128)*file_instrument_size, SEEK_SET)==-1) {
			log_message(GSI_MESSAGE_ERROR, "(oss) %s: open failed (%s)", drums_file, strerror(errno));
			return -1;
		}
		if (read(drum_fd, buf, file_instrument_size) != file_instrument_size) {
			log_message(GSI_MESSAGE_ERROR, "(oss) %s: open failed (%s)", drums_file, strerror(errno));
			return -1;
		}
	}

	if (hw_status->flags & USE_OPL3) {
		/* 4OP/2OP */
		if (strncmp(buf, "4OP", 3) == 0) {
			instrument.key = OPL3_PATCH;
			instrument_size = 22;
		}
		else if (strncmp(buf, "2OP", 3) == 0 || strncmp(buf, "SBI", 3) == 0) {
			instrument.key = FM_PATCH;
			instrument_size = 11;
		}
		else {
			DEB1(DPRT("%s: Invalid OPL3 instrument, instr=%d\n", name, pgm & 127));
			DEB1(DPRT("  buf %2x%2x%2x (%c%c%c)\n", buf[0], buf[1], buf[2], buf[0], buf[1], buf[2]));
			return -1;
		}
	}
	else {
		/* 2OP/SBI */
		if (strncmp(buf, "2OP", 3) == 0 || strncmp(buf, "SBI", 3) == 0) {
			instrument.key = FM_PATCH;
			instrument_size = 11;
		}
		else {
			DEB1(DPRT("%s: Invalid OPL2 instrument, instr=%d\n", name, pgm & 127));
			return -1;
		}
	}


	DEB1(DPRT("   OPL instrument %d is %s\n", pgm, !strncmp(buf, "4OP", 3) ? "4OP (OPL3)" : (!strncmp(buf, "2OP", 3) ? "2OP" : (!strncmp(buf, "SBI", 3) ? "SBI" : "Invalid" ) ) ));


	for (i=0; i<instrument_size; i++)
		instrument.operators[i] = buf[i+36];
	for (i=instrument_size; i<32; i++)
		instrument.operators[i] = 0;

	/* send the instrument to the sound card */
	/* Note that the OSS macros use seqfd */
#define seqfd hw_status->fd
	SEQ_WRPATCH (&instrument, sizeof (instrument));

	//OLD card_info[ dev ].instrument_loaded[ pgm ] = ((instrument.key == FM_PATCH) ? _INSTRUMENT_OPL2: _INSTRUMENT_OPL3);
	hw_status->instrtype[ pgm ] = ((instrument.key == FM_PATCH) ? _INSTRUMENT_OPL2: _INSTRUMENT_OPL3);
	hw_status->instrmap[ pgm ] = pgm;
	DEB1(DPRT("#opl_load#\n"));

	return 0;
}


int  oss_opl_load_instrument(gsisynth_device_t *hw, gsisynth_devicedata_t *hw_status, int pgm) {
	// load default instrument
#warning finish oss_opl_load_instrument!
	return -1;
}


int  oss_opl_write_instrument (gsisynth_device_t *hw, gsisynth_devicedata_t *hw_status, int pgm, int instrument_type, int instrument_len, char *instrument_data) {
	struct sbi_instrument instrument;
	int  i;

	if (instrument_type == GSI_INSTRUMENT_OPL2) {
		instrument.key = FM_PATCH;
		instrument_len = 11;
	}
	else if (instrument_type == GSI_INSTRUMENT_OPL3) {
		instrument.key = OPL3_PATCH;
		instrument_len = 22;
	}
	else {
		log_message(GSI_MESSAGE_ERROR, "(oss) %s: BAD OPL patch %d", hw->device, pgm);
		return -1;
	}

	instrument.device  = hw->listno;
	instrument.channel = pgm;

	for (i=0; i<instrument_len; i++)
		instrument.operators[i] = instrument_data[i];
	for (i=instrument_len; i<32; i++)
		instrument.operators[i] = 0;

	/* Send the instrument to the sound card */
#define seqfd hw_status->fd
	SEQ_WRPATCH (&instrument, sizeof (instrument));
	//card_info[ card ].instrument_loaded[ pgm ] = ((instrument.key == FM_PATCH) ? _INSTRUMENT_OPL2: _INSTRUMENT_OPL3);

	hw_status->instrtype[ pgm ] = ((instrument.key == FM_PATCH) ? _INSTRUMENT_OPL2: _INSTRUMENT_OPL3);

	hw_status->instrmap[ pgm ] = pgm;

	return 0;
}


/* we can load all instruments so each instr is 1, and we have 256 available */
int  oss_opl_sizeof_default_instrument (gsisynth_device_t *hw, gsisynth_devicedata_t *hw_status, int pgm) {
	return 1;
}


int  oss_opl_instrument_memory_available (struct gsisynth_device *hw, struct gsisynth_devicedata *hw_status) {

	return 256;
}


/* NOP: */
int oss_opl_clear_instrument (gsisynth_device_t *hw, gsisynth_devicedata_t *hw_status, int pgm) {

	return 0;
}


int oss_opl_clear_instruments (gsisynth_device_t *hw, gsisynth_devicedata_t *hw_status)
{
	int  pgm;

	DEB1(DPRT("OPL clear instruments\n"));

	for (pgm=0; pgm<256; pgm++) {
		//OLD card_info[ dev ].instrument_loaded[ pgm ] = _INSTRUMENT_NONE;
		hw_status->instrtype[ pgm ] = _INSTRUMENT_NONE;
	}

	for (pgm=0; pgm<256; pgm++) {
		hw_status->instrmap[ pgm ] = 0;
	}
	hw_status->instrmap[0] = -1;

	return 0;
}
#  endif /* USE_OPL3_SYNTH */

#endif /* USE_OSS */


/*      EOF synth_oss_opl.c */
