/* GSI - general sound interface
 *
 * synth_ultra.c (ultrasound MIDI playing init)
 *
 * Lower level API dependencies:
 *   - Ultrasound sequencer/synth playing (only MIDI at the moment)
 *
 *
 * Copyright (C) W.H.Scholten 1998 - 2001
 *   - First version: July 1998
 *
 * Redistribution and use in source and binary forms, with or
 * without modification, are permitted provided that the following
 * conditions are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS
 * IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
 * AUTHOR OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


/*__________________INCLUDES___________________ */
//#include <gsi/gsisynth_config.h>
#include <gsi/config.h>
#if  USE_ULTRASOUND

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <unistd.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/stat.h>

#include <limits.h>


#include <gsi/whs_defines.h>
#include <gsi/whs_types.h>
#define DEBUG_PREFIX "GSISYNTH_"
#include <gsi/gsi_debug.h>
#include <gsi/device.h>

#include "../gsisynth.h"

#include "../instrument.h"
#include "../api.h"
#include "../log.h"

#include <libgus.h>


/*__________________VARIABLES___________________
 */
int  gus_midi_card = -1;
int  gus_midi_device = -1;
char *conf_file = NULL;
char *instr_file = NULL;

gus_midi_device_t *ultra_device;

char *instr_source = NULL;

// all this should be put in the synth struct:
int  reverb_type = 4;
int  chorus_type = 2;

int  gus_emul = -1;

int  preload = False;
int  midi_type;
static int ultra_open = False;


gsisynth_device_t default_ultrasound_synth_device = {
	NULL,
	"default synth",/* identification string */

	api_init_synth,
	api_close_synth,
	api_write_synth,
	api_synth_set_timeout,
	api_synth_buffer_status,

	NULL, /* api_load_instrument, */
	NULL, /* api_write_instrument, */
	NULL, /* api_clear_instrument, */
	NULL, /* api_clear_instruments, */
	NULL, /* api_sizeof_default_instrument, */
	api_reset_synth,

	NULL, /* instrument_memory_available */

	0, /* total_instrument_memory */
	100, /* synth_rate */

	// Hardware info:
	0, /* type */
	0, /* num_voices */
};

/* attach
 */
char *api_attach_synth_device (char *device, int i) {
	gsisynth_device_t hw;
	gsisynth_devicedata_t hw_status;
	/* device is not used, as it's set by the ultra config file */

	memset(&hw, 0, sizeof(gsisynth_device_t));
	memset(&hw_status, 0, sizeof(gsisynth_devicedata_t));
	//hw->device = device;

	device = api_attach_synth(&hw, &hw_status, i);
	api_close_synth(hw, hw_status);

	return device;
}

char *api_attach_synth (gsisynth_device_t **hw_out, gsisynth_devicedata_t **hw_status_out, int i) {
	if (i > 0) return NULL;

	return "(ultra) GUS";
}



int  api_write_synth (gsisynth_device_t *hw, gsisynth_devicedata_t *hw_status) {
	int  status;

	if (ultra_open) {
		if (!api_synth_buffer_full(hw, hw_status)) {
			status = gus_midi_write();
			//status = gus_midi_flush();
		}
		//gus_queue_flush(gus_synth_handle);

		//TEST:
		api_synth_buffer_full(hw, hw_status);
	}

	return 0;
}

void api_synth_set_timeout (gsisynth_device_t *hw, gsisynth_devicedata_t *hw_status, struct timeval *timeout) {
	timeout->tv_usec = 20000; /* 20 ms, so we write often enough to the sequencer */
}

int api_reset_midi(gsi_song_playstatus_t *spdata) {
	gsi_midi_status_t *mdata;
	int  status;

	gsisynth_device_t *hw = spdata->hw;
	gsisynth_devicedata_t *hw_status = spdata->hw_status;

	if (!spdata) return -1;
	mdata = spdata->song;
	if (!mdata) return -2;

	status = api_reset_synth(hw, hw_status);

	return status;
}

int api_reset_synth (gsisynth_device_t *hw, gsisynth_devicedata_t *hw_status) {
#if 0
	//gus_memory_reset(gus_synth_handle, GUS_DOWNLOAD_MODE_NORMAL);
	gus_midi_timer_stop();
	gus_midi_reset();
	for (i = 0; i < 16; i++) {
		if (!IS_PERC(i)) {
			gus_midi_control(GUS_MIDID_COMMON, i, GUS_MCTL_E3_CHORUS_DEPTH, 20);
		}
	}
	gus_midi_memory_reset_all();
#endif
	return 0;
}

/*---------------------------------------------------------------------------*
 *      Close the synth device.
 *---------------------------------------------------------------------------*/
int api_close_synth(gsisynth_device_t *hw, gsisynth_devicedata_t *hw_status)
{
	DEB1(DPRT("api_close_synth (ultra)\n"));
//XXX	if (dev != 0) return -1; // current limit 1 synth
	if (hw == NULL || hw_status == NULL) return -1;

	if (ultra_open) {
		gus_midi_icfg_close(GUS_MIDID_COMMON);
		gus_midi_close();
		ultra_open = False;
		//seqfd = -1;
	}

	if (conf_file != NULL) { free(conf_file); conf_file = NULL; }
	if (instr_file != NULL) { free(instr_file); instr_file = NULL; }
	if (instr_source != NULL) { free(instr_source); instr_source = NULL; }
	DEB1(DPRT("#api_close_synth# (ultra)\n"));

	return 0;
}


/*---------------------------------------------------------------------------*
 *      Open the synth device.
 *---------------------------------------------------------------------------*/
int api_open_synth(gsisynth_device_t *hw, gsisynth_devicedata_t *hw_status)
{
	DEB1(DPRT("api_open_synth (ultra)\n"));
//XXX	if (dev != 0) return -1; // current limit 1 synth
	if (hw == NULL || hw_status == NULL) return -1;

	DEB(DPRT("#api_open_synth# (ultra)\n"));
	return 0;
}


/*---------------------------------------------------------------------------*
 *      Initialize all synthesizer devices found.
 *---------------------------------------------------------------------------*/
int api_init_synth(gsisynth_device_t *hw, gsisynth_devicedata_t *hw_status /*, int type*/)
{
	int  i;

//XXX	if (dev != 0) return -1; // current limit 1 synth
	if (hw == NULL || hw_status == NULL) return -1;

	DEB1(DPRT("("__FILE__"): *api_init_synth* (ultra)\n"));

	if (ultra_open) {
		DEB1(DPRT("ultra: already open.\n"));
		return 0;
	}

	instr_source = GUS_ICFG_SOURCE_AUTO;
	DEB1(DPRT("ultra: GUS_ICFG_SOURCE_AUTO="GUS_ICFG_SOURCE_AUTO"\n"));
	midi_type = _MIDI_GM; /* temp, user should select it */

	switch(midi_type) {
	case _MIDI_GM:
		gus_emul = GUS_MIDI_EMUL_GM; break; /* general MIDI */
	case _MIDI_GS:
		gus_emul = GUS_MIDI_EMUL_GS; break; /* roland GS */
	case _MIDI_MT32:
		gus_emul = GUS_MIDI_EMUL_MT32; break; /* MT 32 */
	default:
		log_message(GSI_MESSAGE_ERROR, "bad MIDI emulation type %d", midi_type);
		return -1;
	}

	if (gus_midi_card < 0 || gus_midi_device < 0) {
		gus_midi_device = GUS_MIDID_COMMON; // let the driver select
	}
	else {
		gus_midi_device = (gus_midi_card << 4) | gus_midi_device;
		//DEB(DPRT("gus_midi_device=%x", gus_midi_device));
	}

	if (gus_midi_device == GUS_MIDID_COMMON) {
		DEB1(DPRT("  gus_midi_device = GUS_MIDID_COMMON\n"));
		 // let the driver select the best output:
		if (gus_midi_open_intelligent(GUS_MIDI_OUT, conf_file, 8 * 1024, GUS_MIDI_OF_ECHO_ENABLE)) {
			log_message(GSI_MESSAGE_ERROR, "open midi error: %s", gus_midi_error());
			return -1;
		}
		ultra_open = True;
	}
	else {
		/* we have given a specific output device (gus_midi_device): */
		ultra_device = malloc(sizeof(gus_midi_device_t));
		if (ultra_device == NULL) {
			log_message(GSI_MESSAGE_ERROR, "Out of memory in %s", "open_midi");
			return -1;
		}

		gus_midi_fill_device_structure(ultra_device, GUS_MIDI_OPEN_MODE_WRITE, gus_midi_device, 0xffff);
		if (gus_midi_open(GUS_MIDI_OUT, ultra_device, 8 * 1024, GUS_MIDI_OF_ECHO_ENABLE)) {
			log_message(GSI_MESSAGE_ERROR, "open midi error: %s", gus_midi_error());
			return -1;
		}
		ultra_open = True;
	}

	WARN("assuming 1 ultra synth");
	gsisynth_num_synths = 1; //temp..

	// very similar to code in oss.c:
	if (gsisynth_num_synths > 0) {
//		card_info = malloc(gsisynth_num_synths*sizeof(struct gsisynthhw));

//		voices = calloc(nrsynths, sizeof(void *));
		/* at exit: free(synth_types); free(card_info); free(voices[0-nrsynths-1]), free(voices); */

		// Synth is fixed at 16 devices at the moment so not needed:
		//X synth = malloc(gsisynth_num_synths*sizeof(gsisynth_t));
	}
	gsisynth_devices[ 0 ].type = SYNTH_HW_GUS;
	gsisynth_devices[ 0 ].num_voices = 0; // temp.., doesn't really matter as we use the ultra lib's midi emulation.
//ELSEWHERE	card_info[0].midi_mode = midi_type;

//ELSEWHERE	gsisynth_devices[0].voices = malloc(sizeof(struct voice_status) * card_info[0].nr_voices);


	gus_midi_timer_stop();
	gus_midi_reset();
	for (i = 0; i < 16; i++) {
//XXX		if (!IS_PERC(i)) {
			gus_midi_control(GUS_MIDID_COMMON, i, GUS_MCTL_E3_CHORUS_DEPTH, 20);
//XXX		}
	}
	gus_midi_memory_reset_all();


	for (ultra_device = gus_midi_output_devices(); ultra_device!=NULL; ultra_device = ultra_device->next) {
		DEB1(DPRT("  ultra_device=%0x, gus_emul=%d\n", (unsigned int) ultra_device, gus_emul));
		if (gus_emul >= 0) {
			gus_midi_emulation_set(ultra_device->device, gus_emul);
		}

		if (ultra_device->cap & GUS_MIDI_CAP_MEMORY) {
			gus_info_t ginfo;
			struct GUS_STRU_EFFECT effect;

			if (gus_midi_icfg_open(ultra_device->device, instr_file, instr_source, "")) {
				log_message(GSI_MESSAGE_ERROR, "open icfg failed");
				gus_midi_close();
				return -1;
			}
			if (gus_midi_synth_info(ultra_device->device, &ginfo) >= 0) {
				if (ginfo.flags & GUS_STRU_INFO_F_ENHANCED) {
					gus_effect_interwave(&effect, reverb_type, chorus_type);
					gus_midi_effect_setup(ultra_device->device, &effect);
				}
			}
			if (preload) {
				DEB1(DPRT("preloading instruments\n"));
			if (gus_midi_icfg_preload(ultra_device->device, "all", NULL) < 0) {
					log_message(GSI_MESSAGE_ERROR, "preloading instruments failed");
			}
//			gus_midi_preload_bank(GUS_MIDID_COMMON, "ALL");
			DEB1(DPRT("preloading done\n"));
			}
		}
//		gus_queue_write_set_size(ultra_device, 256); /* size of write queue in 'events' */
	}

	gus_midi_timer_base(100); // = synth_rate ?
	gus_midi_timer_tempo(100);
//XXX	gsisynth_devices[dev].synth_rate = 100;
	hw->synth_rate = 100;

//XXX	gsisynth_devices[dev].fd = gus_midi_get_file_descriptor();
	hw_status->fd = gus_midi_get_file_descriptor();

//XXX	DEB1(DPRT("ultra fd =%d\n", gsisynth_devices[dev].fd));
	DEB1(DPRT("ultra fd =%d\n", hw_status->fd));

	// set to ultra midi writing to non-blocking (will it work this way?)
	//oldflags = fcntl(socket_fd, F_GETFL);
	//if (oldflags != -1) fcntl(hw_status->fd, F_SETFL, oldflags | O_NONBLOCK);
	//gus_midi_realtime( 1 /*int enable*/);

	//TEST:
//	{ int volume = 15;
//	gus_midi_master_volume(GUS_MIDID_COMMON, volume > 15 ? 127 : volume << 3);
//	}
	DEB1(DPRT("#ultra_init_synth#\n"));

	return 0;
}



int  api_gus_load (gsisynth_device_t *hw, gsisynth_devicedata_t *hw_status, int pgm, int type) {
	gus_midi_device_t *card;

//XXX	if (gsisynth_devices[ dev ].instrtype[ pgm ] == _INSTRUMENT_NONE) {
	if (hw_status->instrtype[ pgm ] == _INSTRUMENT_NONE) {

		card = gus_midi_output_devices();
		while ( card ) {
//			if ((card->channels & (1 << chn)) && (card->cap & GUS_MIDI_CAP_MEMORY)) {
			if ((card->cap & GUS_MIDI_CAP_MEMORY)) {
				gus_midi_icfg_download_program(card->device, &pgm, 1);
//				DEB1(DPRT("downloading instrument %d (ch=%d)\n", pgm, chn));
//XXX				gsisynth_devices[ dev ].instrtype[ pgm ] = _INSTRUMENT_GUS_16BIT; //temp
				hw_status->instrtype[ pgm ] = _INSTRUMENT_GUS_16BIT; //temp

				DEB1(DPRT("downloading instrument %d\n", pgm));
			}
			card = card->next;
		}
	}
	return 0;
}


int api_gus_unload (gsisynth_device_t *hw, gsisynth_devicedata_t *hw_status, int pgm, int type)
{
	int  status;
//	gus_instrument_t  instr;

	DEB1(DPRT("("__FILE__"): *api_gus_unload* (ULTRA)\n"));

	status = 0;
//	status = gus_midi_memory_free(GUS_MIDID_COMMON, &instr);
	DEB1(DPRT("#api_gus_unload#\n"));

	return status;
}


int  api_clear_samples (gsisynth_device_t *hw, gsisynth_devicedata_t *hw_status) {
	int  pgm;

	gus_midi_memory_reset_all();

	for (pgm=0; pgm<256; pgm++) {
//XXX		gsisynth_devices[ dev ].instrtype[ pgm ] = _INSTRUMENT_NONE;
		hw_status->instrtype[ pgm ] = _INSTRUMENT_NONE;
	}

	for (pgm=0; pgm<256; pgm++) {
//XXX		gsisynth_devices[ dev ].instrmap[ pgm ] = 0;
		hw_status->instrmap[ pgm ] = 0;
	}
//XXX	gsisynth_devices[ dev ].instrmap[0] = -1;
	hw_status->instrmap[0] = -1;

	return 0;
}


#if 0
//	for (ultra_device = gus_midi_output_devices(); ultra_device; ultra_device = ultra_device->next) {
//		if (ultra_device->cap & GUS_MIDI_CAP_MEMORY) {
//			gus_midi_icfg_close(ultra_device->device);
//		}
//	}

static void ultra_download_instrument(void *priv, gus_midi_device_t *mdev, unsigned int pgm) {
	if (mdev->cap & GUS_MIDI_CAP_MEMORY) {
		if (gus_midi_icfg_download_program(mdev->device, &pgm, 1) < 0)
			log_message(GSI_MESSAGE_ERROR, "out of GUS memory, instrument %d (%d) cannot be loaded...", pgm >> 16, (pgm & 0xffff) + 1);
	}
}
#endif


static gus_midi_callbacks_t ultra_callback = {
	17,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL
};


/*---------------------------------------------------------------------------*
 *      Some sequencer buffer management functions.
 *---------------------------------------------------------------------------*/
// Buffer status: 0=empty, 1=used, 2=full
int api_synth_buffer_status(gsisynth_device_t *hw, gsisynth_devicedata_t *hw_status) {
	if (ultra_open)
		return 1;

	return 0;
}

// is the sequencer (or buffer) ready to accept data or not?
int api_synth_buffer_full(gsisynth_device_t *hw, gsisynth_devicedata_t *hw_status) {
	int  status;
	struct timeval ultra_tv = { 0, 0 };
	fd_set read_fds, write_fds;

	//DEB(DPRT("synth_buffer_full? free=%d\n", gus_queue_write_free(ultra_synth_handle)));
	//if (gus_queue_write_free(ultra_synth_handle) <10) return True;

	FD_ZERO(&read_fds);
	FD_ZERO(&write_fds);
	FD_SET(hw_status->fd, &read_fds);
	FD_SET(hw_status->fd, &write_fds);
	status = select(hw_status->fd + 1, &read_fds, &write_fds, NULL, &ultra_tv);
	if (FD_ISSET(hw_status->fd, &read_fds)) {
		//DEB(DPRT("ultra midi: input callbacks ..\n"));
		gus_midi_input(&ultra_callback);
	}
	if (FD_ISSET(hw_status->fd, &write_fds)) {
		//DEB(DPRT("ultra midi: we can write\n"));
		return False;
	}

	//DEB(DPRT("ultra midi: we can NOT write\n"));

	return True;
}
#endif /* USE_ULTRASOUND */


/*      EOF synth_ultra.c */
