/* GSI - general sound interface
 *
 * midi_play.c (currently only MIDI type playing (i.e. also hmp))
 *
 * Lower level API dependencies:
 *   - unix gettimeofday and sysex decoding (MIDI specifications).
 *   - sysex decoding should also be removed from the server
 *     (put midi+sysex decoding in libgsisong ?)
 *   - libgsif header files
 *
 * Copyright (C) W.H.Scholten 1998 - 2001
 *
 * Redistribution and use in source and binary forms, with or
 * without modification, are permitted provided that the following
 * conditions are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS
 * IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
 * AUTHOR OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

//#include <gsi/gsisynth_config.h>
#include <gsi/config.h>

#include <stdio.h>
#include <stdlib.h>
//#include <string.h>

#include <limits.h>

#include <gsi/whs_types.h>
#define DEBUG_PREFIX "GSISYNTH_"
#include <gsi/gsi_debug.h>

#include <gsi/gsif.h>
#include <gsi/gsif_midi.h>

#include "gsisynth.h"

#include "api.h"


void gsisynth_clear (gsi_song_playstatus_t *spdata)
{
	gsi_midi_status_t *mdata = spdata->song;

	DEB1(DPRT("MIDI ("__FILE__"): *close_synth*\n"));
	/* Card data. */
//	if (card_info != NULL) {
//		free(card_info);
//		card_info = NULL;
//	}

	/* Play data, should keep this for possible re grabbing. */
	if (mdata->voices != NULL) {
//		for (i=0; i<num_synths; i++) {
//			if (music_client->voices[i] != NULL)
//				free(music_client->voices[i]);
//		}
		free(mdata->voices);
		mdata->voices = NULL;
	}

	spdata->song_play_status = _SONG_NONE;
	spdata->song_play_ct = 0; // not needed any more

	gsisynth_close(spdata->hw, spdata->hw_status);
	DEB1(DPRT("MIDI #close_synth#\n"));
}


/*---------------------------------------------------------------------------*
 *      Timing.
 *---------------------------------------------------------------------------*/
void gsisynth_song_init_time (gsi_song_playstatus_t *spdata	/* OR gsi_midi_status_t *mdata ? */, double offset)
{
	int  offset_int, offset_frac;
	struct timeval start_tv;

	if (spdata == NULL) return;

	spdata->start_synth_ticks = (int) (offset * gsisynth_rate); /* OSS bug workaround. */

	offset_int  = (int) offset;
	offset_frac = (int) ((offset - (double) offset_int) * 1e6);
	DEB1(DPRT("MIDI   init_midi_time: offset=%f  start_synth_ticks=%d\n", (float) offset, spdata->start_synth_ticks));

	gettimeofday(&start_tv, NULL);
	start_tv.tv_usec -= offset_frac;
	start_tv.tv_sec  -= offset_int;
	if (start_tv.tv_usec < 0) {
		start_tv.tv_usec +=1000000;
		start_tv.tv_sec--;
	}
	spdata->start_tv = start_tv;
}


double gsisynth_song_get_time (gsi_song_playstatus_t *spdata)
{
	struct timeval new_tv;

	gettimeofday(&new_tv, NULL);
	//if (new_tv.tv_usec <start_tv.tv_usec) {
	//}
	return (double) (new_tv.tv_sec - spdata->start_tv.tv_sec) + (double) (new_tv.tv_usec - spdata->start_tv.tv_usec) * 1.0e-6;
}


/* Called when a new song has been loaded for the current client (do also when
 * the synth was released?).
 */
void gsisynth_song_prepare (gsi_song_playstatus_t *spdata) {
	/* Reset voices (replace by instrument cache algorithm?): */
	gsisynth_reset_voices(spdata->hw, spdata->hw_status);

	gsisynth_clear_instruments(spdata->hw, spdata->hw_status);
}


/*---------------------------------------------------------------------------*
 *      Initialize for midi (& hmp) playing
 *---------------------------------------------------------------------------*/
int gsisynth_midi_init_play (gsi_song_playstatus_t *spdata)
{
	int  dev;
	int  i;
	gsi_midi_status_t *mdata = spdata->song;

	dev = spdata->dev;

	DEB1(DPRT("MIDI ("__FILE__"): *play_midi_init*\n"));
	if (spdata->song == NULL) {
		spdata->song_play_status = _SONG_NONE;
		spdata->song_play_ct = 0; /* Not needed any more. */
		return -1;
	}

//CHANGE	if (open_synth(spdata) == -1) {
//CHANGE		return -1;
//CHANGE	}

	mdata->midi_default_tempo = 500000;

	/* Actual midi playing: */
	for (i = 0; i < 16; i++)
		mdata->midi_force_chn_pgm[i] = mdata->midi_force_chn_volume[i] = 0;	/* no pgm/vol override */

	gsisynth_preload_instruments(spdata);

	gsisynth_seq_reset(spdata);
	gsisynth_song_init_time(spdata, 0);

	if (spdata->song_status == NULL)
		spdata->song_status = gsif_make_song_status(spdata->song);
	gsif_reset_song_status(spdata->song_status);


	/* Instead of that seq control this (will need to check song type itself
	 * and do init): */
	gsisynth_seq_prepare(spdata);

	gsisynth_seq_start_timer(spdata);
	gsisynth_seq_flush(spdata);

	spdata->song_time = 0.0;

	spdata->song_play_status = _SONG_PLAYING;
	spdata->song_events_done = False;

	DEB1(DPRT("MIDI #play_midi_init#\n"));

	return 0;
}


/*---------------------------------------------------------------------------*
 *      Play the midi events on internal sythesizers or external midi.
 *      Main loop.
 *---------------------------------------------------------------------------*/
// Some often used abbreviations
#define CMD		song_tracks[track]->running
#define CHN		(cmd & 0xf)
#define NOTE	data[0]
#define VEL		data[1]

int gsisynth_midi_play_events(gsi_song_playstatus_t *spdata, int max_dt)
{
	int  instrument_loaded;
	double time_of_earliest;
	gsi_midi_status_t *mdata;

	gsisynth_device_t *hw;
	gsisynth_devicedata_t *hw_status;

	int  dev;

	struct gsi_song_status *song_status;

	int  cmd;
	byte *data;


	DEB2(DPRT("MIDI ("__FILE__"): *play_midi_events*\n"));

	if (spdata == NULL) return -1;
	dev = spdata->dev;
	hw = spdata->hw;
	hw_status = spdata->hw_status;

	if (spdata->song == NULL) return -1;
	mdata = spdata->song;

	if (spdata->song_status == NULL) {
		spdata->song_status = gsif_make_song_status(spdata->song);
	}

	song_status = spdata->song_status;

	if (!gsisynth_buffer_full(hw, hw_status)) {
		for (;;) {
			struct gsi_song_event event;
			int  ev_status;

			if (gsisynth_buffer_full(hw, hw_status)) { break; }
			if (gsif_midi_next_event(-1, song_status, -1, -1, &event) < 0) {
				break; /* error .. */
			}

			/* if the next midi event is later than eg. max(0.4s, 4* fragtime),
			 * leave and do it next time.
			 *
			 * Note: max_dt is in s. */

			time_of_earliest = song_status->time + ((event.ticks - song_status->ticks) * ((double) song_status->midi_tempo *1.0e-6)) / (double) song_status->midi_division;
			//DEB(DPRT("midi_time=%6.2f  (ss->time=%6.2f ev.ticks=%5d ss->ticks=%5d)  time_of_earliest=%6.2f\n", midi_time, song_status->time, event.ticks, song_status->ticks, time_of_earliest));

			if (time_of_earliest > spdata->song_time+max_dt * 1.0e-6) {
				break; /* We don't handle this event yet. */
			}

			/* midi ticks -> seconds:
			 *    time = (ticks * midi_tempo * 1e-6) / midi_division
			 * seconds -> midi ticks:
			 *    ticks = (time * midi_division * 1e6) / midi_tempo
			 */

			/* now get it really (could combine the two gsif_midi_next_event()
			 * calls by passing max_ticks = ticks_from_time(current_time +
			 * max_dt*1.0e-6)).
			 */
			ev_status = gsif_midi_next_event(INT_MAX, song_status, -1, -1, &event);
			if (ev_status == 2) {
				spdata->song_events_done = True;

				song_status->song_end_time = song_status->time;
				break; /* No more events. */
			}

			cmd = event.event;
//CHANGE			((gsi_midi_status_t *) current->song_status->song)->time_of_last_midi_command = time_of_earliest;
			((gsi_midi_status_t *) spdata->song)->time_of_last_midi_command = time_of_earliest;

//X			use_dev = mdata->midi_channel_output[CHN];
//X			DEB1(DPRT("MIDI   -> use_dev=%d\n", use_dev));

			data = event.data;
			if (cmd == MIDIMETA_SET_TEMPO) {
				song_status->midi_tempo = ((data[0] << 16) | (data[1] << 8) | data[2]);
			}


			if (event.ticks > song_status->last_ticks) {
				int  synth_ticks;

				synth_ticks = (int) (song_status->time * gsisynth_rate);
				//if (synth_ticks> (int) (status->last_song_time*gsisynth_rate)) {
				if (synth_ticks > song_status->last_synth_ticks) {
					//status->last_song_time = status->song_time;
					song_status->last_synth_ticks = synth_ticks;
					DEB1(DPRT("MIDI   -> wait %d ticks\n", (int) synth_ticks));

					//gsisynth_seq_wait_time(spdata, synth_ticks);
					// WHS: OSS bug workaround code:
					//DEB(DPRT("synth_ticks = %d, start_synth_ticks=%d\n", synth_ticks, spdata->start_synth_ticks));
					gsisynth_seq_wait_time(spdata, synth_ticks - spdata->start_synth_ticks);
					gsisynth_seq_flush(spdata);
				}
			}

			instrument_loaded = 0; /* Used for NOTE_ON & KEY_PRESSURE */

			if (!spdata->song_events_done && (cmd > 0x7f) && IS_PLAYING(CHN)) {
				switch (cmd & 0xf0) {
				case MIDI_NOTE_OFF:
//OBSOLETE#if  USE_ULTRASOUND && ULTRA_TEST
//OBSOLETE					gus_midi_note_off(GUS_MIDID_COMMON, CHN, NOTE, VEL);	
//OBSOLETE#else
					gsisynth_seq_stop_note(spdata /*use_dev*/, CHN, NOTE, VEL);
//OBSOLETE#endif
					break;

				case MIDI_NOTE_ON:
					DEB2(DPRT("MIDI     MIDI_NOTE_ON: chn %d, note %d\n", CHN, NOTE));
//OBSOLETE#if  USE_ULTRASOUND && ULTRA_TEST
//OBSOLETE					gus_midi_note_on( GUS_MIDID_COMMON, CHN, NOTE, VEL);	
//OBSOLETE#else
					if (IS_PERC(CHN) && VEL && (!IS_MIDI(CHN)) ) {
						instrument_loaded = gsisynth_seq_set_instrument(spdata /*use_dev*/, CHN, NOTE + 128);
					}
					DEB2(DPRT("MIDI     instrument %d: loaded=%d\n", CHN, instrument_loaded));
					if (VEL && mdata->midi_force_chn_volume[ CHN ])
						VEL = mdata->midi_force_chn_volume[ CHN ] - 1;
					//ORIG if (instrument_loaded != -1)
					if (instrument_loaded == 0)
						gsisynth_seq_start_note(spdata /*use_dev*/, CHN, NOTE, VEL);
//OBSOLETE#endif
					break;

				case MIDI_KEY_PRESSURE:
#if  USE_ULTRASOUND && ULTRA_TEST
					gus_midi_note_pressure(GUS_MIDID_COMMON, CHN, NOTE, VEL);
#else
					if (IS_PERC(CHN) && VEL && (!IS_MIDI(CHN)))
						instrument_loaded = gsisynth_seq_set_instrument(spdata /*use_dev*/, CHN, NOTE + 128);
					//ORIG if (instrument_loaded != -1)
					if (instrument_loaded == 0)
						gsisynth_seq_key_pressure(spdata /*use_dev*/, CHN, NOTE, VEL);
#endif
					break;

				case MIDI_CTL_CHANGE:
#if  USE_ULTRASOUND && ULTRA_TEST
					gus_midi_control(GUS_MIDID_COMMON, CHN, NOTE, VEL);
#else
					gsisynth_seq_control(spdata /*use_dev*/, CHN, NOTE, VEL);
#endif
					break;

				case MIDI_PGM_CHANGE:
#if  USE_ULTRASOUND && ULTRA_TEST
					gus_midi_program_change(GUS_MIDID_COMMON, CHN, NOTE);
#else
					if (IS_MIDI(CHN) || !IS_PERC(CHN))
						NOTE = gsisynth_seq_set_instrument(spdata /*use_dev*/, CHN, NOTE);
#endif
					break;

				case MIDI_CHN_PRESSURE:
#if  USE_ULTRASOUND && ULTRA_TEST
					gus_midi_channel_pressure(GUS_MIDID_COMMON, CHN, NOTE);
#else
					gsisynth_seq_chn_pressure(spdata /*use_dev*/, CHN, NOTE);
#endif
					break;

				case MIDI_PITCH_BEND:
					/* Note that NOTE and VEL are actually the LSB/MSB part of
					 * the (14 bit) pitchbend value. */
#if  USE_ULTRASOUND && ULTRA_TEST
					gus_midi_bender(GUS_MIDID_COMMON, CHN, NOTE | (VEL << 7));
#else
					gsisynth_seq_bender(spdata /*use_dev*/, CHN, NOTE, VEL);
#endif
					break;

				case MIDI_SYSTEM_PREFIX:
#if  USE_ULTRASOUND && ULTRA_TEST
					gus_midi_sysex(GUS_MIDID_COMMON, &data[0], event.length);
#else

					/* The sysex decoding is very incomplete at the moment. */
					/* GS & GM:
					 * gm_on  = { 0x7e, 0x7f, 0x09, 0x01 }
					 * gm_off = { 0x7e, 0x7f, 0x09, 0x02 }
					 * gs_on  = { 0x41, 0x10, 0x42, 0x12, 0x40, 0x00, 0x7f, 0x00, 0x41 }
					 * gs_off = { 0x41, 0x10, 0x42, 0x12, 0x40, 0x00, 0x7f, 0x7f, 0x41 }
					 */

					if (event.length > 1) {
						unsigned int  i, j;

						DEB1(DPRT("MIDI   -> decode_sysex\n"));
						if (event.length > 7 && data[0] == 0x41 && data[2] == 0x42 && data[3] == 0x12) {
							/* GS data set messages: */
							if (data[4] == 0x40 && (data[5] & 0xf0) == 0x10 && data[6] == 0x15) {
								/* Use rythm part (rythm channels used for drums). */
								if (!(i = (data[5] & 0xf)))
									i = 0x09;
								else if (i < 10)
									i--;
								i = 1<<i;
								if (data[7])
									mdata->midi_percussion_chn |= i;
								else
									mdata->midi_percussion_chn &= ~i;
							}
							if ( (data[4] == 0x40 || data[4] == 0) &&
								 data[5] == 0x00 && data[6] == 0x7f) {
								/* GS reset: */
								mdata->midi_percussion_chn = 0x0200; /* Percussion only in channel 10 (no more rythm channels for drums). */
								for (i = 0; i < 16; i++) {	/* Set status info. */
									for (j = 0; j < 128; j++)
										mdata->midi_note_vel[i][j] = 0;
									mdata->midi_channel[i].bender = 8192;
									mdata->midi_channel[i].bender_range = 2;
									mdata->midi_channel[i].controller[ MIDICTL_PAN ] = 64;
									mdata->midi_channel[i].controller[ MIDICTL_SUSTAIN ] = 0;
								}
							}
							/* All this handles the first of alienz' sysex
							 * messages, but the second, hmm. */
						}


//XXX						// Send sysex to Midi out:
//XXX						for (j=0; j<num_synths; j++) {
//XXX							if (gsisynth_devices[j].type == MIDI_HW) {
//XXX								if (cmd == MIDI_SYSTEM_PREFIX)
//XXX									gsisynth_seq_midi_out(dev, j, MIDI_SYSTEM_PREFIX);
//XXX								for (i = 0; i < event.length; i++)
//XXX									gsisynth_seq_midi_out(dev, j, data[i]);
//XXX							}
//XXX						}
				}
#endif
					break;
				default:
					break;
				} /* switch (cmd &0xf0) */
			}
		} /* for (;;) */
	}
	else { gsisynth_seq_flush(spdata); }

	DEB2(DPRT("MIDI #play_midi_events#\n"));

	return 1;
}


void gsisynth_preload_instruments (gsi_song_playstatus_t *spdata) {
	struct gsi_song_event event;
	int  instruments = 0;
	gsi_midi_status_t *mdata = spdata->song;
	gsisynth_device_t *hw = spdata->hw;
	gsisynth_devicedata_t *hw_status = spdata->hw_status;


	// For ISPERC() :
	//int percussion_chn = spdata->midi_percussion_chn;
	//int *channel_output = spdata->midi_channel_output;


	/*
	 * Add a check here if we can load all instruments
	 * (OPL, GUS with enough memory).
	 */

	if (spdata->song_status == NULL)
		spdata->song_status = gsif_make_song_status(spdata->song);

	gsif_reset_song_status(spdata->song_status);

	for (;;) {
		int  pgm, cmd, chn;
		int  ev_status;

		ev_status = gsif_midi_next_event(INT_MAX, spdata->song_status, -1, -1, &event);
		if (ev_status) break;
		pgm = event.data[0];
		cmd = event.event &0xf0;
		chn = event.event &0x0f;

		if (!(pgm & 0x80)) {
			if ((cmd == MIDI_PGM_CHANGE && !IS_PERC(chn)) || (cmd == MIDI_NOTE_ON && IS_PERC(chn))) {
				int instrument = pgm;

				if (IS_PERC(chn))
					instrument |= 128;
				else if (mdata->midi_force_chn_pgm[chn])
					instrument = mdata->midi_force_chn_pgm[chn]-1;

				instruments++;
				gsisynth_load_instrument (hw, hw_status, instrument /* pgm */);
			}
		}
	}
	if (instruments == 0) {
		//DEB(DPRT("extra: preloading piano\n"));
		instruments ++;
		gsisynth_load_instrument (hw, hw_status, 0 /* pgm */);
	}
}

/* EOF midi_play.c */
