/* GSI - general sound interface
 *
 * utils.c - libgsicd, cd utils (msf <-> lba etc)
 *
 * Lower level API dependencies:
 *
 * Copyright (C) 1998 - 2000 W.H.Scholten
 *
 * Redistribution and use in source and binary forms, with or
 * without modification, are permitted provided that the following
 * conditions are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS
 * IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
 * AUTHOR OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */




/*__________________INCLUDES___________________*/
#include <stdio.h>
#include <stdlib.h>


#include "gsicd.h"



int _gsicd_nearest_track(int cd)
{
	int  lba_pos;
	int  nearest;
	struct gsicd_status pos;

	_gsicd_get_status (cd, &pos);
	lba_pos = gsicd_lba_from_msf (pos.min, pos.sec, pos.frame);
	if (abs(lba_pos - gsicd_lba_track_pos(cd, pos.track)) <
		abs(lba_pos - gsicd_lba_track_pos(cd, pos.track+1))) {
		nearest = pos.track;
	}
	else {
		nearest = pos.track+1;
	}

	return nearest;
}


/*---------------------------------------------------------------------------*
 *      lba <-> msf utils
 *---------------------------------------------------------------------------*/
int gsicd_m_from_lba (int lba) {
	return lba/(75*60);
}
int gsicd_s_from_lba (int lba) {
	return (lba/75) %60;
}
int gsicd_f_from_lba (int lba) {
	return (lba % 75);
}
int gsicd_lba_from_msf (int m, int s, int f) {
	return m*60*75+s*75+f;
}

int gsicd_lba_track_len(int cd, int i) {
	//DEB(DPRT("lba track len for %d, num_tracks=%d\n", i, cd_status[ cd ]->num_tracks));
	if (i > gsicd_status[ cd ]->num_tracks) i = gsicd_status[ cd ]->num_tracks;

	return gsicd_lba_track_pos(cd, i+1) - gsicd_lba_track_pos(cd, i);
}

int gsicd_lba_track_pos(int cd, int i) {
	if (i > gsicd_status[ cd ]->num_tracks+1) i = gsicd_status[ cd ]->num_tracks+1;

	return gsicd_lba_from_msf (gsicd_status[ cd ]->track[i].min, gsicd_status[ cd ]->track[i].sec, gsicd_status[ cd ]->track[i].frame);
}


/*      EOF utils.c */
