/* GSI - general sound interface
 *
 * pcm_ultra.c - pcm mode setting
 * THIS CODE (AT LEAST FRAGMENT SETTING) DOESN'T WORK FOR SOME REASON. HELP!
 *
 * Lower level API dependencies:
 *   - ultrasound PCM code
 *
 *
 * Copyright (C) 1996 - 2000 W.H.Scholten
 *
 * Redistribution and use in source and binary forms, with or
 * without modification, are permitted provided that the following
 * conditions are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS
 * IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
 * AUTHOR OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#if USE_ULTRASOUND
#  include <libgus.h>

/* from mixer.c: */
extern int  ultra_card;
extern int  ultra_dev;

void *ultra_pcm_handle = NULL; /* do we need different handles for each device opened (pcm/mixer/synth) ? */

void ultra_sync_pcm();


/*---------------------------------------------------------------------------*
 *      setup_pcm (ultrasound):
 *        opens device (pcmdev; usually /dev/pcm)
 *        sets fragsize (2 fragments of size 'fragsize', or 3 of fragsize/2).
 *        sets samplerate ('samplerate')
 *        sets mono/stereo ('channels')
 *        sets sample type; 8 bit mulaw/8 bit unsigned/16 bit ('type')
 *---------------------------------------------------------------------------*/
int api_setup_pcm (GSIPCM_HW, int samplerate, int channels, int type, int log2_fragsize) {
	int  status;
	int  value;
	int  format;
#if  USE_OSS
	int  snd_formats[] = { AFMT_MU_LAW, AFMT_U8, AFMT_S16_LE };
#endif
#if  USE_ULTRASOUND /* ultrasound constants are same as OSS constants so not really needed */
	int  snd_formats[] = { AFMT_MU_LAW, AFMT_U8, GUS_PCM_MODE_S16_LE };
#endif
	char *snd_format_strings[] = { "mulaw", "unsigned 8 bit", "signed 16 bit" };
	int  first_pcm_type;
	/* pcmdev not needed for ultra driver: */
	//char *pcmdev = pcm_devices[dev].device;


	DEB1(DPRT("("__FILE__"): *setup_pcm* (ultrasound)\n"));

	/* A few sanity checks, put this later in the code ? */
	samplerate = min(samplerate, 48000);
	samplerate = max(samplerate, 4000);
	type  = min(type, GSI_16BIT);
	type  = max(type, GSI_8BIT_MULAW);
	log2_fragsize = min(log2_fragsize, 13); /* max 8K buffer at the moment */


	/* Fragment size. */
	DEB1(DPRT("  setting fragment to 2^%d\n", log2_fragsize));
	if (log2_fragsize<=3) {
		/* Choose the best fragsize
		 *                   1  2  3  4  5  6  7  8  9 10 11 12 */
		int  base_log[ 12 ] = { 0, 1, 2, 2, 2, 3, 3, 3, 3, 3, 3, 4 };
		log2_fragsize += base_log[ (samplerate+2000)/4000 -1 ]+7+(channels >> 1)+(gsipcm_bytes_per_sample[ type ] >> 1);
		/* e.g. 8000Hz mono 8 bit=256, 48000Hz, stereo 16 bit=8192 */
		DEB1(DPRT("  auto log2_fragsize=%d\n", log2_fragsize));
	}
	log2_fragsize = max(log2_fragsize, 4); /* sanity check */
	log2_fragsize = min(log2_fragsize, 13); /* sanity check */

	if (pcm < 0) { /* not yet opened */
		pcm = gus_pcm_open(&ultra_pcm_handle, ultra_card, ultra_dev, GUS_PCM_DIRECTION_PLAYBACK /*direction*/, GUS_PCM_OF_NONE);
		if (pcm == 0) pcm = 1; else { pcm = -1; errno = EBUSY; }

		if (pcm < 0) {
			if (errno == EBUSY) {
				return GSIPCM_BUSY;
			}
			else {
				return GSIPCM_NOT_PRESENT;
			}
			return -1;
		}
	}



	/* Set fragment no/size. */
	hw_status->fragsize = gus_pcm_dma_size_get(ultra_pcm_handle, GUS_PCM_DIRECTION_PLAYBACK /*direction*/, 0);
	DEB1(DPRT("  gus frag size=%x\n", hw_status->fragsize));
	status = gus_pcm_dma_size_set(ultra_pcm_handle, GUS_PCM_DIRECTION_PLAYBACK /*direction*/, 3 /*fragments*/, (1 << /*hw_status->log2_fragsize*/ log2_fragsize) /*fragment_size*/);
//	gus_queue_flush(ultra_pcm_handle);
//	gus_do_flush(ultra_pcm_handle);
	DEB1(DPRT("  set frag: status=%d, frag req=%d\n", status, 1 << log2_fragsize));


	hw_status->samplerate = samplerate;
	hw_status->log2_fragsize = log2_fragsize;
	hw_status->channels = channels;
	hw_status->type = type;



	/* Set number of channels. */
	value = hw_status->channels-1; //0=mono, 1=stereo
	status = gus_pcm_channels_set(ultra_pcm_handle, GUS_PCM_DIRECTION_PLAYBACK /*direction*/, value+1);

	if (hw_status->channels == 2 && value != 1) {
		DEB1(DPRT(" device doesn't support stereo, using mono.\n"));
	}
	hw_status->channels = value+1;
	DEB1(DPRT("  pcm_channels set to %d\n", value+1));



	/* Set sample format. */
	first_pcm_type = hw_status->type;
	for (;;) {
		/* Sample size: 8 bit mulaw, 8 bit unsigned or 16 bit */
		format = snd_formats[ hw_status->type - 1 ];
//DEB(DPRT("gus_pcm_mode_supported=%x (format=%d)\n\n", gus_pcm_mode_supported(ultra_pcm_handle, GUS_PCM_DIRECTION_PLAYBACK), format));
		if ((gus_pcm_mode_supported(ultra_pcm_handle, GUS_PCM_DIRECTION_PLAYBACK /*direction*/) & format))
			break;

		DEB1(DPRT("audio format %s not supported\n", snd_format_strings[ hw_status->type -1 ]));
		hw_status->type++;
		if (hw_status->type>GSI_16BIT) hw_status->type = GSI_8BIT_MULAW;
		if (hw_status->type == first_pcm_type) {
			//log_message(GSI_MESSAGE_ERROR, "The pcm device doesn't support any of the formats");
			exit(1);
		}
	}
	DEB1(DPRT("  pcm_format set to %s\n", snd_format_strings[ hw_status->type -1 ]));



	/* Set samplerate. */
	value = samplerate;
	value = min(value, gus_pcm_rate_supported(ultra_pcm_handle, GUS_PCM_DIRECTION_PLAYBACK /*direction*/));
	ultra_sync_pcm();
	if (gus_pcm_rate_set(ultra_pcm_handle, GUS_PCM_DIRECTION_PLAYBACK /*direction*/, hw_status->samplerate) < 0) {
		//log_message(GSI_MESSAGE_ERROR, "ultrasound: Couldn't set pcm rate");
		//return -1;
	}
	value = gus_pcm_rate_get(ultra_pcm_handle, GUS_PCM_DIRECTION_PLAYBACK /*direction*/);
//}

	DEB1(DPRT("  pcm_samplerate set to %d\n", value));
	hw_status->samplerate = value;



	/* Fragment size check ------------------- */
	hw_status->fragsize = 0;
	//hw_status->fragsize = gus_pcm_dma_size_get(ultra_pcm_handle, GUS_PCM_DIRECTION_PLAYBACK /*direction*/, 0);

	hw_status->fragsize = 1 << log2_fragsize;

	hw_status->bps = gsipcm_bytes_per_sample[ hw_status->type ];

	hw_status->samples = hw_status->fragsize/(hw_status->bps*hw_status->channels);

	hw_status->max_write_space = api_pcm_write_space(hw, hw_status);

	/* Should make sure that hw_status->samples/hw_status->samplerate >.02 s
	 * for most unices (.02s = minimal repeated usleep time for .01 s system
	 * resolution). */

	api_pcm_set_mode (hw, hw_status, hw_status->modus);

	DEB1(DPRT("  pcm settings: bps=%d, samples=%d [ samplerate=%d, channels=%d, type=%d, log2_frag=%d, frag=%d ]\n", hw_status->bps, hw_status->samples, hw_status->samplerate, hw_status->channels, hw_status->type, hw_status->log2_fragsize, hw_status->fragsize));

	DEB1(DPRT("#setup_pcm#\n"));

	return 0;
}


int api_pcm_write (GSIPCM_HW, void *buf, int bytes_to_write) {
	return gus_pcm_write(ultra_pcm_handle, buf, bytes_to_write);
}


/*---------------------------------------------------------------------------*
 *      Some utils
 *---------------------------------------------------------------------------*/
/* Sync is needed when changing samplerate, stereo etc. during output. */
void ultra_sync_pcm() {
	if ( gus_pcm_sync(ultra_pcm_handle, GUS_PCM_DIRECTION_PLAYBACK /*direction*/) < 0 ) {
		//log_message(GSI_MESSAGE_ERROR, "ultrasound: sync failed");
		exit (-1);
	}
}


int api_pcm_set_mode (GSIPCM_HW, int modus) {
	int status = -1;

	switch(modus) {
	case GSIPCM_MODE_NON_BLOCKING:
		break;
	case GSIPCM_MODE_BLOCKING:
		break;
	case GSIPCM_MODE_DMA:
		break;
	}

	return status;
}


int api_pcm_write_space (GSIPCM_HW) {
	int  status;
	int  audio_space = 0;

	status = -1; /* Temp, which libgus function gives this information ? */
	DEB(DPRT("ultra pcm outspace = %d (st=%d)\n", audio_space, status));
	if (status < 0) audio_space = -1;

	return audio_space;
}

int api_pcm_buffer_status (GSIPCM_HW, int *writespace, int *readdata) {
	int  status;

	if (writespace) {
		*writespace = api_pcm_write_space(hw, hw_status);
	}
	if (readdata) {
		/* not done yet */
	}

	return 0;
}
#endif /* USE_ULTRASOUND */


/*      EOF pcm_ultrasound.c */
