/* GSI - general sound interface
 *
 * io.h - byte order stuff, splitting int16/int32, reading multiple bytes etc.
 *
 * 1996 - 2000 by W.H.Scholten
 *
 * This file is released to the public domain.
 */

#ifndef _GSI__gsi__io_h
#define _GSI__gsi__io_h


#include <gsi/whs_types.h>
#include <gsi/gsi_config.h>

#define SPLIT_MSB_INT32(a)   ((a) >> 24), ((a) >> 16), ((a) >> 8), (a)
#define SPLIT_MSB_INT16(a)   ((a) >> 8), (a)

#define swap_int64(a) ((((a) & 255) << 56) | (((a) & 0xff00) << 40) | ((((a) & 0xff0000) << 24) | (((a) & 0xff000000) << 8)    |     (((a) >> 8) & 0xff000000) | (((a) >>24) &0xff0000)) | (((a) >> 40) & 0xff00) | (((a) >>56) &0xff))
#define swap_int32(a) ((((a) & 255) << 24) | (((a) & 0xff00) << 8) | (((a) >> 8) & 0xff00) | (((a) >>24) &0xff))
#define swap_int16(a) ((((a) & 0xff) << 8) | (((a) >> 8) & 0xff))

#ifdef SYSCONFIG_LITTLE_ENDIAN_MACHINE
#  define msb_from_native_int32(a) swap_int32(a)
#  define msb_from_native_int16(a) swap_int16(a)
#  define lsb_from_native_int32(a) (a)
#  define lsb_from_native_int16(a) (a)
#else
#  define msb_from_native_int32(a) (a)
#  define msb_from_native_int16(a) (a)
#  define lsb_from_native_int32(a) swap_int32(a)
#  define lsb_from_native_int16(a) swap_int16(a)
#endif


/* low level */
void gsi_flush_until_free (gsi_server_t *c, int nfree);
void gsi_send_byte (gsi_server_t *c, int cmd);
void gsi_send_string(gsi_server_t *c, char *string);

/* direct to the pipe/socket: */
void _gsi_send_block (gsi_server_t *c, byte *data, int len);


#endif /* _GSI__gsi__io_h */
