/* GSI - general sound interface
 *
 * spec_to_header.c: Makes header files; the first defines commands, the
 * second has a table with the length of the commands (bytes), the third has
 * a table of strings with the command names.
 *
 * By W.H.Scholten 1998 - 2000
 *
 * This file is public domain.
 *
 * Use: spec_to_header input_file [ cmd_h_name [ cmd_length_h_name [ cmd_names_h_name ] ] ] type
 * type = 1 for command with GSI_CMD_CURRENT adjustment, 0 otherwise.
 */


/*__________________INCLUDES___________________*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define False 0
#define True  1


/*__________________PROGRAMMA___________________*/
int main(int argc, char *argv[]) {
	char  bestand_in[100];
	FILE *fp_spec, *fp_command_h, *fp_command_length_h, *fp_string_h;
	char *command_h = NULL;
	char *command_length_h = NULL;
	char *string_h = NULL;
	int   i;

	int   entry_no;
	char  config_line[8192];

	int   command_no = 0;
	char *command_string[256];
	int   command_length[256];

	int   current_adjust;

	if (argc < 2 || (argc == 2 && !strcmp(argv[1], "-h"))) {
		//use();
		exit(1);
	}

	printf("Making header files from specification file %s:\n", argv[1]);

	if (argc < 2) {
		printf("interactive modus\n");
		printf("Name of the specification file ? ");
		scanf("%50s", bestand_in);
	}
	else strcpy(bestand_in, argv[1]);

	if (argc >= 3) command_h = argv[2];
	if (argc >= 4) command_length_h = argv[3];
	if (argc >= 5) string_h = argv[4];

	if (argc < 6) {
		fprintf(stderr, "missing arg\n");
		exit(1);
	}
	current_adjust = atol(argv[5]);

	for (i=0; i<256; i++) {
		command_length[i] = 0;
		command_string[i] = NULL;
	}

	fp_spec = fopen(bestand_in, "r");
	if (fp_spec == NULL) {
		fprintf(stderr, "File %s cannot be opened\n\n", bestand_in);
		exit(1);
	}

	printf(" %s -> %s / %s / %s\n\n", bestand_in, command_h, command_length_h, string_h);

	fseek(fp_spec, 0L, SEEK_SET);

	/* Initialize */
	for (i=0; i<256; i++) command_length[i] = 0;

	entry_no = 0;
	for (;;) {
		char  *token;
		static int  line_number = 0;
		int    done;

		fgets(config_line, 8100, fp_spec); config_line [ 8101 ] = 0;
		if (feof(fp_spec)) break;
		line_number++;
		if (strlen(config_line) == 0) break; /* empty entry */

		/* Get rid of the linefeed */
		if (config_line[(int) strlen(config_line) -1] == 10)
			config_line[(int) strlen(config_line) -1] = 0;

		token = NULL;
		for (;;) {
			if (token == NULL) {
				token = strtok(config_line, " \t");
				if (token == NULL) break;
			}
			else {
				token = strtok(NULL, " \t");
				if (token == NULL) break;
			}

			done = False;
			if (token[0] == '#' || !strncmp(token, "--", 2) || !strncmp(token, "//", 2))
				break; /* Rest of line is a comment. */

			switch (entry_no) {
			case 0:
				command_no = atol(token);
				break;
			case 1:
				if (command_string[ command_no ] != NULL) {
					fprintf(stderr, "Specification error in %s line %d: %s (%d) redefined as %s\n", argv[1], line_number, command_string[ command_no ], command_no, token); 
					exit(-1);
				}
				command_string[ command_no ] = strdup(token);
				if (command_string[ command_no ] == NULL) {
					fprintf(stderr, "Out of memory\n");
					exit(1);
				}
				break;
			case 2:
				command_length[ command_no ] = atol(token);
				break;
			}

			entry_no ++;
			if (entry_no == 3) entry_no = 0;
		}
	}

	if (current_adjust) {
		/* command lengths for CURRENT (only commands, not events): */
		for (i=0; i<32; i++) {
			int  tmp;

			if (command_length[ 64+32+i ] != 0) {
				fprintf(stderr, "Specification error in %s: CURRENT version for command %s overwrites command %s\n", argv[1], command_string[ 64+i ], command_string[ 64+32+i ]); 
				exit(1);
			}
			if (command_length[ 128+32+i ] != 0) {
				fprintf(stderr, "Specification error in %s: CURRENT version for command %s overwrites command %s\n", argv[1], command_string[ 128+i ], command_string[ 128+32+i ]); 
				exit(1);
			}

			tmp = command_length[  64+i];
			command_length[  64+32+i] = (tmp > 0 ? tmp-1 : (tmp < 0 ? tmp+1 : 0));

			tmp = command_length[ 128+i];
			command_length[ 128+32+i] = (tmp > 0 ? tmp-1 : (tmp < 0 ? tmp+1 : 0));
		}
	}

	/* Write the output files: */
	/* command length: */
	fp_command_length_h = fopen(command_length_h, "w");
	for (i=0; i<256; i++) {
		fprintf(fp_command_length_h, "\t%d,\n", command_length[i]);
	}
	fprintf(fp_command_length_h, "};\n");

	/* commands: */
	fp_command_h = fopen(command_h, "w");
	for (i=0; i<256; i++) {
		if (command_length[i] != 0 && command_string[i] != NULL) {
			fprintf(fp_command_h, "#define %s %d\n", command_string[i], i);
		}
	}

	/* strings: */
	fp_string_h = fopen(string_h, "w");
	for (i=0; i<256; i++) {
		if (command_length[i] != 0) {
			if (command_string[i] != NULL) {
				fprintf(fp_string_h, "\t\"%s\",\n", command_string[i]);
			}
			else if (current_adjust) {
				fprintf(fp_string_h, "\t\"(CURRENT) %s\",\n", command_string[i-32]);
			}
		}
		else {
			fprintf(fp_string_h, "\t\"invalid\",\n");
		}
	}

	fclose (fp_command_h);
	fclose (fp_command_length_h);
	fclose (fp_string_h);

	return 0;
}

/* EOF spec_to_header.c */
