#!/bin/sh
# GSI - general sound interface
#
# script for conversion of command/event specification -> C headers
# By W.H.Scholten, 1998 - 2000
#
# This file is released to the public domain.
#
# Go go gadget copter.


if [ ! -d "work" ]; then
    mkdir work
fi

cd work

# ----------------------------------------------------------------------------#
# Commands
# ----------------------------------------------------------------------------#
if ! ../spec_to_header ../../protocol/commands.spec tmp_commands.h  tmp_command_lengths.h tmp_command_strings.h 1
then
    echo "Invalid spec command result"
    exit 1
fi

# make gsi_commands.h
f=gsi_commands.h
echo "/* GSI - general sound interface" > $f
echo " *" >> $f
echo " * Command definitions (automatically generated from commands.spec)" >> $f
echo " */" >> $f
echo >> $f
echo "#ifndef _GSI__gsi_commands_h" >> $f
echo "#define _GSI__gsi_commands_h" >> $f
echo >> $f
cat tmp_commands.h >> $f
echo >> $f
echo "#endif /* _GSI__gsi_commands_h */" >> $f


# make gsi_command_lengths.h
f=gsi_command_lengths.h
echo "/* GSI - general sound interface" > $f
echo " *" >> $f
echo " * Command lengths in bytes (automatically generated from commands.spec)" >> $f
echo " */" >> $f
echo >> $f
echo "static signed char gsi_command_length[256] = {" >> $f
cat tmp_command_lengths.h >> $f
echo >> $f
echo "/* EOF command_lengths.h */" >> $f

#make debug file with command names
f=gsi_command_names.h
echo "/* GSI - general sound interface" > $f
echo " *" >> $f
echo " * Command names (automatically generated from commands.spec)" >> $f
echo " */" >> $f
echo >> $f
echo "static char *gsi_command_name[256] = {" >> $f
cat tmp_command_strings.h >> $f
echo "};" >> $f
echo >> $f
echo "/* EOF command_names.h */" >> $f

rm -f tmp_commands.h  tmp_command_lengths.h  tmp_command_strings.h


# ----------------------------------------------------------------------------#
# Events
# ----------------------------------------------------------------------------#
if ! ../spec_to_header ../../protocol/events.spec   tmp_events.h    tmp_event_lengths.h tmp_event_strings.h 0
then
    echo "Invalid spec event result"
    exit 1
fi


# make gsi_events.h
f=gsi_events.h
echo "/* GSI - general sound interface" > $f
echo " *" >> $f
echo " * Event definitions (automatically generated from events.spec)" >> $f
echo " */" >> $f
echo >> $f
echo "#ifndef _GSI__gsi_events_h" >> $f
echo "#define _GSI__gsi_events_h" >> $f
echo >> $f
cat tmp_events.h >> $f
echo >> $f
echo "#endif /* _GSI__gsi_events_h */" >> $f

# make gsi_event_lengths.h
f=gsi_event_lengths.h
echo "/* GSI - general sound interface" > $f
echo " *" >> $f
echo " * Event lengths in bytes (automatically generated from events.spec)" >> $f
echo " */" >> $f
echo >> $f
echo "static signed char gsi_event_length[256] = {" >> $f
cat tmp_event_lengths.h >> $f
echo >> $f
echo "/* EOF event_lengths.h */" >> $f

#make debug file with event names
f=gsi_event_names.h
echo "/* GSI - general sound interface" > $f
echo " *" >> $f
echo " * Event names (automatically generated from commands.spec)" >> $f
echo " */" >> $f
echo >> $f
echo "static char *gsi_event_name[256] = {" >> $f
cat tmp_event_strings.h >> $f
echo "};" >> $f
echo >> $f
echo "/* EOF event_names.h */" >> $f


rm -f tmp_events.h    tmp_event_lengths.h  tmp_event_strings.h

# Done

cd ..


# EOF
