/* GSI - general sound interface
 *
 * error.c - gsif library; error messages
 *
 * Lower level API dependencies: 
 *
 *
 * Copyright (C) 2001 W.H.Scholten
 *
 * Redistribution and use in source and binary forms, with or
 * without modification, are permitted provided that the following
 * conditions are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS
 * IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
 * AUTHOR OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */



/*__________________INCLUDES___________________*/
//#include <gsi/gsif_config.h>
#include <stdio.h>

#include <gsi/config.h>
#include <gsi/whs_types.h>
#define DEBUG_PREFIX "GSIF_"
//#include <gsi/gsi_debug.h>

#include "include/gsif.h"


char *gsif_strerror(int _errno) {
	switch (_errno) {
		case GSIF_OK: return "ok";
		case GSIF_ERROR_STATUS_UNKNOWN: return "general failure";
		case GSIF_ERROR_OUT_OF_MEMORY: return "out of memory";

		case GSIF_ERROR_BAD_ARGUMENTS: return "Bad arguments";

		case GSIF_ERROR_NO_SUCH_FILE: return "no such file";
		case GSIF_ERROR_CANT_OPEN_FILE: return "cannot open file";

		case GSIF_ERROR_NULL_DEREFERENCE: return "NULL derefence";
		case GSIF_ERROR_FILE_CORRUPT: return "corrupt file";
		case GSIF_ERROR_UNHANDLED_FILE_TYPE: return "unhandled file type";
		case GSIF_ERROR_UNHANDLED_FORMAT_CONVERSION: return "unhandled format conversion";
		case GSIF_ERROR_UNKNOWN_TYPE: return "unknown sample type";

			/* Low level errors: */
		case GSIF_ERROR_WRITE_TO_FILE_FAILED: return "low level write failed";
		case GSIF_ERROR_READ_FROM_FILE_FAILED: return "low level read failed";

//		case GSIFMIDI_STATUS_NO_MORE_EVENTS: return "no more events";
//		case GSIFMIDI_STATUS_NO_EVENTS_WITHIN_LIMIT: return "no more events within limit";

	default:
		return "";
	}
}

/*      EOF io.c */
