/* GSI - general sound interface
 *
 * formats.c - gsif library; list of supported file formats
 *
 * Lower level API dependencies: 
 *
 *
 * Copyright (C) 2001 W.H.Scholten
 *
 * Redistribution and use in source and binary forms, with or
 * without modification, are permitted provided that the following
 * conditions are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS
 * IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
 * AUTHOR OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */



/*__________________INCLUDES___________________*/
//#include <gsi/gsif_config.h>
#include <stdio.h>

#include <gsi/config.h>
#include <gsi/whs_types.h>
#define DEBUG_PREFIX "GSIF_"
//#include <gsi/gsi_debug.h>

#include "include/gsif.h"


typedef struct _gsif_format {
	char *name;
	char *description;
} gsif_format_t;

/* Could be made extendable (gsif modules) by making this dynamic */
gsif_format_t gsif_formats[ 10 ] = {
	{ "AU", "Sun/NeXT" },
	{ "aiff/aifc", "Apple" },
	{ "VOC", "Creative Labs" },
	{ "WAV", "" },
	{ "MIDI", "" },
	{ "HMP/HMI", "Human machine interface" },
	{ "MUS", "" },
};


/* returns 0 if ok, -1 if the format number does not exist, use as follows:
 * call this function for 0, 1, 2,... until it returns -1. This gives a list
 * of all supported file formats.
 */
int gsif_get_format(char **name, char **description, int index) {
	*name = NULL;
	if (index < 0)
		return -1;

//	if (!--index) *format_name = "AU";
	if (index < (sizeof(gsif_formats) / sizeof(gsif_format_t))) {
		*name = gsif_formats [ index ].name;
		*description = gsif_formats [ index ].description;
	}

	if (*name == NULL)
		return -1;

	return 0;
}

/*      EOF io.c */
