/* GSI - general sound interface: libgsif
 *
 * io.h - byte order stuff, reading multiple bytes etc.
 *
 * 1998 - 2001 by W.H.Scholten
 *
 * This file is released to the public domain.
 */


#ifndef _GSI__gsif__io_h
#define _GSI__gsif__io_h


#include <gsi/whs_types.h>
//#include <gsi/gsif_config.h>
#include <gsi/config.h>

#define swap_int64(a) ((((a) & 255) << 56) | (((a) & 0xff00) << 40) | ((((a) & 0xff0000) << 24) | (((a) & 0xff000000) << 8)    |     (((a) >> 8) & 0xff000000) | (((a) >> 24) &0xff0000)) | (((a) >> 40) & 0xff00) | (((a) >> 56) &0xff))
#define swap_int32(a) ((((a) & 255) << 24) | (((a) & 0xff00) << 8) | (((a) >> 8) & 0xff00) | (((a) >> 24) &0xff))
#define swap_int16(a) ((((a) & 0xff) << 8) | (((a) >> 8) & 0xff))

#ifdef SYSCONFIG_LITTLE_ENDIAN_MACHINE
#  define msb_from_native_int32(a) swap_int32(a)
#  define msb_from_native_int16(a) swap_int16(a)
#  define lsb_from_native_int32(a) (a)
#  define lsb_from_native_int16(a) (a)
#else
#  define msb_from_native_int32(a) (a)
#  define msb_from_native_int16(a) (a)
#  define lsb_from_native_int32(a) swap_int32(a)
#  define lsb_from_native_int16(a) swap_int16(a)
#endif

#define msb_int32_at(buf)   msb_from_native_int32(*((int32 *) (buf)))
#define msb_int16_at(buf)   msb_from_native_int16(*((int16 *) (buf)))

#define lsb_int32_at(buf)   lsb_from_native_int32(*((int32 *) (buf)))
#define lsb_int16_at(buf)   lsb_from_native_int16(*((int16 *) (buf)))


int16 gsif_msb_int16_at(byte *buf);
int32 gsif_msb_int32_at(byte *buf);
int32 gsif_lsb_int32_at(byte *p);
int16 gsif_lsb_int16_at(byte *p);

#if 1
/* File functions with status chaining: */

/* Read from/write to a file. status is set to -1 (or some other non zero
 * value) in case of a file reading/writing error, it is NOT changed if all
 * went well, so set it to 0 before calling one or more of these functions.
 * The status can then be checked after calling one (or more) of these
 * functions. This eliminates having to check the status of all reads/writes.
*/
int32  gsif_r_msb_int32(int fd, int *status);
int32  gsif_r_msb_int24(int fd, int *status);
int16  gsif_r_msb_int16(int fd, int *status);
int32  gsif_r_lsb_int32(int fd, int *status);
int32  gsif_r_lsb_int24(int fd, int *status);
int16  gsif_r_lsb_int16(int fd, int *status);
byte   gsif_r_byte(int fd, int *status);
void   gsif_r_block(int fd, void *data, int length, int *status);

void gsif_w_msb_int32(int fd, int32 i, int *status);
void gsif_w_msb_int24(int fd, int32 i, int *status);
void gsif_w_msb_int16(int fd, int16 i, int *status);
void gsif_w_lsb_int32(int fd, int32 i, int *status);
void gsif_w_lsb_int24(int fd, int32 i, int *status);
void gsif_w_lsb_int16(int fd, int16 i, int *status);
void gsif_w_byte(int fd, byte i, int *status);
void gsif_w_block(int fd, void *data, int length, int *status);

#else
/* Standard read/write functions: */
/* Read from file. Return value = 0 if OK, -1 in case of file reading error. */
int  gsif_r_msb_int32(int fd, int32 *val);
int  gsif_r_msb_int24(int fd, int32 *val);
int  gsif_r_msb_int16(int fd, int16 *val);
int  gsif_r_lsb_int32(int fd, int32 *val);
int  gsif_r_lsb_int24(int fd, int32 *val);
int  gsif_r_lsb_int16(int fd, int16 *val);
int  gsif_r_byte(int fd, byte *val);

/* Write to file. Return value = 0 if OK, -1 in case of file writing error
 * (actually any non-zero value is an error). */
int  gsif_w_msb_int32(int fd, int32 i);
int  gsif_w_msb_int24(int fd, int32 i);
int  gsif_w_msb_int16(int fd, int16 i);
int  gsif_w_lsb_int32(int fd, int32 i);
int  gsif_w_lsb_int24(int fd, int32 i);
int  gsif_w_lsb_int16(int fd, int16 i);
int  gsif_w_byte(int fd, byte i);
int  gsif_w_block(int fd, void *data, int length);
#endif

int32 gsif_swap_int32(int32 a);
int16 gsif_swap_int16(int16 a);


#endif /* _GSI__gsif__io_h */
