/* GSI - general sound interface
 *
 * fill_buffer.c - gsif library; fill a buffer with a given sample format of
 * given value.
 *
 * Lower level API dependencies: none (standard file system stuff)
 *
 *
 * Copyright (C) 1998 - 2001 W.H.Scholten
 *
 * Redistribution and use in source and binary forms, with or
 * without modification, are permitted provided that the following
 * conditions are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS
 * IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
 * AUTHOR OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */


/*__________________INCLUDES___________________*/
//#include <gsi/gsif_config.h>
#include <gsi/config.h>

#include <stdio.h>
#include <stdlib.h>

#include <gsi/whs_defines.h>
#include <gsi/whs_types.h>
#define DEBUG_PREFIX "GSIF_"
//#include <gsi/gsi_debug.h>

#include "../include/gsif.h"

#include "sample_format.h"
#include "../io.h"
#include "mulaw.h"


/*---------------------------------------------------------------------------*
 *      Fill a buffer with a given 32 bit value (rounded down to whatever
 *      accuracy the buffer uses).
 *---------------------------------------------------------------------------*/
int  gsif_fill_buffer(byte *buffer, int num_samples, int channels, int type, int value) {
	int  i, myvalue;

	switch (type) {
	case GSI_8BIT_ALAW:
		/* not yet handled */
		return -1;
		break;
	case GSI_8BIT_MULAW:
	{
		myvalue = linear16_to_mulaw(value >> 16); /* signed shift. */
		for (i=0; i<num_samples*channels; i++)
			buffer[i] = myvalue;
		break;
	}
	case GSI_8BIT:
		myvalue = (value >> 24) ^ 128;
		for (i=0; i<num_samples*channels; i++)
			buffer[i] = (byte) myvalue;
		break;
	case GSI_8BIT_SIGNED:
		myvalue = (value >> 24);
		for (i=0; i<num_samples*channels; i++)
			buffer[i] = (byte) myvalue;
		break;
	case GSI_16BIT:
	{
		//int16 *mybuffer= (int16 *) buffer;

		/* MSB int16: */
		myvalue = (value >> 16);
		for (i=0; i<num_samples*channels*2; i++) {
			buffer[i++] = (byte) myvalue >> 8;
			buffer[i] = (byte) myvalue;
		}
		break;
	}
	case GSI_24BIT:
	{
		/* MSB int24: */
		myvalue = (value >> 8);
		for (i=0; i<num_samples*channels*3; i++) {
			buffer[i++] = (byte) myvalue >> 16;
			buffer[i++] = (byte) myvalue >> 8;
			buffer[i] = (byte) myvalue;
		}
		break;
	}
	default:
		/* unhandled format */
		return -1;
	}
	return 0;
}


/*      EOF sample_format.c */
