/* GSI - general sound interface; libgsif
 *
 * sample_format.h - GSI supported file formats
 *
 * 1998 - 2001 by W.H.Scholten
 *
 * This file is released to the public domain.
 */


#ifndef _GSI__gsif__sample__sample_format_h
#define _GSI__gsif__sample__sample_format_h

#include "../file.h"


int gsif_get_structure_aif (int fd, struct gsi_sound *snd);
int gsif_get_structure_au (int fd, struct gsi_sound *snd);
int gsif_get_structure_voc (int fd, struct gsi_sound *snd);
int gsif_get_structure_wav (int fd, struct gsi_sound *snd);


/*---------------------------------------------------------------------------*
 *      Sun/Next .au/.snd format
 *---------------------------------------------------------------------------*/
//#define MAGIC_AU 0x2E736E64 /* ".snd" = SIG('.','s','n','d') */
#define MAGIC_AU   SIG('.','s','n','d')

/* the sound format in .au's have 0-255 reserved.
 * 
 */

#define AU_ENCODING_MULAW_8        (1)     /* 8-bit ISDN mu-law */
#define AU_ENCODING_LINEAR_8       (2)     /* 8-bit linear PCM */
#define AU_ENCODING_LINEAR_16      (3)     /* 16-bit linear PCM */
#define AU_ENCODING_LINEAR_24      (4)     /* 24-bit linear PCM */
#define AU_ENCODING_LINEAR_32      (5)     /* 32-bit linear PCM */
#define AU_ENCODING_FLOAT          (6)     /* 32-bit IEEE floating point */
#define AU_ENCODING_DOUBLE         (7)     /* 64-bit IEEE floating point */

#define AU_FORMAT_4BIT_G721       (23)    /* 4-bit CCITT g.721 ADPCM */
#define AU_FORMAT_3BIT_G723       (25)    /* CCITT g.723 3-bit ADPCM */
#define AU_FORMAT_5BIT_G723       (26)    /* CCITT g.723 5-bit ADPCM */
#define AU_FORMAT_ALAW_8          (27)    /* 8-bit ISDN A-law */
#define AU_FORMAT_ALT_4BIT_G721  (101) /* same a format 23 */
#define AU_FORMAT_ALT_3BIT_G722  (102)
#define AU_FORMAT_ALT_5BIT_G723  (103)

#if 0
typedef struct {
	int  magic;               /* magic number SND_MAGIC */
	int  dataLocation;        /* offset or pointer to the data */
	int  dataSize;            /* number of bytes of data */
	int  dataFormat;          /* the data format code */
	int  samplingRate;        /* the sampling rate */
	int  channelCount;        /* the number of channels */
	char info[4];             /* optional text information */
} SNDSoundStruct;
#endif


/*---------------------------------------------------------------------------*
 *      Creative labs VOC format.
 *---------------------------------------------------------------------------*/
#define VOC_TERMINATOR          0
#define VOC_DATA                1
#define VOC_CONTINUE            2
#define VOC_SILENCE             3
#define VOC_MARKER              4
#define VOC_TEXT                5
#define VOC_REPEAT              6
#define VOC_REPEAT_END          7
#define VOC_EXTENDED            8 /* extends VOC_DATA */
#define VOC_DATA_2              9 /* more data types, supersedes VOC_DATA/VOC_EXTENDED */

#define MAGIC_VOC               SIG('C','r','e','a')
#define VOC_ID                  "Creative Voice File\032"
#define VOC_ID_SIZE             20
//#define VOC_DATA_OFFSET         0x001a
#define VOC_VERSION110          0x010a

#define VOC_VERSION120          0x0114



/*---------------------------------------------------------------------------*
 *      The following is from the Audio file formats FAQ (v3.13, 25 May 1998)
 *---------------------------------------------------------------------------*/
/*
Creative Voice (VOC) file format
--------------------------------

From: galt@dsd.es.com

(byte numbers are hex!)

    HEADER (bytes 00-19)
    Series of DATA BLOCKS (bytes 1A+) [Must end w/ Terminator Block]

- ---------------------------------------------------------------

HEADER:
-------
     byte #     Description
     ------     ------------------------------------------
     00-12      "Creative Voice File"
     13         1A (eof to abort printing of file)
     14-15      Offset of first datablock in .voc file (std 1A 00
                in Intel Notation)
     16-17      Version number (minor,major) (VOC-HDR puts 0A 01)
     18-19      2's Comp of Ver. # + 1234h (VOC-HDR puts 29 11)

- ---------------------------------------------------------------

DATA BLOCK:
-----------

   Data Block:  TYPE(1-byte), SIZE(3-bytes), INFO(0+ bytes)
   NOTE: Terminator Block is an exception -- it has only the TYPE byte.

      TYPE   Description     Size (3-byte int)   Info
      ----   -----------     -----------------   -----------------------
      00     Terminator      (NONE)              (NONE)
      01     Sound data      2+length of data    *
      02     Sound continue  length of data      Voice Data
      03     Silence         3                   **
      04     Marker          2                   Marker# (2 bytes)
      05     ASCII           length of string    null terminated string
      06     Repeat          2                   Count# (2 bytes)
      07     End repeat      0                   (NONE)
      08     Extended        4                   ***

      *Sound Info Format:       **Silence Info Format:
       ---------------------      ----------------------------
       00   Sample Rate           00-01  Length of silence - 1
       01   Compression Type      02     Sample Rate
       02+  Voice Data

    ***Extended Info Format:
       ---------------------
       00-01  Time Constant: Mono: 65536 - (256000000/sample_rate)
                             Stereo: 65536 - (25600000/(2*sample_rate))
       02     Pack
       03     Mode: 0 = mono
                    1 = stereo


  Marker#           -- Driver keeps the most recent marker in a status byte
  Count#            -- Number of repetitions + 1
                         Count# may be 1 to FFFE for 0 - FFFD repetitions
                         or FFFF for endless repetitions
  Sample Rate       -- SR byte = 256-(1000000/sample_rate)
  Length of silence -- in units of sampling cycle
  Compression Type  -- of voice data
                         8-bits    = 0
                         4-bits    = 1
                         2.6-bits  = 2
                         2-bits    = 3
                         Multi DAC = 3+(# of channels) [interesting--
                                       this isn't in the developer's manual]

Detailed description of new data blocks (VOC files version 1.20 and above):

        (Source is fax from Barry Boone at Creative Labs, 405/742-6622)

BLOCK 8 - digitized sound attribute extension, must preceed block 1.
          Used to define stereo, 8 bit audio
        BYTE bBlockID;       // = 8
        BYTE nBlockLen[3];   // 3 byte length
        WORD wTimeConstant;  // time constant = same as block 1
        BYTE bPackMethod;    // same as in block 1
        BYTE bVoiceMode;     // 0-mono, 1-stereo

        Data is stored left, right

BLOCK 9 - data block that supersedes blocks 1 and 8.  
          Used for stereo, 16 bit.

        BYTE bBlockID;          // = 9
        BYTE nBlockLen[3];      // length 12 plus length of sound
        DWORD dwSamplesPerSec;  // samples per second, not time const.
        BYTE bBitsPerSample;    // e.g., 8 or 16
        BYTE bChannels;         // 1 for mono, 2 for stereo
        WORD wFormat;           // see below
        BYTE reserved[4];       // pad to make block w/o data 
                                // have a size of 16 bytes

        Valid values of wFormat are:

                0x0000  8-bit unsigned PCM
                0x0001  Creative 8-bit to 4-bit ADPCM
                0x0002  Creative 8-bit to 3-bit ADPCM
                0x0003  Creative 8-bit to 2-bit ADPCM
                0x0004  16-bit signed PCM
                0x0006  CCITT a-Law
                0x0007  CCITT u-Law
                0x02000 Creative 16-bit to 4-bit ADPCM

        Data is stored left, right

 */


/*---------------------------------------------------------------------------*
 *	.wavs (I think I lost my mind supporting this trash):
 *---------------------------------------------------------------------------*/
/* 
   "RIFF"
   length (4 byte LSB)
   "WAVE"
   "fmt "
   ?? (4b)
   format (2b LSB) [ should be RIFF_WAVE_FORMAT_PCM=0x0001 ]
   channels (2b LSB)
   samplerate (4b LSB),
   ?? (4b LSB)
   ?? (2b LSB)
   bits per sample: 2 b LSB (i.e. 8/16)
   "data"
   ?? (4b)

   data starts at 44 (0x2c)

   I don't support any other stuff, multiple blocks etc. If GSI complains,
   you're screwed. Get a real conversion utility to rid yourself of the junk.
*/


#endif /* _GSI__gsif__sample__sample_format_h */
