/* GSI - general sound interface
 *
 * sample_get_struture_wav.c - gsif library, sample structure loading
 *
 * Lower level API dependencies: none (standard file system stuff)
 *
 *
 * Copyright (C) 1998 - 2001 W.H.Scholten
 *
 * Redistribution and use in source and binary forms, with or
 * without modification, are permitted provided that the following
 * conditions are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS
 * IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
 * AUTHOR OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */


/*__________________INCLUDES___________________*/
//#include <gsi/gsif_config.h>
#include <gsi/config.h>

#include <stdio.h>
#include <stdlib.h>

#include <unistd.h>

#include <gsi/whs_defines.h>
#include <gsi/whs_types.h>
#define DEBUG_PREFIX "GSIF_"
#include <gsi/gsi_debug.h>

#include "../include/gsif.h"

#include "sample_format.h"
#include "../io.h"
//#include "mulaw.h"


/*---------------------------------------------------------------------------*
 *      Returns 0 if recognized as a valid WAV that the library knows how to
 *      handle.
 *      return codes: GSIF_OK/GSIF_ERROR_* from gsif.h
 *---------------------------------------------------------------------------*/
int gsif_get_structure_wav(int fd, struct gsi_sound *new_snd) {
	int  format_status = GSIF_ERROR_STATUS_UNKNOWN;
	int  size, data_start;
	int  channels = 0, type = -1, samplerate = 0;

	int  sig;
	//int  file_pos;
	int  fmt_blocksize;
	int  file_status = 0;

	DEB1(DPRT("   -> sig = MAGIC_WAV\n"));

	// Position just after the 4 byte signature we checked to get here:
	lseek(fd, new_snd->offset + 4, SEEK_SET);

	/*size = gsif_r_msb_int32(fd);*/ lseek(fd, 4, SEEK_CUR);
	if (gsif_r_msb_int32(fd, &file_status) != SIG('W','A','V','E')) return GSIF_ERROR_STATUS_UNKNOWN;
	if (gsif_r_msb_int32(fd, &file_status) != SIG('f','m','t',' ')) return GSIF_ERROR_STATUS_UNKNOWN;
	fmt_blocksize = gsif_r_lsb_int32(fd, &file_status);
DEB(DPRT("blksize = %d\n", fmt_blocksize));

	sig = gsif_r_msb_int16(fd, &file_status);
DEB(DPRT("wav type %04x\n", sig));
	if (sig != 0x0100 /* = BE(RIFF_WAVE_FORMAT_PCM) */ )
		return GSIF_ERROR_STATUS_UNKNOWN;
DEB(DPRT("riff pcm OK\n"));

	channels = gsif_r_lsb_int16(fd, &file_status);
	samplerate = gsif_r_lsb_int32(fd, &file_status);
	if (samplerate < 1 || channels < 1) {
		return GSIF_ERROR_FILE_CORRUPT;
	}

DEB(DPRT("sample rate & chan OK\n"));

	lseek(fd, 6, SEEK_CUR); // 4 for byterate (samplerate * channels * bytes per sample), 2 for bytes per sample

	type = gsif_r_lsb_int16(fd, &file_status) /8 +1;
	if (type == GSI_16BIT) type = GSI_16BIT_LE;
	if (type == GSI_24BIT) type = GSI_24BIT_LE;
	/* For 16/24 bit, do a lsb->msb conversion for all samples? */

	//if (fmt_blocksize==18) lseek(fd, 2, SEEK_CUR); //type seems a 32 bit int.
	lseek(fd, new_snd->offset + 16+4+fmt_blocksize, SEEK_SET);

	//file_pos = lseek(fd, 0, SEEK_CUR);
	for (;;) { /* loop to read all chunks */
		sig = gsif_r_msb_int32(fd, &file_status);
		DEB2(DPRT("   -> new chunk: sig=%c%c%c%c\n", (char) (sig>>24), (char) (sig>>16), (char) (sig>>8), (char) sig));
		if (sig == SIG('I','N','F','O')) {
			//int  info_len = gsif_r_lsb_int32(fd, &file_status);
			sig = gsif_r_msb_int32(fd, &file_status);
			if (sig == SIG('I','N','A','M')) {
				int  comment_len = gsif_r_lsb_int32(fd, &file_status);
				new_snd->comment = calloc(1, comment_len+1);
				if (new_snd->comment == NULL) return GSIF_ERROR_OUT_OF_MEMORY;

				read(fd, new_snd->comment, comment_len);
				new_snd->comment[ comment_len ] = 0;
			}
			//lseek(fd, new_snd->offset + file_pos+info_len+8, SEEK_SET);
		}
		if (sig == SIG('d','a','t','a')) {
//XXX			data_start = 44;
//XXX			size = lseek(fd, 0, SEEK_END) - data_start - new_snd->offset;
			int chunk_size = gsif_r_lsb_int32(fd, &file_status);
			size = chunk_size;
			data_start = lseek(fd, 0, SEEK_CUR) - new_snd->offset;
DEB(DPRT("data size = %d\n", size));

			gsif_add_block(new_snd, samplerate, channels, type, data_start, size);
			gsif_make_linear_block_list(new_snd);
			//gsif_load_block_data(fd, new_snd, /*0*/new_snd->blocks-1);

			new_snd->file_format = GSI_FILE_WAV;
			break; /* not correct to break out but who cares. */
		}
	} /* chunk loop */

	format_status = GSIF_OK;

	return format_status;
}




/*---------------------------------------------------------------------------*
 *      EOF sample_get_structure_wav.c
 *---------------------------------------------------------------------------*/
