/* GSI - general sound interface
 *
 * save.c - gsif library; save samples
 *
 * Lower level API dependencies: 
 *
 *
 * Copyright (C) 1998 - 2001 W.H.Scholten
 *
 * Redistribution and use in source and binary forms, with or
 * without modification, are permitted provided that the following
 * conditions are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS
 * IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
 * AUTHOR OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */



/*__________________INCLUDES___________________*/
//#include <gsi/gsif_config.h>
#include <gsi/config.h>

#include <stdio.h>
#include <stdlib.h>

#include <unistd.h>
#include <fcntl.h>

#include <gsi/whs_types.h>
#include <gsi/whs_defines.h>
#define DEBUG_PREFIX "GSIF_"
#include <gsi/gsi_debug.h>

#include "../include/gsif.h"

#include "../io.h"
#include "../log.h"


/*__________________EXTERN___________________*/
extern int  _gsif_Bps[];
extern int  _gsif_type_value[];



/*---------------------------------------------------------------------------*
 *      Save raw data as a file.
 *
 *      Returns 0 if OK, -1 if unable to save the file (e.g. unsupported save
 *      format, or no write permission).
 *---------------------------------------------------------------------------*/
int  gsif_save_raw_sample (char *filename, int save_format, int rate, int channels, int type, char *data, int data_length, int offset)
{

	return -1;
}


/*---------------------------------------------------------------------------*
 *      Save a gsi_sound/gsi_block sound file:
 *
 *      Returns 0 if OK, -1 if unable to save the file (e.g. unsupported save
 *      format, or no write permission).
 *---------------------------------------------------------------------------*/
int  gsif_save_sample (struct gsi_sound *snd, int file_type, int offset)
{
	int  i, j;
	int  data_length;
	int  fd;
	int  status = GSIF_OK; /* write status, 0 = OK */

	DEB1(DPRT("("__FILE__") gsif_save_sample:\n"));
	fd = open(snd->filename, O_WRONLY);
	if (fd < 0) return GSIF_ERROR_WRITE_TO_FILE_FAILED;

	switch(file_type) {
	case GSI_FILE_AU:
		for (i=0; i < snd->blocks; i++) {
			data_length = snd->block[i]->data_length;
			if (snd->block[i]->data == NULL && snd->block[i]->data_length > 0) {
				/* silence -> add silence of length data_length */
				/* GSI_8BIT */
				byte b = 128;
				for (j=0; j<data_length; j++) {
					gsif_w_block(fd, &b, 1, &status);
				}
			}
			else {
				/* Save block of data */
				/* should convert data to requested format */
				gsif_w_block(fd, snd->block[i]->data, data_length, &status);
			}
		}
		break;
	default:
		log_message(0, "Saving to file type %d not yet implemented", file_type);
		return GSIF_ERROR_WRITE_TO_FILE_FAILED;
	}
	DEB1(DPRT("-> gsif_save_sample: done\n"));

	return status;
}


/*      EOF save.c */
