/* GSI - general sound interface
 *
 * save_aiff.c - gsif library; file functions: save a sound in aiff fileformat
 *
 * Lower level API dependencies: 
 *
 *
 * Copyright (C) 1998 W.H.Scholten
 *
 * Redistribution and use in source and binary forms, with or
 * without modification, are permitted provided that the following
 * conditions are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS
 * IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
 * AUTHOR OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */


/*__________________INCLUDES___________________*/
//#include <gsi/gsif_config.h>
#include <gsi/config.h>

#include <stdio.h>

#include <fcntl.h>

#include <gsi/whs_types.h>
#define DEBUG_PREFIX "GSIF_"
#include <gsi/gsi_debug.h>

#include "../include/gsif.h"

#include "../io.h"



/*---------------------------------------------------------------------------*
 *      Save a sound as aiff.
 * NB. This is an INTERNAL function. It's only available publicly for testing.
 *---------------------------------------------------------------------------*/
int gsif_save_aiff(struct gsi_sound *snd)
{
	int  samplerate, channels, type;
	//int  data_start;
	int  data_length;
	int  i;
	char *filename;
	int  fd;
	int  status = GSIF_OK; /* write status, 0 = OK */


	filename = snd->filename;

	fd = open(filename, O_WRONLY | O_CREAT, 0664);
	if (fd < 0) return GSIF_ERROR_WRITE_TO_FILE_FAILED;

	/* aiff header */
	gsif_w_block(fd, "FORM", 4, &status);

	/* write size after writing all blocks .. */
	gsif_w_block(fd, "AIFC", 4, &status);

	/* write a comment block: */
	if (snd->comment != NULL) {
		data_length = strlen(snd->comment)+1;
		gsif_w_block(fd, "COMT", 4, &status);
		gsif_w_msb_int32(fd, 1, &status); /* sample frames */
		gsif_w_msb_int32(fd, data_length, &status); /* chunk len */
		gsif_w_block(fd, snd->comment, data_length, &status);
	}


	for (i=0; i< snd->blocks; i++) {
		/* write each block (a format block + a data block) */

		/* write sound format block */
		DEB2(DPRT("  writing COMM\n"));
		data_length = 4+2+2;
		gsif_w_block(fd, "COMM", 4, &status);

		gsif_w_msb_int32(fd, data_length, &status);

		samplerate = snd->block[i]->samplerate;
		channels = snd->block[i]->channels;
		type = snd->block[i]->type;

		gsif_w_msb_int16(fd, channels, &status);
		gsif_w_msb_int32(fd, 1, &status); /* sample frames => should be data_len /bps/channels? */

		gsif_w_msb_int32(fd, type, &status);

		/* Apple dipsticks! using a long double, ridiculous */
		{  long double a = (long double) samplerate;
#ifdef SYSCONFIG_LITTLE_ENDIAN_MACHINE
		for (i=0; i<10; i++) {
			gsif_w_block(fd, &a +9-i, 1, &status);
		}
#else
			gsif_w_block(fd, &a, 10, &status);
#endif
		}

		gsif_w_msb_int32(fd, channels, &status);


		/* write sound data block */
		DEB2(DPRT("  writing SSND\n"));
		data_length = snd->block[i]->data_length;
		gsif_w_block(fd, "SSND", 4, &status);

		gsif_w_msb_int32(fd, data_length, &status);

		gsif_w_block(fd, snd->block[i]->data, data_length, &status);
	}

	return status;
}


void gsif_write_aiff_block(int sig, byte *data) {
}


/*      EOF save_aiff.c */
