/* GSI - general sound interface
 *
 * save_au.c - gsif library; file functions: save a sound in au fileformat
 *
 * Lower level API dependencies: 
 *
 *
 * Copyright (C) 1998 W.H.Scholten
 *
 * Redistribution and use in source and binary forms, with or
 * without modification, are permitted provided that the following
 * conditions are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS
 * IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
 * AUTHOR OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */


/*__________________INCLUDES___________________*/
//#include <gsi/gsif_config.h>
#include <gsi/config.h>

#include <stdio.h>

#include <fcntl.h>

#include <gsi/whs_types.h>
#define DEBUG_PREFIX "GSIF_"
//#include <gsi/gsi_debug.h>

#include "../include/gsif.h"

#include "../io.h"


/*---------------------------------------------------------------------------*
 *      Save a sound as .au: saves block 0, so convert a multi block sound
 *      before calling this function.
 * NB. This is an INTERNAL function. It's only available publicly for testing.
 *---------------------------------------------------------------------------*/
int gsif_save_au(struct gsi_sound *snd)
{
	int  samplerate, channels, type;
	int  data_start, data_length;
	char *filename;
	int  fd;
	char *beschrijving;
	int  status = GSIF_OK; /* write status, 0 = OK */

	data_length = snd->block[0]->data_length;

	samplerate = snd->block[0]->samplerate;
	channels = snd->block[0]->channels;
	type = snd->block[0]->type;


	filename = snd->filename;
	beschrijving = snd->comment;


	fd = open(filename, O_WRONLY | O_CREAT, 0664);

	lseek(fd, 0L, SEEK_SET);

	/* .au header */
	gsif_w_block(fd, ".snd", 4, &status);

	if (beschrijving != NULL)
		data_start = 6*4+strlen(beschrijving)+1;
	else
		data_start = 6*4;
	/* Note: audio format faq says 28 = minimum value for data location
	 * (see also the SND struct), hence: */
	if (data_start < 28) data_start = 28;

	gsif_w_msb_int32(fd, data_start, &status);

	gsif_w_msb_int32(fd, data_length, &status);

	gsif_w_msb_int32(fd, type, &status);

	gsif_w_msb_int32(fd, samplerate, &status);

	gsif_w_msb_int32(fd, channels, &status);

	if (beschrijving != NULL) {
		gsif_w_block(fd, beschrijving, strlen(beschrijving)+1, &status);
	}

	lseek(fd, data_start /* offset ? */, SEEK_SET);
	gsif_w_block(fd, snd->block[0]->data, snd->block[0]->data_length, &status);

	return status;
}


/*      EOF save_au.c */
