/* GSI - general sound interface
 *
 * load_mus.c: gsif library; load a mus file and convert for internal use.
 *
 * Lower level API dependencies: standard file functions
 *
 *
 * Copyright (C) W.H.Scholten 1998 - 2001
 *
 * Redistribution and use in source and binary forms, with or
 * without modification, are permitted provided that the following
 * conditions are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS
 * IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
 * AUTHOR OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */


/*__________________INCLUDES___________________*/
//#include <gsi/gsif_config.h>
#include <gsi/config.h>

#include <stdio.h>
#include <stdlib.h>

#include <gsi/whs_defines.h>
#include <gsi/whs_types.h>
#define DEBUG_PREFIX "GSIF_"
#include <gsi/gsi_debug.h>

#include "../include/gsif.h"

#include "../io.h"
#include "../file.h" /* For SIG() */


int mus_to_midi(struct gsi_song *sng);

/*---------------------------------------------------------------------------*
 *      Load a midi in the file at the given offset.
 *
 *      Returns number of tracks (0 if invalid midi file).
 *---------------------------------------------------------------------------*/
//int gsif_load_song(char *filename, int offset, struct gsi_song *new_sng) {

int load_song_mus(int fd, struct gsi_song *sng)
{
	/* mus format (doom/doom2/heretic/raptor ..): */
	int  score_len;
	int  score_start;
	int  channels;
	int  sec_channels;
	int  instr_count;

	int  status = GSIF_OK; /* write status, 0 = OK */

	DEB1(DPRT("L   Found MUS\n"));
	sng->file_format = GSI_FILE_MUS;

	score_len = (int) (uint16) gsif_r_lsb_int16(fd, &status);
	score_start = gsif_r_lsb_int16(fd, &status);
	channels = gsif_r_lsb_int16(fd, &status);
	sec_channels = gsif_r_lsb_int16(fd, &status);
	instr_count = gsif_r_lsb_int16(fd, &status);

#if 0
	/* For debugging etc,
	 * write the MUS to a file: */
	{
		int  pos;
		byte fbyte;
		int  ii;

		int  tmp_fd = open("/home/wouter/tmpfile", O_WRONLY | O_CREAT, 0666);
		DEB(DPRT("tmp_fd=%d\n", tmp_fd));
		pos = lseek(fd, 0, SEEK_CUR);
		lseek(fd, sng->offset, SEEK_SET);
		for (ii=0; ii < score_len+score_start; ii++) {
			gsif_r_block(fd, &fbyte, 1, &status);
			gsif_w_block(tmp_fd, &fbyte, 1, &status);
		}
		close(tmp_fd);
		lseek(fd, pos, SEEK_SET);
	}
#endif

	DEB2(DPRT("L   MUS score_len = %d\n", score_len));
	DEB2(DPRT("L   MUS score_start = %d\n", score_start));
	DEB2(DPRT("L   MUS channels = %d\n", channels));
	DEB2(DPRT("L   MUS secondary channels = %d\n", sec_channels));
	DEB2(DPRT("L   MUS intruments used = %d\n", instr_count));

	lseek(fd, 2, SEEK_CUR); /* skip 2 dummy bytes */

	DEB2(int  j; for (j=0; j<instr_count; j++) {
		int  instr = gsif_r_lsb_int16(fd, &status);
		DPRT("L   Instrument %2d is %3d (%s)\n", j, instr, instr <128 ? gsi_gmvoice_names[instr] : gsi_gmdrum_names[instr -100]);
	}
		);

	lseek(fd, score_start + sng->offset, SEEK_SET);

	sng->data = malloc(score_len);
	if (sng->data == NULL) return GSIF_ERROR_OUT_OF_MEMORY;

	gsif_r_block(fd, sng->data, score_len, &status);
	if (status) return -1;

	sng->data_length = score_len; /* temp */

	/* now convert MUS -> MIDI (should this be called separately?)
	 */
	status = mus_to_midi(sng);

DEB(DPRT("mus_to_midi status = %d\n", status));
DEB(DPRT("mus_to_midi sng->data_length = %d\n", sng->data_length));

	return status;
}


/*      EOF load_mus.c */
