/* GSI - general sound interface
 *
 * midi_utils.c: gsif library; stuff to read/write midi files.
 *
 * Lower level API dependencies: standard file stuff
 *
 *
 * Copyright (C) W.H.Scholten 1998 - 2001
 *
 * Redistribution and use in source and binary forms, with or
 * without modification, are permitted provided that the following
 * conditions are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS
 * IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
 * AUTHOR OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */


/*__________________INCLUDES___________________*/
//#include <gsi/gsif_config.h>
#include <gsi/config.h>

#include <stdio.h>

#include <gsi/whs_types.h>
#include <gsi/whs_defines.h>
#define DEBUG_PREFIX "GSIF_"
#include <gsi/gsi_debug.h>

#include "midi_utils.h"


/*---------------------------------------------------------------------------*
 *  MIDI utilities.
 *---------------------------------------------------------------------------*/
/* Write a variable-length number. Returns the number of bytes written. */
int gsif_midi_write_vl(byte *ptr, int val) {
	int  i, chunks;

	DEB3(DPRT("WRITE_VL   vl=%d\n", val));

	chunks = gsif_midi_length_of_vl(val);
	DEB3(DPRT("WRITE_VL   chunks = %d\n", chunks));

	for (i=chunks-1 ; i>=0; i--) {
		int  tmp = (val >> (i*7)) & 127;
		if (i > 0) tmp |= 128;
		DEB3(DPRT("WRITE_VL   writing 0x%02x\n", tmp));
		*ptr++ = tmp;
	}

	return chunks;
}


/* Number of bytes a given integer would need in the form of a vl-number. */
int gsif_midi_length_of_vl(int val) {
	int  i, chunks;

	for (i=4; i>=0; i--) {
		int  tmp = val & (127 << (i*7));
		if (tmp != 0) break;
	}
	if (i == -1) i = 0;
	chunks = i+1;

	return chunks;
}


/*      EOF song_utils.c */
