/* GSI - general sound interface
 *
 * save_midi.c - gsif library; file functions: save a sound in MIDI fileformat
 *
 * Lower level API dependencies: 
 *
 *
 * Copyright (C) 1998 - 2001 W.H.Scholten
 *
 * Redistribution and use in source and binary forms, with or
 * without modification, are permitted provided that the following
 * conditions are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS
 * IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
 * AUTHOR OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */


/*__________________INCLUDES___________________*/
//#include <gsi/gsif_config.h>
#include <gsi/config.h>

#include <stdio.h>
#include <fcntl.h>

#include <gsi/whs_types.h>
#define DEBUG_PREFIX "GSIF_"
//#include <gsi/gsi_debug.h>

#include "../include/gsif.h"

#include "../io.h"


/*---------------------------------------------------------------------------*
 *      Save a song as midi.
 * NB. This is an INTERNAL function. It's only available publicly for testing.
 *---------------------------------------------------------------------------*/
int gsif_save_midi(struct gsi_song *sng)
{
	int  data_start = 0;
	char *filename;
	int  fd;
	char *beschrijving;
	int  i;
	int  status = 0; /* write status, 0 = OK */

	filename = sng->filename;
	beschrijving = sng->comment;

	fd = open(filename, O_WRONLY | O_CREAT | O_TRUNC, 0664);
	/* Should use O_TRUNC when one wants a new file (or just let the
	 * application remove any old file first?).
	 *
	 * Perhaps use offset =-1 to indicate a new file (as -1 would wrap around
	 * the file anyway (when long=int)).
	 */

	lseek(fd, data_start /* offset */, SEEK_SET);

	/* midi signature & header */
	gsif_w_block(fd, "MThd", 4, &status);

	gsif_w_msb_int32(fd, 6, &status); /* header track length */
	gsif_w_msb_int16(fd, sng->midi_format, &status);
	gsif_w_msb_int16(fd, sng->tracks, &status);
	gsif_w_msb_int16(fd, sng->midi_division, &status);

	for (i=0; i<sng->tracks; i++) {
		/* midi signature & track length & track data */
		gsif_w_block(fd, "MTrk", 4, &status);

		gsif_w_msb_int32(fd, sng->track[i]->data_length, &status);
		gsif_w_block(fd, sng->track[i]->data, sng->track[i]->data_length, &status);


		// perhaps some renaming is in order:
//		gsif_w_msb_int32(fd, sng->track[i]->data_length);
//gsif_w_block (fd, sng->track[i]->data, sng->track[i]->data_length, &status);
	}

	return status;
}


/*      EOF save_midi.c */
