/*
 * $Id: midiin.c,v 1.1 1998/06/04 05:33:28 keithp Exp $
=> upgraded to 1.2 1998/08/14
 *
 * Modified by W.H.Scholten
 *
 * Copyright  1998 Keith Packard
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of Keith Packard not be used in
 * advertising or publicity pertaining to distribution of the software without
 * specific, written prior permission.  Keith Packard makes no
 * representations about the suitability of this software for any purpose.  It
 * is provided "as is" without express or implied warranty.
 *
 * KEITH PACKARD DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
 * EVENT SHALL KEITH PACKARD BE LIABLE FOR ANY SPECIAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
 * DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 */


/* WHS: Midi input: reads raw midi data, like coming in from a midi port
 * I.e. stream midi..
 */

#include <stdio.h>
#include <fcntl.h>
#include <stdlib.h>
#include "../kpmidi.h"

typedef struct _midiIn {
	FILE       *file;
	Boolean    done;
	Boolean    pipe;
	MidiParseState  state;
} MidiIn;

void
MidiInDispose (MidiData *d)
{
	MidiIn *i = d->source;

	if (i->pipe)
		pclose (i->file);
	else
		fclose (i->file);
	free (i);

	d->source = 0;
}

int
MidiInRead (MidiData *d, char *name)
{
	MidiIn  *i;
	int     flags;

	i = calloc (1, sizeof (MidiIn));
	if (!i)
		return 0;

	if (*name == '|')
	{
		i->file = popen (name+1, "r");
		i->pipe = True;
	}
	else if (!strcmp (name,"-"))
	{
		i->file = stdin;
		i->pipe = False;
	}
	else
	{
		i->file = fopen (name, "r");
		i->pipe = False;
	}
	if (!i->file)
	{
		free (i);
		return 0;
	}
	MidiInitState (&i->state);
	i->state.obeyRealtime = True;
	flags = fcntl (fileno (i->file), F_GETFL);
	flags |= O_NONBLOCK;
	(void) fcntl (fileno (i->file), F_SETFL, flags);
	d->eventTime = 0;
	d->source = i;

	return 1;
}

void
MidiInReadyCallback (void *closure, int *source)
{
	MidiData *d = closure;
	MidiIn   *i = d->source;
	unsigned char   buf[1024];
	unsigned char   *b;
	int  n;
	int  m;

	n = fread (buf, 1, sizeof (buf), i->file);
	if (n <= 0)
	{
		i->done = True;
		d->eventTime = d->time;

		return;
	}
	b = buf;

	while (n)
	{
		m = MidiParseOne (d, &i->state, b, n, False);
		if (m > 0)
		{
			b += m;
			n -= m;
		}
		else
			n = m;
	}
}


int
MidiInEvent (MidiData *d, Boolean seek)
{
	MidiIn  *i = d->source;

	d->eventTime = 1e38;
	if (i->done)
		return 0;

	return 1;
}

void
MidiInReset (MidiData *d)
{
	d->eventTime = 0;
}

SourceFuncs MidiInSource = {
	MidiInRead,
	MidiInEvent,
	MidiInDispose,
	MidiInReset,
	MidiSample,
};
