%{
/*
 * $Id: patchlex.l,v 1.4 1999/08/26 18:58:48 keithp Exp $
 *
 * Modified by W.H.Scholten
 *
 * Copyright  1998 Keith Packard
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of Keith Packard not be used in
 * advertising or publicity pertaining to distribution of the software without
 * specific, written prior permission.  Keith Packard makes no
 * representations about the suitability of this software for any purpose.  It
 * is provided "as is" without express or implied warranty.
 *
 * KEITH PACKARD DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
 * EVENT SHALL KEITH PACKARD BE LIABLE FOR ANY SPECIAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
 * DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 */

#include "../kpmidi.h"
#include "patchgram.h"

void	    skipcomment (void);
void	    skipline (void);
    
extern int  MidiPatch_line;
int lexEscape (int c);

%}
%x comment
%%
"/*"         BEGIN(comment);
<comment>[^*\n]*        /* eat anything that's not a '*' */
<comment>"*"+[^*/\n]*   /* eat up '*'s not followed by '/'s */
<comment>\n             ++MidiPatch_line;
<comment>"*"+"/"        BEGIN(INITIAL);
^#.*$		;
release		return RELEASE;
rate		return RATE;
low		return LOW;
high		return HIGH;
root		return ROOT;
scale		return SCALE;
loop		return LOOP;
tremelo		return TREMELO;
vibrato		return VIBRATO;
chorus		return CHORUS;
always		return ALWAYS;
fixed_frequency	return FIXED_FREQUENCY;
fixed_velocity	return FIXED_VELOCITY;
release_loop	return RELEASE_LOOP;
auto_pan	return AUTO_PAN;
"{"		return OC;
"}"		return CC;
","		return COMMA;
";"		return SEMI;
"="		return EQUAL;
"<"		return LESS;
">"		return RIGHT;
[-+]?[0-9]+	{ MidiPatch_lval.ival = atoi (yytext); return INTEGER; }
[-+]?(([0-9]+((\.[0-9]*(\{[0-9]+\})?)?))|(\.[0-9]+)|(\.[0-9]*\{[0-9]+\}))(([Ee][-+]?[0-9]+)?) {
		MidiPatch_lval.fval = atof (yytext); return FLOAT; }
\"([^\n\"]|\\\")*\" {
			char	*c = malloc (yyleng);
			char	*s;
			s = yytext + 1;
			MidiPatch_lval.sval = c;
			while (*s) {
				if (*s == '\\')
					*c++ = lexEscape (*++s);
				else
					*c++ = *s;
				++s;
			}
			*c = '\0';
			return STRING;
			}
" "
"\t"
"\n"		++MidiPatch_line;
%%
int
lexEscape (int c)
{
    switch (c) {
    case '0':
	return '\0';
	break;
    case 'b':
	return '\b';
	break;
    case 'n':
	return '\n';
	break;
    case 't':
	return '\t';
	break;
    case 'f':
	return '\f';
	break;
    default:
	return c;
    }
}
