/*
 * $Id: mididata.c,v 1.13 1998/05/04 06:16:04 keithp Exp $
 *
 * Modified by W.H.Scholten (dd 2000 - 2001)
 *
 * Copyright  1998 Keith Packard
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of Keith Packard not be used in
 * advertising or publicity pertaining to distribution of the software without
 * specific, written prior permission.  Keith Packard makes no
 * representations about the suitability of this software for any purpose.  It
 * is provided "as is" without express or implied warranty.
 *
 * KEITH PACKARD DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
 * EVENT SHALL KEITH PACKARD BE LIABLE FOR ANY SPECIAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
 * DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 */

//#include "kpmidi.h" /* For MIN_DRUM, MAX_DRUM */
#include "midi_data.h"

const char *patchNames[128] = {
	/* 1-8 PIANO */
	"acoustic_grand,piano",
	"bright_acoustic,piano",
	"electric_grand,piano",
	"honky_tonk,piano",
	"electric_piano_1,piano",
	"electric_piano_2,piano",
	"harpsichord,piano",
	"clav,piano",

	/* 9-16 CHROM PERCUSSION */
	"celesta,chrom_percussion",
	"glockenspiel,chrom_percussion",
	"music_box,chrom_percussion",
	"vibraphone,chrom_percussion",
	"marimba,chrom_percussion",
	"xylophone,chrom_percussion",
	"tubular_bells,chrom_percussion",
	"dulcimer,chrom_percussion",

	/* 17-24 ORGAN */
	"drawbar_organ,organ",
	"percussive_organ,organ",
	"rock_organ,organ",
	"church_organ,organ",
	"reed_organ,organ",
	"accordian,organ",
	"harmonica,organ",
	"tango_accordian,organ",

	/* 25-32 GUITAR */
	"acoustic_guitar_nylon,guitar",
	"acoustic_guitar_steel,guitar",
	"electric_guitar_jazz,guitar",
	"electric_guitar_clean,guitar",
	"electric_guitar_muted,guitar",
	"overdriven_guitar,guitar",
	"distortion_guitar,guitar",
	"guitar_harmonics,guitar",

	/* 33-40 BASS */
	"acoustic_bass,bass",
	"electric_bass_finger,bass",
	"electric_bass_pick,bass",
	"fretless_bass,bass",
	"slap_bass_1,bass",
	"slap_bass_2,bass",
	"synth_bass_1,bass",
	"synth_bass_2,bass",

	/* 41-48 STRINGS */
	"violin,strings",
	"viola,strings",
	"cello,strings",
	"contrabass,strings",
	"tremolo_strings,strings",
	"pizzicato_strings,strings",
	"orchestral_strings,strings",
	"timpani,strings",

	/* 49-56 ENSEMBLE */
	"string_ensemble_1,ensemble",
	"string_ensemble_2,ensemble",
	"synth_strings_1,ensemble",
	"synth_strings_2,ensemble",
	"choir_aahs,ensemble",
	"voice_oohs,ensemble",
	"synth_voice,ensemble",
	"orchestra_hit,ensemble",

	/* 57-64 BRASS */
	"trumpet,brass",
	"trombone,brass",
	"tuba,brass",
	"muted_trumpet,brass",
	"french_horn,brass",
	"brass_section,brass",
	"synth_brass_1,brass",
	"synth_brass_2,brass",

	/* 65-72 REED */
	"soprano_sax,reed",
	"alto_sax,reed",
	"tenor_sax,reed",
	"baritone_sax,reed",
	"oboe,reed",
	"english_horn,reed",
	"bassoon,reed",
	"clarinet,reed",

	/* 73-80 PIPE */
	"piccolo,pipe",    	/* piccolo */
	"flute,pipe",
	"recorder,pipe",
	"pan_flute,pipe",
	"blown_bottle,pipe",
	"shakuhachi,pipe",
	"whistle,pipe",
	"ocarina,pipe",

	/* 81-88 SYNTH LEAD */
	"lead_1_square,synth_lead",
	"lead_2_sawtooth,synth_lead",
	"lead_3_calliope,synth_lead",
	"lead_4_chiff,synth_lead",
	"lead_5,synth_lead",
	"lead_6_voice,synth_lead",
	"lead_7_fifths,synth_lead",
	"lead_8_bass_lead,synth_lead",

	/* 89-96 SYNTH PAD */
	"pad_1_new_age,synth_pad",		
	"pad_2_warm,synth_pad",
	"pad_3_polysynth,synth_pad",
	"pad_4_choir,synth_pad",
	"pad_5_bowed,synth_pad",
	"pad_6_metallic,synth_pad",
	"pad_7_halo,synth_pad",
	"pad_8_sweep,synth_pad",
	
	/* 97-104 SYNTH EFFECTS */
	"fx_1_rain,synth_effects",
	"fx_2_soundtrack,synth_effects",
	"fx_3_crystal,synth_effects",
	"fx_4_atmosphere,synth_effects",
	"fx_5_brightness,synth_effects",
	"fx_6_goblins,synth_effects",
	"fx_7_echoes,synth_effects",
	"fx_8_sci_fi,synth_effects",

	/* 105-112 ETHNIC */
	"sitar,ethnic",
	"banjo,ethnic",
	"shamisen,ethnic",
	"koto,ethnic",
	"kalimba,ethnic",
	"bagpipes,ethnic",
	"fiddle,ethnic",
	"shannai,ethnic",

	/* 113-120 PERCUSSIVE */
	"tinkle_bell,percussive",
	"agogo,percussive",
	"steel_drums,percussive",
	"woodblock,percussive",
	"taiko_drum,percussive",
	"melodic_tom,percussive",
	"synth_tom,percussive",
	"reverse_cymbal,percussive",

	/* 121-128 SOUND EFFECTS */
	"guitar_fret_noise,sound_effects",
	"breath_noise,sound_effects",
	"seashore,sound_effects",
	"bird_tweet,sound_effects",
	"telephone_ring,sound_effects",
	"helicopter,sound_effects",
	"applause,sound_effects",
	"gunshot,sound_effects",
};

const char *drumNames[ MAX_DRUM - MIN_DRUM + 1 ] = {
	"acoustic_bass_drum",
	"bass_drum_1",
	"side_stick",
	"acoustic_snare",
	"hand_clap",
	"electric_snare",
	"low_floor_tom",
	"closed_hi_hat",
	"high_floor_tom",
	"pedal_hi_hat",
	"low_tom",
	"open_hi_hat",
	"low_mid_tom",
	"hi_mid_tom",
	"crash_cymbal_1",
	"high_tom",
	"ride_cymbal_1",
	"chinese_cymbal",
	"ride_bell",
	"tambourine",
	"splash_symbal",
	"cowbell",
	"crash_cymbal_2",
	"vibraslap",
	"ride_cymbal_2",
	"hi_bongo",
	"low_bongo",
	"mute_hi_conga",
	"open_hi_conga",
	"low_conga",
	"high_timbale",
	"low_timbale",
	"high_agogo",
	"low_agogo",
	"cabasa",
	"maracas",
	"short_whistle",
	"long_whistle",
	"short_guiro",
	"long_guiro",
	"claves",
	"hi_wood_block",
	"low_wood_block",
	"mute_cuica",
	"open_cuica",
	"mute_triangle",
	"open_triangle",
};

/*
 * Frequency of each note in mHz
 */

const double        equal440Table[12] = {
	440.00000000000,    /* A        */
	466.16376151809,    /* B flat   */
	493.88330125612,    /* B        */
	523.25113060119,    /* C        */
	554.36526195374,    /* C sharp  */
	587.32953583481,    /* D        */
	622.25396744416,    /* E flat   */
	659.25511382574,    /* E        */
	698.45646286600,    /* F        */
	739.98884542326,    /* F sharp  */
	783.99087196349,    /* G        */
	830.60939515989,    /* G sharp  */
};

const double        equal415Table[12] = {
	415.304697579945,   /* A */
	440.000000000000,   /* B flat */
	466.163761518090,   /* B */
	493.883301256124,   /* C */
	523.251130601197,   /* C sharp */
	554.365261953744,   /* D */
	587.329535834815,   /* E flat */
	622.253967444162,   /* E */
	659.255113825740,   /* F */
	698.456462866008,   /* F sharp */
	739.988845423269,   /* G */
	783.990871963499,   /* G sharp/A flat */
};

const double        mean415Table[12] = {
	415.304697579945,   /* A       */
	444.369584188405,   /* B flat  */
	464.324767581875,   /* B       */
	496.820298689295,   /* C       */
	519.130871974931,   /* C sharp */
	555.461980235506,   /* D       */
	594.335702504800,   /* E flat  */
	621.025373361619,   /* E       */
	664.487516127912,   /* F       */
	694.327475294383,   /* F sharp */
	742.919628131000,   /* G       */
	785.541865831669,   /* G sharp */
};

const double        werk3Table[12] = {
	415.304697579945,   /* A */
	441.991980407396,   /* B flat */
	467.217784777438,   /* B */
	497.240977958320,   /* C */
	523.842347149506,   /* C sharp */
	555.618713913140,   /* D */
	589.322640543194,   /* E flat */
	622.957046369918,   /* E */
	662.987970611093,   /* F */
	698.456462866008,   /* F sharp */
	743.338943790938,   /* G */
	785.763520724259,   /* G sharp/A flat */
};

const double        vallottiTable[12] = {
	415.304697579945,   /* A */
	441.493141444760,   /* B flat */
	465.112116089953,   /* B */
	495.559295860625,   /* C */
	523.251130601197,   /* C sharp */
	554.991634067452,   /* D */
	588.657521926347,   /* E flat */
	621.551682024120,   /* E */
	662.239712167140,   /* F */
	697.668174134930,   /* F sharp */
	741.662003053957,   /* G */
	784.876695901796,   /* G sharp/A flat */
};

// Moved to MidiData:
//OLD double            noteTable[128];

const float volumeTable[ 1 << VOLUME_BITS ] = {
	0.00000000f,  /*     0 */
	0.00026100f,  /*     1 */
	0.00052629f,  /*     2 */
	0.00079595f,  /*     3 */
	0.00107004f,  /*     4 */
	0.00134863f,  /*     5 */
	0.00163181f,  /*     6 */
	0.00191964f,  /*     7 */
	0.00221221f,  /*     8 */
	0.00250959f,  /*     9 */
	0.00281185f,  /*    10 */
	0.00311909f,  /*    11 */
	0.00343138f,  /*    12 */
	0.00374880f,  /*    13 */
	0.00407144f,  /*    14 */
	0.00439939f,  /*    15 */
	0.00473273f,  /*    16 */
	0.00507155f,  /*    17 */
	0.00541595f,  /*    18 */
	0.00576600f,  /*    19 */
	0.00612181f,  /*    20 */
	0.00648348f,  /*    21 */
	0.00685109f,  /*    22 */
	0.00722474f,  /*    23 */
	0.00760454f,  /*    24 */
	0.00799058f,  /*    25 */
	0.00838297f,  /*    26 */
	0.00878182f,  /*    27 */
	0.00918722f,  /*    28 */
	0.00959928f,  /*    29 */
	0.01001813f,  /*    30 */
	0.01044386f,  /*    31 */
	0.01087659f,  /*    32 */
	0.01131643f,  /*    33 */
	0.01176351f,  /*    34 */
	0.01221794f,  /*    35 */
	0.01267984f,  /*    36 */
	0.01314934f,  /*    37 */
	0.01362655f,  /*    38 */
	0.01411162f,  /*    39 */
	0.01460466f,  /*    40 */
	0.01510580f,  /*    41 */
	0.01561519f,  /*    42 */
	0.01613295f,  /*    43 */
	0.01665923f,  /*    44 */
	0.01719416f,  /*    45 */
	0.01773788f,  /*    46 */
	0.01829055f,  /*    47 */
	0.01885231f,  /*    48 */
	0.01942330f,  /*    49 */
	0.02000368f,  /*    50 */
	0.02059360f,  /*    51 */
	0.02119322f,  /*    52 */
	0.02180270f,  /*    53 */
	0.02242221f,  /*    54 */
	0.02305190f,  /*    55 */
	0.02369194f,  /*    56 */
	0.02434251f,  /*    57 */
	0.02500378f,  /*    58 */
	0.02567592f,  /*    59 */
	0.02635911f,  /*    60 */
	0.02705354f,  /*    61 */
	0.02775938f,  /*    62 */
	0.02847683f,  /*    63 */
	0.02920608f,  /*    64 */
	0.02994732f,  /*    65 */
	0.03070075f,  /*    66 */
	0.03146656f,  /*    67 */
	0.03224497f,  /*    68 */
	0.03303618f,  /*    69 */
	0.03384039f,  /*    70 */
	0.03465784f,  /*    71 */
	0.03548872f,  /*    72 */
	0.03633326f,  /*    73 */
	0.03719169f,  /*    74 */
	0.03806424f,  /*    75 */
	0.03895113f,  /*    76 */
	0.03985261f,  /*    77 */
	0.04076891f,  /*    78 */
	0.04170028f,  /*    79 */
	0.04264696f,  /*    80 */
	0.04360921f,  /*    81 */
	0.04458728f,  /*    82 */
	0.04558143f,  /*    83 */
	0.04659193f,  /*    84 */
	0.04761905f,  /*    85 */
	0.04866305f,  /*    86 */
	0.04972422f,  /*    87 */
	0.05080284f,  /*    88 */
	0.05189920f,  /*    89 */
	0.05301358f,  /*    90 */
	0.05414629f,  /*    91 */
	0.05529762f,  /*    92 */
	0.05646788f,  /*    93 */
	0.05765739f,  /*    94 */
	0.05886646f,  /*    95 */
	0.06009540f,  /*    96 */
	0.06134456f,  /*    97 */
	0.06261425f,  /*    98 */
	0.06390482f,  /*    99 */
	0.06521661f,  /*   100 */
	0.06654998f,  /*   101 */
	0.06790526f,  /*   102 */
	0.06928284f,  /*   103 */
	0.07068306f,  /*   104 */
	0.07210631f,  /*   105 */
	0.07355296f,  /*   106 */
	0.07502339f,  /*   107 */
	0.07651801f,  /*   108 */
	0.07803720f,  /*   109 */
	0.07958137f,  /*   110 */
	0.08115094f,  /*   111 */
	0.08274631f,  /*   112 */
	0.08436791f,  /*   113 */
	0.08601618f,  /*   114 */
	0.08769155f,  /*   115 */
	0.08939447f,  /*   116 */
	0.09112539f,  /*   117 */
	0.09288478f,  /*   118 */
	0.09467309f,  /*   119 */
	0.09649080f,  /*   120 */
	0.09833841f,  /*   121 */
	0.10021640f,  /*   122 */
	0.10212526f,  /*   123 */
	0.10406552f,  /*   124 */
	0.10603767f,  /*   125 */
	0.10804226f,  /*   126 */
	0.11007981f,  /*   127 */
	0.11215086f,  /*   128 */
	0.11425596f,  /*   129 */
	0.11639568f,  /*   130 */
	0.11857059f,  /*   131 */
	0.12078125f,  /*   132 */
	0.12302827f,  /*   133 */
	0.12531223f,  /*   134 */
	0.12763375f,  /*   135 */
	0.12999344f,  /*   136 */
	0.13239193f,  /*   137 */
	0.13482986f,  /*   138 */
	0.13730788f,  /*   139 */
	0.13982665f,  /*   140 */
	0.14238683f,  /*   141 */
	0.14498910f,  /*   142 */
	0.14763417f,  /*   143 */
	0.15032273f,  /*   144 */
	0.15305550f,  /*   145 */
	0.15583320f,  /*   146 */
	0.15865658f,  /*   147 */
	0.16152638f,  /*   148 */
	0.16444337f,  /*   149 */
	0.16740833f,  /*   150 */
	0.17042203f,  /*   151 */
	0.17348530f,  /*   152 */
	0.17659893f,  /*   153 */
	0.17976376f,  /*   154 */
	0.18298063f,  /*   155 */
	0.18625039f,  /*   156 */
	0.18957392f,  /*   157 */
	0.19295210f,  /*   158 */
	0.19638582f,  /*   159 */
	0.19987601f,  /*   160 */
	0.20342359f,  /*   161 */
	0.20702950f,  /*   162 */
	0.21069470f,  /*   163 */
	0.21442017f,  /*   164 */
	0.21820689f,  /*   165 */
	0.22205588f,  /*   166 */
	0.22596817f,  /*   167 */
	0.22994478f,  /*   168 */
	0.23398678f,  /*   169 */
	0.23809524f,  /*   170 */
	0.24227126f,  /*   171 */
	0.24651594f,  /*   172 */
	0.25083042f,  /*   173 */
	0.25521584f,  /*   174 */
	0.25967337f,  /*   175 */
	0.26420420f,  /*   176 */
	0.26880953f,  /*   177 */
	0.27349059f,  /*   178 */
	0.27824861f,  /*   179 */
	0.28308487f,  /*   180 */
	0.28800066f,  /*   181 */
	0.29299728f,  /*   182 */
	0.29807605f,  /*   183 */
	0.30323834f,  /*   184 */
	0.30848551f,  /*   185 */
	0.31381896f,  /*   186 */
	0.31924010f,  /*   187 */
	0.32475039f,  /*   188 */
	0.33035129f,  /*   189 */
	0.33604428f,  /*   190 */
	0.34183088f,  /*   191 */
	0.34771263f,  /*   192 */
	0.35369109f,  /*   193 */
	0.35976786f,  /*   194 */
	0.36594455f,  /*   195 */
	0.37222280f,  /*   196 */
	0.37860429f,  /*   197 */
	0.38509070f,  /*   198 */
	0.39168378f,  /*   199 */
	0.39838526f,  /*   200 */
	0.40519694f,  /*   201 */
	0.41212062f,  /*   202 */
	0.41915815f,  /*   203 */
	0.42631140f,  /*   204 */
	0.43358227f,  /*   205 */
	0.44097269f,  /*   206 */
	0.44848464f,  /*   207 */
	0.45612010f,  /*   208 */
	0.46388112f,  /*   209 */
	0.47176975f,  /*   210 */
	0.47978809f,  /*   211 */
	0.48793828f,  /*   212 */
	0.49622248f,  /*   213 */
	0.50464290f,  /*   214 */
	0.51320178f,  /*   215 */
	0.52190140f,  /*   216 */
	0.53074406f,  /*   217 */
	0.53973212f,  /*   218 */
	0.54886797f,  /*   219 */
	0.55815405f,  /*   220 */
	0.56759282f,  /*   221 */
	0.57718678f,  /*   222 */
	0.58693851f,  /*   223 */
	0.59685058f,  /*   224 */
	0.60692563f,  /*   225 */
	0.61716636f,  /*   226 */
	0.62757547f,  /*   227 */
	0.63815573f,  /*   228 */
	0.64890997f,  /*   229 */
	0.65984105f,  /*   230 */
	0.67095186f,  /*   231 */
	0.68224537f,  /*   232 */
	0.69372458f,  /*   233 */
	0.70539254f,  /*   234 */
	0.71725236f,  /*   235 */
	0.72930719f,  /*   236 */
	0.74156024f,  /*   237 */
	0.75401476f,  /*   238 */
	0.76667408f,  /*   239 */
	0.77954156f,  /*   240 */
	0.79262061f,  /*   241 */
	0.80591473f,  /*   242 */
	0.81942744f,  /*   243 */
	0.83316235f,  /*   244 */
	0.84712309f,  /*   245 */
	0.86131340f,  /*   246 */
	0.87573703f,  /*   247 */
	0.89039784f,  /*   248 */
	0.90529971f,  /*   249 */
	0.92044662f,  /*   250 */
	0.93584259f,  /*   251 */
	0.95149171f,  /*   252 */
	0.96739816f,  /*   253 */
	0.98356615f,  /*   254 */
	1.00000000f,  /*   255 */
};

const double centsTable[100] = {
	1.000000000000000000,    /* 0 */
	1.000577789506554849,    /* 1 */
	1.001155912853823660,    /* 2 */
	1.001734370234695914,    /* 3 */
	1.002313161842173006,    /* 4 */
	1.002892287869367127,    /* 5 */
	1.003471748509503048,    /* 6 */
	1.004051543955916115,    /* 7 */
	1.004631674402054031,    /* 8 */
	1.005212140041476188,    /* 9 */
	1.005792941067853663,    /* 10 */
	1.006374077674969669,    /* 11 */
	1.006955550056719328,    /* 12 */
	1.007537358407109007,    /* 13 */
	1.008119502920258981,    /* 14 */
	1.008701983790399659,    /* 15 */
	1.009284801211874694,    /* 16 */
	1.009867955379140314,    /* 17 */
	1.010451446486764437,    /* 18 */
	1.011035274729427558,    /* 19 */
	1.011619440301923190,    /* 20 */
	1.012203943399156314,    /* 21 */
	1.012788784216146043,    /* 22 */
	1.013373962948022733,    /* 23 */
	1.013959479790029983,    /* 24 */
	1.014545334937524412,    /* 25 */
	1.015131528585975440,    /* 26 */
	1.015718060930965505,    /* 27 */
	1.016304932168189845,    /* 28 */
	1.016892142493456275,    /* 29 */
	1.017479692102687405,    /* 30 */
	1.018067581191917093,    /* 31 */
	1.018655809957293323,    /* 32 */
	1.019244378595077771,    /* 33 */
	1.019833287301645131,    /* 34 */
	1.020422536273483338,    /* 35 */
	1.021012125707194462,    /* 36 */
	1.021602055799493147,    /* 37 */
	1.022192326747209279,    /* 38 */
	1.022782938747284653,    /* 39 */
	1.023373891996776086,    /* 40 */
	1.023965186692854079,    /* 41 */
	1.024556823032802599,    /* 42 */
	1.025148801214020189,    /* 43 */
	1.025741121434019298,    /* 44 */
	1.026333783890425400,    /* 45 */
	1.026926788780980315,    /* 46 */
	1.027520136303538223,    /* 47 */
	1.028113826656068097,    /* 48 */
	1.028707860036653932,    /* 49 */
	1.029302236643493629,    /* 50 */
	1.029896956674899666,    /* 51 */
	1.030492020329299319,    /* 52 */
	1.031087427805233769,    /* 53 */
	1.031683179301360553,    /* 54 */
	1.032279275016450226,    /* 55 */
	1.032875715149388807,    /* 56 */
	1.033472499899177333,    /* 57 */
	1.034069629464932083,    /* 58 */
	1.034667104045884134,    /* 59 */
	1.035264923841379581,    /* 60 */
	1.035863089050879093,    /* 61 */
	1.036461599873960360,    /* 62 */
	1.037060456510314754,    /* 63 */
	1.037659659159749337,    /* 64 */
	1.038259208022187075,    /* 65 */
	1.038859103297666397,    /* 66 */
	1.039459345186340755,    /* 67 */
	1.040059933888479948,    /* 68 */
	1.040660869604468797,    /* 69 */
	1.041262152534808694,    /* 70 */
	1.041863782880116052,    /* 71 */
	1.042465760841123634,    /* 72 */
	1.043068086618680335,    /* 73 */
	1.043670760413750953,    /* 74 */
	1.044273782427416197,    /* 75 */
	1.044877152860873348,    /* 76 */
	1.045480871915434928,    /* 77 */
	1.046084939792531809,    /* 78 */
	1.046689356693709216,    /* 79 */
	1.047294122820629170,    /* 80 */
	1.047899238375071596,    /* 81 */
	1.048504703558931661,    /* 82 */
	1.049110518574221329,    /* 83 */
	1.049716683623070024,    /* 84 */
	1.050323198907722855,    /* 85 */
	1.050930064630543059,    /* 86 */
	1.051537280994009782,    /* 87 */
	1.052144848200719185,    /* 88 */
	1.052752766453385336,    /* 89 */
	1.053361035954838654,    /* 90 */
	1.053969656908027241,    /* 91 */
	1.054578629516015997,    /* 92 */
	1.055187953981987059,    /* 93 */
	1.055797630509241358,    /* 94 */
	1.056407659301195068,    /* 95 */
	1.057018040561383376,    /* 96 */
	1.057628774493458934,    /* 97 */
	1.058239861301191853,    /* 98 */
	1.058851301188469707,    /* 99 */
};

const double semiTable[128] = {
	1,    /* 0 */
	1.059463094359295310,    /* 1 */
	1.122462048309373017,    /* 2 */
	1.189207115002721249,    /* 3 */
	1.259921049894873191,    /* 4 */
	1.334839854170034368,    /* 5 */
	1.414213562373095145,    /* 6 */
	1.498307076876681743,    /* 7 */
	1.587401051968199583,    /* 8 */
	1.681792830507429226,    /* 9 */
	1.781797436280678770,    /* 10 */
	1.887748625363387500,    /* 11 */
	2,    /* 12 */
	2.118926188718590620,    /* 13 */
	2.244924096618746034,    /* 14 */
	2.378414230005442498,    /* 15 */
	2.519842099789746381,    /* 16 */
	2.669679708340068736,    /* 17 */
	2.828427124746190291,    /* 18 */
	2.996614153753363485,    /* 19 */
	3.174802103936399167,    /* 20 */
	3.363585661014858452,    /* 21 */
	3.563594872561357541,    /* 22 */
	3.775497250726775000,    /* 23 */
	4,    /* 24 */
	4.237852377437181239,    /* 25 */
	4.489848193237492069,    /* 26 */
	4.756828460010884996,    /* 27 */
	5.039684199579492763,    /* 28 */
	5.339359416680137471,    /* 29 */
	5.656854249492380582,    /* 30 */
	5.993228307506726971,    /* 31 */
	6.349604207872798334,    /* 32 */
	6.727171322029716904,    /* 33 */
	7.127189745122715081,    /* 34 */
	7.550994501453550001,    /* 35 */
	8,    /* 36 */
	8.475704754874362479,    /* 37 */
	8.979696386474984138,    /* 38 */
	9.513656920021769992,    /* 39 */
	10.079368399158985525,    /* 40 */
	10.678718833360274942,    /* 41 */
	11.313708498984761164,    /* 42 */
	11.986456615013453941,    /* 43 */
	12.699208415745596668,    /* 44 */
	13.454342644059433809,    /* 45 */
	14.254379490245430162,    /* 46 */
	15.101989002907100001,    /* 47 */
	16,    /* 48 */
	16.951409509748724957,    /* 49 */
	17.959392772949968275,    /* 50 */
	19.027313840043539983,    /* 51 */
	20.158736798317971051,    /* 52 */
	21.357437666720549885,    /* 53 */
	22.627416997969522328,    /* 54 */
	23.972913230026907883,    /* 55 */
	25.398416831491193335,    /* 56 */
	26.908685288118867618,    /* 57 */
	28.508758980490860324,    /* 58 */
	30.203978005814200003,    /* 59 */
	32,    /* 60 */
	33.902819019497449915,    /* 61 */
	35.918785545899936551,    /* 62 */
	38.054627680087079966,    /* 63 */
	40.317473596635942101,    /* 64 */
	42.714875333441099770,    /* 65 */
	45.254833995939044655,    /* 66 */
	47.945826460053815765,    /* 67 */
	50.796833662982386670,    /* 68 */
	53.817370576237735236,    /* 69 */
	57.017517960981720648,    /* 70 */
	60.407956011628400006,    /* 71 */
	64,    /* 72 */
	67.805638038994899830,    /* 73 */
	71.837571091799873102,    /* 74 */
	76.109255360174159932,    /* 75 */
	80.634947193271884203,    /* 76 */
	85.429750666882199539,    /* 77 */
	90.509667991878089310,    /* 78 */
	95.891652920107631530,    /* 79 */
	101.593667325964773340,    /* 80 */
	107.634741152475470471,    /* 81 */
	114.035035921963441297,    /* 82 */
	120.815912023256800012,    /* 83 */
	128,    /* 84 */
	135.611276077989799660,    /* 85 */
	143.675142183599746204,    /* 86 */
	152.218510720348319865,    /* 87 */
	161.269894386543768405,    /* 88 */
	170.859501333764399078,    /* 89 */
	181.019335983756178621,    /* 90 */
	191.783305840215263061,    /* 91 */
	203.187334651929546681,    /* 92 */
	215.269482304950940943,    /* 93 */
	228.070071843926882593,    /* 94 */
	241.631824046513600024,    /* 95 */
	256,    /* 96 */
	271.222552155979599320,    /* 97 */
	287.350284367199492408,    /* 98 */
	304.437021440696639729,    /* 99 */
	322.539788773087536811,    /* 100 */
	341.719002667528798156,    /* 101 */
	362.038671967512357242,    /* 102 */
	383.566611680430526121,    /* 103 */
	406.374669303859093361,    /* 104 */
	430.538964609901881886,    /* 105 */
	456.140143687853765186,    /* 106 */
	483.263648093027200048,    /* 107 */
	512,    /* 108 */
	542.445104311959198640,    /* 109 */
	574.700568734398984816,    /* 110 */
	608.874042881393279458,    /* 111 */
	645.079577546175073621,    /* 112 */
	683.438005335057596312,    /* 113 */
	724.077343935024714483,    /* 114 */
	767.133223360861052242,    /* 115 */
	812.749338607718186722,    /* 116 */
	861.077929219803763772,    /* 117 */
	912.280287375707530373,    /* 118 */
	966.527296186054400096,    /* 119 */
	1024,    /* 120 */
	1084.890208623918397279,    /* 121 */
	1149.401137468797969632,    /* 122 */
	1217.748085762786558917,    /* 123 */
	1290.159155092350147243,    /* 124 */
	1366.876010670115192625,    /* 125 */
	1448.154687870049428966,    /* 126 */
	1534.266446721722104485,    /* 127 */
};

const double sineTable[ 1 << SINE_BITS ] = {
	0,    /*     0 */
	0.01227154,    /*     1 */
	0.02454123,    /*     2 */
	0.03680722,    /*     3 */
	0.04906767,    /*     4 */
	0.06132074,    /*     5 */
	0.07356456,    /*     6 */
	0.08579731,    /*     7 */
	0.09801714,    /*     8 */
	0.1102222,    /*     9 */
	0.1224107,    /*    10 */
	0.1345807,    /*    11 */
	0.1467305,    /*    12 */
	0.1588581,    /*    13 */
	0.1709619,    /*    14 */
	0.1830399,    /*    15 */
	0.1950903,    /*    16 */
	0.2071114,    /*    17 */
	0.2191012,    /*    18 */
	0.2310581,    /*    19 */
	0.2429802,    /*    20 */
	0.2548657,    /*    21 */
	0.2667128,    /*    22 */
	0.2785197,    /*    23 */
	0.2902847,    /*    24 */
	0.3020059,    /*    25 */
	0.3136817,    /*    26 */
	0.3253103,    /*    27 */
	0.3368899,    /*    28 */
	0.3484187,    /*    29 */
	0.359895,    /*    30 */
	0.3713172,    /*    31 */
	0.3826834,    /*    32 */
	0.393992,    /*    33 */
	0.4052413,    /*    34 */
	0.4164296,    /*    35 */
	0.4275551,    /*    36 */
	0.4386162,    /*    37 */
	0.4496113,    /*    38 */
	0.4605387,    /*    39 */
	0.4713967,    /*    40 */
	0.4821838,    /*    41 */
	0.4928982,    /*    42 */
	0.5035384,    /*    43 */
	0.5141027,    /*    44 */
	0.5245897,    /*    45 */
	0.5349976,    /*    46 */
	0.545325,    /*    47 */
	0.5555702,    /*    48 */
	0.5657318,    /*    49 */
	0.5758082,    /*    50 */
	0.5857979,    /*    51 */
	0.5956993,    /*    52 */
	0.605511,    /*    53 */
	0.6152316,    /*    54 */
	0.6248595,    /*    55 */
	0.6343933,    /*    56 */
	0.6438315,    /*    57 */
	0.6531728,    /*    58 */
	0.6624158,    /*    59 */
	0.671559,    /*    60 */
	0.680601,    /*    61 */
	0.6895405,    /*    62 */
	0.6983762,    /*    63 */
	0.7071068,    /*    64 */
	0.7157308,    /*    65 */
	0.7242471,    /*    66 */
	0.7326543,    /*    67 */
	0.7409511,    /*    68 */
	0.7491364,    /*    69 */
	0.7572088,    /*    70 */
	0.7651673,    /*    71 */
	0.7730105,    /*    72 */
	0.7807372,    /*    73 */
	0.7883464,    /*    74 */
	0.7958369,    /*    75 */
	0.8032075,    /*    76 */
	0.8104572,    /*    77 */
	0.8175848,    /*    78 */
	0.8245893,    /*    79 */
	0.8314696,    /*    80 */
	0.8382247,    /*    81 */
	0.8448536,    /*    82 */
	0.8513552,    /*    83 */
	0.8577286,    /*    84 */
	0.8639729,    /*    85 */
	0.870087,    /*    86 */
	0.8760701,    /*    87 */
	0.8819213,    /*    88 */
	0.8876396,    /*    89 */
	0.8932243,    /*    90 */
	0.8986745,    /*    91 */
	0.9039893,    /*    92 */
	0.909168,    /*    93 */
	0.9142098,    /*    94 */
	0.9191139,    /*    95 */
	0.9238795,    /*    96 */
	0.9285061,    /*    97 */
	0.9329928,    /*    98 */
	0.937339,    /*    99 */
	0.9415441,    /*   100 */
	0.9456073,    /*   101 */
	0.9495282,    /*   102 */
	0.953306,    /*   103 */
	0.9569403,    /*   104 */
	0.9604305,    /*   105 */
	0.9637761,    /*   106 */
	0.9669765,    /*   107 */
	0.9700313,    /*   108 */
	0.97294,    /*   109 */
	0.9757021,    /*   110 */
	0.9783174,    /*   111 */
	0.9807853,    /*   112 */
	0.9831055,    /*   113 */
	0.9852776,    /*   114 */
	0.9873014,    /*   115 */
	0.9891765,    /*   116 */
	0.9909026,    /*   117 */
	0.9924795,    /*   118 */
	0.993907,    /*   119 */
	0.9951847,    /*   120 */
	0.9963126,    /*   121 */
	0.9972905,    /*   122 */
	0.9981181,    /*   123 */
	0.9987955,    /*   124 */
	0.9993224,    /*   125 */
	0.9996988,    /*   126 */
	0.9999247,    /*   127 */
	1,    /*   128 */
	0.9999247,    /*   129 */
	0.9996988,    /*   130 */
	0.9993224,    /*   131 */
	0.9987955,    /*   132 */
	0.9981181,    /*   133 */
	0.9972905,    /*   134 */
	0.9963126,    /*   135 */
	0.9951847,    /*   136 */
	0.993907,    /*   137 */
	0.9924795,    /*   138 */
	0.9909026,    /*   139 */
	0.9891765,    /*   140 */
	0.9873014,    /*   141 */
	0.9852776,    /*   142 */
	0.9831055,    /*   143 */
	0.9807853,    /*   144 */
	0.9783174,    /*   145 */
	0.9757021,    /*   146 */
	0.97294,    /*   147 */
	0.9700313,    /*   148 */
	0.9669765,    /*   149 */
	0.9637761,    /*   150 */
	0.9604305,    /*   151 */
	0.9569403,    /*   152 */
	0.953306,    /*   153 */
	0.9495282,    /*   154 */
	0.9456073,    /*   155 */
	0.9415441,    /*   156 */
	0.937339,    /*   157 */
	0.9329928,    /*   158 */
	0.9285061,    /*   159 */
	0.9238795,    /*   160 */
	0.9191139,    /*   161 */
	0.9142098,    /*   162 */
	0.909168,    /*   163 */
	0.9039893,    /*   164 */
	0.8986745,    /*   165 */
	0.8932243,    /*   166 */
	0.8876396,    /*   167 */
	0.8819213,    /*   168 */
	0.8760701,    /*   169 */
	0.870087,    /*   170 */
	0.8639729,    /*   171 */
	0.8577286,    /*   172 */
	0.8513552,    /*   173 */
	0.8448536,    /*   174 */
	0.8382247,    /*   175 */
	0.8314696,    /*   176 */
	0.8245893,    /*   177 */
	0.8175848,    /*   178 */
	0.8104572,    /*   179 */
	0.8032075,    /*   180 */
	0.7958369,    /*   181 */
	0.7883464,    /*   182 */
	0.7807372,    /*   183 */
	0.7730105,    /*   184 */
	0.7651673,    /*   185 */
	0.7572088,    /*   186 */
	0.7491364,    /*   187 */
	0.7409511,    /*   188 */
	0.7326543,    /*   189 */
	0.7242471,    /*   190 */
	0.7157308,    /*   191 */
	0.7071068,    /*   192 */
	0.6983762,    /*   193 */
	0.6895405,    /*   194 */
	0.680601,    /*   195 */
	0.671559,    /*   196 */
	0.6624158,    /*   197 */
	0.6531728,    /*   198 */
	0.6438315,    /*   199 */
	0.6343933,    /*   200 */
	0.6248595,    /*   201 */
	0.6152316,    /*   202 */
	0.605511,    /*   203 */
	0.5956993,    /*   204 */
	0.5857979,    /*   205 */
	0.5758082,    /*   206 */
	0.5657318,    /*   207 */
	0.5555702,    /*   208 */
	0.545325,    /*   209 */
	0.5349976,    /*   210 */
	0.5245897,    /*   211 */
	0.5141027,    /*   212 */
	0.5035384,    /*   213 */
	0.4928982,    /*   214 */
	0.4821838,    /*   215 */
	0.4713967,    /*   216 */
	0.4605387,    /*   217 */
	0.4496113,    /*   218 */
	0.4386162,    /*   219 */
	0.4275551,    /*   220 */
	0.4164296,    /*   221 */
	0.4052413,    /*   222 */
	0.393992,    /*   223 */
	0.3826834,    /*   224 */
	0.3713172,    /*   225 */
	0.359895,    /*   226 */
	0.3484187,    /*   227 */
	0.3368899,    /*   228 */
	0.3253103,    /*   229 */
	0.3136817,    /*   230 */
	0.3020059,    /*   231 */
	0.2902847,    /*   232 */
	0.2785197,    /*   233 */
	0.2667128,    /*   234 */
	0.2548657,    /*   235 */
	0.2429802,    /*   236 */
	0.2310581,    /*   237 */
	0.2191012,    /*   238 */
	0.2071114,    /*   239 */
	0.1950903,    /*   240 */
	0.1830399,    /*   241 */
	0.1709619,    /*   242 */
	0.1588581,    /*   243 */
	0.1467305,    /*   244 */
	0.1345807,    /*   245 */
	0.1224107,    /*   246 */
	0.1102222,    /*   247 */
	0.09801714,    /*   248 */
	0.08579731,    /*   249 */
	0.07356456,    /*   250 */
	0.06132074,    /*   251 */
	0.04906767,    /*   252 */
	0.03680722,    /*   253 */
	0.02454123,    /*   254 */
	0.01227154,    /*   255 */
	0.00000000,    /*   256 */
	-0.01227154,    /*   257 */
	-0.02454123,    /*   258 */
	-0.03680722,    /*   259 */
	-0.04906767,    /*   260 */
	-0.06132074,    /*   261 */
	-0.07356456,    /*   262 */
	-0.08579731,    /*   263 */
	-0.09801714,    /*   264 */
	-0.1102222,    /*   265 */
	-0.1224107,    /*   266 */
	-0.1345807,    /*   267 */
	-0.1467305,    /*   268 */
	-0.1588581,    /*   269 */
	-0.1709619,    /*   270 */
	-0.1830399,    /*   271 */
	-0.1950903,    /*   272 */
	-0.2071114,    /*   273 */
	-0.2191012,    /*   274 */
	-0.2310581,    /*   275 */
	-0.2429802,    /*   276 */
	-0.2548657,    /*   277 */
	-0.2667128,    /*   278 */
	-0.2785197,    /*   279 */
	-0.2902847,    /*   280 */
	-0.3020059,    /*   281 */
	-0.3136817,    /*   282 */
	-0.3253103,    /*   283 */
	-0.3368899,    /*   284 */
	-0.3484187,    /*   285 */
	-0.359895,    /*   286 */
	-0.3713172,    /*   287 */
	-0.3826834,    /*   288 */
	-0.393992,    /*   289 */
	-0.4052413,    /*   290 */
	-0.4164296,    /*   291 */
	-0.4275551,    /*   292 */
	-0.4386162,    /*   293 */
	-0.4496113,    /*   294 */
	-0.4605387,    /*   295 */
	-0.4713967,    /*   296 */
	-0.4821838,    /*   297 */
	-0.4928982,    /*   298 */
	-0.5035384,    /*   299 */
	-0.5141027,    /*   300 */
	-0.5245897,    /*   301 */
	-0.5349976,    /*   302 */
	-0.545325,    /*   303 */
	-0.5555702,    /*   304 */
	-0.5657318,    /*   305 */
	-0.5758082,    /*   306 */
	-0.5857979,    /*   307 */
	-0.5956993,    /*   308 */
	-0.605511,    /*   309 */
	-0.6152316,    /*   310 */
	-0.6248595,    /*   311 */
	-0.6343933,    /*   312 */
	-0.6438315,    /*   313 */
	-0.6531728,    /*   314 */
	-0.6624158,    /*   315 */
	-0.671559,    /*   316 */
	-0.680601,    /*   317 */
	-0.6895405,    /*   318 */
	-0.6983762,    /*   319 */
	-0.7071068,    /*   320 */
	-0.7157308,    /*   321 */
	-0.7242471,    /*   322 */
	-0.7326543,    /*   323 */
	-0.7409511,    /*   324 */
	-0.7491364,    /*   325 */
	-0.7572088,    /*   326 */
	-0.7651673,    /*   327 */
	-0.7730105,    /*   328 */
	-0.7807372,    /*   329 */
	-0.7883464,    /*   330 */
	-0.7958369,    /*   331 */
	-0.8032075,    /*   332 */
	-0.8104572,    /*   333 */
	-0.8175848,    /*   334 */
	-0.8245893,    /*   335 */
	-0.8314696,    /*   336 */
	-0.8382247,    /*   337 */
	-0.8448536,    /*   338 */
	-0.8513552,    /*   339 */
	-0.8577286,    /*   340 */
	-0.8639729,    /*   341 */
	-0.870087,    /*   342 */
	-0.8760701,    /*   343 */
	-0.8819213,    /*   344 */
	-0.8876396,    /*   345 */
	-0.8932243,    /*   346 */
	-0.8986745,    /*   347 */
	-0.9039893,    /*   348 */
	-0.909168,    /*   349 */
	-0.9142098,    /*   350 */
	-0.9191139,    /*   351 */
	-0.9238795,    /*   352 */
	-0.9285061,    /*   353 */
	-0.9329928,    /*   354 */
	-0.937339,    /*   355 */
	-0.9415441,    /*   356 */
	-0.9456073,    /*   357 */
	-0.9495282,    /*   358 */
	-0.953306,    /*   359 */
	-0.9569403,    /*   360 */
	-0.9604305,    /*   361 */
	-0.9637761,    /*   362 */
	-0.9669765,    /*   363 */
	-0.9700313,    /*   364 */
	-0.97294,    /*   365 */
	-0.9757021,    /*   366 */
	-0.9783174,    /*   367 */
	-0.9807853,    /*   368 */
	-0.9831055,    /*   369 */
	-0.9852776,    /*   370 */
	-0.9873014,    /*   371 */
	-0.9891765,    /*   372 */
	-0.9909026,    /*   373 */
	-0.9924795,    /*   374 */
	-0.993907,    /*   375 */
	-0.9951847,    /*   376 */
	-0.9963126,    /*   377 */
	-0.9972905,    /*   378 */
	-0.9981181,    /*   379 */
	-0.9987955,    /*   380 */
	-0.9993224,    /*   381 */
	-0.9996988,    /*   382 */
	-0.9999247,    /*   383 */
	-1,    /*   384 */
	-0.9999247,    /*   385 */
	-0.9996988,    /*   386 */
	-0.9993224,    /*   387 */
	-0.9987955,    /*   388 */
	-0.9981181,    /*   389 */
	-0.9972905,    /*   390 */
	-0.9963126,    /*   391 */
	-0.9951847,    /*   392 */
	-0.993907,    /*   393 */
	-0.9924795,    /*   394 */
	-0.9909026,    /*   395 */
	-0.9891765,    /*   396 */
	-0.9873014,    /*   397 */
	-0.9852776,    /*   398 */
	-0.9831055,    /*   399 */
	-0.9807853,    /*   400 */
	-0.9783174,    /*   401 */
	-0.9757021,    /*   402 */
	-0.97294,    /*   403 */
	-0.9700313,    /*   404 */
	-0.9669765,    /*   405 */
	-0.9637761,    /*   406 */
	-0.9604305,    /*   407 */
	-0.9569403,    /*   408 */
	-0.953306,    /*   409 */
	-0.9495282,    /*   410 */
	-0.9456073,    /*   411 */
	-0.9415441,    /*   412 */
	-0.937339,    /*   413 */
	-0.9329928,    /*   414 */
	-0.9285061,    /*   415 */
	-0.9238795,    /*   416 */
	-0.9191139,    /*   417 */
	-0.9142098,    /*   418 */
	-0.909168,    /*   419 */
	-0.9039893,    /*   420 */
	-0.8986745,    /*   421 */
	-0.8932243,    /*   422 */
	-0.8876396,    /*   423 */
	-0.8819213,    /*   424 */
	-0.8760701,    /*   425 */
	-0.870087,    /*   426 */
	-0.8639729,    /*   427 */
	-0.8577286,    /*   428 */
	-0.8513552,    /*   429 */
	-0.8448536,    /*   430 */
	-0.8382247,    /*   431 */
	-0.8314696,    /*   432 */
	-0.8245893,    /*   433 */
	-0.8175848,    /*   434 */
	-0.8104572,    /*   435 */
	-0.8032075,    /*   436 */
	-0.7958369,    /*   437 */
	-0.7883464,    /*   438 */
	-0.7807372,    /*   439 */
	-0.7730105,    /*   440 */
	-0.7651673,    /*   441 */
	-0.7572088,    /*   442 */
	-0.7491364,    /*   443 */
	-0.7409511,    /*   444 */
	-0.7326543,    /*   445 */
	-0.7242471,    /*   446 */
	-0.7157308,    /*   447 */
	-0.7071068,    /*   448 */
	-0.6983762,    /*   449 */
	-0.6895405,    /*   450 */
	-0.680601,    /*   451 */
	-0.671559,    /*   452 */
	-0.6624158,    /*   453 */
	-0.6531728,    /*   454 */
	-0.6438315,    /*   455 */
	-0.6343933,    /*   456 */
	-0.6248595,    /*   457 */
	-0.6152316,    /*   458 */
	-0.605511,    /*   459 */
	-0.5956993,    /*   460 */
	-0.5857979,    /*   461 */
	-0.5758082,    /*   462 */
	-0.5657318,    /*   463 */
	-0.5555702,    /*   464 */
	-0.545325,    /*   465 */
	-0.5349976,    /*   466 */
	-0.5245897,    /*   467 */
	-0.5141027,    /*   468 */
	-0.5035384,    /*   469 */
	-0.4928982,    /*   470 */
	-0.4821838,    /*   471 */
	-0.4713967,    /*   472 */
	-0.4605387,    /*   473 */
	-0.4496113,    /*   474 */
	-0.4386162,    /*   475 */
	-0.4275551,    /*   476 */
	-0.4164296,    /*   477 */
	-0.4052413,    /*   478 */
	-0.393992,    /*   479 */
	-0.3826834,    /*   480 */
	-0.3713172,    /*   481 */
	-0.359895,    /*   482 */
	-0.3484187,    /*   483 */
	-0.3368899,    /*   484 */
	-0.3253103,    /*   485 */
	-0.3136817,    /*   486 */
	-0.3020059,    /*   487 */
	-0.2902847,    /*   488 */
	-0.2785197,    /*   489 */
	-0.2667128,    /*   490 */
	-0.2548657,    /*   491 */
	-0.2429802,    /*   492 */
	-0.2310581,    /*   493 */
	-0.2191012,    /*   494 */
	-0.2071114,    /*   495 */
	-0.1950903,    /*   496 */
	-0.1830399,    /*   497 */
	-0.1709619,    /*   498 */
	-0.1588581,    /*   499 */
	-0.1467305,    /*   500 */
	-0.1345807,    /*   501 */
	-0.1224107,    /*   502 */
	-0.1102222,    /*   503 */
	-0.09801714,    /*   504 */
	-0.08579731,    /*   505 */
	-0.07356456,    /*   506 */
	-0.06132074,    /*   507 */
	-0.04906767,    /*   508 */
	-0.03680722,    /*   509 */
	-0.02454123,    /*   510 */
	-0.01227154,    /*   511 */
};

const float midiInterp[ 1 << INTERP_BITS ] = {
	0.00000f, /*   0 */
	0.01563f, /*   1 */
	0.03125f, /*   2 */
	0.04688f, /*   3 */
	0.06250f, /*   4 */
	0.07813f, /*   5 */
	0.09375f, /*   6 */
	0.10938f, /*   7 */
	0.12500f, /*   8 */
	0.14063f, /*   9 */
	0.15625f, /*  10 */
	0.17188f, /*  11 */
	0.18750f, /*  12 */
	0.20313f, /*  13 */
	0.21875f, /*  14 */
	0.23438f, /*  15 */
	0.25000f, /*  16 */
	0.26563f, /*  17 */
	0.28125f, /*  18 */
	0.29688f, /*  19 */
	0.31250f, /*  20 */
	0.32813f, /*  21 */
	0.34375f, /*  22 */
	0.35938f, /*  23 */
	0.37500f, /*  24 */
	0.39063f, /*  25 */
	0.40625f, /*  26 */
	0.42188f, /*  27 */
	0.43750f, /*  28 */
	0.45313f, /*  29 */
	0.46875f, /*  30 */
	0.48438f, /*  31 */
	0.50000f, /*  32 */
	0.51563f, /*  33 */
	0.53125f, /*  34 */
	0.54688f, /*  35 */
	0.56250f, /*  36 */
	0.57813f, /*  37 */
	0.59375f, /*  38 */
	0.60938f, /*  39 */
	0.62500f, /*  40 */
	0.64063f, /*  41 */
	0.65625f, /*  42 */
	0.67188f, /*  43 */
	0.68750f, /*  44 */
	0.70313f, /*  45 */
	0.71875f, /*  46 */
	0.73438f, /*  47 */
	0.75000f, /*  48 */
	0.76563f, /*  49 */
	0.78125f, /*  50 */
	0.79688f, /*  51 */
	0.81250f, /*  52 */
	0.82813f, /*  53 */
	0.84375f, /*  54 */
	0.85938f, /*  55 */
	0.87500f, /*  56 */
	0.89063f, /*  57 */
	0.90625f, /*  58 */
	0.92188f, /*  59 */
	0.93750f, /*  60 */
	0.95313f, /*  61 */
	0.96875f, /*  62 */
	0.98438f, /*  63 */
};
