/*
 * $Id: patchmap.c,v 1.1 1998/05/08 00:16:31 keithp Exp $
 *
 * Modified by W.H.Scholten
 *
 * Copyright  1998 Keith Packard
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of Keith Packard not be used in
 * advertising or publicity pertaining to distribution of the software without
 * specific, written prior permission.  Keith Packard makes no
 * representations about the suitability of this software for any purpose.  It
 * is provided "as is" without express or implied warranty.
 *
 * KEITH PACKARD DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
 * EVENT SHALL KEITH PACKARD BE LIABLE FOR ANY SPECIAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
 * DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 */

#include <unistd.h>
#include <sys/mman.h>

/* File stuff */
#include <fcntl.h>
#include <sys/types.h>

#include <stdlib.h>
#include <string.h>
#include "kpmidi.h"
#include "utils/log.h"

int
MidiMapSize (const Instrument *ip)
{
	return (sizeof (Instrument) +
			ip->nsamples * sizeof (SampleElement) +
			ip->envelopeTotal * sizeof (EnvelopeElement) +
			ip->dataTotal * sizeof (float));
}

void
MidiPatchMap_free (const MidiPatch *p)
{
	munmap ((void *) p->instrument, MidiMapSize (p->instrument));
	free ((void *) p->name);
	free ((void *) p);
}

const MidiPatch *
MidiPatchMap (const char *file)
{
	int         fd;
	MidiPatch   *p;
	void        *start;
	Instrument  t;
	const char  *dot;
	char        *name;

	fd = open (file, 0);
	if (fd < 0)
	{
		gsimidipcm_log_message (GSI_MESSAGE_ERROR, "MidiPatchMap: can't map %s", file);
		goto bail1;
	}

	if (read (fd, &t, sizeof (Instrument)) != sizeof (Instrument))
	{
		gsimidipcm_log_message (GSI_MESSAGE_ERROR, "MidiPatchMap: can't read %s", file);
		goto bail2;
	}

	if (strcmp (t.magic, _GSIMIDIPCM_INSTRUMENT_NATIVE_BINARY__MAGIC) != 0)
	{
		gsimidipcm_log_message (GSI_MESSAGE_ERROR, "MidiPatchMap: bad magic %s", file);
		goto bail2;
	}

	p = calloc (1, sizeof (MidiPatch));
	if (!p)
	{
		gsimidipcm_log_message (GSI_MESSAGE_ERROR, "Out of memory");
		goto bail3;
		//return 0;
	}

	dot = strrchr (file, '.');
	if (!dot)
		dot = file + strlen (file);
	name = malloc (dot - file + 1);
	if (!name) {
		gsimidipcm_log_message (GSI_MESSAGE_ERROR, "Out of memory");
		goto bail3;
	}

	strncpy (name, file, dot - file);
	name[dot-file] = '\0';
	p->name = name;

	start = mmap (0, MidiMapSize (&t), PROT_READ, MAP_SHARED, fd, 0);
	if (!start)
	{
		gsimidipcm_log_message (GSI_MESSAGE_ERROR, "Can't map %s", file);
		goto bail3;
	}
	p->instrument = start;
	p->samples = (SampleElement *) (p->instrument + 1);
	p->envelopes = (EnvelopeElement *) (p->samples + p->instrument->nsamples);
	p->data = (float *) (p->envelopes + p->instrument->envelopeTotal);
	p->free = MidiPatchMap_free;
	close (fd);
	return p;

  bail3:
	free (p);

  bail2:
	close (fd);

  bail1:
	return 0;
}
