/*
 * $Id: debug.c,v 1.2 1998/02/10 23:02:22 keithp Exp $
 *
 * Modified by W.H.Scholten
 *
 * Copyright  1998 Keith Packard
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of Keith Packard not be used in
 * advertising or publicity pertaining to distribution of the software without
 * specific, written prior permission.  Keith Packard makes no
 * representations about the suitability of this software for any purpose.  It
 * is provided "as is" without express or implied warranty.
 *
 * KEITH PACKARD DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
 * EVENT SHALL KEITH PACKARD BE LIABLE FOR ANY SPECIAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
 * DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 */

#include <stdio.h>
#include "debug.h"

#ifdef KPMIDI_DEBUG

#define STANDARD_DEBUG_PREFIX  "KPMIDI: "

/*
 * Debugging printfs -- each bit controls debugging in different
 * areas of the driver.
 */

#define DEBUG_THREAD     0
#define DEBUG_TIMESTAMP  1

long DebugExtra = 
	(1 << DEBUG_THREAD) |
	(1 << DEBUG_TIMESTAMP);

long DebugLevel = 
	(1 << DEBUG_FAILURE) |
	(1 << DEBUG_INIT) |
	(0 << DEBUG_INITV) |
	(0 << DEBUG_INPUT) |
	(0 << DEBUG_REPLY) |
	(0 << DEBUG_NETWORK) |
	(0 << DEBUG_WAVE) |
	(0 << DEBUG_PAUSE) |
	(1 << DEBUG_MIDI) |
	(1 << DEBUG_NOTE) |
	(0 << DEBUG_ENVELOPE) |
	(1 << DEBUG_PROGRAM) |
	(0 << DEBUG_MIDIOUT) |
	(0 << DEBUG_MIX);

unsigned long   lastTime;

#include <stdarg.h>
#include <sys/time.h>

unsigned long 
ms (void)
{
	struct timeval  tp;

	gettimeofday(&tp, 0);
	return(tp.tv_sec * 1000) + (tp.tv_usec / 1000);
}

void
_gsimidipcm_debug_print(long DebugPrintLevel, char *DebugMessage, ...)
{
	va_list ap;
	int    head_len;
	int    header = 1;
	unsigned long   now;

	va_start(ap, DebugMessage);

	if (DebugPrintLevel < 0) {
		header = 0;
		DebugPrintLevel = -DebugPrintLevel;
	}

#define Check(i)    ((i) <= DebugPrintLevel && DebugPrintLevel < (i) + 32)

	if ((Check(0) && (DebugLevel & (1 << (DebugPrintLevel - 0))))) {
		char buffer[1024];
		int  len;

		/* We prepend the STANDARD_DEBUG_PREFIX to each string, and
		 * append a new-line character to the end:
		 */

		if (header) {
			strcpy(buffer, STANDARD_DEBUG_PREFIX);
			if (DebugExtra & (1 << DEBUG_TIMESTAMP)) {
				now = ms ();
				sprintf (buffer + strlen (buffer), "%9ld %6ld ", now, now - lastTime);
				lastTime = now;
			}
			head_len = strlen (buffer);
		}
		else
			head_len = 0;
		len = vsprintf(buffer + head_len, DebugMessage, ap);

		if (buffer[ head_len + len - 1 ] !=  '\n') {
			buffer[ head_len + len ]     = '\n';
			buffer[ head_len + len + 1 ] = '\0';
		}

		fprintf (stderr, "%s", buffer);
	}

	va_end(ap);
} 

#endif /* KPMIDI_DEBUG */
