/* GSI - general sound interface
 *
 * log.c - gsi* library; info & error message handling.
 *
 * Lower level API dependencies: standard io.
 *
 *
 * Copyright (C) 2000 W.H.Scholten
 *
 * Redistribution and use in source and binary forms, with or
 * without modification, are permitted provided that the following
 * conditions are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS
 * IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
 * AUTHOR OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */



/*__________________INCLUDES___________________*/
#include <stdio.h>

//#include <gsi/gsimidipcm_config.h>
#include <gsi/config.h>
#include <gsi/whs_types.h>
#define DEBUG_PREFIX "GSIMIDIPCM_"
//#include <gsi/gsi_debug.h>

#include "log.h"


static void (*_log_simple_message) (int msg_type, char *msg) = NULL;

void gsimidipcm_log_set_handler (void (*handler) (int msg_type, char *msg)) {
	_log_simple_message = handler;
}

static int  be_quiet = 0;

static void log_simple_message (int msg_type, char *msg) {
	if (!be_quiet) {
		//fprintf(stderr, "%s: %s\n", "libgsimidipcm: ", msg);
		fprintf(stderr, "%s\n", msg);
	}
}


#define MSG_PREFIX "MIDI softsynth (gsimidipcm): "
//void log_message (int type, char *format, ...)
void gsimidipcm_log_message (int type, char *format, ...)
{
	va_list args;
	char    buf[ 4096 ];

	va_start (args, format);
	//vsnprintf (buf, 4000, format, args);
	strcpy (buf, MSG_PREFIX);
	vsnprintf (buf+strlen(MSG_PREFIX), 4096-strlen(MSG_PREFIX), format, args);

	if (_log_simple_message)
		_log_simple_message(type, buf);
	else
		log_simple_message (type, buf);

	va_end (args);
}

/*      EOF log.c */
