/*
 * Read/create DEFAULTDIR/gsi.kpmidi (volume for each midi instrument)
 *
 *
 * $Id: volume.c,v 1.2 1998/04/06 20:59:20 keithp Exp $
 *
 * Modified by W.H.Scholten
 *
 * Copyright  1998 Keith Packard
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of Keith Packard not be used in
 * advertising or publicity pertaining to distribution of the software without
 * specific, written prior permission.  Keith Packard makes no
 * representations about the suitability of this software for any purpose.  It
 * is provided "as is" without express or implied warranty.
 *
 * KEITH PACKARD DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
 * EVENT SHALL KEITH PACKARD BE LIABLE FOR ANY SPECIAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
 * DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 */

#include <stdio.h>

#include "utils/string_match.h"
#include "midi_data.h"
#include "volume.h"
#include "utils/log.h"
#include "utils/debug.h"

float  patchVolume[ 128 ];
float  drumVolume[ MAX_DRUM - MIN_DRUM + 1 ];


FILE *
OpenMidiVolumes (char *mode)
{
	char    file[ 1024 ];

	//snprintf (file, 1024, "%s/.gsi.kpmidi.volume", getenv ("HOME"));
	snprintf (file, 1024, "%s/gsi.kpmidi.volume", DEFAULT_DIR);

	DEB(DPRT("volume: trying to open %s\n", file));

	return fopen (file, mode);
}


void
InitMidiVolumes (void)
{
	char    name[1024];
	float   volume;
	FILE    *f;
	int     i;

	DEB(DPRT("volume: InitMidiVolumes\n"));

	/* Initialize the volumes for all instruments to 1.0 */ 
	for (i = 0; i < 128; i++)
		patchVolume[i] = 1.0;

	for (i = MIN_DRUM; i <= MAX_DRUM; i++)
		drumVolume[ i-MIN_DRUM ] = 1.0;

	/* If we can open the volume file, change the volumes of the instruments */
	f = OpenMidiVolumes ("r");
	if (!f) {
		gsimidipcm_log_message(GSI_MESSAGE_ERROR, "volume: OpenMidiVolumes (read) failed\n");
		//DEB(DPRT("patchVolume[10]=%6.3f\n", patchVolume[10]));
		return;
	}

	while (fscanf (f, "%s %f", name, &volume) == 2)
	{
		for (i = 0; i < 128; i++) {
			if (match_any (name, patchNames[i]))
			{
				patchVolume[i] = volume;
				break;
			}
		}

		if (i < 128)
			continue;

		for (i = MIN_DRUM; i <= MAX_DRUM; i++) {
			if (match_any (name, drumNames[i-MIN_DRUM]))
			{
				drumVolume[i-MIN_DRUM] = volume;
				break;
			}
		}
	}
	fclose (f);
}


void
SaveMidiVolumes (void)
{
	FILE    *f;
	char    part[1024];
	int     i;

	f = OpenMidiVolumes ("r");
	if (!f)
	{
		for (i = 0; i < 128; i++)
			if (patchVolume[i] != 1.0)
				break;
		if (i == 128)
		{
			for (i = MIN_DRUM; i <= MAX_DRUM; i++)
				if (drumVolume[i-MIN_DRUM] != 1.0)
					break;
			if (i == MAX_DRUM+1)
				return;
		}
	}
	else {
		fclose (f);
	}

	f = OpenMidiVolumes ("w");
	if (!f) {
		gsimidipcm_log_message(GSI_MESSAGE_ERROR, "volume: OpenMidiVolumes (write) failed\n");
		return;
	}

	for (i = 0; i < 128; i++)
	{
		if (patchVolume[i] != 1.0)
		{
			match_next (patchNames[i], part);
			fprintf (f, "%s %g\n", part, (double) patchVolume[i]);
		}
	}

	for (i = MIN_DRUM; i <= MAX_DRUM; i++)
	{
		if (drumVolume[i-MIN_DRUM] != 1.0)
		{
			match_next (drumNames[i-MIN_DRUM], part);
			fprintf (f, "%s %g\n", part, (double) drumVolume[i-MIN_DRUM]);
		}
	}

	fclose (f);
}
