/* GSI - general sound interface
 *
 * host_allow.c - 
 *
 * Lower level API dependencies:
 *     - standard file system stuff
 *
 *
 * Copyright (C) 2000 W.H.Scholten
 *
 * Redistribution and use in source and binary forms, with or
 * without modification, are permitted provided that the following
 * conditions are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS
 * IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
 * AUTHOR OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */



/*__________________INCLUDES___________________*/
#include <gsi/gsi_config.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <sys/stat.h>

#include <gsi/whs_types.h>
//#define DEBUG_PREFIX "GSI_SERVER (CONF)"
#define DEBUG_PREFIX "(CONF)"
#include <gsi/gsi_debug.h>

#include <gsi/version.h>


/*__________________DEFINITIES___________________*/
#define _GSI_HOST_ALLOW_FILE           "gsi.hosts"
#define _GSI_MASTER_HOST_ALLOW_FILE    "/etc/gsi.hosts"


/*__________________EXTERN___________________*/


/*__________________VARIABELEN___________________*/


/*__________________FUNCTIE PROTOTYPES___________________*/
int  host_allow(char *file, char *name);

#warning "Finish this file"

/* ------------------------------------------------------------------------- *
 *      Read config file (/etc/gsi.hosts)                                    *
 */
int  host_allow()
{
	int  status;

	snprintf(my_configfile, 512, "%s/.%s", getenv("HOME"), _GSI_HOST_ALLOW_FILE);

	/* First try the config file in the home directory. */
	status = read_config(my_configfile);
	if (status > 0) {
		strcpy(my_configfile, _GSI_MASTER_HOST_ALLOW_FILE);
		status = read_config(my_configfile);
	}

	return status;
}


/* ------------------------------------------------------------------------- *
 *      Returns 0 if OK, -1 if the host is not allowed to connect            *
 */
int host_allow(char *file, char *name)
{
	int  i;
	char config_line[ 256 ];
	int  line_number = 0;
	int  bad_config_file_version = 0;
	FILE *fp;

	DEB1(DPRT("("__FILE__"): *read_host_allow*\n"));

	if (name == NULL) return -1;
	if (strlen(name) == 0) return -1;

	fp = fopen(name, "r");
	if (fp == NULL) {
		return -1;
	}

	bad_config_file_version = 0;
	for(;;) {
		fgets(config_line, 127, fp); config_line[ 127 ] = 0;
		if (feof(fp)) {
			return -1;
		}

		line_number++;
		if (strlen(config_line) == 0) {
			log_message(GSI_MESSAGE_ERROR, "Config file %s: empty file", name);
			return -2; /* Empty file. */
		}

		/* Get rid of the linefeed. */
		if (config_line [ (int) strlen(config_line) -1 ] == 10)
			config_line [ (int) strlen(config_line) -1 ] = 0;

		if (!strncmp(config_line, "server_version ", 15)) {
			if (strcmp(config_line + 15, GSI_VERSION_STRING)) {
				log_message(GSI_MESSAGE_ERROR, "Config file %s: wrong server version", name);
				//return -1;
				bad_config_file_version = 1;
			}
			break;
		}
	}

	if (bad_config_file_version) {
		log_message(GSI_MESSAGE_ERROR, "Will try to use this config file anyway...");
	}


	/* go through rest of the file for all settings */
	for (;;) {
		int  bad_entry;
		char *token;
		static int  device = _NONE;
		int  done;

		fgets(config_line, 127, fp); config_line[ 127 ] = 0;
		if (feof(fp)) break;
		line_number++;
		if (strlen(config_line) == 0) break; /* Empty file. */

		/* Get rid of the linefeed. */
		if (config_line [ (int) strlen(config_line) -1 ] == 10)
			config_line [ (int) strlen(config_line) -1 ] = 0;

		token = strtok(config_line, " \t");
		done = False;
		if (token == NULL) continue;
		if (token[0] == '#') continue;
		for (i=0; i<_NONE; i++) {
			if (!strcmp(token, device_string[i])) {
				device = i; done = True;
				DEB2(DPRT("token = no %d (%s)\n", i, device_string[i]));
			}
		}
		if (done) continue;

		bad_entry = False;
	}
}


/*      EOF host_allow.c */
