/* GSI - general sound interface
 *
 * client_data.c - shared sound server, client management
 *
 * Lower level API dependencies: standard socket code
 *
 *
 * Copyright (C) 1998 - 2001 W.H.Scholten
 *
 * Redistribution and use in source and binary forms, with or
 * without modification, are permitted provided that the following
 * conditions are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS
 * IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
 * AUTHOR OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */



/*__________________INCLUDES___________________*/
#include <gsi/gsi_config.h>

#include <stdio.h>
#include <stdlib.h>
//#include <string.h>


#include <gsi/whs_types.h>
#define DEBUG_PREFIX "GSI_SERVER (CLIENT)"
//#include <gsi/gsi_debug.h>
#include <gsi/gsi_commands.h>
#include <gsi/gsimixer.h>
#include <gsi/pcm_data.h>

#include "client.h"
#include "client_data.h"
#include "gsi_server.h"
#include "load/slot.h"


/*__________________DEFINITIES___________________*/


/*__________________EXTERN___________________*/


/*__________________VARIABELEN___________________*/


/*---------------------------------------------------------------------------*
 *      Init client data
 *---------------------------------------------------------------------------*/
int initialize_client_data(struct gsi_client *new, int socket_fd) {
	int i;

	new->input_cmd = -1;
	new->input__slot = -1;
	new->mach1 = 256 << 8;
	new->max_channels = MAX_CHANNELS;
	new->base_width = 256;
	new->last_channel = -1;

	for (i=0; i<GSIMIXER_NUM_DEVICE_TYPES; i++) {
		int  j;
		new->application_volume[i] = malloc(sizeof(int) * gsimixer_num_devices[i]);
		for (j=0; j<gsimixer_num_devices[i]; j++) {
			new->application_volume[i][j] = 256;
		}
	}
	new->new_pcm_samplerate = 8000;
	new->new_pcm_channels = 1;
	new->new_pcm_type = GSI_8BIT;
	new->new_pcm_log2_frag = 8;

	/* Here we indicate that the PCM, SYNTH, CD are not (yet) requested */
	new->mixer_request = GSI_CMD_RELEASE_MIXER;
	new->pcm_request = GSI_CMD_RELEASE_PCM;
	new->synth_request = GSI_CMD_RELEASE_SYNTH;
	for (i=0; i<16; i++)
		new->cd_request[i] = GSI_CMD_RELEASE_CD;

	new->socket_fd = socket_fd;

	return 0;
}


/*---------------------------------------------------------------------------*
 *      For the given client, remove all allocated space (samples etc).
 *---------------------------------------------------------------------------*/
int free_client_data(int client)
{
	int  i;

	if ((client & 255) < num_clients) {
		current = client_list[ client ];

		free_all_slots(current);
		for (i=0; i<256; i++) {
			if (current->channel[i] != NULL) free(current->channel[i]);
		}

		for (i=0; i<GSIMIXER_NUM_DEVICE_TYPES; i++) {
			if (current->application_volume[i])
				free(current->application_volume[i]);
		}

		return 0;
	}

	return -1;
}


/*      EOF client.c */
