/* GSI - general sound interface
 *
 * client_device_requests.c - shared sound server, grab/release devices
 *
 * Lower level API dependencies:
 *   - libgsipcm, unix file descriptors & their use in select.
 *
 *
 * Copyright (C) 1996 - 2000 W.H.Scholten
 *
 * Redistribution and use in source and binary forms, with or
 * without modification, are permitted provided that the following
 * conditions are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS
 * IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
 * AUTHOR OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


/*__________________INCLUDES___________________*/
#include <gsi/config.h>

#include <stdio.h>
#include <string.h>

#include <gsi/whs_types.h>
#define DEBUG_PREFIX "GSI_SERVER"
#include <gsi/gsi_debug.h>
#include <gsi/gsi_events.h>
#include <gsi/gsi_commands.h>

#include <gsi/gsipcm.h>
#include <gsi/gsicd.h>

#include "client_device_requests.h"
#include "client.h"
#include "client_data.h"
#include "gsi_server.h"

#include "device/pcm/pcm.h"
#include "play/cd/cd.h"

#include "io.h"


/*__________________DEFINITIES___________________*/


/*__________________EXTERN___________________*/


/*__________________VARIABELEN___________________*/


/*__________________FUNCTIE PROTOTYPES___________________*/


/*__________________PROGRAMMA___________________*/
/* When there are no clients connected, status is set to this: */
void init_device_requests() {
	/* Should make this an option in gsi.conf: */
	autorelease_pcm = True;
}


/*---------------------------------------------------------------------------*
 *      Client stuff: remove_client, remove_pcm_request, remove_synth_request,
 *      remove_cd_request.
 *---------------------------------------------------------------------------*/
void remove_device_requests(int client) {
	int  i;

	remove_pcm_request(client);
	for (i=0; i<pcm_requests; i++)
		if (pcm_request[i] > client) pcm_request[i]--;

	total_playnum -= client_list[ client ]->channel_playnum;

	remove_synth_request(client);
	if (synth_owner > client && synth_owner < 65536) synth_owner--;

	for (i=0; i<16; i++) {
		remove_cd_request(i, client);
		if (cd_owner[i] > client && cd_owner[i] < 65536) cd_owner[i]--;
	}

	remove_mixer_request(client);
	if (mixer_owner > client && mixer_owner < 65536) mixer_owner--;

//	remove_client_data(client);
//	num_clients--;
//
//	/* Should make this an option in gsi.conf: */
//	if (num_clients == 0) autorelease_pcm = True;
}


void remove_pcm_request(int client) {
	int  i;

	DEB1(DPRT("("__FILE__"): <- remove_pcm_request\n"));
	DEB1(DPRT("-> removing pcm request for client %d\n", client));
	DEB1(DPRT("-> before: pcm_requests=%d\n", pcm_requests));
	if (pcm_requests > 0) {
		int  next = -1; /* next client to take the pcm */

		DEB2({ int  clt;
		for (clt=0; clt<pcm_requests; clt++) {
			DPRT("-> pcm_request[%d]=%d\n", clt, pcm_request[clt]);
		} }
			);

		if (pcm_request[0] == client) {
			/* same code as when client pcm_request[0] gives up the PCM: */
			for (next=1; next<pcm_requests; next++) {
				struct gsi_client  *c = client_list[ pcm_request[ next ] ];

				/* Use delayed pcm setup when we are playing sounds? We wait
				 * with switching until no sounds are playing when using
				 * autoset_pcm, also do this for standard GSI_CMD_GRAB_PCM ?
				 */
				autorelease_pcm = c->autorelease_pcm;
				autoset_pcm = c->autoset_pcm;
				if (!autorelease_pcm) {
					if (!autoset_pcm) {
						setup_pcm(server_pcm, server_pcm_status, /* dev,*/ c->new_pcm_samplerate, c->new_pcm_channels, c->new_pcm_type, c->new_pcm_log2_frag);
					}
					break;
				}

				/* autorelease clients only get the PCM if they use it */
				if (c->channel_playnum) {
					if (!autoset_pcm) {
						setup_pcm(server_pcm, server_pcm_status, /* dev,*/ c->new_pcm_samplerate, c->new_pcm_channels, c->new_pcm_type, c->new_pcm_log2_frag);
					}
					break;
				}
			}
			DEB(DPRT("> client 0: next = %d\n", next));
			if (next == pcm_requests) { /* No other clients so free the PCM */
				next = 0; /* for below, client 1 is provisionally assigned first spot */
				autorelease_pcm = False;
				autoset_pcm = False; /* Correct? or with SET_PCM_SAMPLERATE etc? */
				DEB(DPRT("calling GSIPCM_CLOSE\n"));
				gsipcm_close(server_pcm, server_pcm_status);
				DEB(DPRT("> GSIPCM_CLOSE done\n"));
			}
		}

		DEB(DPRT("> all: next = %d\n", next));
		if (next > 0) {
			/* client 0 is gone, client 'next' takes over */
			pcm_request[0] = next;
			client = next; /* this will move all requests down as if client 'next' gave up his request */
		}
		for (i=0; i<pcm_requests; i++) {
			if (pcm_request[i] == client) {
				if (pcm_requests-i-1 > 0)
					memmove(&pcm_request[i], &pcm_request[i+1], pcm_requests-i-1);
				pcm_requests--;
				break;
			}
		}
	}

	DEB2({ int  clt;
	DPRT("after: pcm_requests=%d\n", pcm_requests);
	for (clt=0; clt<pcm_requests; clt++) {
		DPRT("pcm_request[%d]=%d\n", clt, pcm_request[clt]);
	} }
		);

	DEB1(DPRT("-> remove_pcm_request done\n"));
}


/* Move to play/song/synth.c ? */
void remove_synth_request(int client) {
	struct gsi_client *cld;

	cld = client_list[ client ];
}


/* Move to cd/cd.c ? */
void remove_cd_request(int cdno, int client) {
	int  i;

//	for (cdno == 0; cdno <16; cdno++) {
		if (cd_owner[ cdno ] == client) {
			gsicd_done(cdno);
			cd_owner[ cdno ] = -1;
			// now send all other clients GSI_EV_CD_RELEASED
			for (i=0; i<num_clients; i++) {
				if (i != client) {
					send_list(client_list[i], 2, GSI_EV_CD_RELEASED, cdno);
				}
			}
		}
		else if (cd_owner[ cdno ] > 65535 && current->cd_request[ cdno ] == GSI_CMD_SHARE_CD) {
			cd_owner[ cdno ]--;
			if (cd_owner[ cdno ] == 65536) {
				// almost same code as above:
				gsicd_done(cdno);
				cd_owner[ cdno ] = -1;
				// now send all clients GSI_EV_CD_RELEASED
				for (i=0; i<num_clients; i++) {
					send_list(client_list[i], 2, GSI_EV_CD_RELEASED, cdno);
				}
			}
		}
//	} /* loop through all CD's */
}


/* Move to mixer/mixer.c ? */
void remove_mixer_request(int client) {
	int  i;

	if (mixer_owner == client) {
		/* The following close would not allow us to set e.g. synth volume
		 * (the mixer device can be shared with other apps anyway): */
		//close_mixer();

		mixer_owner = -1;

		/* now send all other clients GSI_EV_MIXER_RELEASED */
		for (i=0; i<num_clients; i++) {
			if (i != client) {
				send_byte(client_list[i], GSI_EV_MIXER_RELEASED);
			}
		}
	}
}


/*      EOF device_control.c */
