/* GSI - general sound interface
 *
 * commands.c - shared sound server, command decoding
 *
 * Lower level API dependencies:
 *   - 
 *
 *
 * Copyright (C) 1996 - 2000 W.H.Scholten
 *
 * Redistribution and use in source and binary forms, with or
 * without modification, are permitted provided that the following
 * conditions are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS
 * IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
 * AUTHOR OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


/*__________________INCLUDES___________________*/
#include <gsi/config.h>

#include <stdio.h>
#include <stdlib.h>
//#include <string.h>

#include <gsi/whs_types.h>
//#define DEBUG_PREFIX "GSI_SERVER (CMD)"
#define DEBUG_PREFIX "(PRE CMD)"
#include <gsi/gsi_debug.h>

#include <gsi/gsi_events.h>
#include <gsi/gsi_eventmasks.h>

#include "client.h"

#include "io.h"
#include "command_decode.h"


/*__________________DEFINITIES___________________*/


/*__________________EXTERN___________________*/


/*__________________VARIABELEN___________________*/


/*__________________PROTOTYPES___________________*/


/*---------------------------------------------------------------------------*
 *      Decode error handling
 *---------------------------------------------------------------------------*/
void _decode_nuke_command(gsi_client_t *c, uint32 length) {
	c->input_stage = 0;
	c->input_cmd = -1;
	c->input_nuke = length;
}

void _decode_error__bad_slot(gsi_client_t *c, int slot) {
	send_byte(c, GSI_EV_BAD_SLOT);
	send_byte(c, slot);
}
void _decode_error__slot_busy(gsi_client_t *c, int slot) {
	send_byte(c, GSI_EV_SLOT_BUSY);
	send_byte(c, slot);
}
void _decode_error__no_room(gsi_client_t *c, int slot, int length) {
	send_byte(c, GSI_EV_NO_ROOM);
	send_byte(c, slot);
	send_int32(c, length);
}
void _decode_error__no_such_file(gsi_client_t *c, int slot) {
	send_list(c, 2, GSI_EV_NO_SUCH_FILE, slot);
//	if (nuke) _decode_nuke_command(nuke);
}


/*---------------------------------------------------------------------------*
 *      Read from client; async versions, using ring buffer & direct reading.
 *      Returns 'False' if the complete data has not yet been transmitted to
 *      the server, 'True' if it has...
 *---------------------------------------------------------------------------*/
int  cmd_bytes_used = 0;
int  cmd_block_bytes_used = 0;
int  cmd_decode_finished = False;


/*---------------------------------------------------------------------------*
 *      Returns True if we're done reading the block. False otherwise.
 *---------------------------------------------------------------------------*/
int cmd_init_direct_block(gsi_client_t *c, int offset, void *data, int len) {
	int  copy_len1, copy_len2;
	int  copy_len;
	int  ostart;

	DEB(DPRT("- cmd_init_direct_block: offset=%d, len=%d\n", offset, len));
	NANA(I(len > 0));
	c->input_block_size = len;
	c->input_block = data;
	c->input_block_len = 0;


	ostart = c->input_buffer_start + offset;
//DEB(DPRT(">   ostart = %d\n", ostart));
	if (ostart >= RING_BUFFER_SIZE) ostart -= RING_BUFFER_SIZE;
//DEB(DPRT(">   after wraparound adjust: ostart = %d\n", ostart));
//DEB(DPRT(">   c->input_buffer_len=%d, offset=%d, len=%d\n", c->input_buffer_len, offset, len));

	copy_len = min(c->input_buffer_len - offset, len);
	copy_len1 = min(ostart + copy_len, RING_BUFFER_SIZE) - ostart;
//DEB(DPRT(">   copy_len  = %d\n", copy_len));
//DEB(DPRT(">   copy_len1 = %d\n", copy_len1));
	memcpy(c->input_block, c->input_buffer + ostart, copy_len1);
	c->input_block_len += copy_len1;

	copy_len2 = copy_len - copy_len1;
//DEB(DPRT(">   copy_len2 = %d\n", copy_len2));
	if (copy_len2 > 0) {
		memcpy(c->input_block + copy_len1, c->input_buffer, copy_len2);
		c->input_block_len += copy_len2;
	}

	if (c->input_block_len < c->input_block_size)
		c->input_read_block = True;

	cmd_block_bytes_used += copy_len;
//DEB(DPRT("EXIT CMD_INIT_DIRECT_BLOCK: c->input_block_len = %d, c->input_read_block = %d, c->input_block_size=%d\n", c->input_block_len, c->input_read_block, c->input_block_size));
	return (c->input_block_len == c->input_block_size);
}


/*---------------------------------------------------------------------------*
 *      Returns True if we're done reading the block. False if not or if
 *      memory allocation failed. In that case error is set to -1.
 *---------------------------------------------------------------------------*/
int cmd_init_block(gsi_client_t *c, int offset, int alloc_len, int len, int *error) {
	NANA(I(len > 0));
	DEB(if (len < 0) { DEB(DPRT("Error: cmd_init_block: len < 0\n")); exit(1); });
//DEB(DPRT("CMD_INIT_BLOCK: alloc_len=%d, len=%d\n", alloc_len, len));
	if (c->input_block_alloc_len != alloc_len) {
		char *new_buf = realloc(c->input_block, alloc_len);
		if (new_buf == NULL) {
			*error = -1;
			return False;
		}
		c->input_block_alloc_len = alloc_len;
		c->input_block = new_buf;
	}
	c->input_block_size = len;

	return cmd_init_direct_block(c, offset, c->input_block, len);
}


/*---------------------------------------------------------------------------*
 *      Reset counters etc.
 *---------------------------------------------------------------------------*/
void post_decode_command(gsi_client_t *c) {
	int  total_used = cmd_bytes_used + cmd_block_bytes_used;
//DEB(DPRT("post decode: total bytes used = %d\n", total_used));

	NANA(I(total_used >= 0));

	if (total_used) {
		c->input_buffer_start += total_used;
//BAD		if (c->input_buffer_start > RING_BUFFER_SIZE)
		if (c->input_buffer_start >= RING_BUFFER_SIZE)
			c->input_buffer_start -= RING_BUFFER_SIZE;

		c->input_buffer_len -= total_used;
//DEB(DPRT("post decode: now input_buffer_len = %d\n", c->input_buffer_len));

		c->input_cmd_len -= cmd_bytes_used;
		cmd_bytes_used = 0;
		cmd_block_bytes_used = 0;
	}
	if (cmd_decode_finished) {
//DEB(DPRT("post decode: decode is FINISHED!\n"));
		c->input_cmd = -1;
		c->input_cmd_len = 0;

		c->input_read_block = False;
// 	Reclaim excessive space used:
//		if (c->input_block_size > 2048) {
//			free(c->input_block); c->input_block = NULL; c->input_block_len = 0;
//		}
	}

	DEB( if (c->input_buffer_len < 0) { DPRT("c->input_buffer_len < 0!\n"); exit(1); } );

}


void adjust_for_cmd_bytes_used (gsi_client_t *c) {
	int  total_used = cmd_bytes_used + cmd_block_bytes_used;
//DEB(DPRT("=>   adjust_for_cmd_bytes_used: %d bytes\n", total_used));
	NANA(I(total_used >= 0));

	c->input_buffer_start += total_used;
//BAD	if (c->input_buffer_start > RING_BUFFER_SIZE)
	if (c->input_buffer_start >= RING_BUFFER_SIZE)
		c->input_buffer_start -= RING_BUFFER_SIZE;

	c->input_buffer_len -= total_used;
	c->input_cmd_len -= cmd_bytes_used;

	cmd_bytes_used = 0;
	cmd_block_bytes_used = 0;
//DEB(DPRT(">   input_buffer_start = %d, input_buffer_len = %d, input_cmd_len = %d\n", c->input_buffer_start, c->input_buffer_len, c->input_cmd_len));
//DEB(DPRT("EXIT adjust_for_cmd_bytes_used\n"));
}


/*      EOF commands.c */
