/* GSI - general sound interface
 *
 * attach_device.c - attach devices
 *
 * Lower level API dependencies: libgsicd, mixer, synth, pcm
 *
 *
 * Copyright (C) 1996 - 2001 W.H.Scholten
 *
 * Redistribution and use in source and binary forms, with or
 * without modification, are permitted provided that the following
 * conditions are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS
 * IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
 * AUTHOR OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */




/*__________________INCLUDES___________________*/
#include <gsi/config.h>

#include <stdio.h>

#include <stdlib.h>
#include <string.h>

#include <gsi/whs_types.h>
#define DEBUG_PREFIX "GSI_SERVER (device attaching)"
//#include <gsi/gsi_debug.h>
#include <gsi/whs_defines.h>
#include <gsi/gsimixer.h>
#include <gsi/gsicd.h>
//#include <gsi/gsisynth.h>

#include "../log.h"
#include "attach_device.h"
#include "pcm/pcm.h"

#include <gsi/device.h>


int gsidev_attach_device (char *device, char *description, void *dev, void *dev_status, int dev_size, int dev_status_size, int dev_type, int *dev_no);

int gsidev_remove_last_device (void *dev, void *dev_status, int dev_type, int *dev_no);


/* TEMP: */
gsipcm_device_t *server_pcm;
gsipcm_devicedata_t *server_pcm_status;



int attach_cd_device(char *device, char *description) {
	int  status;

	if (gsidev_attach_device(device, description, &gsicd_devices, &gsicd_status, sizeof(struct gsicd_device), sizeof(struct gsicd_status), GSI_CD, &gsicd_num_cds))
		return -1; /* out of memory */

	gsicd_status[ gsicd_num_cds-1 ]->fd = -1;
	gsicd_devices[ gsicd_num_cds-1 ]->num_slots = 1; /* can be overridden in gsi.conf */

	/* Test if the device is really present: */ 
	status = gsicd_init(gsicd_num_cds-1);
	if (status == GSICD_ERROR_NOT_PRESENT) {
		gsidev_remove_last_device (&gsicd_devices, &gsicd_status, GSI_CD, &gsipcm_num_pcms);
		log_message(GSI_MESSAGE_ERROR, "cd attach: %s is not a valid device", device);
		return -1;
	}
	/* Close device if we succeeded in opening: */
	if (status == GSICD_OK) {
		gsicd_done(gsicd_num_cds-1);
	}

	return 0;
}


int attach_pcm_device(char *device, char *description) {
	int  status;

	if (gsidev_attach_device(device, description, &gsipcm_devices, &gsipcm_status, sizeof(struct gsipcm_device), sizeof(struct gsipcm_devicedata), GSI_PCM, &gsipcm_num_pcms))
		return -1; /* out of memory */

fprintf(stderr, "num pcms=%d\n", gsipcm_num_pcms);
	/* Test if the device is really present: */
	status = gsipcm_init (gsipcm_devices[ gsipcm_num_pcms-1 ], gsipcm_status[ gsipcm_num_pcms-1 ], GSIPCM_MODE_BLOCKING);
	if (status == GSIPCM_NOT_PRESENT) {
		gsidev_remove_last_device (&gsipcm_devices, &gsipcm_status, GSI_PCM, &gsipcm_num_pcms);
		log_message(GSI_MESSAGE_ERROR, "pcm attach: %s is not a valid device", device);
		return -1;
	}
	/* Close device if we succeeded in opening: */
	if (status == GSIPCM_OK) {
		gsipcm_close(gsipcm_devices[ gsipcm_num_pcms-1 ], gsipcm_status[ gsipcm_num_pcms-1 ]);
	}

	/* TEMP: */
	server_pcm = gsipcm_devices[ 0 ];
	server_pcm_status = gsipcm_status [ 0 ];
	server_pcm_status->fd = -1;

	return 0;
}


int attach_synth_device(char *device, char *description) {
#if 0
	int  i;
	char extended_description[ 32 ];

	/* Need to demultiplex synthesizer devices (e.g. OSS, synth device = midi,
	 * opl2, opl3, gus, awe ...). */

	for (i=0; i<256; i++) {
		char *device_type;

		device_type = gsisynth_attach_device(device, i);
		if (device_type == NULL) break;
		snprintf(extended_description, 32, "%s/%s", description, device_type);

		if (gsidev_attach_device(device, description, gsisynth_devices, gsisynth_status, sizeof(struct gsisynth_device), sizeof(struct gsisynth_devicedata), GSI_SYNTH, &gsisynth_num_synths))
			return -1; /* out of memory */
	}

	if (i == 0) { /* nothing found when demultiplexing */
		log_message(GSI_MESSAGE_ERROR, "synth attach: %s is not a valid device", device);
		return -1;
	}
#endif /* disabled */

	return 0;
}


/* Generic attach code ===================================================== */
int gsidev_attach_device (char *device, char *description, void *dev, void *dev_status, int dev_size, int dev_status_size, int dev_type, int *dev_no) {
	int  i;
	void *dev_alloc;
	struct gsi_generic_device ***gdev = (gsi_generic_device_t ***) dev;
	struct gsi_generic_devicestatus ***gdev_status = (gsi_generic_devicestatus_t ***) dev_status;

	/* check if it's already in the list: */
	for (i=0; i<*dev_no; i++) {
		struct gsi_generic_device *d = (*gdev)[i];
		if (!strcmp(d->device, device) && !strcmp(d->id, description)) return 0;
	}

	/* allocate room for device information */
	dev_alloc = realloc(*gdev, sizeof (void *) * (*dev_no + 1));
	if (dev_alloc == NULL) {
		log_message(GSI_MESSAGE_ERROR, "Out of memory attaching device %s (%s)", device, description);
		return -1;
	}
	*gdev = dev_alloc;
fprintf(stderr, "a0\n");

	dev_alloc = calloc(1, dev_size);
	(*gdev) [ *dev_no ] = dev_alloc;
	if (dev_alloc == NULL) {
		log_message(GSI_MESSAGE_ERROR, "Out of memory attaching device %s (%s)", device, description);
		return -1;
	}
fprintf(stderr, "a1\n");
	/* allocate room for device status */
	dev_alloc = realloc(*gdev_status, sizeof (void *) * (*dev_no + 1));
	if (dev_alloc == NULL) {
		free((*gdev) [ *dev_no ]);
		log_message(GSI_MESSAGE_ERROR, "Out of memory attaching device %s (%s)", device, description);
		return -1;
	}
	*gdev_status = dev_alloc;
fprintf(stderr, "a2\n");

	dev_alloc = calloc(1, dev_status_size);
	(*gdev_status) [ *dev_no ] = dev_alloc;
	if (dev_alloc == NULL) {
		free((*gdev) [ *dev_no ]);
		log_message(GSI_MESSAGE_ERROR, "Out of memory attaching device %s (%s)", device, description);
		return -1;
	}
fprintf(stderr, "a3\n");

	/* Fill in device information */
	dev_alloc = strdup(device);
	(*gdev) [ *dev_no ]->device = dev_alloc;
	if (dev_alloc == NULL) {
		free((*gdev) [ *dev_no ]);
		free((*gdev_status) [ *dev_no ]);
		log_message(GSI_MESSAGE_ERROR, "Out of memory attaching device %s (%s)", device, description);
		return -1;
	}
	strncpy((*gdev) [ *dev_no ]->id, description, 32);
fprintf(stderr, "a4\n");

	(*dev_no)++;
	gsimixer_num_devices[ dev_type ] = (*dev_no);

	return 0;
}


int gsidev_remove_last_device (void *dev, void *dev_status, int dev_type, int *dev_no) {
	struct gsi_generic_device ***gdev = (gsi_generic_device_t ***) dev;
	struct gsi_generic_devicestatus ***gdev_status = (gsi_generic_devicestatus_t ***) dev_status;

	(*dev_no)--;
	if ((*gdev)[ *dev_no ]) {
		if ((*gdev)[ *dev_no ]->device)
			free((*gdev)[ *dev_no ]->device);

		free((*gdev)[ *dev_no ]);
	}
	if ((*gdev_status)[ *dev_no ]) free((*gdev_status)[ *dev_no ]);

	gsimixer_num_devices[ dev_type ] = (*dev_no);

	return 0;
}


/* Modules ================================================================= */
#if 0
int num_pcm_modules = 0;
int attach_pcm_module_device(char *device, gsipcm_module_t *module_fns, char *description, char *module_name) {
	int   i;
	void *tmp;

	/* check it's not already in the list: */
	for (i=0; i<num_pcm_modules; i++) {
		if (!strcmp(pcm_module[i].device, device) && !strcmp(pcm_module[i].id, description)) return 0;
	}

	/* Allocate space */
	tmp = realloc(pcm_module, sizeof(gsipcmmodule_t) * (num_pcm_modules + 1));
	if (tmp == NULL) {
		log_message(GSI_MESSAGE_ERROR, "Out of memory attaching pcm module");
		return -1;
	}
	pcm_module = tmp;

	tmp = realloc(pcm_module_fd, sizeof(int) * (num_pcm_modules + 1));
	if (tmp == NULL) {
		log_message(GSI_MESSAGE_ERROR, "Out of memory attaching pcm module");
		return -1;
	}
	pcm_module_fd = tmp;

	/* Initalize structures */
	pcm_module [ num_pcm_modules ].device = strdup(device);
	strncpy(pcm_module [ num_pcm_modules ].id, description, 32);
	pcm_module [ num_pcm_modules ].id[ 31 ] = 0;

	pcmmodule_fd [ num_pcm_modules ] = -1;

	num_pcm_modules++;

	return 0;
}
#endif
/*      EOF attach_device.c */
