/* GSI - general sound interface
 *
 * pcm.c - shared sound server, pcm mode setting
 *
 * Lower level API dependencies: libgsipcm
 *
 *
 * Copyright (C) 1996 - 2000 W.H.Scholten
 *
 * Redistribution and use in source and binary forms, with or
 * without modification, are permitted provided that the following
 * conditions are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS
 * IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
 * AUTHOR OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */




/*__________________INCLUDES___________________*/
#include <gsi/config.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <syslog.h>

#include <gsi/whs_types.h>
#include <gsi/whs_defines.h>
#define DEBUG_PREFIX "GSI_SERVER (device pcm)"
//#include <gsi/gsi_debug.h>
#include <gsi/gsi_events.h>
#include <gsi/gsipcm.h>

#include "../../gsi_server.h"
#include "../../io.h"
#include "pcm.h"


/* tables.c: */
extern int bytes_per_sample[];

/* From attach_pcm.c: */
extern gsipcm_device_t *server_pcm;
extern gsipcm_devicedata_t *server_pcm_status;


/*---------------------------------------------------------------------------*
 *      setup_pcm:
 *        opens device (hw->device) and sets
 *          fragsize
 *          samplerate
 *          channels
 *          type (8 bit mulaw/8 bit unsigned/16 bit)
 *---------------------------------------------------------------------------*/
int setup_pcm (GSIPCM_HW, /*int dev,*/ int samplerate, int channels, int type, int log2_fragsize) {
	int  status;
	int  dev = 0; /* temp */


	status = gsipcm_set_pcm (hw, hw_status, samplerate, channels, type, log2_fragsize);
	/* Note that gsipcm_set_pcm returns the file descriptor. */

	if (status != 0) {
		if (status == GSIPCM_BUSY) {
			send_byte(current, GSI_EV_PCM_BUSY);
		}
		else if (status == GSIPCM_NOT_PRESENT) {
			syslog(LOG_ERR, "%s: Couldn't open PCM %s", server_name, gsipcm_devices[ dev ]->device);
			send_byte(current, GSI_EV_NO_PCM);
		}
		return -1;
	}

	if (status == 0) {
		hw_status->fragment = pcm_final; /* see gsi_server.c */

		/* Set pcm mode to non-blocking: */
		gsipcm_set_mode(hw, hw_status, GSIPCM_MODE_NON_BLOCKING);
	}

	return 0;
}


/*      EOF pcm.c */
