/* GSI - general sound interface: server
 *
 * io.h - byte order stuff, reading multiple bytes etc.
 *
 * 1996 - 2000 by W.H.Scholten
 *
 * This file is released to the public domain.
 */


#ifndef _GSI__server__io_h
#define _GSI__server__io_h

#include <gsi/whs_types.h>
#include <gsi/gsi_config.h>

#define swap_int64(a) ((((a) & 255) << 56) | (((a) & 0xff00) << 40) | ((((a) & 0xff0000) << 24) | (((a) & 0xff000000) << 8)    |     (((a) >> 8) & 0xff000000) | (((a) >>24) &0xff0000)) | (((a) >> 40) & 0xff00) | (((a) >>56) &0xff))
#define swap_int32(a) ((((a) & 255) << 24) | (((a) & 0xff00) << 8) | (((a) >> 8) & 0xff00) | (((a) >>24) &0xff))
#define swap_int16(a) ((((a) & 0xff) << 8) | (((a) >> 8) & 0xff))

#ifdef SYSCONFIG_LITTLE_ENDIAN_MACHINE
#  define msb_from_native_int32(a) swap_int32(a)
#  define msb_from_native_int16(a) swap_int16(a)
#  define lsb_from_native_int32(a) (a)
#  define lsb_from_native_int16(a) (a)
#else
#  define msb_from_native_int32(a) (a)
#  define msb_from_native_int16(a) (a)
#  define lsb_from_native_int32(a) swap_int32(a)
#  define lsb_from_native_int16(a) swap_int16(a)
#endif


#define msb_int32_at(buf)   msb_from_native_int32(*((int32 *) (buf)))
#define msb_int16_at(buf)   msb_from_native_int16(*((int16 *) (buf)))

#define next_msb_int32(buf) ( ((((byte *) (buf))++)[0] << 24) | ((((byte *) (buf))++)[1] << 16) | ((((byte *) (buf))++)[2] << 8) | (((byte *) (buf))++)[3] )
#define next_msb_int16(buf) ( ((((byte *) (buf))++)[0] << 8) | (((byte *) (buf))++)[1] )
#define next_byte(buf)      ((((byte *) (buf))++)[0])


/* writing to clients */
#define SEND_CURRENT          1
#define SEND_ALL_BUT_CURRENT  2
#define SEND_ALL              4
#define SEND_CD_OWNERS        8

#include <stdarg.h>
//#include "gsi_server.h"
#include "client_data.h"
void   send_byte (gsi_client_t *c, int data);
void   send_int32 (gsi_client_t *c, int data);
void   send_list (gsi_client_t *c, int argc, ...);
/* how much faster? */
//void   current_send_byte (int data);
//void   current_send_int32 (int data);
//void   current_send_list (int argc, ...);

void   send_list_to_cd_owners (int cd, int argc, ...);
//void  send_list_to (int whom, int argc, ...);

void   flush_if_no_room_for (gsi_client_t *c, int bytes);
void   flush (gsi_client_t *c);
void   flush_all (void); /* all clients */

#endif /* _GSI__server__io_h */
