/* GSI - general sound interface
 *
 * log.c: error & info message logging.
 *
 *
 * Copyright (C) W.H.Scholten 2000
 *
 * Redistribution and use in source and binary forms, with or
 * without modification, are permitted provided that the following
 * conditions are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS
 * IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
 * AUTHOR OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <gsi/gsi_config.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <syslog.h>

#include <gsi/whs_defines.h>
#include <gsi/whs_types.h>
#define DEBUG_PREFIX "GSI_ (log)"
//#include <gsi/gsi_debug.h>

#include "gsi_server.h" /* for server_name */
#include "log.h"


#include <signal.h>

static void (*_log_simple_message) (int msg_type, char *msg) = NULL;

int server_be_quiet = 1;

/*---------------------------------------------------------------------------*
 * Send an info/error message to stderr or only to syslog depending on the
 * flag 'be_quiet'.
 *---------------------------------------------------------------------------*/
void log_message (int msg_type, char *format, ...)
{
	va_list args;
	char    buf[ 4096 ];

	va_start (args, format);
	vsnprintf (buf, 4000, format, args);
if (_log_simple_message != NULL) kill(0, SIGSEGV);
	if (_log_simple_message)
		_log_simple_message(msg_type, buf);
	else
		log_simple_message (msg_type, buf);

	va_end (args);
}


void log_simple_message (int msg_type, char *msg) {
	if (!server_be_quiet) {
		fprintf(stderr, "%s: %s\n", server_name, msg);
	}

	if (msg_type == GSI_MESSAGE_NOTICE)
		syslog(LOG_NOTICE, msg);
	else
		syslog(LOG_ERR, msg);
}



#if 0 /* OLD */
void log_message (int type, char *format, ...)
{
	va_list args;
//	char   *msg_prefix = server_name;
	char    buf[ 4096 ];

	va_start (args, format);
//	strcpy (buf, msg_prefix);
//	vsnprintf (buf + strlen(msg_prefix), 4000, format, args);
	vsnprintf (buf, 4000, format, args);

	error_message2(type, buf);

	va_end (args);
}
#endif

/*      EOF log.c */
