/* GSI - general sound interface
 *
 * cd.c - shared sound server, cd stuff
 *
 * Lower level API dependencies: libgsicd
 *
 *
 * Copyright (C) 1996 - 2000 W.H.Scholten
 *
 * Redistribution and use in source and binary forms, with or
 * without modification, are permitted provided that the following
 * conditions are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS
 * IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
 * AUTHOR OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */




/*__________________INCLUDES___________________*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <gsi/whs_types.h>
#define DEBUG_PREFIX "GSI_SERVER (play cd)"
//#include <gsi/gsi_debug.h>
#include <gsi/gsi_events.h>

#include <gsi/gsicd.h>

#include <gsi/gsi_commands.h>

#include "../../client.h"
#include "../../io.h"
#include "cd.h"


volatile sig_atomic_t cd_track_alarm = False;

void cd_lowlevel_track_event (void);


void cd_lowlevel_track_event ()
{
	cd_track_alarm = True;
	/* Main loop is now interrupted if in select..
	 * That will in turn call gsicd_generate_events(); which calls cd_event(..)
	 */
}


void cd_track_event (int event, int cd, int track)
{
	int client_cd_event [2] = { GSI_EV_CD_TRACK_STARTED, GSI_EV_CD_TRACK_END };

	if (event < 0 || event > 4) {
		fprintf(stderr, "bad cd event %d\n", event);
		return;
	}

	if (event < 2)
		send_list_to_cd_owners (cd, 3, client_cd_event [event], cd, track);
	else
		send_list_to_cd_owners (cd, 6, GSI_EV_CD_STATUS, event-2, cd, track, gsicd_status[ cd ]->current_m, gsicd_status[ cd ]->current_f, gsicd_status[ cd ]->current_f);
	// XXX or add GSI_EV_CD_PAUSE/GSI_EV_CD_RESUME/GSI_EV_CD_STOP events ?
}


int cd_init() {
	gsicd_set_lowlevel_event_function( &cd_lowlevel_track_event );
	gsicd_set_event_function( &cd_track_event );

	return 0;
}


/*---------------------------------------------------------------------------*
 *      Check if the client may give cd commands
 *---------------------------------------------------------------------------*/
int is_a_cd_owner(int cd, int client) {
fprintf(stderr, "is_a_cd_owner: cd %d; cd_owner[ %d ] = %d, client = %d\n", cd, cd, cd_owner[ cd ], client);

	if (cd_owner[ cd ] == client || (cd_owner[ cd ] > 65535 && client_list[ client ]->cd_request[ cd ] == GSI_CMD_SHARE_CD))
		return True;

	return False;
}


/*      EOF cd.c */
