/* GSI - general sound interface
 *
 * channel.h
 *
 * by W.H.Scholten 1998 - 2001
 *
 * This file is released to the public domain.
 */


#ifndef _GSI__server__play__channel_h
#define _GSI__server__play__channel_h


/* Internal sound flags for actions on channels & channel status: */

//#define CHANNEL_FLAG_REPEAT            (1<<0)
#define CHANNEL_FLAG_DOPPLER           (1<<1)
/* Positional placement after sound changes: */
#define CHANNEL_FLAG_RECALC_3D_ONCE    (1<<2) /* eg. when x is set with GSI_CMD_CHANNEL_SET_X */
#define CHANNEL_FLAG_RECALC_3D         (1<<3) /* when either dx,dy,dz are not zero */
#define CHANNEL_FLAG_3D                (1<<4) /* Necessary?? 3d if set, pan otherwise */
#define CHANNEL_FLAG_INIT_POSITION     (1<<5) /* To init radius[i] etc. */

#define CHANNEL_FLAG_ECHO              (1<<6)
#define CHANNEL_FLAG_OBSERVER_POS      (1<<7) /* alter sound positions/speeds according to the observer pos and rotation, i.e. all outside sounds, not e.g. sounds inside a car or game error beeps and stuff */

#define CHANNEL_FLAG_REPEAT_STREAM_ON_UNDERFLOW (1<<8)

#define CHANNEL_FLAG_PLAYING           (1<<9)
#define CHANNEL_FLAG_PAUSED            (1<<10)


/* -------------------------------------------------------------------------
 * CHANNEL structure:
 * channels structures are allocated when used (start a sound -> check for
 * channel struct, malloc if not yet done). Never free them, except when the
 * client releases the PCM (or disconnects, of course).
 * ------------------------------------------------------------------------- */
typedef struct gsis_channel_info {
	int     type; /* type of slot being playing (at the moment: sample/midi) */

	int     slot;
	/*      slot being played */

	short   volume;
	short   mix_volume;
	/*      = (volume * client->pcm_volume) >> 8; */

	int     play_ct;

	/* Samples: */
//	union sound_data {
	struct gsis_sample_data {
		/* ==== position system ==== */
		short   pan;
		short   depth;
		short   heigth;

		/* ==== 3d/movement system ==== */
		float   old_dist; /* For doppler calculations */
		/*      saved, used when a sound doen't move to prevent recalculation. */

		float   x, y, z;
		short   dx, dy, dz;

		int     last_position_time;
		/*      Time at which the position was last set/adjusted with GSI_CMD_SET_X
		 *      etc, and the current_time (set to zero at init). This is measured
		 *      in 1/32 s, 24 bit int, 8 bit fraction. */

		/* ==== Effects ==== */
		short   echo_dt; /* dt*32 */
		byte    echo_fvol;
		/*      fractional volume change, should be 1/(8R^3) where R is distance to
		 *      the bounce point => log2: -3 -3* 2log(R) ? => dvol? */

		int     pitch_f;
		/*      16.16 pitch factor */

		/* Sample rate conversion factors from sound to pcm samplerate and back
		 * (calculated with the slot's samplerate, the pcm's samplerate, sound_f,
		 * and doppler):
		 * This method will be replaced by an RL algorithm.
		 * These values aren't used at the moment (always recalculated each
		 * fragment even if sound_f, doppler, pcm sample rate are unchanged). */
		int     ch_samplerate_conversion;
		int     ch_samplerate_conversion2;

		int     position;
		/*      Current position in file */

		int     flags;
		/*      wether to use the 3d system, echo, playing state etc. */
	} sample;

	/* Songs: */
	struct gsis_song_data {
		void  *playstatus;
		int   (*song_init)(void *playstatus);
		int   (*song_set_time)(void *playstatus, double time);
		int   (*song_get_time)(void *playstatus, double *time);
		int   (*song_pause)(void *playstatus);
		int   (*song_resume)(void *playstatus);
		int   (*song_stop)(void *playstatus);
		int   (*song_seek)(void *playstatus, double time);
		int   (*song_play)(void *playstatus, double dt);
		double duration;

		int    flags;
		/*     used for what? */
	} song;
//	}; /* union sound_data */

} gsis_channel_info_t;


#endif /* _GSI__server__play__channel_h */
