/* GSI - general sound interface
 *
 * internal_to_pcm_format.c - convert internal pcm buffer to format of the pcm
 * output device.
 *
 * Lower level API dependencies:
 *   -
 *
 *
 * Copyright (C) 1996 - 2000 W.H.Scholten
 *
 * Redistribution and use in source and binary forms, with or
 * without modification, are permitted provided that the following
 * conditions are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS
 * IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
 * AUTHOR OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


	/* Clip the sample size from 32 bit (24 bit + overflow) to 8 bit,
	 * 16 bit or 24 bit. */
	switch(server_pcm_status->type) {
	case GSI_8BIT:
		DEB3(DPRT("write to PCM: samplerate %d channels %d type GSI_8BIT\n", server_pcm_status->samplerate, server_pcm_status->channels));
		{
			int qq, fin = server_pcm_status->samples * server_pcm_status->channels;
			for (qq=0; qq< fin; qq++) {
				int  pmb = pcm_premix[qq]>>16 /* 17 */;
				pcm_final[qq] = (pmb>127) ? 255 : (pmb>=-128) ? (pmb ^ 128) : 0;
			}
		}
		break;
	case GSI_8BIT_MULAW:
		DEB3(DPRT("write to PCM: samplerate %d channels %d type GSI_8BIT_MULAW\n", server_pcm_status->samplerate, server_pcm_status->channels));
		for (i=0; i<server_pcm_status->samples *server_pcm_status->channels; i++) {
			int  pcm_final_tmp = pcm_premix[i];
			pcm_final[i] = linear16_to_mulaw(pcm_final_tmp >>8 /*>>9*/);
		}
		break;

	case GSI_16BIT:
		DEB3(DPRT("write to PCM: samplerate %d channels %d type GSI_16BIT\n", server_pcm_status->samplerate, server_pcm_status->channels));
		if (server_pcm_status->byte_order == server_byte_order) {
			for (i=0; i<server_pcm_status->samples *server_pcm_status->channels; i++) {
				int  pcm_final_tmp = pcm_premix[i] /*>>1*/;
				pcm_final[  i<<1     ] = (pcm_final_tmp >>  8) &0xff;
				pcm_final[ (i<<1) +1 ] = (pcm_final_tmp >> 16) &0xff;
			}
		}
		else {
			/* Swap bytes. */
			for (i=0; i<server_pcm_status->samples *server_pcm_status->channels; i++) {
				int  pcm_final_tmp = pcm_premix[i] /*>>1*/;
				pcm_final[  i<<1     ] = (pcm_final_tmp >> 16) &0xff;
				pcm_final[ (i<<1) +1 ] = (pcm_final_tmp >>  8) &0xff;
			}
		}
		break;

	case GSI_24BIT:
		DEB3(DPRT("write to PCM: samplerate %d channels %d type GSI_24BIT\n", server_pcm_status->samplerate, server_pcm_status->channels));
		/* Do 24 bit device use 32 bits to write(2) ?
		 * if so: ((int *)pcm_final)[i] = pcm_final_tmp; else: */
		if (server_pcm_status->byte_order == server_byte_order) {
			for (i=0; i<server_pcm_status->samples *server_pcm_status->channels; i++) {
				int  pcm_final_tmp = pcm_premix[i] /*>>1*/;

				pcm_final[ i*3    ] = (pcm_final_tmp      ) &0xff;
				pcm_final[ i*3 +1 ] = (pcm_final_tmp >>  8) &0xff;
				pcm_final[ i*3 +2 ] = (pcm_final_tmp >> 16) &0xff;
			}
		} else {
			/* Swap bytes 1 & 3 */
			for (i=0; i<server_pcm_status->samples *server_pcm_status->channels; i++) {
				int  pcm_final_tmp = pcm_premix[i] /*>>1*/;

				pcm_final[ i*3    ] = (pcm_final_tmp >> 16) &0xff;
				pcm_final[ i*3 +1 ] = (pcm_final_tmp >>  8) &0xff;
				pcm_final[ i*3 +2 ] = (pcm_final_tmp      ) &0xff;
			}
		}
		break;
	}
