/* GSI - general sound interface
 *
 * play_fragment.c - calculate a fragment of sound for the given channel and
 * add it to the internal mixing buffer
 *
 * Lower level API dependencies: none
 *
 *
 * Copyright (C) 1996 - 2001 W.H.Scholten
 *
 * Redistribution and use in source and binary forms, with or
 * without modification, are permitted provided that the following
 * conditions are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS
 * IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
 * AUTHOR OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

//#include <math.h>

/*---------------------------------------------------------------------------*
 *      Calculate one fragment of a sound playing on a given channel.
 *---------------------------------------------------------------------------*/
void play_fragment(struct gsi_client *c, int k) {
	int     j;
	int     i = c->channel_used[ k ];

	/* buffer management (for repeat/stream): */
	int     remaining_frag_samples; // equiv. to pcm_samples - mix_index
	int     bufsize;
	int     mix_index;
	int     *my_premix;

	int     sound_volume_L, sound_volume_R; // stereo position volume factors

	// Volume factors for stereo sounds (is this useful? Perhaps only in case
	// of music)
	//int   sound_volume_L_L, sound_volume_L_R;
	//int   sound_volume_R_L, sound_volume_R_R;

	char   *sample_position;
	int     sample_length;
	int     samplerate_c;  /* samplerate conversion factor */
	int     samplerate_c2; /* inverse samplerate conversion factor */
	int     sound_to_mixbuffer;

	/* 3d system: */
	float   radius, radius_sq; /* x-z plane distance */
	float   dist, dist_sq;     /* the 3d distance */
	float   x, z, y;           /* position without the 8 bit fraction */

	int     f; /* frequency adjustment factor (doppler effect) */

	struct gsis_channel_info *ch = c->channel[i];
	struct gsis_slot          *s = c->slot[ ch->slot ];
	struct gsis_slot_sample  *ss = c->slot[ ch->slot ]->data.sample;
	struct gsis_sample_data  *ch_sample = &ch->sample;

	//stereo_shift = channels-sound_channels[i];

	//SPEED TEST:
	//SPEED_INIT_TIME();

	DEB3(DPRT("("__FILE__"): *play_fragment* (for ch %d)\n", i));
	DEB1(DPRT("server_pcm_status->samples=%d\n", server_pcm_status->samples));

	/* --------------------- The 3D system ------------------------------ */
	if (ch_sample->flags & (CHANNEL_FLAG_RECALC_3D | CHANNEL_FLAG_RECALC_3D_ONCE | CHANNEL_FLAG_INIT_POSITION) ) {
		DEB3(DPRT("3d update\n"));
		/*   Should I put the time adjust here instead of in the init at
		 *   GSI_CMD_CHANNEL_SET_X etc?
		 *   First use this:
		 *       dt = current_time-last_position_time[j];
		 *   Then:
		 *       last_position_time[j] = current_time; */
		DEB3(DPRT("3d: (dx[i]*32*pcm_samples)/pcm_samplerate=%d\n", (ch_sample->dx*32*server_pcm_status->samples)/server_pcm_status->samplerate));

		/* correct the change which is designed for 1/32s to the actual time
		 * used per fragment:
		 */
		pcm_time = (float) server_pcm_status->samples / (float) server_pcm_status->samplerate;
		ch_sample->x += (ch_sample->dx*32)*pcm_time;
		ch_sample->z += (ch_sample->dz*32)*pcm_time;
		ch_sample->y += (ch_sample->dy*32)*pcm_time;
		/*   could approximate these by sound_x[i] += (sound_dx[i]*g)>>8 etc,
		 *   where g=(32*server_pcm_status->samples<<8)/server_pcm_status->samplerate;
		 *   (as in frame time after pcm write)
		 */


		/* remove the 8 bit fraction: */
		x = ch_sample->x;
		z = ch_sample->z;
		y = ch_sample->y;
		radius_sq = x*x+y*y;
		radius = sqrt(radius_sq);
		if (radius <64.0) radius = 64.0;
		ch_sample->pan = (x*256)/radius;
		dist_sq = radius_sq+z*z;

		if (dist_sq < 4096.0) {
			dist = 64.0; dist_sq = 4096.0;
		}
		else {
			dist = sqrt(dist_sq);
		}
		ch_sample->flags &= ~CHANNEL_FLAG_INIT_POSITION;
	}
	else {
		dist = ch_sample->old_dist;
		dist_sq = dist*dist;
	}
	ch_sample->flags &= ~CHANNEL_FLAG_RECALC_3D_ONCE;
/*---------------------------------------------------------------------------*
 * 3d: R = x,z-plane radius
 *         1-(x/R)                                1+(x/R)
 *  vol_L= -------- (r0/r)^2 *fspeed*vol,  vol_R= -------- (r0/r)^2 *fspeed*vol
 *            2                                      2
 *
 * Pan:
 *         (1-pan/256)                (1+pan/256)
 *  vol_L= ----------- *vol    vol_R= ----------- *vol
 *              2                          2
 *---------------------------------------------------------------------------*/


//fprintf(stderr, "base width =%d\n", c->base_width);
	// --------------------------- VOLUME ---------------------------
	//sound_volume_L= ((sound_volume[i]>>1)*r0*r0)/dist_sq;

#   define r0  64
	/* stereo: */
	sound_volume_L = ( ((((256*256 - (ch_sample->pan*c->base_width))) *ch->mix_volume)>>16) *r0*r0)/ (int) dist_sq; /* incomplete, f */
	sound_volume_R = ( ((((256*256 + (ch_sample->pan*c->base_width))) *ch->mix_volume)>>16) *r0*r0)/ (int) dist_sq; /* incomplete, f */
#   undef  r0

	/* For mono, use (sound_volume_L+sound_volume_R)/2 */
	DEB2(DPRT("(2d) channel: pan = %d, volume = %d ** client: base_width = %d\n", ch_sample->pan, ch->mix_volume, c->base_width));
	DEB3(DPRT("(2d) vol=%d\n", (((256*256 - (ch_sample->pan*c->base_width))) *ch->mix_volume)));


	/* -------------- DOPPLER (3D SYSTEM) / PITCH FACTOR ------------ */
	/* doppler_freq=freq* vg/(vg+(r'-r)/dt) */
	if (ch_sample->flags & CHANNEL_FLAG_DOPPLER) {
		/* This is not accurate enough (with integers.. use again with FP?):
		 *    f=(mach1<<8)/(mach1+(old_dist[i]-dist))
		 * Therefore: */
		int  A;
		// A = sound_dx[i]*(sound_dx[i]+old_x*2) + sound_dy[i]*(sound_dy[i]+old_y*2)  + sound_dy[i]*(sound_dy[i]+old_y*2);
		// but as x,y,z have been updated:

		A = (ch_sample->dx*(((ch_sample->x*2)-ch_sample->dx)/256.0)+ch_sample->dy*(((ch_sample->y*2)-ch_sample->dy)/256.0)+ch_sample->dz*(((ch_sample->z*2)-ch_sample->dz)/256.0));
		/* We should correct A with a factor (32*server_pcm_status->samples)/server_pcm_status->samplerate
		 * as above in the x,y,z update. Also use this for current_time update.
		 */

		f = (c->mach1<<8) / (c->mach1 - (((32*A)/ch_sample->old_dist)/2.0) );
		/* should correct f with ch_sample->pitch_f too .. */

		DEB3(DPRT("(3d) A=%d, f=%d, (((32*A)/old_dist[i])/2.0)=%f\n", A, f, (((32*A)/ch_sample->old_dist)/2.0)));
		DEB3(DPRT("(3d) mach1=%d\n, old_dist[i]=%f, dist=%f\n", c->mach1, ch_sample->old_dist, dist));
	}
	else f = ch_sample->pitch_f >> 8;

	/* some sanity checking: */
	if (f < 4) f = 4; else if (f > (65536/4)) f = (65536/4);
	DEB3(DPRT("(3d) doppler: f=%d\n", f));

	/* we're done with radius, so now set old_dist. */
	ch_sample->old_dist = dist;


	/* -------------- PITCH FACTOR (FROM 3D OR DIRECTLY) ------------ */
	samplerate_c  = ( ((ss->samplerate << 8)/server_pcm_status->samplerate)* 256) /f;
	samplerate_c2 = ( ((server_pcm_status->samplerate << 8)/ss->samplerate)* f) /256;
	DEB2(fprintf(stderr, "samplerate_c  = %d, samplerate_c2=%d\n", samplerate_c, samplerate_c2));

	/* ---------------------------- VOLUME -------------------------- */
	/* adjust volume so we remove the need to shift each sample (except 24 bit)
	 * later on: (could also put the >>1 in here for stereo ->mono (e.g. S8_M))
	 */
	switch (s->type) {
	case GSI_8BIT:
		sound_volume_L <<= 8; sound_volume_R <<= 8; break;
	case GSI_8BIT_MULAW:
	case GSI_16BIT:
	case GSI_24BIT:
		break;
	}

	// should really use two sections above so we don't calculate L and R
	// for mono pcm:
	if (server_pcm_status->channels == 1) sound_volume_L = (sound_volume_L+sound_volume_R)>>1;
	DEB3(DPRT("vol L=%d\n", sound_volume_L));


	/* ------------------------------- GO --------------------------- */
	/* Calculate how many samples we make from the given sound at this time */
	remaining_frag_samples = server_pcm_status->samples; // replace by server_pcm_status->samples-mix_index
//fprintf(stderr, "remaining_frag_samples = %d\n", remaining_frag_samples);
	mix_index = 0;

	//SPEED_INIT_TIME();
	DEB2(DPRT("c\n"));

  calculate_fragment:
	/* compute how many samples we're handling this time.. */
	DEB2(DPRT("c1\n"));

//OLD	if (ch_sample->flags & CHANNEL_FLAG_STREAM) {
//	if (c->slot[ slot ]->data.sample->flags = _GSI_INPUT_STREAM) {
	if (ss->input == _GSI_INPUT_STREAM) {
		/* take care of wrap around: */
		if (ch_sample->position >= ss->size) {
			ch_sample->position -= ss->size;
		}
		bufsize = min(ss->current_size, ss->size - ch_sample->position);
		sample_length = min(remaining_frag_samples, ( (bufsize/(_server_gsi_bytes_per_sample[ ss->type ] * ss->channels)) * samplerate_c2) >> 8);

		DEB2(DPRT("stream: slot = %d bytes, ch_ptr = %d\n", bufsize, ch_sample->position));
		DEB2(DPRT("  ,,  : sample_len = %d (remaining = %d, slot avail = %d)\n", sample_length, remaining_frag_samples, ( (bufsize/(_server_gsi_bytes_per_sample[ ss->type ] * ss->channels)) * samplerate_c2) >> 8));

		if (bufsize < 0) {
			bufsize += ss->size;
		}
		if (bufsize == 0) {
			int  bytes_underflowed = remaining_frag_samples;

			if (ch_sample->flags & CHANNEL_FLAG_REPEAT_STREAM_ON_UNDERFLOW) {
			//NEW: if (ch->play_ct != 1) {
				sample_length = remaining_frag_samples; // XXX wrap around etc..
				ch_sample->position -= (((remaining_frag_samples *(_server_gsi_bytes_per_sample[ ss->type ] * ss->channels)) * samplerate_c) >> 8);
				if (ch_sample->position < 0) {
					ch_sample->position += ss->size;
				}
				/* now calc sample length again.. (for slot wrap around) */
				bufsize = min(ss->current_size, ss->size - ch_sample->position);
				sample_length = min(remaining_frag_samples, ( (bufsize/(_server_gsi_bytes_per_sample[ ss->type ] * ss->channels)) * samplerate_c2) >> 8);

			}
			else {
				/* silence.. skip remaining part of the fragment */
				sample_length = 0;
				remaining_frag_samples = 0;
				mix_index = server_pcm_status->samples;
			}
			send_byte(current, GSI_EV_STREAM_UNDERFLOW);
			send_byte(current, ch->slot);
			send_int32(current, ((bytes_underflowed*samplerate_c) >> 8) * _server_gsi_bytes_per_sample[ ss->type ] * ss->channels); // no. of bytes that were needed but not available.
		}
		else {
			int  bytes_done;

			if (sample_length == 0) {
				/* a small remainder; happens when the sound's samplerate is
				 * greater than the pcm's rate (e.g. 22 kHz file -> 8 kHz pcm).
				 */
				ch_sample->position += (((bufsize * (_server_gsi_bytes_per_sample[ ss->type ] * ss->channels)) * samplerate_c) >> 8);
				bytes_done = bufsize;

				DEB2(DPRT("Remainder! bufsize=%d (ch ptr =%d)\n", bufsize, ch_sample->position));
				DEB2(DPRT("   ,,      slot ch=%d, type=%d (%d bytes), rate=%d\n", ss->channels, ss->type, _server_gsi_bytes_per_sample[ ss->type ], ss->samplerate));
			}
			else {
				bytes_done = ((sample_length*samplerate_c)>>8) * _server_gsi_bytes_per_sample[ ss->type ] * ss->channels;
			}

			send_byte(current, GSI_EV_STREAM_UPDATE);
			send_byte(current, ch->slot);
			send_int32(current, bytes_done);

			ss->current_size -= bytes_done;
		}
	}
	else {
		/* No stream. */
		bufsize = ss->size;

		/* 23-4-2000: added (int64) cast to prevent overflow on very big files
		 * loaded entirely in memory: */
		sample_length = min (
			remaining_frag_samples /*server_pcm_status->samples*/,
			((( (int64_t) ss->size - ch_sample->position)*samplerate_c2)>>8) / (ss->channels * _server_gsi_bytes_per_sample[ ss->type ])
			);
		DEB2(DPRT("XXX remaining_frag_samples = %d, s->size=%d\n", remaining_frag_samples, ss->size));
	}

	sample_position = ss->buffer + ch_sample->position;

	DEB3(DPRT("(2d/3d) ch %d (slot %d), samplerate_c=%d, samplerate_c2=%d\n", i, c->channel[i]->slot, samplerate_c, samplerate_c2);
		 DPRT("sample_position = %d\n", ch_sample->position);
		 DPRT("stereo_pos=%f, stereo_speed=%d, observer_x=%f\n", ch_sample->x, ch_sample->dx, c->observer_x);
		 DPRT("vol L,R=%d,%d\n", sound_volume_L, sound_volume_R);
		);
	//SPEED_PRINT_TIME("divs");


	/* Write samples to the internal mixing buffer:
	 * --------------------------------------------
	 * The internal mixing buffer is 24 bit + 8 bit for overflow (a 32 bit int)
	 * We use a multiplication factor fake_volume = vol<<8 for 8 bit, vol for
	 * 16 bit, vol/8 for 24 bit (i.e. first multiply with vol, then shift right
	 * 8 times) which gives 24 bit data and 8 bits for overflow detection (i.e.
	 * the maximum possible overflow for 256 channels).
	 * Then shift right the internal mixing buffer values 8 or 16 times for
	 * the final mixing buffer (i.e. what will be written to the pcm device).
	 */

	/* The following only works with 1 or 2 channels of the sound and the pcm
	 * device: */
	sound_to_mixbuffer = ((ss->type) << 2) | ((ss->channels-1) << 1) | (server_pcm_status->channels-1);

	/*         sound type | stereo/mono sound | stereo/mono pcm */
#   define S24_M  (((GSI_24BIT      <<2) | (1<<1)) | 0)
#   define S16_M  (((GSI_16BIT      <<2) | (1<<1)) | 0)
#   define  S8_M  (((GSI_8BIT       <<2) | (1<<1)) | 0)
#   define Smu_M  (((GSI_8BIT_MULAW <<2) | (1<<1)) | 0)
#   define M24_M  (((GSI_24BIT      <<2) | (0<<1)) | 0)
#   define M16_M  (((GSI_16BIT      <<2) | (0<<1)) | 0)
#   define  M8_M  (((GSI_8BIT       <<2) | (0<<1)) | 0)
#   define Mmu_M  (((GSI_8BIT_MULAW <<2) | (0<<1)) | 0)
#   define S24_S  (((GSI_24BIT      <<2) | (1<<1)) | 1)
#   define S16_S  (((GSI_16BIT      <<2) | (1<<1)) | 1)
#   define  S8_S  (((GSI_8BIT       <<2) | (1<<1)) | 1)
#   define Smu_S  (((GSI_8BIT_MULAW <<2) | (1<<1)) | 1)
#   define M24_S  (((GSI_24BIT      <<2) | (0<<1)) | 1)
#   define M16_S  (((GSI_16BIT      <<2) | (0<<1)) | 1)
#   define  M8_S  (((GSI_8BIT       <<2) | (0<<1)) | 1)
#   define Mmu_S  (((GSI_8BIT_MULAW <<2) | (0<<1)) | 1)
	/* Mmu_S = mono mulaw sound to stereo buffer etc. */

	//SPEED_PRINT_TIME("");
	DEB3(DPRT("calculating (part of) fragment for channel %d (slot %d), sample length=%d (mix_index=%d)\n", i, c->channel[i]->slot, sample_length, mix_index));

	my_premix = &pcm_premix[ mix_index ];

	switch(sound_to_mixbuffer) {
	case S16_M:  // stereo 16 bit sound to mono 24 bit buffer
		DEB3(DPRT("conversion: S16_M\n"));
		for (j=0; j<sample_length; j++) {
			int16  b1, b2;
			int    position_rest = (((samplerate_c*j)>>6) & ~3);

			b1  = ((int) *(sample_position+position_rest+0))<<8;
			b1 |= (byte) *(sample_position+position_rest+1);
			b2  = ((int) *(sample_position+position_rest+2))<<8;
			b2 |= (byte) *(sample_position+position_rest+3);

			my_premix[j] += ((b1+b2)*sound_volume_L)>>(1+0/*+8*/);
		}
		ch_sample->position += ((sample_length*samplerate_c)>>6) & ~3;
		break;

	case Smu_M:  // stereo 8 bit mulaw sound to mono 24 bit buffer
		DEB3(DPRT("conversion: Smu_M\n"));

		for (j=0; j<sample_length; j++) {
			int16  b1, b2;
			int    position_rest = (((samplerate_c*j)>>7)&~1);

			b1  = mulaw_to_linear16_inline( (byte) *(sample_position+position_rest+0) );
			b2  = mulaw_to_linear16_inline( (byte) *(sample_position+position_rest+1) );

			my_premix[j] += (((int) b1+ (int) b2)*sound_volume_L)>> (1/*+8*/);
		}
		ch_sample->position += ((sample_length*samplerate_c)>>7) & ~1;
		break;

	case S8_M:   // stereo 8 bit sound to mono 24 bit buffer
		DEB3(DPRT("conversion: S8_M\n"));

		for (j=0; j<sample_length; j++) {
			int16  b1, b2;
			int    position_rest = (((samplerate_c*j)>>7)&~1);

			b1  = (int16) *(sample_position+position_rest+0);
			b2  = (int16) *(sample_position+position_rest+1);

			//my_premix[j] += (((int) b1+ (int) b2)*sound_volume_L) >>1;
			my_premix[j] += (((int) b1+ (int) b2)*sound_volume_L) << (8-1);
		}
		ch_sample->position += ((sample_length*samplerate_c)>>7) & ~1;
		break;

	case M16_M:  // mono 16 bit sound to mono 24 bit buffer
		DEB3(DPRT("conversion: M16_M\n"));

		for (j=0; j<sample_length; j++) {
			int16  b1;
			int    position_rest = (((samplerate_c*j)>>7) & ~1);

			b1  = ((int) (*(sample_position+position_rest+0))) <<8;
			b1 |=  (byte) *(sample_position+position_rest+1);

			my_premix[j] += (((int) b1)*sound_volume_L) >> (0/*+8*/);
		}
		ch_sample->position += ((sample_length*samplerate_c)>>7) & ~1;
		break;

	case Mmu_M:  // mono 8 bit mulaw sound to mono 24 bit buffer
		DEB3(DPRT("conversion: Mmu_M\n"));

		if (samplerate_c == 256) { //no doppler or samplerate conversion, faster code
			for (j=0; j<sample_length; j++) {
				int  position_rest = j;

				my_premix[j] += ( ((int) mulaw_to_linear16_inline( (int) (byte) (*(sample_position+position_rest)) )) * sound_volume_L) >> (0/*+8*/);
			}
		}
		else {
			for (j=0; j<sample_length; j++) {
				int  position_rest = ((samplerate_c*j)>>8);

				my_premix[j] += ( ((int) mulaw_to_linear16_inline( (int) (byte) (*(sample_position+position_rest)) )) * sound_volume_L) >> (0/*+8*/);
			}
		}
		ch_sample->position += ((sample_length*samplerate_c) >> 8);
		break;

	case M8_M:   // mono 8 bit sound to mono 24 bit buffer
		DEB3(DPRT("conversion: M8_M\n"));

		if (samplerate_c == 256) {
			// no doppler or samplerate conversion, faster code
#if 1
			// OLD: 42-44 s (k6-233)
			//SPEED_INIT_TIME();
			for (j=0; j<sample_length; j++) {
				my_premix[j] += ((*(sample_position+(j))) * sound_volume_L ) << 8;
			}
#else
			// NEW: 29-31 s (k6 233)
			int16 *my_volume = volume_table[ sound_volume_L >> 8 ];
			char  *sample_position2 = sample_position+sample_length-1;
			int   *my_premix2 = my_premix+sample_length-1;

			for (j=sample_length-1; j>=0; j--) {
//				*my_premix2-- += (*sample_position2) * sound_volume_L;
//				*my_premix2-- += my_volume[ *sample_position2 ];
				my_premix[j] +=  my_volume[ *sample_position2-- ] << 8;

//				sample_position2 --;
//				my_premix2 --;
			}
#endif
		}
		else {
#if 0 /* New v1 */
			// 125 s (no improvement ..)
			int  position_rest = 0;
			int  position_delta = samplerate_c<<8;

			for (j=0; j<sample_length; j++) {
				//my_premix[j] += (*sample_position) * sound_volume_L;
				my_premix[j] += ((*sample_position) * sound_volume_L) << 8;
				position_rest += position_delta;
				sample_position += (position_rest >>16);
				position_rest &= 65535;
			}
#endif
#if 0 /* New v2, assumes pcm_samples * position_delta < MAX_INT */
			// 125 s, also no improvement over standard mul+shift.
			int  position_rest = 0;
			int  position_delta = samplerate_c<<8;

			for (j=0; j<sample_length; j++) {
				my_premix[j] += (*(sample_position+(position_rest>>16))) * sound_volume_L;
				position_rest += position_delta;

//				if ((j&511) == 0) {
//					sample_position3 += position_rest >>16;
//					position_rest &= 65535;
//				}
			}
			position_rest &= 65535;
			// 192k / 4k = 48 * samplerate factor
			// then also max 8* for doppler and pitch shift =>384 (9 bits)
			// say 10 bits for fraction => 19 bits, 32-19=13 bits for
			// sample length maximum (i.e. 8192 samples = 0.04s at 192k
			// or reduce the sample_length by going through the loop multiple times..
#endif
#if 1
			// 125 s
			for (j=0; j<sample_length; j++) {
//				my_premix[j] += (*(sample_position+((samplerate_c*j + position_rest)>>(0+8)))) * sound_volume_L;
				my_premix[j] += (*(sample_position+((samplerate_c*j)>>(0+8)))) * sound_volume_L;
			}
#endif
		}
		ch_sample->position += ((sample_length*samplerate_c)>>8);
		break;

	case S16_S:  // stereo 16 bit sound to stereo 24 bit buffer
		DEB3(DPRT("conversion: S16_S\n"));

		for (j=0; j<sample_length*2; j+=2) {
			int  b1, b2;
			int  position_rest = (((samplerate_c*j)>>7) & ~3);

			b1  = ((int) *(sample_position+position_rest+0))<<8;
			b1 |= (byte) *(sample_position+position_rest+1);
			b2  = ((int) *(sample_position+position_rest+2))<<8;
			b2 |= (byte) *(sample_position+position_rest+3);

			my_premix[j]   += (b1*sound_volume_L)>>(0/*+8*/);
			my_premix[j+1] += (b2*sound_volume_R)>>(0/*+8*/);
		}
		ch_sample->position += ((sample_length*samplerate_c)>>6) & ~3;
		break;

	case Smu_S:  // stereo 8 bit mulaw sound to stereo 24 bit buffer
		DEB3(DPRT("conversion: Smu_S\n"));

		//if (f == 256) { // faster code
		//else {
		for (j=0; j<sample_length*2; j+=2) {
			int16  b1, b2;
			int  position_rest = (((samplerate_c*j)>>8) & ~1);

			b1  = mulaw_to_linear16_inline( (int) (byte) *(sample_position+position_rest+0) );
			b2  = mulaw_to_linear16_inline( (int) (byte) *(sample_position+position_rest+1) );

			my_premix[j]  += (b1*sound_volume_L)>>(0/*+8*/);
			my_premix[j+1]+= (b2*sound_volume_R)>>(0/*+8*/);
		}
		ch_sample->position += ((sample_length*samplerate_c)>>7) & ~1;
		break;

	case S8_S:   // stereo 8 bit sound to stereo 24 bit buffer
		DEB3(DPRT("conversion: S8_S\n"));

		//if (f == 256) { // faster code
		for (j=0; j<sample_length*2; j+=2) {
			int16 b1, b2;
			int   position_rest = (((samplerate_c*j)>>8)&~1);

			b1  = (int16) *(sample_position+position_rest+0);
			b2  = (int16) *(sample_position+position_rest+1);

			my_premix[j]  += (b1*sound_volume_L) << 8;
			my_premix[j+1]+= (b2*sound_volume_R) << 8;
		}
		ch_sample->position += ((sample_length*samplerate_c)>>7) & ~1;
		break;

	case M16_S:  // mono 16 bit sound to stereo 24 bit buffer
		DEB3(DPRT("conversion: M16_S\n"));

		if (samplerate_c == 256) {
			for (j=0; j<sample_length*2; j+=2) {
				int16 b1;
				int   position_rest = ((j) & ~1);

				b1  = ( (int) (*(sample_position+position_rest+0)) ) <<8;
				b1 |=   (byte) *(sample_position+position_rest+1);

				my_premix[j]   += (((int) b1) *sound_volume_L)>>(0/*+8*/);
				my_premix[j+1] += (((int) b1) *sound_volume_R)>>(0/*+8*/);
			}
		}
		else {
			for (j=0; j<sample_length*2; j+=2) {
				int16 b1;
				int   position_rest = (((samplerate_c*j)>>8) & ~1);

				b1  = ((int16) (*(sample_position+position_rest+0))) <<8;
				b1 |=  (byte)   *(sample_position+position_rest+1);

				my_premix[j]   += (((int) b1) *sound_volume_L)>>(0/*+8*/);
				my_premix[j+1] += (((int) b1) *sound_volume_R)>>(0/*+8*/);
			}
		}
		ch_sample->position += ((sample_length*samplerate_c)>>7) & ~1;
		break;

	case Mmu_S:  // mono 8 bit mulaw sound to stereo 24 bit buffer
		DEB3(DPRT("conversion: Mmu_S\n"));

		if (samplerate_c == 256) {
			for (j=0; j<sample_length*2; j+=2) {
				int16 b1;
				int   position_rest = (j>>1);

				b1  = mulaw_to_linear16_inline( (int) (byte) (*(sample_position+position_rest )) );

				my_premix[j]   += (((int) b1) * sound_volume_L)>>(0/*+8*/);
				my_premix[j+1] += (((int) b1) * sound_volume_R)>>(0/*+8*/);
			}
		}
		else {
			for (j=0; j<sample_length*2; j+=2) {
				int16 b1;
				int   position_rest = (((samplerate_c*j)>>9) );

				b1  = mulaw_to_linear16_inline( (int) (byte) (*(sample_position+position_rest)) );

				my_premix[j]   += (((int) b1) * sound_volume_L)>>(0/*+8*/);
				my_premix[j+1] += (((int) b1) * sound_volume_R)>>(0/*+8*/);
			}
		}
		ch_sample->position += ((sample_length*samplerate_c)>>8);
		break;

	case M8_S:   // mono 8 bit sound to stereo 24 bit buffer
		DEB3(DPRT("conversion: M8_S\n"));

		if (samplerate_c == 256) {
			for (j=0; j<sample_length*2; j+=2) {
				int16 b1;
				int   position_rest = (j>>1);

				b1  = ((int) (*(sample_position+position_rest )));

				//my_premix[j]   += ((int) b1) * sound_volume_L;
				//my_premix[j+1] += ((int) b1) * sound_volume_R;
				my_premix[j]   += (((int) b1) * sound_volume_L) << 8;
				my_premix[j+1] += (((int) b1) * sound_volume_R) << 8;
			}
		}
		else {
			for (j=0; j<sample_length*2; j+=2) {
				int16 b1;
				int   position_rest = (((samplerate_c*j)>>9) );

				b1  = (int) (*(sample_position+position_rest));

				//my_premix[j]   += ((int) b1) * sound_volume_L;
				//my_premix[j+1] += ((int) b1) * sound_volume_R;
				my_premix[j]   += (((int) b1) * sound_volume_L) << 8;
				my_premix[j+1] += (((int) b1) * sound_volume_R) << 8;
			}
		}
		ch_sample->position += ((sample_length*samplerate_c)>>8);
		break;
	default:
	{
		log_message(0, "pcm: unhandled sound->device conversion %d", sound_to_mixbuffer);
	}
		/* should handle this properly by killing the channel */
		goto kill_channel;
		return;
	}

	remaining_frag_samples -= sample_length;
	mix_index += sample_length;

	if (mix_index < server_pcm_status->samples) {
		//OLD if (ch_sample->flags & CHANNEL_FLAG_STREAM) {
		if (ss->input == _GSI_INPUT_STREAM) {
			/* wrap around or underflow */

			goto calculate_fragment;
		}
//		else if (ch->flags & CHANNEL_FLAG_REPEAT) {
		else if (ch->play_ct != 1) {
			DEB2(DPRT("-> finished playing channel %d (slot %d). Repeating it!\n", i, c->channel[i]->slot));
			ch_sample->position = 0;
			sample_position = ss->buffer;
			if (ch->play_ct > 0) ch->play_ct--;
			goto calculate_fragment;
		}
		else {
			DEB2(DPRT("-> finished playing channel %d (slot %d)\n", i, c->channel[i]->slot));
			DEB2(DPRT("-> it was at position %d\n", ch_sample->position));

		  kill_channel:
			/* remove the channel from the play list. */
			ch->play_ct = 0;
			if (c->channel_used[k] == c->last_channel)
				c->last_channel = -1;
			if (k < c->channel_playnum-1) {
				memmove(c->channel_used+k, c->channel_used+k+1, (c->channel_playnum-k-1)*sizeof(int));
			}

			c->channel_playnum--;
			total_playnum--;

			/* These two are used as flags to check if a sound is playing on
			 * the channel, should use e.g.: status_flag[MAX_CHANNELS]
			 */
			ch->slot = -1;

//			if (c->event_mask & GSI_EVMASK_CHANNEL) {
				send_byte(current, GSI_EV_CHANNEL_END);
				send_byte(current, i);
				DEB2(DPRT("-> GSI_EV_CHANNEL_END (%d) in queue\n", i));
//			}
		}
	}
}


/*      EOF play_fragment.c */

