/* GSI - general sound interface
 *
 * slot.h
 *
 * by W.H.Scholten 1998 - 2001
 *
 * This file is released to the public domain.
 */


#ifndef _GSI__server__play__slot_h
#define _GSI__server__play__slot_h

/* This struct contains the first 3 elements of the gsis_slot_sample struct: */
struct gsi_sample {
	int     samplerate;
	byte    channels;
	byte    type;
};

struct gsis_stream {
	int     samplerate; /* same as the slot. REMOVE? */
	byte    channels;   /* same as the slot. REMOVE? */
	byte    type;       /* Need NOT be the same as the slot. e.g. GSI_8BIT will be transformed into signed 8 bit internally (but still named GSI_8BIT) */
	int     start_ptr;  /* in channel struct (position) so remove? */
	int     write_ptr;
};
struct gsis_shm {
	int     samplerate; /* same as the slot. REMOVE? */
	byte    channels;   /* same as the slot. REMOVE? */
	byte    type;       /* Need NOT be the same as the slot. e.g. GSI_8BIT will be transformed into signed 8 bit internally (but still named GSI_8BIT) */
	int     start_ptr;  /* in channel struct (position) so remove? */
	int     write_ptr;
	byte   *buffer;
};


enum { _GSI_INPUT_NONE, _GSI_INPUT_FILE, _GSI_INPUT_STREAM, _GSI_INPUT_SHM } gsis_input_t;
#define _GSI_DATA _GSI_STREAM /* directly loaded data handled same as stream */

enum { _GSI_SOUNDTYPE_NONE, _GSI_SOUNDTYPE_SAMPLE, _GSI_SOUNDTYPE_SONG } gsis_soundtype_t;


/* -------------------------------------------------------------------------
 *     SLOT structure:
 * ------------------------------------------------------------------------- */
typedef struct gsis_slot_sample {
	/* The data converted/obtained from input: */
	int     samplerate;   /* rename to sample_rate ? */
	byte    channels;
	byte    type;
	int     size;
	/*      the allocated size of 'buffer' (bytes => SAMPLES ?) */

	int8   *buffer;
	/*      the actual sound data (a single block) */

	int     meta_sample_size;
	/*      = channels * size_of(type) */

	int     meta_samples;
	/*      = (int) (size / meta_sample_size) */

	/* better: */
	int     input;        /* either _GSI_INPUT_FILE, _GSI_INPUT_STREAM or _GSI_INPUT_SHM */

	union {
		struct gsi_sound   *file;
		struct gsis_stream *stream;
		struct gsis_shm    *shm;
	} data;

	/* The copy/conversion status (from input to the slot itself). */
	int     load_block;    /* the next byte comes from this block */
	int     load_ptr;      /* the next byte can be gotten (possibly with samplerate conversion..) from block 'load_block' at position 'load_ptr'. */

	/* loading status: */
	int     current_size;  /* the current size (bytes loaded) */
	/* with streams, current size is actually the number of the next byte to
	 * be written (modulo size, so if current size=size, then the next byte to
	 * write is at position 0), and the channel using the stream holds the
	 * start of the data (i.e. the first byte that has not been played
	 * (transferred to the mixing buffer) yet). */

//	int  flags; /* e.g. SLOT_FLAG_AUTO_FREE ?? */
} gsis_slot_sample_t;


typedef struct gsis_slot_midi {
	struct gsi_song *file;
} gsis_slot_midi_t;


#define _GSI_SLOTTYPE__UNDEFINED  0
#define _GSI_SLOTTYPE__PCM_SAMPLE 1
#define _GSI_SLOTTYPE__MIDI_SONG  2

typedef struct gsis_slot {
	int  type;
	union {
		gsis_slot_sample_t *sample;
		gsis_slot_midi_t   *midi;
	} data;
} gsis_slot_t;

#endif /* _GSI__server__play__slot_h */
