/* GSI - general sound interface
 *
 * song_play.c - currently only MIDI type playing (i.e. also hmp).
 *
 * Lower level API dependencies:
 *   - unix gettimeofday
 *
 *
 * Copyright (C) W.H.Scholten 1998 - 2000
 *
 * Redistribution and use in source and binary forms, with or
 * without modification, are permitted provided that the following
 * conditions are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS
 * IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
 * AUTHOR OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


/*__________________  INCLUDES  __________________*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>


#include <gsi/whs_defines.h>
#include <gsi/whs_types.h>

#include <gsi/config.h>
#define DEBUG_PREFIX "GSIS_MIDI_"
#include <gsi/gsi_debug.h>

#include <gsi/gsi_events.h>
#include <gsi/gsi_eventmasks.h>
#include <gsi/gsif_midi.h>

#include "song_play.h"
#include "../../gsi_server.h"
#include "../../client.h"
#include "../../io.h"


/*__________________ DEFINITIES __________________*/


/*---------------------------------------------------------------------------*
 *      Timing.
 *---------------------------------------------------------------------------*/
void song_init_time(int client, int ch, double offset)
{
	struct gsi_client *mclient;
	void *spdata;
	struct gsis_channel_info *this_channel;

	mclient = client_list[ client ];

	this_channel = mclient->channel[ ch ];
	if (this_channel->type != _GSI_SOUNDTYPE_SONG) {
//		return -1;
		return;
	}

	spdata = mclient->channel[ ch ]->song.playstatus;

	if (spdata == NULL) return;
	mclient->channel[ ch ]->song.song_init(spdata);
//	return 0;
}


double song_get_time(int client, int ch)
{
	struct gsi_client *mclient;
	struct gsis_channel_info *this_channel;
	int  status;
	void *spdata;
	double  dt; /* time into the song */

	mclient = client_list[ client ];

	this_channel = mclient->channel[ ch ];
	if (this_channel->type != _GSI_SOUNDTYPE_SONG) {
		return -1;
	}
	spdata = mclient->channel[ ch ]->song.playstatus;

	status = mclient->channel[ ch ]->song.song_get_time(spdata, &dt);

	return dt;
}


/*---------------------------------------------------------------------------*
 *      Initialize for midi (& hmp) playing
 *---------------------------------------------------------------------------*/
int song_init(int client, int ch)
{
	struct gsi_client *mclient = client_list[ client ];
	struct gsis_channel_info *this_channel;
	void *spdata;

	this_channel = mclient->channel[ ch ];
	this_channel->type = _GSI_SOUNDTYPE_SONG;

	spdata = mclient->channel[ ch ]->song.playstatus;

	mclient->channel[ ch ]->song.song_init(spdata);

	return 0;
}


/*---------------------------------------------------------------------------*
 *      Play the midi events on internal sythesizers or external midi.
 *      Main loop.
 *---------------------------------------------------------------------------*/
int song_play_events(int client, int ch, int max_dt)
{
	struct gsi_client *mclient = client_list[ client ];
	void *spdata;

	spdata = mclient->channel[ ch ]->song.playstatus;

	mclient->channel[ ch ]->song.song_play(spdata, max_dt/100.0);

	if (song_finished(client, ch)) {

		/* We're done. */
		DEB1(DPRT("MIDI   done playing\n"));
		if (mclient->channel[ ch ]->play_ct > 0) {
			mclient->channel[ ch ]->play_ct--;
			if (mclient->channel[ ch ]->play_ct == 0) {
				mclient->channel[ ch ]->song.flags = _SONG_NONE;
//OBS			if (mclient->event_mask & GSI_EVMASK_SONG) {
//OBS				send_byte(mclient, GSI_EV_SONG_END);
//OBS				DEB1(DPRT("MIDI   -> event in queue\n"));
//OBS			}
			}
			else {
				/* play it again */
				//spdata->song_events_done = False;
//OBS				if (mclient->event_mask & GSI_EVMASK_SONG) {
//OBS					send_byte(mclient, GSI_EV_SONG_STARTED);
//OBS				}
			}
		}
		else if (mclient->channel[ ch ]->play_ct < 0) {
			/* Infinite loop. */
			mclient->channel[ ch ]->song.flags = _SONG_PLAYING;
			//spdata->song_events_done = False;
		}

		if (mclient->channel[ ch ]->song.flags == _SONG_PLAYING) {
			song_init(client, ch);
		}
	}
	DEB2(DPRT("MIDI #play_midi_events#\n"));

	return 1;
}


/* returns True is finished: */
int song_finished (int client, int ch) {
	struct gsi_client *mclient = client_list[ client ];
	void  *spdata;
	double time;

	spdata = mclient->channel[ ch ]->song.playstatus;

	mclient->channel[ ch ]->song.song_get_time(spdata, &time);

	return (time >= mclient->channel[ ch ]->song.duration);
}


/*      EOF song_play.c */
