/* GSI - general sound interface
 *
 * real_time.c - shared sound server, setting RT priorities
 *
 * Lower level API dependencies:
 *   - POSIX RT
 *
 *
 * Copyright (C) 2000 W.H.Scholten
 *
 * Redistribution and use in source and binary forms, with or
 * without modification, are permitted provided that the following
 * conditions are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS
 * IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
 * AUTHOR OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#include <stdio.h>
#include <string.h>
#include <errno.h>

#include <sys/types.h>

#include <gsi/config.h>
#define DEBUG_PREFIX "GSI_SERVER (RT)"
//#include <gsi/gsi_debug.h>

#ifdef USE_POSIX_RT
#  include <sched.h>
#  include "real_time.h"
#  include "vsched.h"
#endif

#include "../log.h"
#include "real_time.h"


/*---------------------------------------------------------------------------*
 *      Set RT priority
 *---------------------------------------------------------------------------*/
void init_real_time() {
#ifdef USE_POSIX_RT
	int  status;
	struct sched_param p;

	p.sched_priority = 1;
	status = vsched_setscheduler(0, SCHED_FIFO, &p);
	if (status < 0) {
		log_message (1, "POSIX real time init: failed to set priority FIFO");
	}
#endif
}


void end_real_time() {
	/* Note: this doesn't work properly (i.e. as expected ) in general (i.e. if
	 * all prio's are RT (RT OS)). See the RT book on how to do this properly.
	 */
#ifdef USE_POSIX_RT
	int  status;

	struct sched_param p;

	p.sched_priority = 0;
	status = vsched_setscheduler(0, SCHED_OTHER, &p);
	if (status < 0) {
		log_message (1, "POSIX real time end: failed to set priority OTHER");
	}
#endif
}


/*      EOF real_time.c */
