/* GSI - general sound interface
 *
 * tables.c - shared sound server, tables
 *
 * Lower level API dependencies: none
 *
 *
 * Copyright (C) 1998 - 2001 W.H.Scholten
 *
 * Redistribution and use in source and binary forms, with or
 * without modification, are permitted provided that the following
 * conditions are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS
 * IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
 * AUTHOR OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


/*__________________INCLUDES___________________*/
#include <gsi/config.h>

#include <math.h>

#define DEBUG_PREFIX "GSIS_"
//#include <gsi/gsi_debug.h>

#include <gsi/whs_types.h>
#include <gsi/whs_defines.h>

#include "tables.h"


int16 volume_table [ 257 ][ 256 ];
int  sinus_table [ 1024 ];  /* for wave playing and observer rotation. */


/*---------------------------------------------------------------------------*
 *      Make a integer sinus table, values are 65536*sinus(2 pi * i/1024)    *
 */
void make_sinus() {
	int  i;

	for (i=0; i<1024; i++) {
		sinus_table[i] = (int) (sin((M_PI*2.0*i)/1024.0) *65536.0 +.5);
	}
}


/*---------------------------------------------------------------------------*
 *      Make a volume multiply table, index=vol (0-256), x. Gives slightly   *
 *      faster volume calculations than MUL.                                 *
 */
void make_volume_table() {
	int  vol, i;

	for (vol=0; vol<257; vol++) {
		for (i=0; i<256; i++) {
			volume_table[vol][i] = vol*(i^128); /* ^128 for signed 8 bit internal samples */
		}
	}
}


/*---------------------------------------------------------------------------*
 *      Bytes used per sample:
 *---------------------------------------------------------------------------*/
int _server_gsi_bytes_per_sample [ GSI_NUM_SAMPLE_TYPES ] = {
	1, 1, 1, 2, 3, 1, 2, 3, -4, -3, -5 /* negative = bits, not bytes. */
};
/* This table is also available in gsif (sample_format.c: _gsif_Bps[]) and
 * gsipcm.c. */

int _server_gsi_types[] = {
	GSI_8BIT_ALAW, GSI_8BIT_MULAW, GSI_8BIT, GSI_16BIT, GSI_24BIT, 
	GSI_8BIT_SIGNED, GSI_16BIT_LE, GSI_24BIT_LE, //add ADPCM
};
char *_server_gsi_type_strings[] = {
	"8BIT_ALAW", "8BIT_MULAW", "8BIT", "16BIT", "8BIT_24BIT", 
	"8BIT_SIGNED", "16BIT_LE", "24BIT_LE", //add ADPCM
};


/*      EOF tables.c */
