/* GSI - general sound interface
 *
 * event: tests libgsicd's event generation
 *
 * By W.H.Scholten 1999 - 2001
 *
 * This software is placed in the public domain and can be used freely
 * for any purpose. It comes without any kind of warranty, either
 * expressed or implied, including, but not limited to the implied
 * warranties of merchantability or fitness for a particular purpose.
 * Use it at your own risk. the author is not responsible for any damage
 * or consequences raised by use or inability to use this program.
 */


/*
 *  export APP_DEBUG_LEVEL= n for debug output.
 *
 */


/*__________________INCLUDES___________________*/
#include <stdio.h>
#include <stdlib.h>
//#include <string.h>
#include <unistd.h>

#include <gsi/gsicd.h>
#include <gsi/p_signal.c> /* needed for libgsicd */

/* These two includes are really for internal GSI use .. */
#include <gsi/whs_types.h>
#define DEBUG_PREFIX "APP_"
//#include <gsi/gsi_debug.h>



/*__________________PROGRAM___________________*/
#include "alloc_device.inc.c"
#include "status.inc.c"

volatile sig_atomic_t cd_track_alarm = False;


void cd_lowlevel_event ()
{
	cd_track_alarm = True;
	printf("cd_lowlevel_event\n");
	/* main loop is now interrupted if in select..
	 * then that will call gsicd_generate_events(); which calls cd_event(..) */
}


void cd_event (int event, int cd, int track)
{
	/* send an event to the relevant client */
	char *client_cd_event [ 2 ] = { "TRACK_STARTED", "TRACK_END" };

	if (event < 0 || event > 1) {
		fprintf(stderr, "bad cd event %d\n", event);
		return;
	}
	printf("cd_event: callback %s for CD %d track %d\n", client_cd_event[ event ], cd, track);
}


int main(int argc, char *argv[]) {
	int  i;
	int  lba, m,s,f;
	int  status;
	int  cd; /* cd no we're using */

	printf("%s\n", argv[0]);
	printf("Checking utility functions\n");
	if (gsicd_lba_from_msf(20,40,74) != ((20*60 + 40)*75 +74) ) {
		fprintf(stderr, "%s failure\n", "gsicd_lba_from_msf");
		exit(1);
	}
	if (gsicd_m_from_lba(123456) != 27 ) {
		fprintf(stderr, "%s failure\n", "gsicd_m_from_lba");
		exit(1);
	}
	if (gsicd_s_from_lba(123456) != 26 ) {
		fprintf(stderr, "%s failure\n", "gsicd_s_from_lba");
		exit(1);
	}
	if (gsicd_f_from_lba(123456) != 6 ) {
		fprintf(stderr, "%s failure\n", "gsicd_f_from_lba");
		exit(1);
	}
	printf("Ok\n\n");


	/* allocate space for device & status, and set it up. */
	alloc_cd_device();

	/* setup event handlers */
	gsicd_set_lowlevel_event_function( &cd_lowlevel_event );
	gsicd_set_event_function( &cd_event );

	cd = 0;
#define MYCD cd

	status = gsicd_init( MYCD );
	if (status != 0) {
		fprintf(stderr, "gsicd_init failed\n");
		fprintf(stderr, "cd status = %d (%s)\n", gsicd_status[ MYCD ]->status, cd_status_string[ gsicd_status[ MYCD ]->status ]);
		exit(1);
	}

	/* print track listing: */
	gsicd_update_status( MYCD );
	printf("Tracks:\n");
	for (i=0; i<=gsicd_status[ MYCD ]->num_tracks; i++) {
		printf("%2d type = %s, msf=%3d %02d %02d\n", i, (gsicd_status[ MYCD ]->track[i].type == GSICD_DATA ? "DATA " : "AUDIO"), gsicd_status[ MYCD ]->track[i].min, gsicd_status[ MYCD ]->track[i].sec, gsicd_status[ MYCD ]->track[i].frame);
	}


	/* 1st test:
	 * set to 1 s before end of first audio track, and start playing: */
	printf("Test 1 (play 2 - 1,0,0 to 3 + 0,0,0), usleep timeout\n");
printf("first_audio_track = %d\n", gsicd_status[ MYCD ]->first_audio_track);
	lba = gsicd_lba_track_len( MYCD , gsicd_status[ MYCD ]->first_audio_track) - 1*75;
printf("1: lba=%d\n", lba);

	m = gsicd_m_from_lba (lba);
	s = gsicd_s_from_lba (lba);
	f = gsicd_f_from_lba (lba);
	printf("1: playing %d, msf %d %2d %2d to %d, msf %d %2d %2d\n", 1, m,s,f, 3, 0,0,0);

	printf("1: lba trk 1 = %d, len = %d\n", gsicd_lba_track_pos( MYCD, 1), gsicd_lba_track_len( MYCD, 1));
	printf("1: lba trk 2 = %d, len = %d\n", gsicd_lba_track_pos( MYCD, 2), gsicd_lba_track_len( MYCD, 2));
//exit(1);
	gsicd_play( MYCD, 1, m,s,f, 3, 0,0,0);

	/* sleep 10 s: */
	printf("1: usleep 10s\n");
	usleep(10 * 1000000);
	printf("1: usleep done (should have been interrupted)\n");
	printf("1: cd_track_alarm = %d\n", cd_track_alarm);
	printf("1: cd position: msf=%3d %2d %2d\n", gsicd_status[ MYCD ]->current_m, gsicd_status[ MYCD ]->current_s, gsicd_status[ MYCD ]->current_f);
	printf("1: you should see 'TRACK_STARTED for CD 0 track 2' now:\n");
	if (cd_track_alarm) gsicd_generate_events();
	gsicd_stop( MYCD );
	gsicd_generate_events();




	/* 2nd test: */
	printf("Test 2 (play 2 - 1,0,0 to 2 + 0,0,0)\n");
	/* Now play 1 s before end of first audio track, to exactly 2 +0,0,0. */
	gsicd_play( MYCD, 1, m,s,f, 2, 0,0,0);
	printf("2: usleep 10s\n");
	usleep(10 * 1000000);
	printf("2: usleep done (should have been interrupted)\n");
	printf("2: cd_track_alarm = %d\n", cd_track_alarm);
	printf("2: cd position: msf=%3d %2d %2d\n", gsicd_status[ MYCD ]->current_m, gsicd_status[ MYCD ]->current_s, gsicd_status[ MYCD ]->current_f);
	printf("2: you should see 'TRACK_STARTED for CD 0 track 2' & 'TRACK_END for CD 0 track 2' now:\n");
	if (cd_track_alarm) gsicd_generate_events();
//	gsicd_stop( MYCD );
	gsicd_generate_events();
	usleep(5);
	printf("more events:\n");
	gsicd_generate_events();

	gsicd_stop( MYCD );
	gsicd_generate_events();

	return 0;
}

/* EOF event.c */
