/* GSI - general sound interface
 *
 * jockey: tests libgsicd's ejecting, inserting, changing.
 *
 * By W.H.Scholten 1999 - 2001
 *
 * This software is placed in the public domain and can be used freely
 * for any purpose. It comes without any kind of warranty, either
 * expressed or implied, including, but not limited to the implied
 * warranties of merchantability or fitness for a particular purpose.
 * Use it at your own risk. the author is not responsible for any damage
 * or consequences raised by use or inability to use this program.
 */


/*
 *  export APP_DEBUG_LEVEL= n for debug output.
 *
 */


/*__________________INCLUDES___________________*/
#include <stdio.h>
#include <stdlib.h>
//#include <string.h>

#include <gsi/gsicd.h>
#include <gsi/p_signal.c> /* needed for libgsicd */

/* These two includes are really for internal GSI use .. */
#include <gsi/whs_types.h>
#define DEBUG_PREFIX "APP_"
//#include <gsi/gsi_debug.h>



#define MYCD 0


/*__________________PROGRAM___________________*/
#include "alloc_device.inc.c"
#include "status.inc.c"

volatile sig_atomic_t cd_track_alarm = False;


void cd_lowlevel_event ()
{
	cd_track_alarm = True;
	printf("cd_lowlevel_event\n");
	/* main loop is now interrupted if in select..
	 * then that will call gsicd_generate_events(); which calls cd_event(..) */
}


void cd_event (int event, int cd, int track)
{
	/* send an event to the relevant client */
	char *client_cd_event [ 2 ] = { "TRACK_STARTED", "TRACK_END" };

	if (event < 0 || event > 1) {
		fprintf(stderr, "bad cd event %d\n", event);
		return;
	}
	printf("cd_event: callback %s for CD %d track %d\n", client_cd_event[ event ], cd, track);
}


int main(int argc, char *argv[]) {
	int  status;
	int  cd; /* cd no we're using */


	printf("%s\n", argv[0]);

	/* allocate space for device & status, and set it up. */
	alloc_cd_device();

	/* setup event handlers */
	gsicd_set_lowlevel_event_function( &cd_lowlevel_event );
	gsicd_set_event_function( &cd_event );

	printf("initializing cd\n");
	status = gsicd_init(MYCD);
	if (status != 0) {
		fprintf(stderr, "gsicd_init failed\n");
		fprintf(stderr, "cd status = %d (%s)\n", gsicd_status[ MYCD ]->status, cd_status_string[ gsicd_status[ MYCD ]->status ]);
	}

	printf("\nejecting cd\n");
	status = gsicd_eject(MYCD);
	printf("status = %d (%s)\n", status, cd_error_string[-status]);

	printf("\ninserting cd\n");
	status = gsicd_insert(MYCD);
	printf("status = %d (%s)\n", status, cd_error_string[-status]);

	printf("\nchanging to CD 1\n");
	status = gsicd_select_slot(MYCD, 1);
	printf("status = %d (%s)\n", status, cd_error_string[-status]);

	printf("\nchanging to CD 0\n");
	status = gsicd_select_slot(MYCD, 1);
	printf("status = %d (%s)\n", status, cd_error_string[-status]);

	printf("\nchanging to CD 5\n");
	status = gsicd_select_slot(MYCD, 1);
	printf("status = %d (%s)\n", status, cd_error_string[-status]);

	return 0;
}

/* EOF jockey.c */
