/* GSI - general sound interface
 *
 * gsimixertest (main.c): tests libgsimixer
 *
 * By W.H.Scholten 1999 - 2000
 *
 * This software is placed in the public domain and can be used freely
 * for any purpose. It comes without any kind of warranty, either
 * expressed or implied, including, but not limited to the implied
 * warranties of merchantability or fitness for a particular purpose.
 * Use it at your own risk. the author is not responsible for any damage
 * or consequences raised by use or inability to use this program.
 */


/*
 *  export APP_DEBUG_LEVEL= n for debug output.
 *
 *  use: gsimixertest <device>; default device is /dev/mixer
 */


/*__________________INCLUDES___________________*/
#include <stdio.h>
#include <stdlib.h>
//#include <string.h>

#include <gsi/gsimixer.h>

/* These two includes are really for internal GSI use .. */
#include <gsi/whs_types.h>
#define DEBUG_PREFIX "APP_"
//#include <gsi/gsi_debug.h>



/*__________________VARIABLES___________________*/
gsimixer_device_t mixer_hw;
gsimixer_devicedata_t mixer_hw_status;


/*__________________PROGRAM___________________*/
void use(char *prg_name) {
	fprintf(stderr, "Use: %s -h\n", prg_name);
	fprintf(stderr, "     %s [ mix device ]\n", prg_name);
	fprintf(stderr, "default mixer device is /dev/mixer\n");
	exit(1);
}


int main(int argc, char *argv[]) {
	char *dev_string[6] = {
		"GSI_MASTER",
		"GSI_PCM",
		"GSI_SYNTH",
		"GSI_CD",

		"GSI_LINE_IN",
		"GSI_MICROPHONE",
	};
	int  devtype, devno;
	int  status;

	if (argc > 1 && !strcmp(argv[1], "-h")) use(argv[0]);

	/* Mixer device, default is /dev/mixer, overridden by GSIMIXER_DEVICE and
	 * arg 1.
	 */
	mixer_hw.device = getenv("GSIMIXER_DEVICE");
	if (mixer_hw.device == NULL)
		mixer_hw.device = "/dev/mixer";

	if (argc > 1) mixer_hw.device = argv[1];


	gsimixer_device = mixer_hw.device;

	gsimixer_num_devices [ GSI_MASTER ] = 1;
	gsimixer_num_devices [ GSI_PCM ] = 1;
	gsimixer_num_devices [ GSI_SYNTH ] = 1;
	gsimixer_num_devices [ GSI_CD ] = 1;

	status = gsimixer_init();
	if (status != 0) {
		fprintf(stderr, "gsimixer_init failed\n");
		exit(1);
	}

	/* Print volumes: */
	printf("Current volumes\n");
	for (devtype = 0; devtype < GSIMIXER_NUM_DEVICE_TYPES; devtype ++) {
		printf("device type %d: (%s)\n", devtype, dev_string[ devtype ]);
		printf("device type %d: %d devices\n", devtype, gsimixer_num_devices [ devtype ]);
		for (devno = 0; devno < gsimixer_num_devices [ devtype ]; devno ++) {
			int  vol = gsimixer_get_device_volume(devtype, devno);
			printf("device type %d, no %d has volume %d\n", devtype, devno, vol);
		}
	}

	printf("Setting volume of device type %d (%s) no %d to %d\n", GSI_PCM, dev_string[ GSI_PCM ], 0, 77);
	status = gsimixer_set_device_volume(GSI_PCM, 0, 77);
	if (status != 0) {
		fprintf(stderr, "gsimixer_set_device_volume failed\n");
		exit(1);
	}

	printf("Reading volume of device type %d no %d gives %d\n", GSI_PCM, 0, gsimixer_get_device_volume(GSI_PCM, 0));

	return 0;
}

/* EOF gsimixertest (main.c) */
