/* GSI - general sound interface
 *
 * gsipcmtest (main.c): tests libgsipcm
 *
 * By W.H.Scholten 1999 - 2000
 *
 * This software is placed in the public domain and can be used freely
 * for any purpose. It comes without any kind of warranty, either
 * expressed or implied, including, but not limited to the implied
 * warranties of merchantability or fitness for a particular purpose.
 * Use it at your own risk. the author is not responsible for any damage
 * or consequences raised by use or inability to use this program.
 */


/*
 *  export APP_DEBUG_LEVEL= n for debug output.
 *
 */


/*__________________INCLUDES___________________*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <gsi/gsipcm.h>
#include <gsi/gsif.h>

/* These two includes are really for internal GSI use .. */
#include <gsi/whs_types.h>
#define DEBUG_PREFIX "APP_"
//#include <gsi/gsi_debug.h>


int play_file(char *filename);

#define min(x,y) ((x)<(y) ? (x) : (y))


/*__________________VARIABLES___________________*/
int  pcm_samplerate = 8000;
int  pcm_channels = 1;
int  pcm_type = GSI_8BIT;
int  pcm_log2_fragsize = 8;

gsipcm_device_t pcm_hw;
gsipcm_devicedata_t pcm_hw_status;

char *files[ 8 ] = {
	"../../../../gsi_data/sample/au/m_8_8k.au",
	"../../../../gsi_data/sample/au/m_16_8k.au",
	"../../../../gsi_data/sample/au/s_8_8k.au",
	"../../../../gsi_data/sample/au/s_16_8k.au",
	"../../../../gsi_data/sample/au/interleaved_16bit.au",
};


char *pcm_error_string[ 4 ] = {
	"GSIPCM_OK",          /*  0 */
	"GSIPCM_NOT_PRESENT", /* -1 */
	"GSIPCM_BUSY",        /* -2 */
	"GSIPCM_NOT_OPENED",  /* -3 */
};


/*__________________PROGRAM___________________*/
void use(char *prg_name) {
	fprintf(stderr, "Use: %s -h\n", prg_name);
	fprintf(stderr, "     %s [ filename ]\n", prg_name);
	exit(1);
}

int main(int argc, char *argv[]) {
	int  i;
	int  status;

	if (argc > 1 && !strcmp(argv[1], "-h")) use(argv[0]);

#ifdef __OpenBSD__
	pcm_hw.device = "/dev/sound";
#else
	pcm_hw.device = "/dev/dsp";
#endif

	status = gsipcm_init(&pcm_hw, &pcm_hw_status, GSIPCM_MODE_BLOCKING);
	if (status != 0) {
		fprintf(stderr, "gsipcm_init failed (device %s: error = %s)\n", pcm_hw.device, pcm_error_string[-status]);
		exit(1);
	}

	if (argc == 1) {
		for (i=0; i<4; i++) {
			play_file (files[i]);
			//gsipcm_close(&pcm_hw, &pcm_hw_status);
			printf("\n");
			sleep(1);
		}

		play_file (files[4]);
#if 0
		/* interleaved file test: */
		printf("Stereo byte order test\n");
		filename = files[4];
		snd = gsif_load_sample (filename, 0 /*offset*/, &status);
		if (snd == NULL) {
			fprintf(stderr, "Couldn't load %s (gsif: %s)\n", filename, gsif_strerror(status));
			exit(1);
		}
		gsipcm_write(&pcm_hw, &pcm_hw_status, snd->block[0]->data, snd->block[0]->data_length);
#endif
	}
	else {
		play_file (argv[1]);
	}

	return 0;
}


int play_file(char *filename) {
	char *buffer = malloc(16384);
	struct gsi_sound *snd;
	int  start, length;
	int  status;

	pcm_hw_status.fragment = buffer;

	snd = gsif_load_sample (filename, 0 /*offset*/, &status);
	if (snd == NULL) {
		fprintf(stderr, "Couldn't load %s (gsif: %s)\n", filename, gsif_strerror(status));
		exit(1);
	}

	/* pcm setting = as file */
	pcm_samplerate = snd->block[0]->samplerate;
	pcm_channels = snd->block[0]->channels;
	pcm_type = snd->block[0]->type;

	status = gsipcm_set_pcm(&pcm_hw, &pcm_hw_status, pcm_samplerate, pcm_channels, pcm_type, pcm_log2_fragsize);
	if (status != 0) {
		fprintf(stderr, "gsipcm_set_pcm failed (device %s: error = %s)\n", pcm_hw.device, pcm_error_string[-status]);
		exit(1);
	}

	/* Show what values were really set: */
	printf("PCM was set to:\n");
	printf(" samplerate:    %6d\n", pcm_hw_status.samplerate);
	printf(" channels:      %6d\n", pcm_hw_status.channels);
	printf(" type:          %6d\n", pcm_hw_status.type);
	printf(" log2_fragsize: %6d\n", pcm_hw_status.log2_fragsize);
	printf(" byte order: %s\n", pcm_hw_status.byte_order == GSI_BIG_ENDIAN ? "big endian" : "little endian");

	if (pcm_hw_status.channels != pcm_channels ||
		pcm_hw_status.type != pcm_type) {
		fprintf(stderr, "Couldn't set requested channel/type combination, skipping file\n");
		return -1;
	}
			

	/* Copy part of the sound data to the buffer: */
	length = min(16384, snd->block[0]->data_length);
	memcpy(pcm_hw_status.fragment, snd->block[0]->data, length);

	/* if sound data is 16 bit (gsif always returns MSB data) and the
	 * audio device is 16 bit LSB then swap bytes): */
	if (pcm_hw_status.type == GSI_16BIT && pcm_hw_status.byte_order == GSI_LITTLE_ENDIAN) {
		swab(pcm_hw_status.fragment, pcm_hw_status.fragment, length);
	}

	/* Now write the buffer to the pcm device: */
	start = 0;
	while (start < length) {
		status = gsipcm_write_fragment(&pcm_hw, &pcm_hw_status);
		printf("gsipcm_write_fragment status = %d\n", status);
		if (status >= 0) {
			start += status; /* bytes written */
			pcm_hw_status.fragment += status;
		}
		else {
			printf("gsipcm_write_fragment returned %d\n", status);
			exit(1);
		}
	}
	gsif_free_sound(snd);

	return 0;
}

/* EOF gsipcmtest (main.c) */
