/* GSI - general sound interface
 *
 * memtest (main.c): tests free'ing of malloced space by libgsif
 *
 * By W.H.Scholten 1998 - 2001
 *
 * This software is placed in the public domain and can be used freely
 * for any purpose. It comes without any kind of warranty, either
 * expressed or implied, including, but not limited to the implied
 * warranties of merchantability or fitness for a particular purpose.
 * Use it at your own risk. the author is not responsible for any damage
 * or consequences raised by use or inability to use this program.
 */


/*
 *  export APP_DEBUG_LEVEL= n for debug output.
 */


/*__________________INCLUDES___________________*/
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include <gsi/gsif.h>

/* These two includes are really for internal GSI use .. */
#include <gsi/whs_types.h>
#define DEBUG_PREFIX "APP_"
//#include <gsi/gsi_debug.h>

#ifdef USE_CURSES
#  include <ncurses.h>
#endif


/*__________________PROGRAM___________________*/
int  display_init = False;

void init_display() {
#ifdef USE_CURSES
	initscr();
	cbreak();
	intrflush(stdscr, FALSE);
	keypad(stdscr, TRUE);
#endif

	display_init = True;
}

void close_display() {
	if (display_init) {
#ifdef USE_CURSES
		nocbreak(); /* buffer IO again */
		endwin();
#endif
	}

	display_init = False;
}


int main(int argc, char *argv[]) {
	int   i;
	char  pwd_buf[ 256+20 ];
	int   status;
	char  *ptr;

	init_display();
	atexit(close_display);

	printf("GSIF test program: check for memory leaks\n\n");

	ptr = getcwd(pwd_buf, 256); /* longer and you're screwed */
	if (ptr == NULL) { perror("getcwd"); exit(1); }
	strcpy(pwd_buf + strlen(pwd_buf), "/../../../gsi_data");


	printf("chdir to %s\n", pwd_buf);
	status = chdir(pwd_buf);
	if (status == -1) perror("chdir");

	// ----------------------------------------------------------------
	/* Sleep a while .. */
	printf("check memory use (sleeping 5 s)\n\n");
	sleep(5);

	printf("malloc test\n");
	{
		char *a;
		for (i=0; i<10000; i++) {
			a = calloc(1, 10000);
			free(a);
		}
	}
	printf("malloc test done\n");

	/* Sleep a while .. */
	printf("check memory use (sleeping 5 s)\n\n");
	sleep(5);

	printf("GSIF tests start now:\n");
	// ----------------------------------------------------------------
	/* gsif sample loading/freeing test: */
	printf("load a sample:\n");
	for (i=0; i<10000; i++) {
		struct gsi_sound *snd;
		char *sample;
		int  status;

		//sample = "sample/aiff/220.aiff";
		sample = "sample/au/marvin.au";
		snd = gsif_load_sample(sample, 0, &status);
		if (snd == NULL) {
			printf("hmm. couldn't load sample %s (gsif: %s)\n", sample, gsif_strerror(status));
			break;
		}
		gsif_free_sound(snd);
	}
	printf("direct sample loading done:\n");

	/* Sleep a while .. */
	printf("check memory use (sleeping 5 s)\n\n");
	sleep(5);

	printf("loading a different sound:\n");
	{
		struct gsi_sound *snd;
		char *sample;
		int  status;

		sample = "sample/aiff/220.aiff";
		snd = gsif_load_sample(sample, 0, &status);
		if (snd == NULL) {
			printf("hmm. couldn't load sample %s (gsif: %s)\n", sample, gsif_strerror(status));
		}
		gsif_free_sound(snd);
	}
	printf("direct sample loading done:\n");

	/* Sleep a while .. */
	printf("check memory use (sleeping 5 s)\n\n");
	sleep(5);


	// ----------------------------------------------------------------
	/* gsif song loading/freeing test: */
	printf("loading a song:\n");
	for (i=0; i<10000; i++) {
		struct gsi_song *sng;
		char *sample;
		int  status;

		//sample = "music/midi/InstantX.mid";
		sample = "music/mus/doom/e3m5.mus";
		//sample = "music/hmp/descent_pc_sw10/briefing.hmp";
		sng = gsif_load_song(sample, 0, &status);
		if (sng == NULL) {
			printf("hmm. couldn't load song %s (gsif: %s)\n", sample, gsif_strerror(status));
			break;
		}
		gsif_free_song(sng);
	}
	printf("direct song loading done:\n");

	/* Sleep a while .. */
	printf("check memory use (sleeping 5 s)\n\n");
	sleep(5);

	// ----------------------------------------------------------------

	close_display();

	printf("\nProgram terminated\n");

	return 0;
}

/* EOF memory_leak.c */
