/* GSI - general sound interface
 *
 * gsimidi_pcm_test (main.c): tests libgsimidi_pcm
 *
 * By W.H.Scholten 1999 - 2001
 *
 * This software is placed in the public domain and can be used freely
 * for any purpose. It comes without any kind of warranty, either
 * expressed or implied, including, but not limited to the implied
 * warranties of merchantability or fitness for a particular purpose.
 * Use it at your own risk. the author is not responsible for any damage
 * or consequences raised by use or inability to use this program.
 */


/*
 *  export APP_DEBUG_LEVEL= n for debug output.
 *
 *  use: gsimdipcm_test [ filename ];
 *  default file is ../../../gsi_data/music/midi/Pushit.mid
 */


/*__________________INCLUDES___________________*/
#include <stdio.h>
#include <stdlib.h>

/* These two includes are really for internal GSI use .. */
#include <gsi/whs_types.h>
#define DEBUG_PREFIX "APP_"
#include <gsi/gsi_debug.h>

#include <gsi/gsipcm.h>
#include <gsi/gsimidi_pcm.h>


void  init_time();
float delta_time();


/*__________________PROGRAM___________________*/
float last_printed_time = 0.0;
float dt;

void use(char *prg_name) {
	fprintf(stderr, "Use: %s -h\n", prg_name);
	fprintf(stderr, "     %s [ filename ]\n", prg_name);
	exit(1);
}

int main(int argc, char *argv[]) {
	char *pcm_error_string[ 4 ] = {
		"GSIPCM_OK",          /*  0 */
		"GSIPCM_NOT_PRESENT", /* -1 */
		"GSIPCM_BUSY",        /* -2 */
		"GSIPCM_NOT_OPENED",  /* -3 */
	};

	int  status;
	char *filename;
	struct midi_softsynth softsynth;
	struct gsipcmmodule m;
	void *d;
	int  i;

	/* pcm: */
	gsipcm_device_t pcm_hw;
	gsipcm_devicedata_t pcm_hw_status;
	int  max_write_space = 0;


	if (argc > 1 && !strcmp(argv[1], "-h")) use(argv[0]);
	if (argc > 2) use(argv[0]);

#ifdef __OpenBSD__
	pcm_hw.device = "/dev/sound";
#else
	pcm_hw.device = "/dev/dsp";
#endif

	status = gsipcm_init(&pcm_hw, &pcm_hw_status, GSIPCM_MODE_BLOCKING);
	if (status != 0) {
		fprintf(stderr, "gsipcm_init failed (device %s: error = %s)\n", pcm_hw.device, pcm_error_string[-status]);
		exit(1);
	}

	/* pcm setting = CD quality: */
	status = gsipcm_set_pcm(&pcm_hw, &pcm_hw_status, 44100 /* rate */, 2 /* channels */, GSI_16BIT, 13 /* log2 8192 */);
	if (status != 0) {
		fprintf(stderr, "gsipcm_init failed (device %s: error = %s)\n", pcm_hw.device, pcm_error_string[-status]);
		exit(1);
	}
	max_write_space = gsipcm_write_space(&pcm_hw, &pcm_hw_status);

	/* Show what pcm values were really set: */
	printf("pcm was set to:\n");
	printf("  samplerate: %6d\n", pcm_hw_status.samplerate);
	printf("  channels:   %6d\n", pcm_hw_status.channels);
	printf("  type:       %6d\n", pcm_hw_status.type);
	printf("  fragsize:   %6d\n", pcm_hw_status.fragsize);
	printf("  (log2_frag:  %6d)\n", pcm_hw_status.log2_fragsize);
	printf("  byte order: %s\n", pcm_hw_status.byte_order == GSI_BIG_ENDIAN ? "big endian" : "little endian");


	/* Initialize gsimidi_pcm */
	if (gsimidipcm_open (&softsynth, &d)) {
		printf("gsimidipcm_open failed\n");
		exit(1);
	}

	gsimidipcm_set_output_format(d, GSI_16BIT, 44100, 2);

	gsimidipcm_clear_samples (d);

	if (argc == 2)
		filename = argv[1];
	else
		filename = "../../../gsi_data/music/midi/Pushit.mid";

	gsimidipcm_input_file (d, filename);

	//status = gsimidipcm_set_temperment (d, "vallotti");
	//printf ("temperment st=%d\n", status);
	/* "equal", "meanTone", "WerkmeisterIII", "equal415", "vallotti" */

	//status = gsimidipcm_set_reverb (d, "room");
	//printf ("reverb st=%d\n", status);
	/* "none", "room", "concert", "cathedral" */

	for (i=128; i<255; i++) gsimidipcm_set_instrument_volume(d, i, 2.0);

	init_time();

	/* Now start playing */
	gsimidipcm_get_status (d, &m);
	while (!m.done || m.queued > 0) {
		status = 0;
		if (m.queued >= pcm_hw_status.fragsize) {
			pcm_hw_status.fragment = m.buf + m.get;
			status = gsipcm_write_fragment(&pcm_hw, &pcm_hw_status);
		}
		else if (m.done) {
			status = gsipcm_write(&pcm_hw, &pcm_hw_status, m.buf+m.get, m.queued);
		}

		if (status >= 0) {
			gsimidipcm_remove (d, status);
			gsimidipcm_get_status (d, &m);
		}
		else {
			printf("gsipcm_write_fragment returned %d\n", status);
			exit(1);
		}


		while (!m.done && m.queued < m.bufSamples) {
			gsimidipcm_calculate (d, m.bufSamples - m.queued);
			gsimidipcm_get_status(d, &m);
		}

		if (m.done && m.queued == 0) break;

		if (gsipcm_write_space(&pcm_hw, &pcm_hw_status) < max_write_space - 16384) {
			struct timeval tv = { 0, 60000 };
			select (0, NULL, NULL, NULL, &tv);
		}

		dt = delta_time();
		if (dt - last_printed_time > 2.0) {
			last_printed_time = dt;
			printf("time %4.1f\n", dt);
		}
	}

	/* should drain pcm buffer; simple drain replacement: */
	sleep(1);

	return 0;
}


#include <sys/time.h>
struct timeval start_tv;
void init_time() {
	gettimeofday(&start_tv, NULL);
}

float delta_time() {
	struct timeval tv;
	float dt;

	gettimeofday(&tv, NULL);
	dt = (tv.tv_sec - start_tv.tv_sec) + (float) (tv.tv_usec - start_tv.tv_usec) * 1.0e-6;
	return dt;
}

/* EOF gsimidi_pcm_test (main.c) */
