/* GSI - general sound interface
 *
 * playtest (main.c): tests the gsi server.
 *
 * By W.H.Scholten 1998 - 2001
 *
 * This software is placed in the public domain and can be used freely
 * for any purpose. It comes without any kind of warranty, either
 * expressed or implied, including, but not limited to the implied
 * warranties of merchantability or fitness for a particular purpose.
 * Use it at your own risk. the author is not responsible for any damage
 * or consequences raised by use or inability to use this program.
 */


/*
 *  export APP_DEBUG_LEVEL= n for debug output.
 *
 *
 *
 *  Don't complain about lack of docs on this test program!
 *  It's MY test and it changes a lot ..
 *
 *  Shift + 1 - 8: set pcm to:
 *     mono/stereo/8 bit mulaw/8 bit unsigned/16bit/8000 Hz/16000 Hz/22050 Hz
 *
 *  1 - 0 = play slot  0 -  9
 *  z = position sound at (x,z,y)=(-64,-64,0) [ left ]
 *  x = position sound at (x,z,y)=(  0,-64,0) [ center ]
 *  c = position sound at (x,z,y)=(+64,-64,0) [ right ]
 *
 *  , = move sound to the left
 *  . = move sound to the right
 *
 *
 *  b = load song InstantX.mid
 *  n = start song
 *  m = stop song
 *  k = continue song
 *
 */


/*__________________INCLUDES___________________*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <unistd.h>

#include <gsi/gsi.h>
#include <gsi/gsif.h>

/* These two includes are really for internal GSI use .. */
#include <gsi/whs_types.h>
#define DEBUG_PREFIX "TEST_"
//#include <gsi/gsi_debug.h>

#ifdef USE_CURSES
#  include <ncurses.h>
#endif


/*__________________VARIABLES___________________*/
int  pcm_channels = 2;
int  pcm_type = GSI_8BIT;
int  pcm_samplerate = 8000;
int  pcm_fragsize = 8;
#define SOUNDFILE "test.sounds"


/*__________________PROGRAM___________________*/
int  display_init = False;

void init_display() {
	if (!display_init) {
#ifdef USE_CURSES
		initscr();
		cbreak();
		intrflush(stdscr, FALSE);
		keypad(stdscr, TRUE);
#endif

		display_init = True;
	}
}
void close_display() {
	if (display_init) {
#ifdef USE_CURSES
		nocbreak(); /* buffer IO again */
		endwin();
#endif

		display_init = False;
	}
}


char *pcm_mode_string(int mode) {
	int  pcm_modes[] = {
		GSI_CMD_RELEASE_PCM, GSI_CMD_GRAB_PCM, GSI_CMD_AUTO_PCM
	};
	char *pcm_mode_strings[] = {
		"GSI_CMD_RELEASE_PCM", "GSI_CMD_GRAB_PCM", "GSI_CMD_AUTO_PCM"
	};
	int  i;

	for (i=0; i<3; i++) {
		if (mode == pcm_modes[i]) return pcm_mode_strings[i];
	}
	return NULL;
}


int main(int argc, char *argv[]) {
	char  *pcm_type_string[] = {
		"GSI_8BIT_ALAW", "GSI_8BIT_MULAW", "GSI_8BIT", "GSI_16BIT", "GSI_24BIT"
	};

	int   end;
	char  ch;
	int   x, dx, dopplerspeed;
	char  pwd_buf[ 256+20 ];
	int   status;
	char  *ptr;
	int   pcm_mode = GSI_CMD_GRAB_PCM;
	int   s = 0; /* sound range, 0=first 10 samples, 10=second 10 samples. */
	gsi_server_t *c;

	int  i;

	init_display();
	atexit(close_display);

	printf("GSI test program: common server functions\n\n");

	ptr = getcwd(pwd_buf, 256); /* longer and you're screwed */
	if (ptr == NULL) { perror("getcwd"); exit(1); }
	strcpy(pwd_buf + strlen(pwd_buf), "/../../../../gsi_data");

	fprintf(stderr, "pwd_buf = %s\n", pwd_buf);

	pcm_fragsize = 8+pcm_channels-1+gsif_sizeof_sample(pcm_type)-1;
	// PUT bytes per sample in gsif?
	// pcm_fragsize = 8+pcm_channels-1+gsif_bytes_per_sample(pcm_type)-1;
	// gsif_sample_size/gsif_sizeof_sample ?

	c = gsi_init(NULL); /* NULL means (try to) connect to the local host */
	if (c == NULL) {
		close_display();
		fprintf(stderr, "Failed to connect to gsi_server\n");
		return -1;
	}
	gsi_chdir(c, pwd_buf);

	gsi_init_pcm(c, pcm_samplerate, pcm_channels, pcm_type, pcm_fragsize);

	for (i=0; i<10; i++) {
		gsi_load_sound_from_list(c, SOUNDFILE, "MAN_DOWN", 1);
	}

	gsi_enable_events(c, GSI_EVMASK_CHANNEL | GSI_EVMASK_SONG);

	{
#define TEST1
#define TEST_SHM

#ifdef TEST1
		/* Load sample using gsif and send it via the pipe/socket. */
		struct gsi_sound *snd;
		char *sample;

		//sample = "../../../gsi_data/sample/aiff/220.aiff";
		sample = "../../../gsi_data/sample/au/man_down.au";
		snd = gsif_load_sample(sample, 0, &status);
		if (snd == NULL) {
			close_display();
			printf("unable to load sample %s (gsif: %s)\n", sample, gsif_strerror(status));
			exit(1);
		}

		/* We're only sending block 0. */
fprintf(stderr, "snd->block[0]->type = %d\n", snd->block[0]->type);
		gsi_send_raw_sample(c, 19, snd->block[0]->samplerate, snd->block[0]->channels, snd->block[0]->type, snd->block[0]->data_length, snd->block[0]->data);
		gsif_free_sound(snd);
#endif
#ifdef TEST2
		/* Load sample directly in the server. */
		gsi_load_sound(c, 19, "sample/aiff/220.aiff");
#endif
		gsi_load_sound(c, 18, "sample/au/marvin.au");

#ifdef TEST_SHM
		{ /* SYSV SHM: */
		struct gsi_sound *snd;
		char *sample;
		struct gsi_shm *shmdata;

		sample = "../../../gsi_data/sample/voc/duke3d/duknuk14.voc";
		snd = gsif_load_sample(sample, 0, &status);
		if (snd == NULL) {
			close_display();
			printf("unable to load sample %s (gsif: %s)\n", sample, gsif_strerror(status));
			exit(1);
		}

		gsi_send_commands(c, 3, GSI_CMD_SHMSLOT, 17);
		gsi_send_int32(c, gsif_sizeof_sample(snd->block[0]->type) * snd->block[0]->channels * snd->block[0]->data_length);

		/* We're only going to write block 0. */
		status = gsi_make_shm(c, snd->block[0]->data_length, (char **) &shmdata);
		if (status < 0) {
			close_display(); printf("unable to get shm\n"); exit(1);
		}

		memcpy(shmdata + shmdata->header_size + shmdata->write_end_ptr, snd->block[0]->data, snd->block[0]->data_length);
		gsi_update_shm_write(c, snd->block[0]->data_length, (char *) shmdata);
		/* Notifies that we've written data by updating the write ptr. */

//		gsi_send_raw_sample(c, 19, snd->block[0]->samplerate, snd->block[0]->channels, snd->block[0]->type, snd->block[0]->data_length, snd->block[0]->data);
		gsif_free_sound(snd);
		}
#endif /* TEST_SHM */
	}

	end=0;

#ifdef USE_CURSES
	clear(); move(0,0); refresh();
	printf("gsitest: (press ESC to quit)\n");

	printf("PCM: samplerate=%d channels=%d type=%s  [ mode=%s ]\n", pcm_samplerate, pcm_channels, pcm_type_string[pcm_type], pcm_mode_string(pcm_mode));
	clrtoeol();
		//move(2,0); refresh();
		printf("Sound range = %d to %d  ", s, s+9);
	refresh();
#endif

#ifndef USE_CURSES
	printf("gsitest: (press ESC to quit)\n");
#endif
	gsi_grab_synth (c, 0 /* dev */);
	gsi_flush(c);
	do {
		ch = getchar();

#ifdef USE_CURSES
		clear(); move(0,0); refresh();
		printf("gsitest: (press ESC to quit)");
		move(1,0); refresh();
		printf("PCM: samplerate=%d channels=%d type=%s  [ mode=%s ]    ", pcm_samplerate, pcm_channels, pcm_type_string[pcm_type], pcm_mode_string(pcm_mode));
		move(2,0); refresh();
		printf("Sound range = %d to %d", s, s+9);
		move(4,0); refresh();
		printf("COMMAND: %c  ", ch);
		refresh();
#else
		printf("COMMAND: %c\n", ch);
#endif
		switch (ch) {
		case  27:  end=1; break; /* Escape */
		case 'q':  gsi_stop_channels(c); break;

/*---------------------------------------------------------------------------*
 *      Shift 1 - shift 8: Set PCM modes
 *---------------------------------------------------------------------------*/
		case '!' : /* Shift-1 mono PCM */
			printf("PCM channels: mono");
			pcm_channels = 1;
			gsi_send_commands(c, 3, GSI_CMD_SET_PCM_CHANNELS, 1, GSI_CMD_GRAB_PCM); break;
		case '@' : /* Shift-2 stereo PCM */
			printf("PCM channels: stereo");
			pcm_channels = 2;
			gsi_send_commands(c, 3, GSI_CMD_SET_PCM_CHANNELS, 2, GSI_CMD_GRAB_PCM); break;
		case '#' : /* Shift-3 8 bit mulaw PCM */
			printf("PCM type: mulaw");
			pcm_type = GSI_8BIT_MULAW;
			gsi_send_commands(c, 3, GSI_CMD_SET_PCM_TYPE, GSI_8BIT_MULAW, GSI_CMD_GRAB_PCM); break;
		case '$' : /* Shift-4 8 bit unsigned PCM */
			printf("PCM type: 8 bit unsigned");
			pcm_type = GSI_8BIT;
			gsi_send_commands(c, 3, GSI_CMD_SET_PCM_TYPE, GSI_8BIT, GSI_CMD_GRAB_PCM); break;
		case '%' : /* Shift-5 16 bit PCM */
			printf("PCM type: 16 bit signed");
			pcm_type = GSI_16BIT;
			gsi_send_commands(c, 3, GSI_CMD_SET_PCM_TYPE, GSI_16BIT, GSI_CMD_GRAB_PCM); break;
		case '^' : /* Shift-6 8000 Hz bit PCM */
			printf("PCM samplerate: 8000 Hz");
			pcm_samplerate = 8000;
			gsi_send_commands(c, 6, GSI_CMD_SET_PCM_SAMPLERATE, 0,0, 8000>>8, 8000 &0xff, GSI_CMD_GRAB_PCM); break;
		case '&' : /* Shift-7 16000 Hz bit PCM */
			printf("PCM samplerate: 16000 Hz");
			pcm_samplerate = 16000;
			gsi_send_commands(c, 6, GSI_CMD_SET_PCM_SAMPLERATE, 0,0, 16000>>8, 16000 &0xff, GSI_CMD_GRAB_PCM); break;
		case '*' : /* Shift-8 22050 Hz bit PCM */
			printf("PCM samplerate: 22050 Hz");
			pcm_samplerate = 22050;
			gsi_send_commands(c, 6, GSI_CMD_SET_PCM_SAMPLERATE, 0,0, 22050>>8, 22050 &0xff, GSI_CMD_GRAB_PCM); break;

/*---------------------------------------------------------------------------*
 *      1 - 0: play samples
 *---------------------------------------------------------------------------*/

		case 9: s ^= 10; break;

		case '1' : gsi_play_sound(c, s+0, 0, 1); break;
		case '2' : gsi_play_sound(c, s+1, 0, 1); break;
		case '3' : gsi_play_sound(c, s+2, 0, 1); break;
		case '4' : gsi_play_sound(c, s+3, 0, 1); break;
		case '5' : gsi_play_sound(c, s+4, 0, 1); break;
		case '6' : gsi_play_sound(c, s+5, 0, 1); break;
		case '7' : gsi_play_sound(c, s+6, 0, 1); break;
		case '8' : gsi_play_sound(c, s+7, 0, 1); break;
		case '9' : gsi_play_sound(c, s+8, 0, 1); break; /* marvin.au */
		case '0' : gsi_play_sound(c, s+9, 0, 1); break; /* man_down.au */

		case 'a' : gsi_play_sound(c, 9, 0, 0); break;
		case 's' : gsi_stop_channel(c, 9); break;

		case 'Q' :
		case 'W' :
		case 'E' :
		{
			int pitch;
			if (ch == 'Q') pitch =  50000;
			if (ch == 'W') pitch =  80000;
			if (ch == 'E') pitch = 100000;
			gsi_play_sound(c, 0, 9, 1);
			gsi_send_commands(c, 6, GSI_CMD_CHANNEL_SET_SAMPLE_FACTOR, 0, pitch >>24, pitch >>16, pitch >>8, pitch);
			printf("playing slot 9 with pitch=%d", pitch);
			break;
		}

/*---------------------------------------------------------------------------*
 *      z x c , . < > ; \ [ ]: pan & doppler tests
 *---------------------------------------------------------------------------*/
		case 'z' : x=-64; dx=0; //left
			gsi_send_commands(c, 6, GSI_CMD_CHANNEL_SET_X | GSI_CMD_CURRENT, x >>8, x &255, GSI_CMD_CHANNEL_SET_DX | GSI_CMD_CURRENT, dx >>8, dx &255); break;
		case 'x' : x=  0; dx=0; //middle
			gsi_send_commands(c, 6, GSI_CMD_CHANNEL_SET_X | GSI_CMD_CURRENT, x >>8, x &255, GSI_CMD_CHANNEL_SET_DX | GSI_CMD_CURRENT, dx >>8, dx &255); break;
		case 'c' : x= 64; dx=0; //right
			gsi_send_commands(c, 6, GSI_CMD_CHANNEL_SET_X | GSI_CMD_CURRENT, x >>8, x &255, GSI_CMD_CHANNEL_SET_DX | GSI_CMD_CURRENT, dx >>8, dx &255); break;
		case ',' : dx=-80;  //to left
			gsi_send_commands(c, 3, GSI_CMD_CHANNEL_SET_DX | GSI_CMD_CURRENT, dx >>8, dx &255); break;
		case '.' : dx= 80;  //to right
			gsi_send_commands(c, 3, GSI_CMD_CHANNEL_SET_DX | GSI_CMD_CURRENT, dx >>8, dx &255); break;

		case '<' : dx=-256;  //to left
			dopplerspeed=20;
			gsi_send_commands(c, 4, GSI_CMD_CHANNEL_SET_DX | GSI_CMD_CURRENT, dx >>8, dx & 255, GSI_CMD_CHANNEL_DOPPLER_ON | GSI_CMD_CURRENT); break;

		case '>' : dx= 256;  //to right
			dopplerspeed=20;
			gsi_send_commands(c, 4, GSI_CMD_CHANNEL_SET_DX | GSI_CMD_CURRENT, dx >>8, dx & 255, GSI_CMD_CHANNEL_DOPPLER_ON | GSI_CMD_CURRENT); break;


		case ';' : dx=-5;  //to left
			gsi_send_commands(c, 3, GSI_CMD_CHANNEL_SET_DX | GSI_CMD_CURRENT, dx >>8, dx & 255); break;
		case '\'': dx= 5; //to right
			gsi_send_commands(c, 3, GSI_CMD_CHANNEL_SET_DX | GSI_CMD_CURRENT, dx >>8, dx & 255); break;
		case '[' : dx=-5;  //to left
			gsi_send_commands(c, 4, GSI_CMD_CHANNEL_SET_DX, 5, dx >>8, dx &255); break;
		case ']' : dx= 5; //to right
			gsi_send_commands(c, 4, GSI_CMD_CHANNEL_SET_DX, 5, dx >>8, dx &255); break;

/*---------------------------------------------------------------------------*
 *      Other tests:
 *      F: Free sound 0
 *      A: auto pcm   G: grab pcm  R: release pcm
 *      V: play sound 19 on ch 0   v: kill it
 *      D: play sound 0  on ch 1   d: kill it
 *      E: play sound 6  on ch 5   e: kill it
 *---------------------------------------------------------------------------*/
		case 'F': gsi_send_commands(c, 2, GSI_CMD_FREE_SOUND, 0);
			printf("freeing sound 0\n");
			break;

		case 'A': gsi_send_commands(c, 1, GSI_CMD_AUTO_PCM);
			pcm_mode = GSI_CMD_AUTO_PCM;
			printf("PCM: auto_pcm");
			break;
		case 'G': gsi_send_commands(c, 1, GSI_CMD_GRAB_PCM);
			pcm_mode = GSI_CMD_GRAB_PCM;
			printf("PCM: grab_pcm");
			break;
		case 'R': gsi_send_commands(c, 1, GSI_CMD_RELEASE_PCM);
			pcm_mode = GSI_CMD_RELEASE_PCM;
			printf("PCM: release_pcm");
			break;

		case 'V': gsi_play_sound(c, 0, 19, 1); break;
		case 'v': gsi_stop_channel(c, 0); break;

		case 'D': gsi_play_sound(c, 1, 0, 1); break;
		case 'd': gsi_stop_channel(c, 1); break;

			// overwrite test:
		case 'y': // the first sound should keep on playing.
			gsi_play_sound(c, 0, 11, 1);
			gsi_flush(c);
			usleep(0);
			gsi_play_sound(c, 0, 13, 1);
			gsi_flush(c);
			break;
		case 'Y': // the first sound should be replaced by the second.
			gsi_play_sound(c, 0, 11, 1);
			gsi_flush(c);
			usleep(0);
			gsi_stop_channel(c, 0);
			gsi_play_sound(c, 0, 13, 1);
			gsi_flush(c);
			break;

		case 't': // auto channel allocation test, should hear both sounds
			gsi_play_sound(c, -1, 11, 1);
			gsi_play_sound(c, -1, 13, 1);
			gsi_flush(c);
			break;

		case 'T': // auto channel allocation test, should hear sound at double volume
			gsi_play_sound(c, -1, 13, 1);
			gsi_play_sound(c, -1, 13, 1);
			gsi_flush(c);
			break;

		case 'r':
			gsi_play_sound(c, 3, 13, 1);
			gsi_set_mixer_volume(c, GSI_PCM, 0, 128);
			gsi_flush(c);
			break;

/*---------------------------------------------------------------------------*
 *      Sound adjustment tests:
 *      o: current ch -> angle=64, vol=220
 *      p:  ,,        -> vol=100
 *      U:  ,,        -> angle=192
 *      I:  ,,        -> angle=128
 *      O:  ,,        -> angle=64
 *      P:  ,,        -> angle=0
 *---------------------------------------------------------------------------*/
		case 'o': gsi_send_commands(c, 4, GSI_CMD_CHANNEL_SET_ANGLE | GSI_CMD_CURRENT, 64, GSI_CMD_CHANNEL_SET_VOLUME | GSI_CMD_CURRENT, 220); break;

		case 'p' : gsi_send_commands(c, 2, GSI_CMD_CHANNEL_SET_VOLUME | GSI_CMD_CURRENT, 100); break;
		case 'U' : gsi_send_commands(c, 2, GSI_CMD_CHANNEL_SET_ANGLE | GSI_CMD_CURRENT, 192); break;
		case 'I' : gsi_send_commands(c, 2, GSI_CMD_CHANNEL_SET_ANGLE | GSI_CMD_CURRENT, 128); break;
		case 'O' : gsi_send_commands(c, 2, GSI_CMD_CHANNEL_SET_ANGLE | GSI_CMD_CURRENT, 64); break;
		case 'P' : gsi_send_commands(c, 2, GSI_CMD_CHANNEL_SET_ANGLE | GSI_CMD_CURRENT, 0); break;


/*---------------------------------------------------------------------------*
 *      b n m k: Music, midi (load, start, stop, continue of InstantX.mid)
 *---------------------------------------------------------------------------*/
		case 'b': gsi_load_song(c, 0, "music/InstantX.mid");
			break;
		case 'n': gsi_play_song(c, 255 /*synth */, 255 /* song slot */, 0 /* ch */, 1);
			break;
		case 'm': //OBSOLETE gsi_stop_song(c);
			gsi_stop_channel(c, 0);
			break;
		case 'k': //OBSOLETE gsi_continue_song();
			gsi_send_commands(c, 2, GSI_CMD_CHANNEL_RESUME, 0);
			break;
		default :
#ifdef USE_CURSES
			move(4,2); refresh();
			printf("Invalid");
#else
			printf("\n  Invalid");
#endif
			break;
		}

#ifdef USE_CURSES
		move(1,0); refresh();
		printf("PCM: samplerate=%d channels=%d type=%s  [ mode=%s ]    ", pcm_samplerate, pcm_channels, pcm_type_string[pcm_type], pcm_mode_string(pcm_mode));
		clrtoeol(); //doesn't seem to work (need to set bg?)
		move(2,0); refresh();
		printf("Sound range = %d to %d  ", s, s+9);
		refresh();
#else
		printf("\n");
#endif


#ifdef USE_CURSES
		move(4,0);
		refresh();
#endif

		gsi_flush(c);
	} while (!end);

	gsi_close(c);

	close_display();
	printf("\nProgram terminated\n");

	return 0;
}

/* EOF playtest (main.c) */
