/* GSI - general sound interface
 *
 * memtest (main.c): tests free'ing of malloced space by libgsif and gsi_server
 *
 * By W.H.Scholten 1998 - 2000
 *
 * This software is placed in the public domain and can be used freely
 * for any purpose. It comes without any kind of warranty, either
 * expressed or implied, including, but not limited to the implied
 * warranties of merchantability or fitness for a particular purpose.
 * Use it at your own risk. the author is not responsible for any damage
 * or consequences raised by use or inability to use this program.
 */


/*
 *  export APP_DEBUG_LEVEL= n for debug output.
 */


/*__________________INCLUDES___________________*/
#include <stdio.h>
#include <stdlib.h>

#include <unistd.h>

#include <gsi/gsi.h>
#include <gsi/gsif.h>

/* These two includes are really for internal GSI use .. */
#include <gsi/whs_types.h>
#define DEBUG_PREFIX "APP_"
//#include <gsi/gsi_debug.h>

#ifdef USE_CURSES
#  include <ncurses.h>
#endif


/*__________________VARIABLES___________________*/
int  pcm_channels = 2;
int  pcm_fragsize = 8;
int  pcm_type = GSI_8BIT;
int  pcm_samplerate = 8000;
//#define SOUNDFILE "test.sounds"
#define SOUNDFILE ""


/*__________________PROGRAM___________________*/
int  display_init = False;
void init_display() {
#ifdef USE_CURSES
	initscr();
	cbreak();
	intrflush(stdscr, FALSE);
	keypad(stdscr, TRUE);
#endif

	display_init = True;
}
void close_display() {
	if (display_init) {
#ifdef USE_CURSES
		nocbreak(); /* buffer IO again */
		endwin();
#endif
	}

	display_init = False;
}


int main(int argc, char *argv[]) {
	int   i;
	char  pwd_buf[ 256+20 ];
	int   status;
	char *ptr;
	gsi_server_t *c;


	init_display();
	atexit(close_display);

	printf("GSI test program: memory leaks\n\n");

	ptr = getcwd(pwd_buf, 256); /* longer and you're screwed */
	if (ptr == NULL) { perror("getcwd"); exit(1); }
	strcpy(pwd_buf +strlen(pwd_buf), "/../../../gsi_data");


	printf("chdir to %s\n", pwd_buf);
	status = chdir(pwd_buf);
	if (status == -1) perror("chdir");


	// ----------------------------------------------------------------
	printf("check memory use (sleeping 5 s)\n\n");
	sleep(5);

	printf("malloc test:\n");
	{
		char *a;
		for (i=0; i<10000; i++) {
			a = calloc(1, 10000);
			free(a);
		}
	}
	printf("check memory use (sleeping 5 s)\n\n");
	sleep(5);

	// ----------------------------------------------------------------
	printf("\nPART 2: memory use of gsi_server\n");
	pcm_fragsize = 8+pcm_channels-1+gsif_sizeof_sample(pcm_type)-1;

	c = gsi_init(NULL /* =localhost */);
	if (c == NULL) {
		close_display();
		fprintf(stderr, "Failed to connect to gsi_server\n");
		return -1;
	}
	gsi_chdir(c, pwd_buf);

	gsi_init_pcm(c, pcm_samplerate, pcm_channels, pcm_type, pcm_fragsize);
//OBSOLETE	gsi_load_samples_from_list(c, SOUNDFILE);
	printf("connected to server:\n");

	printf("check memory use (sleeping 5 s)\n\n");
	sleep(5);
	printf("..\n");

	/* Load sample directly in the server */
	printf("loading samples in the server\n");
	for (i=0; i<1000; i++) {
		gsi_load_sound(c, 19, "sample/220.aiff");
		gsi_load_sound(c, 19, "sample/marvin.au");
		gsi_send_commands(c, 2, GSI_CMD_FREE_SOUND, 19);
		gsi_flush(c);
	}
	printf("loading samples done:\n");

	printf("check memory use (sleeping 5 s)\n\n");
	sleep(5);
	printf("..\n");

	printf("loading songs in the server\n");
	for (i=0; i<1000; i++) {
		gsi_load_song(c, 255, "music/midi/InstantX.mid");
		gsi_load_song(c, 255, "music/midi/Zzunknown.mid");
		gsi_load_song(c, 255, "music/mus/doom/e1m1.mus");
		gsi_flush(c);
	}
	printf("songs done:\n");
	printf("check memory use (sleeping 5 s)\n\n");
	sleep(5);

	gsi_close(c);

	close_display();

	printf("\nProgram terminated\n");

	return 0;
}

/* EOF memtest (main.c) */
