/* GSI - general sound interface
 *
 * streamtest (main.c): tests streams with the gsi sound server (and is used
 * to develop the stream API).
 *
 * By W.H.Scholten 1998 - 2000
 *
 * This software is placed in the public domain and can be used freely
 * for any purpose. It comes without any kind of warranty, either
 * expressed or implied, including, but not limited to the implied
 * warranties of merchantability or fitness for a particular purpose.
 * Use it at your own risk. the author is not responsible for any damage
 * or consequences raised by use or inability to use this program.
 */


/*
 *  export APP_DEBUG_LEVEL= n for debug output.
 *
 *  keys:
 *  p     pauses playing
 *  r     resumes playing
 *  q     quits
 *
 */


/*__________________INCLUDES___________________*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <ctype.h>
#include <sys/time.h>
#include <unistd.h>

#include <gsi/gsi.h>
#include <gsi/gsif.h>

/* These two includes are really for internal GSI use .. */
#include <gsi/whs_types.h>
#define DEBUG_PREFIX "APP_"
//#include <gsi/gsi_debug.h>

#ifdef USE_CURSES
#  include <ncurses.h>
#endif


#define min(x,y) ((x)<(y) ? (x) : (y))

#define STREAM_VERSION_1
//#define STREAM_VERSION_2
//#define STREAM_VERSION_SHM


/*__________________VARIABLES___________________*/
int  pcm_samplerate = 8000;
int  pcm_channels = 2;
int  pcm_type = GSI_8BIT;
int  pcm_log2_fragsize = 8;


/*__________________PROGRAM___________________*/
int  display_init = False;
void init_display() {
#ifdef USE_CURSES
	initscr();
	cbreak();
	//noecho();

	//nonl(); /* no newline */
	intrflush(stdscr, FALSE);
	keypad(stdscr, TRUE);
#endif

	display_init = True;
}
void close_display() {
	if (display_init) {
#ifdef USE_CURSES
		nocbreak(); /* buffer IO again */
		endwin();
#endif

		display_init = False;
	}
}


//int msb_int32_at(unsigned char *a)
int msb_int32_at(int *a) {
	return (((int) a[0])<<24) | (((int) a[1])<<16) | (((int) a[2])<<8) | (((int) a[3]));
}


gsi_server_t *c;
int  stream_slot, stream_done, underflow_done;
int  lwm, hwm;
void info_callback() {
#ifdef STREAM_VERSION_1
	if (c->gsi_event_data[0] == GSI_EV_STREAM_UPDATE) {
		stream_slot = c->gsi_event_data[1];
		stream_done += msb_int32_at(c->gsi_event_data + 2);
		fprintf(stderr, "stream_done (slot %d) = %d\n", stream_slot, stream_done);
	}
	else if (c->gsi_event_data[0] == GSI_EV_STREAM_UNDERFLOW) {
		underflow_done += msb_int32_at(c->gsi_event_data + 2);
		fprintf(stderr, "underflow (slot %d) = %d\n", stream_slot, underflow_done);
	}
#endif

#ifdef STREAM_VERSION_2
	if (c->gsi_event_data[0] == GSI_EV_STREAM_LWM) lwm = True;
	if (c->gsi_event_data[0] == GSI_EV_STREAM_HWM) hwm = True;
#endif

#ifdef STREAM_VERSION_SHM
#endif
}

void error_callback() {
	fprintf(stderr, "error event %d\n", c->gsi_event_data[0]);
	if (c->gsi_event_data[0] == GSI_EV_NO_ROOM) {
		fprintf(stderr, "no room for slot %d (%d bytes nuked)\n", c->gsi_event_data[1], (int) msb_int32_at(c->gsi_event_data + 2));
	}
}


int main(int argc, char *argv[]) {
	char  pwd_buf[ 256+20 ];
	char *ptr;
	int   pending, pending_out;

	init_display();
	atexit(close_display);

	printf("GSI test program: streaming audio\n\n");

	ptr = getcwd(pwd_buf, 256); /* longer and you're screwed */
	if (ptr == NULL) { perror("getcwd"); exit(1); }
	strcpy(pwd_buf + strlen(pwd_buf), "/../../../../gsi_data");

	pcm_log2_fragsize = 8+pcm_channels-1+gsif_sizeof_sample(pcm_type)-1;
	c = gsi_init(NULL /* =localhost */);
	if (c == NULL) {
		close_display();
		fprintf(stderr, "Failed to connect to gsi_server\n");
		return -1;
	}
	gsi_chdir(c, pwd_buf);

	gsi_init_pcm(c, pcm_samplerate, pcm_channels, pcm_type, pcm_log2_fragsize);
	gsi_enable_events(c, GSI_EVMASK_CHANNEL | GSI_EVMASK_SONG);

	/* For testing it's useful to disable the next command: */
	gsi_send_commands(c, 1, GSI_CMD_USE_BEST_PCM_SETTINGS);

	{
		/* Load sample using gsif and send it as a stream via the socket. */
		struct gsi_sound  *snd;
		int   stream_ptr;
		int   paused;
		int   status;
		char *filename = "../../../../gsi_data/sample/aiff/220.aiff";

		printf("loading file %s\n", filename);
		snd = gsif_load_sample(filename, 0, &status);
		if (snd == NULL) {
			close_display();
			printf("unable to load %s (gsif: %s)\n", filename, gsif_strerror(status));
			exit(1);
		}

		/* Enable events for streams (Low Water Mark, High Water Mark?). */
		gsi_enable_events(c, GSI_EVMASK_STREAM);

		/* We're going to stream block 0;
		 * Set the datatype it's going to receive: */
		gsi_send_commands(c, 2, GSI_CMD_CONFIGURE_SLOT, 17);
		gsi_send_int32(c, snd->block[0]->samplerate);
		gsi_send_byte(c, snd->block[0]->channels);
		gsi_send_byte(c, snd->block[0]->type);

		/* Stream buffer of 16K bytes: */
		gsi_send_int32(c, 16384);

		/* Now stream on channel 7 using slot 17: */
		gsi_send_commands(c, 3, GSI_CMD_STREAM, 7, 17);

		//gsi_send_commands(c, 2, GSI_CMD_REPEAT_ON, 7);
		//gsi_send_commands(c, 2, GSI_CMD_REPEAT_OFF, 7);

		/* Send the data: */

#ifdef STREAM_VERSION_1
		/*-------------------------------------------------------------------
		 * Version 1: Using bufsize, and bytes processed/bytes underflowed
		 * events.
		 *-------------------------------------------------------------------
		 */

		gsi_set_info_callback(c, info_callback);
		gsi_set_error_callback(c, error_callback);

		stream_ptr = 0;
		/*   no. of bytes sent. */

		stream_done = 0;
		/*   no. of bytes the server has processed. */

		underflow_done = 0;
		/*   no. of bytes the server needed but were unavailable: The server
		 *   used in their place either silence or repeated (part of) the last
		 *   fragment (when using GSI_CMD_REPEAT). Use this to skip later parts
		 *   of a sound in case you need to synchronize sound output with e.g.
		 *   graphics (like a game). */

		paused = False;
		printf("Commands: p (pause) r (resume) q (quit)\n");
		for (;;) {
			fd_set  sel_in;
			fd_set  sel_out;
			int     max_sel;

			/* Send a block to slot 0, size is always such that the server's
			 * buffer does not overflow (no need to wait..) and maximal 2048 b
			 * This way, the only possibility is a buffer underflow in the
			 * server. */
			if (stream_ptr - stream_done < 16384) {
				int  blk_size = min(2048, 16384 - (stream_ptr - stream_done));

				blk_size = min(blk_size, snd->block[0]->data_length - stream_ptr);

				fprintf(stderr, " STREAM TEST: blk_size = %d\n", blk_size);
				if (blk_size > 0) {
					gsi_send_commands(c, 2, GSI_CMD_SEND_SLOT_DATA, 17);
					gsi_send_int32(c, blk_size);
					gsi_send_block(c, snd->block[0]->data + stream_ptr, blk_size);
					stream_ptr += blk_size;
				}
			}

			fprintf(stderr, " STREAM TEST: waiting for event\n");

			FD_ZERO(&sel_in);
			FD_SET(STDIN_FILENO, &sel_in);
			FD_SET(gsi_event_fd(c), &sel_in);

			FD_ZERO(&sel_out);
			FD_SET(gsi_event_fd(c), &sel_out);

			max_sel = gsi_event_fd(c)+1;
			pending = gsi_events_pending(c);
			pending_out = gsi_commands_pending(c);
			if (pending < 0) {
				fprintf(stderr, "Server died\n");
				exit(1);
			}
			else if (pending > 0 || pending_out > 0) {
				struct timeval tv = { 0L, 0L };
				fd_set *a = NULL, *b = NULL;

				if (pending > 0) a = &sel_in;
				if (pending_out > 0) b = &sel_out;
				select (max_sel, a, b, NULL, &tv);
			}
			else {
				select (max_sel, &sel_in, NULL, NULL, NULL);
			}

			while (gsi_next_event(c) == 0) ; /* Handle GSI callbacks. */

			if (FD_ISSET(STDIN_FILENO, &sel_in)) {
				/* Keyboard input */
				byte b;
				b = getchar();
				switch(tolower(b)) {
				case 'p':
					if (!paused) {
						//fprintf(stderr, "pause ch 7\n");
						gsi_send_commands(c, 2, GSI_CMD_CHANNEL_PAUSE, 7);
						paused = True;
					}
					break;
				case 'r':
					if (paused) {
						//fprintf(stderr, "continue ch 7\n");
						gsi_send_commands(c, 2, GSI_CMD_CHANNEL_RESUME, 7);
						paused = False;
					}
					break;
				case 'q':
					stream_done = snd->block[0]->data_length;
					/* This will kick us out of the loop. */
					break;
				}
			}
			gsi_flush(c);
			if (stream_done >= snd->block[0]->data_length) break; // all done.
		}
#endif

#ifdef STREAM_VERSION_2
		/* NB. THIS VERSION DOES NOT WORK. IT'S A DESIGN STUDY.
		 * USE THE STREAM_VERSION_1 CODE.
		 *---------------------------------------------------------------------
		 * Version 2: Using LWM to send, HWM to end
		 * default: HWM = bufsize, LWM = 0 ?
		 *---------------------------------------------------------------------
		 */

		/* set hwm for slot 17 */
		gsi_send_commands(c, 2, GSI_CMD_SET_STREAM_HWM, 17);
		gsi_send_int32(c, 16384 - 4096);

		/* set lwm for slot 17 */
		gsi_send_commands(c, 2, GSI_CMD_SET_STREAM_LWM, 17);
		gsi_send_int32(c, 4096);

		gsi_set_info_callback(c, info_callback);

		lwm = True; /* we need data */
		/* (an LWM event is sent when buffer data available in the server gets
		 * below LWM). */

		hwm = False; // no more data or we might overflow!
		/* (a HWM event is sent when buffer data available in the server gets
		 * above HWM). */

		stream_ptr = 0;
		for (;;) {
			/* send a block of 2K to slot 17. */
			//if (! hwm) {
			if (lwm && ! hwm) {
				gsi_send_commands(c, 2, GSI_CMD_SEND_SLOT_DATA, 17);
				gsi_send_int32(c, 2048);
				gsi_send_block(c, snd->block[0]->data + stream_ptr, 2048);
				stream_ptr += 2048;
			}

			gsi_wait_for_event(c, -1,0);
		}
#endif

#ifdef STREAM_VERSION_SHM
		/* NB. THIS VERSION DOES NOT WORK. IT'S A DESIGN STUDY.
		 * USE THE STREAM_VERSION_1 CODE.
		 *
		 *---------------------------------------------------------------------
		 * Version 3: Using SHM mixing buffer
		 *---------------------------------------------------------------------
		 */
		for (;;) {
			if (!shm_buf.locked) {
				if (shm_buf.free) ...
			}
		}
#endif

		/* done. */
		gsif_free_sound(snd);
	}

	/* this sleep is only for testing. */
	//sleep(2);

	gsi_close(c);

	close_display();
	printf("\nProgram terminated\n");

	return 0;
}

/* EOF streamtest */
