/* GSI - general sound interface
 *
 * threadtest (main.c): tests the threading/context in libgsi.
 *
 * By W.H.Scholten 1998 - 2001
 *
 * This software is placed in the public domain and can be used freely
 * for any purpose. It comes without any kind of warranty, either
 * expressed or implied, including, but not limited to the implied
 * warranties of merchantability or fitness for a particular purpose.
 * Use it at your own risk. the author is not responsible for any damage
 * or consequences raised by use or inability to use this program.
 */


/*__________________INCLUDES___________________*/
#include <stdio.h>
#include <stdlib.h>
//#include <string.h>

#include <unistd.h>

#include <gsi/gsi.h>
#include <gsi/gsif.h>

// These includes are really for internal GSI use ..
#include <gsi/whs_types.h>
#define DEBUG_PREFIX "APP_"
//#include <gsi/gsi_debug.h>
#include <gsi/gsi_event_names.h>


#include <pthread.h>

#undef USE_CURSES
#ifdef USE_CURSES
#  include <ncurses.h>
#endif


void start_thread1(void *arg);
void start_thread2(void *arg);


/*__________________VARIABLES___________________*/
int  pcm_channels = 2;
int  pcm_type = GSI_8BIT;
int  pcm_samplerate = 8000;
int  pcm_fragsize = 8;


/*__________________PROGRAM___________________*/
int  display_init = False;

void init_display() {
	if (!display_init) {
#ifdef USE_CURSES
		initscr();
		cbreak();
		intrflush(stdscr, FALSE);
		keypad(stdscr, TRUE);
#endif

		display_init = True;
	}
}
void close_display() {
	if (display_init) {
#ifdef USE_CURSES
		nocbreak(); /* buffer IO again */
		endwin();
#endif

		display_init = False;
	}
}


pthread_t thread1, thread2;
pthread_attr_t thread_attr1, thread_attr2;
gsi_server_t *c1, *c2;


int main(int argc, char *argv[]) {
	char  *ptr;
	char   pwd_buf[256];
	int    status;
	char  *arg1 = "THIS IS THREAD 1";
	char  *arg2 = "THIS IS THREAD 2";
	void  *tstatus;

	init_display();
	atexit(close_display);

	printf("GSI test program: threads\n\n");
//	printf("On this OS, PTHREAD_THREADS_MAX=%d\n", PTHREAD_THREADS_MAX);

	ptr = getcwd(pwd_buf, 256); /* longer and you're screwed */
	if (ptr == NULL) { perror("getcwd"); exit(1); }
	strcpy(pwd_buf + strlen(pwd_buf), "/../../../../gsi_data");

	pcm_fragsize = 8+pcm_channels-1+gsif_sizeof_sample(pcm_type)-1;



	/* --------------------------------------------------------------------- *
	 *
	 *    Context for thread 1:
	 *
	 * --------------------------------------------------------------------- */
	c1 = gsi_init(NULL);
	if (c1 == NULL) {
		close_display();
		fprintf(stderr, "Failed to connect to gsi_server\n");
		return -1;
	}
	gsi_chdir(c1, pwd_buf);

	gsi_init_pcm(c1, pcm_samplerate, pcm_channels, pcm_type, pcm_fragsize);
	gsi_enable_events(c1, GSI_EVMASK_CHANNEL);
	gsi_load_sound (c1, 0, "sample/au/man_down.au");



	/* --------------------------------------------------------------------- *
	 *
	 *    Context for thread 2:
	 *
	 * --------------------------------------------------------------------- */
	c2 = gsi_init(NULL);
	if (c2 == NULL) {
		close_display();
		fprintf(stderr, "Failed to connect to gsi_server\n");
		return -1;
	}
	gsi_chdir(c2, pwd_buf);

	gsi_init_pcm(c2, pcm_samplerate, pcm_channels, pcm_type, pcm_fragsize);
	gsi_enable_events(c2, GSI_EVMASK_CHANNEL);
	gsi_load_sound (c2, 0, "sample/au/300Hz_16bit_l.au");



#ifndef USE_CURSES
	printf("threadtest: (press ESC to quit)\n");
#endif

	/* thread 1 starts man_down.au */
	status = pthread_create(&thread1, NULL /* pthread_attr_default ? */, (void *) &start_thread1, (void *) arg1);
	if (status < 0) {
		fprintf(stderr, "Failed to create thread 1\n");
		exit(1);
	}

	/* Thread 2 starts 300Hz_16bit_l.au */
	status = pthread_create(&thread2, NULL /* pthread_attr_default ? */, (void *) &start_thread2, (void *) arg2);
	if (status < 0) {
		fprintf(stderr, "Failed to create thread 2\n");
		exit(1);
	}

	/* here wait. */
	status = pthread_join(thread1, &tstatus);
	status = pthread_join(thread2, &tstatus);

	gsi_close(c1);
	gsi_close(c2);

	close_display();
	printf("\nProgram terminated\n");

	return 0;
}


void start_thread1(void *arg) {
	printf("thread message: %s\n", (char *) arg);
	gsi_play_sound(c1, 0, 0, 1); /* plays man_down.au on ch 0 */
	gsi_flush(c1);
	for(;;) {
		int  status;

		if (!gsi_events_pending(c1))
			gsi_wait_for_event(c1, -1,0);
		else {
			status = gsi_next_event(c1);
			if (status == 0) {
				printf("thread1: event %d (%s)\n", c1->gsi_event_data[0], gsi_event_name[ c1->gsi_event_data[0] ]);
				if (c1->gsi_event_data[0] == GSI_EV_NO_PCM) {
					fprintf(stderr, "thread1: No pcm available\n");
					break;
				}

				if (c1->gsi_event_data[0] == GSI_EV_CHANNEL_END) {
					printf("thread1: => channel end for ch %d\n", c1->gsi_event_data[1]);
					break;
				}
			}
		}
	}
	printf("thread1 ends\n");
}


void start_thread2(void *arg) {
	printf("thread message: %s\n", (char *) arg);
	gsi_play_sound(c2, 1, 0, 1); /* plays 300Hz_16bit_l.au on ch 1 */
	gsi_flush(c2);
	for(;;) {
		int  status;

		if (!gsi_events_pending(c2))
			gsi_wait_for_event(c2, -1,0);
		else {
			status = gsi_next_event(c2);
			if (status == 0) {
				printf("thread2: event %d (%s)\n", c2->gsi_event_data[0], gsi_event_name[ c2->gsi_event_data[0] ]);
				if (c2->gsi_event_data[0] == GSI_EV_NO_PCM) {
					fprintf(stderr, "thread2: No pcm available\n");
					break;
				}

				if (c2->gsi_event_data[0] == GSI_EV_CHANNEL_END) {
					printf("thread2: => channel end for ch %d\n", c2->gsi_event_data[1]);
					break;
				}
			}
		}
	}
	printf("thread2 ends\n");
}

/*    EOF threadtest */
