/* raw_to_au.c: Conversie van raw formaat naar .au/.snd
 *
 * By W.H.Scholten 1997 - 2000
 *
 * This file is public domain.
 */


/*__________________INCLUDES___________________*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/* Comment out for english text: */
#define NL

#ifdef NL
#  define NL_T(t) t
#  define ENG_T(t)
#else
#  define NL_T(t)
#  define ENG_T(t) t
#endif


/*__________________PROGRAMMA___________________*/
void my_fwrite(const void *ptr, size_t size, size_t nmemb, FILE *stream);

void use() {
#ifdef NL
	printf("\nHULP VOOR RAW_TO_AU:\n"
		   "raw_to_au -h\n"
		   " of:\n"
		   "raw_to_au <bestand.raw> <frequentie> <kanalen> <type> <commentaar>\n\n"
		   "waarbij type mulaw, 8, 16, 24, 16LE of 24LE is.\n\n"
		);
#else
	printf("\nHELP ON RAW_TO_AU:\n"
		   "raw_to_au -h\n"
		   " or:\n"
		   "raw_to_au <file.raw> <samplerate> <channels> <type> <comment>\n\n"
		   "where type is mulaw, 8, 16, 24, 16LE or 24LE.\n\n"
		);
#endif
}


int main(int argc, char *argv[]) {
	char  bestand_in[ 100 ], bestand_uit[ 100 ];
	FILE *fpin, *fpuit;
	char  string[ 100 ];
	char  beschrijving[ 1024 ];
	unsigned char byte;
	int   samplerate, channels, type;
	int   data_start, data_length;
	int   i;

	if (argc == 2 && !strcmp(argv[1], "-h")) {
		use();
		exit(0);
	}

	printf(
		NL_T("\nraw_to_au: audio conversie: raw -> au\n")
		ENG_T("\nraw_to_au: audio conversion: raw -> au\n")
		);


	if (argc < 2) {
		printf(
			NL_T("interactieve modus\n"
				 " [ Direkt: raw_to_au <bestand.raw> <frequentie> <kanalen> <type> <commentaar> ]\nmet type = mulaw, 8, 16, 24, 16LE, 24LE\n")
			ENG_T("interactive mode\n"
				  " [ Direct: raw_to_au <file.raw> <frequency> <channels> <type> <comment line> ]\nwhere type = mulaw, 8, 16, 24, 16LE, 24LE\n")
			);

		printf(
			NL_T("Naam van het om te zetten bestand ? ")
			ENG_T("Name of the file to be converted ? ")
			);
		scanf("%50s", bestand_in);
	}
	else strcpy(bestand_in, argv[1]);

	strcpy(bestand_uit, bestand_in);
	if (!strcmp(bestand_uit+strlen(bestand_uit)-4, ".raw")) bestand_uit[strlen(bestand_uit)-4] = 0;
	strcat (bestand_uit, ".au");


	if (argc < 3) {
		printf(
			NL_T("Frequentie ? ")
			ENG_T("Frequency ? ")
			);
		scanf("%50s", string);
	}
	else strcpy(string, argv[2]);

	samplerate = atol(string);
	if (samplerate < 1) {
		fprintf(stderr,
				NL_T("te klein")
				ENG_T("too small")
			);
		exit(1);
	}


	if (argc < 4) {
		printf(
			NL_T("Kanalen ? ")
			ENG_T("Channels ? ")
			);
		scanf("%50s", string);
	}
	else strcpy(string, argv[3]);

	channels = atol(string);


	if (argc < 5) {
		printf(
			NL_T("Type ?")
			ENG_T("Type ?")
			" (mulaw, 8, 16, 24, 16LE, 24LE) "
			);
		scanf("%50s", string);
	}
	else strcpy(string, argv[4]);

	if (!strcmp(string, "mulaw")) type = 1;
	else if (!strcmp(string, "8")) type = 2;
	else if (!strcmp(string, "16")) type = 3;
	else if (!strcmp(string, "24")) type = 4;
	else if (!strcmp(string, "16LE")) type = 128+3;
	else if (!strcmp(string, "24LE")) type = 128+4;
	else {
		fprintf(stderr,
				NL_T("onbekend type\n")
				ENG_T("unknown type\n")
			);
		exit(1);
	}

	if (argc < 6) {
		char *ptr;
		printf(
			NL_T("Beschrijving ? ")
			ENG_T("Description ? ")
			);
		fgetc(stdin); /* remove newline */
		do {
			ptr = fgets(beschrijving, 1023, stdin);
		} while(ptr == NULL);
	}
	else strncpy(beschrijving, argv[5], 1023);
	beschrijving[ 1023 ] = 0;

	for (i=0; ; i++) {
		if (beschrijving[i] == 10) beschrijving[i] = 0;
		if (beschrijving[i] == 0) break;
	}

	fpin = fopen(bestand_in, "r");
	if (fpin == NULL) {
		printf(
			NL_T("Bestand ")
			ENG_T("File ")
			);
		printf(bestand_in);
		printf(
			NL_T(" is niet te openen\n\n")
			ENG_T(" cannot be opened\n\n")
			);
		exit(1);
	}

	fpuit = fopen(bestand_uit, "w");
	if (fpuit == NULL) {
		fprintf(stderr,
				NL_T("%s kan niet geopend worden om naar te schrijven\n\n")
				ENG_T("%s cannot be opened for writing\n\n")
				, bestand_uit
				);
		exit(1);
	}

	fseek(fpin, 0L, SEEK_SET);

	printf(" %s > %s\n\n", bestand_in, bestand_uit);

	/* au header */
	my_fwrite(".snd", 4, 1, fpuit);

	data_start = 6*4+strlen(beschrijving)+1;
	for (i=3; i>=0; i--) {
		byte = ((data_start >> (i*8))&255);
		my_fwrite(&byte, 1, 1, fpuit);
	}

	fseek(fpin, 0L, SEEK_END);
	data_length = ftell(fpin);
	for (i=3; i>=0; i--) {
		byte = ((data_length >> (i*8))&255);
		my_fwrite(&byte, 1, 1, fpuit);
	}

	for (i=3; i>=0; i--) {
		byte = (( (type & 127) >> (i*8))&255);
		my_fwrite(&byte, 1, 1, fpuit);
	}

	for (i=3; i>=0; i--) {
		byte = ((samplerate >> (i*8))&255);
		my_fwrite(&byte, 1, 1, fpuit);
	}

	for (i=3; i>=0; i--) {
		byte = ((channels >> (i*8)) & 255);
		my_fwrite(&byte, 1, 1, fpuit);
	}
	my_fwrite(beschrijving, strlen(beschrijving)+1, 1, fpuit);

	/* schrijf de gegevens */
	fseek(fpin, 0L, SEEK_SET);
	{
	char data[8];
	int  nbytes = (type & 127);
	int  byteswap = type & 128;

	if (nbytes > 1) nbytes--;
	//fprintf(stderr, "nbytes=%d, byteswap=%d\n", nbytes, byteswap);
	while (1) {
		fread(data, nbytes, 1, fpin);
		if (feof(fpin)) break;
		if (byteswap) {
			if (nbytes == 2) {
				data[7] = data[0];
				data[0] = data[1];
				data[1] = data[7];
			}
			else {
				data[7] = data[0];
				data[0] = data[2];
				data[2] = data[7];
			}
		}
		my_fwrite(data, nbytes, 1, fpuit);
	}
	}

	return 0;
}


void my_fwrite(const void *ptr, size_t size, size_t nmemb, FILE *stream) {
	size_t status;

	status = fwrite(ptr, size, nmemb, stream);
	if (status != nmemb) {
		fprintf(stderr, "Write error: %d of %d bytes written", status, nmemb);
		exit(1);
	}
}

/* EOF raw_to_au.c */
