/* toon.c: maakt een .au bestand met vaste toonhoogte
 *
 * By W.H.Scholten 1997 - 2000
 *
 * This file is public domain.
 */


/*__________________INCLUDES___________________*/
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <unistd.h>

/* Comment out for english text: */
#define NL

#ifdef NL
#  define NL_T(t) t
#  define ENG_T(t)
#else
#  define NL_T(t)
#  define ENG_T(t) t
#endif


void write_vol(float vol, int type, FILE *fpuit);


/*__________________PROGRAMMA___________________*/
void use() {
#ifdef NL
	printf("\nHULP VOOR TOON:\n"
		   "toon -h\n"
		   " of:\n"
		   "toon <bestand.raw> <toonhoogte> <duur> <frequentie> <kanalen> <type>\n\n"
		   "waarbij type 8, 16 of mulaw is, kanalen is 0 voor mono, 1 voor linker, 2 voor rechter kanaal.\n\n"
		);
#else
	printf("\nHELP ON TOON:\n"
		   "toon -h\n"
		   " or:\n"
		   "toon <file.raw> <pitch> <duration> <samplerate> <channels> <type>\n\n"
		   "where type is 8, 16 or mulaw, channels is 0 for mono, 1 for left, 2 voor right channel.\n\n"
		);
#endif
}


int main(int argc, char *argv[]) {
	int    i;
	char   bestand_uit[ 100 ];

	char   toonduur_string[ 100 ];
	char   toon_string[ 100 ];
	char   freq_string[ 100 ];
	char   type_string[ 100 ];
	char   chan_string[ 100 ];
	int    samplerate, channels, type;
	int    toon;
	float  samples;
	float  tijd;
	int    golven;

	FILE  *fpuit;


	if (argc == 2 && !strcmp(argv[1], "-h")) {
		use();
		exit(0);
	}

	printf(
		NL_T("\ntoon: maak vaste toonhoogte bestand L/R\n")
		ENG_T("\ntoon: make file with fixed pitch L/R\n"));

	if (argc < 2) {
		printf(
			NL_T("interactieve modus\n"
				 " [ Direkt: toon <bestand.raw> <toonhoogte> <duur> <snelheid> <kanalen> <type> <commentaar> ]\n")
			ENG_T("interactive mode\n"
				  " [ Direct: toon <file.raw> <note frequency> <duration> <samplerate> <channels> <type> <comment> ]\n"));

		printf(
			NL_T("Naam van het bestand ? ")
			ENG_T("Name of the file ? "));
		scanf("%50s", bestand_uit);
	}
	else strcpy(bestand_uit, argv[1]);


	if (argc < 3) {
		printf(
			NL_T("Toonhoogte Hz ? ")
			ENG_T("Note frequency ? "));
		scanf("%50s", toon_string);
	}
	else strcpy(toon_string, argv[2]);

	toon = atol(toon_string);


	if (argc < 3) {
		printf(
			NL_T("Toonduur s ? ")
			ENG_T("Note duration ? "));
		scanf("%50s", toonduur_string);
	}
	else strcpy(toonduur_string, argv[3]);

	tijd = atof(toonduur_string);


	if (argc < 4) {
		printf(
			NL_T("Bestand: frequentie ? ")
			ENG_T("File: sample frequency ? "));
		scanf("%50s", freq_string);
	}
	else strcpy(freq_string, argv[4]);

	samplerate = atol(freq_string);
	if (samplerate < 1 || samplerate > 1000000) {
		printf(NL_T("Bestand: frequentie buiten bereik\n") ENG_T("File: sample frequency is out of range\n"));
		exit(1);
	}


	channels = -1;
	if (argc < 5) {
		do {
			printf(
				NL_T("Bestand: op welke kanalen ?" " (0=mono, 1=links, 2=rechts, 3=links+rechts) ")
				ENG_T("File: on which channels ?" " (0=mono, 1=left, 2=right, 3=left+right) "));
			scanf("%50s", chan_string);
			channels = atol(chan_string);
		} while (channels < 0 || channels > 3);
	}
	else strcpy(chan_string, argv[5]);

	channels = atol(chan_string);
	if (channels < 0 || channels > 3) {
		printf(
			NL_T("Foutief aantal kanalen: %d\n")
			ENG_T("Bad number of channels: %d\n")
			, channels);
		exit(1);
	}


	type = -1;
	if (argc < 6) {
		do {
			printf(
				NL_T("Sample type ?")
				ENG_T("Sample type ?")
				" (8/16/mulaw) ");
			scanf("%50s", type_string);
			if (!strcmp(type_string, "mulaw")) type = 1;
			else if (!strcmp(type_string, "8")) type = 2;
			else if (!strcmp(type_string, "16")) type = 3;
		} while (type < 1 || type > 3);
	}
	else strcpy(type_string, argv[6]);

	if (!strcmp(type_string, "mulaw")) type = 1;
	else if (!strcmp(type_string, "8")) type = 2;
	else if (!strcmp(type_string, "16")) type = 3;
	else {
		printf(
			NL_T("Sample: %s is een onbekend type\n")
			ENG_T("Sample: %s is an unknown type\n")
			, type_string);
		exit(1);
	}


	fpuit = fopen(bestand_uit, "w");
	if (fpuit == NULL) {
		fprintf(stderr,
				NL_T("%s kan niet geopend worden om naar te schrijven\n\n")
				ENG_T("%s cannot be opened for writing\n\n")
				, bestand_uit
				);
		exit(1);
	}

	/* schrijf de gegevens */
	samples = (float) samplerate / (float) toon;
	golven = (int) (tijd *(float) toon);

	for (i=0; i<golven * samples; i++) {
		int j = i;
		float vol = sin((2*M_PI* j) /(float) samples);
//fprintf(stderr, "vol=%f\n", vol);
		if (channels == 2) {
			/* schrijf links 0: */
			write_vol(0.0, type, fpuit);
		}
		write_vol(vol, type, fpuit);
		if (channels == 1) {
			/* schrijf rechts 0: */
			write_vol(0.0, type, fpuit);
		}
		if (channels == 3) write_vol(vol, type, fpuit);
	}
	fclose(fpuit);

	{
		char *argv[6];
		char beschrijving[ 256 ];
		char *kanalen;

		sprintf(beschrijving, "Toon %dHz, %s", toon, channels == 0 ? "mono" : (channels == 1 ? "linker kanaal" : "rechter kanaal"));
		if (channels == 0) kanalen = "1"; else kanalen = "2";
		argv[0] = "raw_to_au";
		argv[1] = bestand_uit;
		argv[2] = freq_string;
		argv[3] = kanalen;
		argv[4] = type_string;
		argv[5] = beschrijving;
		argv[6] = NULL;
		execvp("raw_to_au", argv);
		fprintf(stderr, "couldn't execvp raw_to_au (check if it's in a dir in PATH)!\n");
	}

	return 0;
}


void write_vol(float vol, int type, FILE *fpuit) {
	unsigned char byte;

	switch (type) {
	case 1:
		//byte = linear_16_tomulaw(vol*64*256);
		//fwrite(&byte, 1, 1, fpuit);
		fprintf(stderr, "mulaw not finished\n");
		exit(1);
		break;

	case 2:
		byte = ((int) (vol*64)) + 128;
		fwrite(&byte, 1, 1, fpuit);
		break;

	case 3:
		byte = (((int) (vol*64*256))) >> 8; /* MSB */
		fwrite(&byte, 1, 1, fpuit);
		byte = (((int) (vol*64*256))) & 255; /* LSB */
		fwrite(&byte, 1, 1, fpuit);
		break;
	}
}

/* EOF toon.c */
